SEAScope Client
===============================================================================

Compilation toolchain dependencies
----------------------------------
* wget
* unzip
* git
* tclsh
* make
* cmake
* pkg-config
* fuse library (development version)

On macOS you can use [Homebrew](https://brew.sh/) to install them:
```bash
brew install wget cmake pkg-config macfuse
```

Dynamic dependencies
--------------------
The SEAScope standalone client uses dynamic linking for OpenGL-related
and platform-dependent libraries.

The following libraries are not only required during compilation (development
version may be required depending on Linux distributions) but also at runtime:

* libX11
* libXrandr
* libXinerama
* libXrender
* libXcursor
* libXxf86vm
* libGL
* libuuid
* libpthread
* libdl

Building the application
------------------------
```bash
# Clone the repository
git clone https://git.oceandatalab.com/seascope/client.git
cd client

# Check that the requirements are met
./developer/build_standalone check

# Retrieve thirdparty source code and build libraries
./developer/build_standalone deps

# Generate bindings for remote interaction
./developer/build_standalone cpp-api

# Build the standalone client
VERSION=release
GLEW_VERSION=2.0.0 #
./developer/build_standalone "${VERSION}" "${GLEW_VERSION}"
```

Bundle macOS application
--------------------------
1. Build the application (see previous section)
2. Get the following files:
    * `./share/colormaps/`
    * `./share/images/`
    * `./share/operators/`
3. Run:
```bash
VERSION=release
VERSION_NUMBER=1.0.0

# Create macOS application
./developer/build_standalone app "${VERSION}" "${VERSION_NUMBER}"

# Create macOS DMG
./developer/build_standalone dmg "${VERSION}" "${VERSION_NUMBER}"
```
4. The following files will be added to `./${VERSION}/`:
    * `SEAScope.app`
    * `SEAScope-${VERSION_NUMBER}.dmg`

Building universal macOS application
------------------------------------
```bash
SHARE_DIR=./share
BRANCH=feature/universal-macos-app
./developer/macos/release.sh "${SHARE_DIR}" "${BRANCH}"
```

**NB:** The `SDKROOT` environment variable might need to be set to the latest
SDK ("macosx" should suffice). For more info see `man xcrun`.

Running the application
-----------------------
The application can be started by simply calling the executable on the command
line but it will only work if it is able to find a configuration file and a
suitable environment.

A SEAScope environment should look like this:

```bash
SEAScope
       \_ colormaps # .rgb files describing the colormaps
       \_ config.ini # SEAScope configuration file
       \_ data # contains the base layer texture + one directory per collection
       \_ GLSL # Copy of the shaders which can be found in this repository
       \_ images
               \_ arrows
                       \_ pgm
                            \_ 0.pgm # Texture for arrows
               \_ windBarbs
                          \_ *.png # Textures for wind barbs (PNG)
                          \_ pgm
                               \_ *.pgm # Textures for wind barbs (PGM)
       \_ operators # TODO
       \_ plugins
                \_ catalogue # Plugins for catalogue services
                \_ index # Plugins for index services
                \_ datastream # Plugins for data streaming services
```

TODO: provide a way to generate an environment

Start the executable with:
```bash
./seascope -c config.ini
```

[brew]: https://brew.sh
