// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "bindings/binary.h"

int readNetCDFWeb(const std::string& file_name, DataSet * data_set)
{
    std::string fileName = file_name;
        
    fileName += "/data";

    FILE* fp = fopen(fileName.c_str(), "rb");
    
    LOG(INFO) << "Load " << fileName.c_str();
    
    if (nullptr == fp)
    {
        printf("File %s not found.\n", fileName.c_str());
        return 1;
    }
    
    LOG(INFO) << "File open";
    
    unsigned char major,minor;

    fread(&major, 1, 1, fp);
    fread(&minor, 1, 1, fp);

    fread(&data_set->dimensionCount, sizeof(unsigned int), 1, fp);
    
    data_set->offset = new float[data_set->dimensionCount];
    data_set->scaleFactor = new float[data_set->dimensionCount];
    
    fread(data_set->offset, sizeof(float), data_set->dimensionCount, fp);
    fread(data_set->scaleFactor, sizeof(float), data_set->dimensionCount, fp);

    fread(&data_set->lon_size, sizeof(unsigned int), 1, fp);
    fread(&data_set->lat_size, sizeof(unsigned int), 1, fp);
    
    unsigned char hasGCP;
    fread(&hasGCP, sizeof(unsigned char), 1, fp);
    
    if (hasGCP > 0)
    {
        unsigned int gcpDataSize = data_set->lon_size * data_set->lat_size;

        data_set->lons = new float[gcpDataSize];
        data_set->lats = new float[gcpDataSize];
        data_set->gcpx = new unsigned short[gcpDataSize];
        data_set->gcpy = new unsigned short[gcpDataSize];
        
        fread(data_set->lons, sizeof(float), gcpDataSize, fp);
        fread(data_set->lats, sizeof(float), gcpDataSize, fp);
        fread(data_set->gcpx, sizeof(unsigned short), gcpDataSize, fp);
        fread(data_set->gcpy, sizeof(unsigned short), gcpDataSize, fp);
    }
    else
    {
        fread(data_set->lons, sizeof(float), data_set->lon_size, fp);
        fread(data_set->lats, sizeof(float), data_set->lat_size, fp);
    }

    fread(&data_set->width, sizeof(unsigned int), 1, fp);
    fread(&data_set->height, sizeof(unsigned int), 1, fp);

    unsigned int dataType;
    fread(&dataType, sizeof(unsigned int), 1, fp);
    
    if (1 == dataType)
    {
        data_set->scaleFactorGL = 255.f;
    }
    else
    {
        data_set->scaleFactorGL = 1.f;
    }
    
    data_set->data = new Image;
    data_set->dataMask = new Image;
    
    data_set->data->allocate(data_set->width, data_set->height, data_set->dimensionCount, (OGL::DataType)dataType, nullptr);
    data_set->dataMask->allocate(data_set->width, data_set->height, 1, OGL::unsigned_byte, nullptr);

    fread(data_set->data->data(), sizeof(unsigned char), data_set->data->dataSize(), fp);
    fread(data_set->dataMask->data(), sizeof(unsigned char), data_set->dataMask->dataSize(), fp);
    
    fclose(fp);
    
    LOG(INFO) << "Load done";
    
    return 0;
}

