// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   buffer.h
 * Author: gleseach
 *
 * Created on 3 septembre 2015, 11:07
 */

#ifndef BUFFER_OGL3_H
#define	BUFFER_OGL3_H

#include "3d/buffer.h"

namespace OGL3
{

class BufferGL3 final: public OGL::Buffer
{
public:
    BufferGL3(OGL::BufferType type);
    ~BufferGL3() override;
    
    void create(OGL::Context* context) override;
    void destroy() override;
    
    void bind() const override;
    void unbind() const override;
    
    /*!
     * \brief Bind the buffer to an output of a transform feedback pass.
     */
    void bindTo(int index) override;
    
    void allocate(unsigned int size, void * data, OGL::BufferUsage usage = OGL::static_draw) override;
    void allocate(const Array& array) override;
    void copy(void * data) override;
    void copy(unsigned int offset, unsigned int size, void* data) override;
    
    void copyTo(OGL::Buffer* buffer, unsigned int readOffset, unsigned int writeOffset, unsigned int size) override;
    
    void* map(OGL::BufferAccess access) override;
    void unmap() override;
};

} // namespace OGL3

#endif	/* BUFFER_OGL3_H */

