// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   context.h
 * Author: gleseach
 *
 * Created on 26 mai 2016, 12:39
 */

#ifndef CONTEXT_OGL3_H
#define CONTEXT_OGL3_H

#include "3d/context.h"

namespace OGL3
{
    
class ContextGL3 : public OGL::Context
{
public:
    ContextGL3();
    
    bool                       init(const SEAScope::RendererResources*) override;
    OGL::Texture*              createTexture(OGL::TextureType type) override;
    OGL::Buffer*               createBuffer(OGL::BufferType type) override;
    OGL::FrameBuffer*          createFramebuffer() override;
    OGL::Program*              createProgram() override;
    OGL::Shader*               createShader(OGL::ShaderType type, const std::string& name) override;
    OGL::RenderBuffer*         createRenderbuffer(OGL::RenderBufferType type) override;
    OGL::VertexArrayObject*    createVertexArrayObject() override;
    OGL::VertexAttribArray*    createVertexAttribArray(unsigned int id, unsigned int component_count, OGL::Buffer * buffer, DataType dataType = DataType::float32, unsigned int stride = 0, unsigned int offset = 0) override;
    OGL::VertexAttribArray*    createVertexAttribArray(unsigned int id, const TypedBuffer& typedBuffer, unsigned int stride = 0, unsigned int offset = 0) override;
    OGL::DrawMode*             createDrawMode() override;
    OGL::Utils2*               createUtils() override;
    OGL::ProgramBindHelper*    createProgramHelper() override;
    OGL::Query*                createQuery() override;
    OGL::Sync*                 createSync() override;
    OGL::VertexAttribDivisor*  createVertexAttribDivisor(OGL::VertexAttribArray* vaa, unsigned int divisor) override;

    std::string getShaderPath() const override;
    void activeTextureUnit(unsigned int id) override;
    void activeTextureUnit(unsigned int id, OGL::Texture* object) override;
    void deactiveTextureUnit(unsigned int id) override;
    void setPolygonMode(OGL::Face face, OGL::RasterizationMode mode) override;
    void setFlag(OGL::Capability cap, bool status) override;
    void setViewport(const glm::ivec4& size) override;
    glm::ivec4 getViewportSize() override;
    void setClearColor(const glm::vec4& color) override;
    void clear(int bits) override;
    void setDepthMask(bool status) override;
    void setColorMask(bool red, bool green, bool blue, bool alpha) override;
    void setBlendFunc(OGL::BlendingFactor sourceFactor, OGL::BlendingFactor destinationFactor) override;
    void setStencilMask(bool status) override;
    void setStencilFunc(OGL::Func func, int ref, unsigned int mask) override;
    void setStencilOp(OGL::StencilOp sfail, OGL::StencilOp dpfail, OGL::StencilOp dppass) override;
	const SEAScope::RendererResources* getResources() const override;
    void setLineWidth(float val) override;

    int getGPUInfoInteger(OGL::GPUInfoInteger val) const override;
    float getGPUInfoFloat(OGL::GPUInfoFloat val) const override;
    
private:
    void loadShaders();
};

} // namespace OGL3

#endif /* CONTEXT_OGL3_H */

