// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef FRAMEBUFFEROBJECT_OGL3_H
#define FRAMEBUFFEROBJECT_OGL3_H

#include "3d/framebuffer.h"

namespace OGL3
{
    
class FrameBufferGL3 final: public OGL::FrameBuffer
{
public:
    FrameBufferGL3();
    FrameBufferGL3(unsigned int renderTargetCount, bool useDepth);
    ~FrameBufferGL3() override;
    
    void create(OGL::Context* context) override;
    void destroy() override;
    void bind() const override;
    void unbind() const override;
    
    bool allocate(unsigned int width, unsigned int height, DataType dataType) override;
    void resize(unsigned int width, unsigned int height) override;
    
    glm::vec3 readPixel(int x, int y) const override;
    
    /*!
     * \brief Async reading of a three components RGB pixel. Bind a PBO before.
     */
    void readPixelAsync(int x, int y) const override;

    void blit(unsigned int width, unsigned int height) override;
    void dumpToImage(Image& img) override;
};

} // namespace OGL3

#endif /* FRAMEBUFFEROBJECT_H */

