// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   Program.h
 * Author: gleseach
 *
 * Created on 6 août 2015, 15:23
 */

#ifndef PROGRAM_OGL3_H
#define	PROGRAM_OGL3_H

#include "3d/program.h"

namespace OGL
{
    class Shader;
}

namespace OGL3
{
    
class ProgramGL3 final: public OGL::Program
{
public:
    ProgramGL3();
    ~ProgramGL3() override;
    
    void create(OGL::Context* context) override;
    void destroy() override;
    
    void attachShaders(const std::vector<OGL::Shader*>& shaders) override;
    void attachShaders(const OGL::Shader* shaders, unsigned int size) override;
    bool link() override;
    
    void bind() const override;
    void unbind() const override;
    
    void setTransformFeedbackVaryings(const std::vector<std::string>& varyings, OGL::TransformFeedbackBufferMode bufferMode) override;
    
    void bindAttribLocation(unsigned int pos, const char *name) override;
    int getUniformLocation(const char *name) override;
    int getAttribLocation(const char *name) override;
    
    //! Return the number of uniform used by the program.
    int getUniformCount() const override;
    
    //! Return the name of the uniform with the id passed in parameter.
    std::string getUniformName(int id) const override;
    
    //! Return the type of the uniform with the id passed in parameter.
    OGL::UniformType getUniformType(int id) const override;
    
    void uniform1i(int location, const int& value) override;
    void uniform1u(int location, const std::uint32_t& value) override;
    void uniform1f(int location, const float& value) override;
    void uniform2fv(int location, const glm::vec2& value) override;
    void uniform3fv(int location, const glm::vec3& value) override;
    void uniform4fv(int location, const glm::vec4& value) override;
    void uniformMatrix4fv(int location, const glm::mat4& value) override;
    void uniform2bv(int location, const glm::bvec2& value) override;
};

} // namespace OGL

#endif	/* PROGRAM_H */
