// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ANCHOR_POOL_NODE_H
#define ANCHOR_POOL_NODE_H

#include <vector>
#include "3d/scene_graph_node.h"

// Forward declarations
class ShapeAnchor;
class AnchorController;
class Icons;

namespace OGL
{
    class Context;
    class StaticMesh;
    class Texture;
}

namespace SEAScope
{
    class AppEventsDispatcher;
}

class AnchorPoolNode : public Node
{
    public:
        AnchorPoolNode(SEAScope::AppEventsDispatcher&);
        ~AnchorPoolNode() override;

        AnchorPoolNode(const AnchorPoolNode&) = delete;
        AnchorPoolNode& operator=(const AnchorPoolNode&) = delete;

        bool initialize(OGL::Context* ctx, const Icons&);

        void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
        void drawPicking(OGL::Context* ctx, const glm::mat4& mvp) override;

        void setDecoration(AnchorType type, ObjectCursorState state, OGL::Texture*);

        void removeAnchor(AnchorType, unsigned int id);
        void clear();

        void showAll();
        void showOnly(AnchorType type, unsigned int id);

        std::vector<std::vector<ShapeAnchor*>>& getAnchors() { return this->_anchors; }

        void addAnchor(AnchorController*, AnchorType type, unsigned int id);

    protected:
        SEAScope::AppEventsDispatcher& _appEventsDispatcher;
        OGL::StaticMesh* _screenQuadMesh;
        std::vector<std::vector<ShapeAnchor*>> _anchors;
        std::vector<std::vector<OGL::Texture*>> _anchorDecoration;
};

#endif
