// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   app_enum.h
 * Author: gleseach
 *
 * Created on 21 septembre 2015, 14:53
 */

#ifndef APP_ENUM_H
#define	APP_ENUM_H

#include <vector>
#include "vendors/_glm.h"

enum RenderMode
{
    planar = 0,
    spherical
};

enum ColorMode
{
    fillColor = 0,
    falseColor
};

enum CameraMode
{
    ortho = planar,
    trackball = spherical,
    freefly = 2,
    count
};

enum StyleState
{
    normal      = 0,
    selected,
    edition,
    styleCount
};

enum RenderableFlags
{
    pickable = 0,
    moveable,
    anchor,
    editable,
    erasable,
    renderableFlagsCount
};

enum AnchorType
{
	PointMoveable = 0,
	PointAdd,
    AnchorTypeCount
};

enum ObjectCursorState
{
	ObjectNormal = 0, // Cursor not over the object
	ObjectHighlight, // Cursor over the object
	ObjectSelected, // Object selected
	ObjectStateCount
};

enum MovementType
{
    teleport
};

typedef std::vector<glm::vec3>      VerticeArray;
typedef std::vector<glm::vec2>      TexCoordArray;
typedef std::vector<glm::vec3>      NormalArray;
typedef std::vector<unsigned int>   IndiceArray;

#endif	/* APP_ENUM_H */

