// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   array.h
 * Author: gleseach
 *
 * Created on 14 juin 2016, 19:04
 */

#ifndef ARRAY_H
#define ARRAY_H

#include "3d/opengl_enum.h"
#include "3d/buffer_desc.h"

/*!
 * \brief Describe the data stored in a raw buffer in an OpenGL way.
 */
class Array
{
public:
    /*!
     * \brief Default constructor.
     */
    Array();
    
    /*!
     * \brief Constructor.
     * param[in] length Number of elements in the array.
     * param[in] count Number of components in an element.
     * param[in] dataType Basic type of a component (int , float ...).
     * param[in] data Pointer to the raw buffer.
     * \note The size of a component is deduced from the data type.
     */
    Array(const BufferDesc& desc, void* data);
    
    //! Pointer to the raw buffer.
    void* getDataPtr() const;
    
    const BufferDesc& getDescription() const;
    
private:
    BufferDesc _desc;
    void* _data;
};

#endif /* ARRAY_H */
