// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef BILLBOARD_H
#define BILLBOARD_H

#include "3d/shape_mesh.h"

namespace OGL
{
    // Forward declarations
    class Context;
    class Texture;
    class StaticMesh;
}

// Forward declarations
class Point;

class Billboard: public ShapeMesh
{
    public:
        Billboard();
        ~Billboard() override;

        void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
        void drawPicking(OGL::Context* ctx, const glm::mat4& mvp) override;

        virtual const glm::vec3 getPosition() const;
        virtual void setPosition(const glm::vec3&);

        void initialize(Point*);
        void setMesh(OGL::StaticMesh*);
        void setTexture(OGL::Texture*);
    protected:
        OGL::StaticMesh* _mesh;
        OGL::Texture* _texture;
        glm::vec3 _position;
};

#endif
