// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   buffer.h
 * Author: gleseach
 *
 * Created on 3 septembre 2015, 11:07
 */

#ifndef BUFFER_H
#define	BUFFER_H

#include "3d/bindable_object.h"
#include "3d/opengl_enum.h"

class Array;

namespace OGL
{

class Buffer : public BindableObject
{
public:
    Buffer(OGL::BufferType type);
    virtual ~Buffer() override;

    virtual void bindTo(int index) = 0;

    virtual void allocate(unsigned int size, void * data, OGL::BufferUsage usage = OGL::static_draw) = 0;
    virtual void allocate(const Array& array) = 0;

    // CPU to GPU, full array
    virtual void copy(void * data) = 0;

    virtual void* map(OGL::BufferAccess access) = 0;
    virtual void unmap() = 0;

    /*! 
     * \brief CPU to GPU, sub array
     * \param[in] writeOffset Position to write in the buffer.
     * \param[in] size Size of the data to write.
     * \param[in] data Pointer to the data to write.
     * */
    virtual void copy(unsigned int writeOffset, unsigned int size, void * data) = 0;

    // GPU to GPU
    virtual void copyTo(OGL::Buffer* buffer, unsigned int readOffset, unsigned int writeOffset, unsigned int size) = 0;

    BufferType getBufferType() const {return _type;}

    // Size of the buffer in bytes.
    unsigned int getSize() const { return this->_size; }

protected:
    BufferType _type;
    unsigned int _size;
};

} // namespace OGL

#endif	/* BUFFER_H */

