// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   BufferDesc.h
 * Author: gleseach
 *
 * Created on 3 octobre 2016, 11:42
 */

#ifndef BUFFERDESC_H
#define BUFFERDESC_H

#include "3d/opengl_enum.h"

/*!
 * \brief Buffer description. Number of element in the array, number of components in an element and size and type of a component.
 */
class BufferDesc
{
public:
    BufferDesc();
    BufferDesc(unsigned int length, unsigned int componentCount, DataType dataType);

    //! Return the number of elements in the array.
    unsigned int getLength() const;
    //! Return the number of components in an element.
    unsigned int getComponentCount() const;
    //! Return the type of a component.
    DataType getDataType() const;

    //! Return the size in byte of a component.
    unsigned int getComponentSize() const;
    //! Return the size in byte of an element.
    unsigned int getElementSize() const;
    //! Return the size in byte of the full array.
    unsigned int getArraySize() const;

private:
    unsigned int _length;
    unsigned int _componentCount;
    DataType _dataType;
};

#endif /* BUFFERDESC_H */
