// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef CAMERA_H
#define CAMERA_H

#include "3d/frustum.h"

class Camera
{
public:
    Camera();
    ~Camera();

    // Matrix transform
    void lookAt(const glm::vec3& eye, const glm::vec3& center, const glm::vec3& up);
    void perspective(float fovy, float aspect, float near, float far);
    void ortho(float left, float right, float bottom, float top);

    // Setter
    void setViewMatrix(const glm::mat4& view);
    void setModelMatrix(const glm::mat4& model);

    // Getter
    const glm::mat4& getProjectionMatrix() const {return _projection;}
    const glm::mat4& getViewMatrix() const {return _view;}
    const glm::mat4& getModelMatrix() const {return _model;}
    const Frustum& getFrustum() const {return _frustum;}

    glm::vec3 getWorldPosition() const;

private:
    glm::mat4 _projection;
    glm::mat4 _view;
    glm::mat4 _model;
    Frustum _frustum;
};

#endif /* CAMERA_H */
