// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   colormap_manager.h
 * Author: gleseach
 *
 * Created on 6 juin 2016, 18:56
 */

#ifndef COLORMAP_MANAGER_OGL_H
#define COLORMAP_MANAGER_OGL_H


namespace SEAScope
{
    class ColormapManager;
}

namespace OGL
{

class Texture;
class Context;

/*!
 * \brief Store colormap object create in gpu memory.
 */
class ColormapManagerGPU
{
public:
    ColormapManagerGPU();
    ~ColormapManagerGPU();

    void init(OGL::Context* ctx, SEAScope::ColormapManager* mngr);
    void clear();

    OGL::Texture* getColormap(OGL::Context* ctx, const std::string& colormapName);
    OGL::Texture * loadColormap(OGL::Context* ctx, const std::vector<std::array<uint8_t, 3>>& rgb);

private:
    SEAScope::ColormapManager* _mngr;
    std::map<std::string, OGL::Texture*> _colormapCache;
};

}

#endif /* COLORMAP_MANAGER_OGL_H */

