// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   data_rendering_param.h
 * Author: gleseach
 *
 * Created on 26 décembre 2016, 17:07
 */

#ifndef DATA_RENDERING_PARAM_H
#define DATA_RENDERING_PARAM_H

#include "vendors/_glm.h"
#include "3d/opengl_enum.h"

namespace OGL
{
    class Texture;
}

struct Renderable;

struct DataRenderingParam
{
    Renderable * data;
    OGL::Texture* colormap;
    bool useRange;
    glm::vec2 range;
    float alpha;
	bool log;
    bool visible;
    unsigned int id;
    float zIndex;
    glm::vec3 color;
    OGL::FilterMode filterMode;
    unsigned int particlesCount;
    unsigned int streamlineLength;
    float streamlineSpeed;
    float lineThickness;

    float billboardsSize;
    float billboardsDensity;

    DataRenderingParam()
    {
        data = nullptr;
        colormap = nullptr;

        useRange = false;
        alpha = 1.f;
        log = false;
        visible = true;
        id = 0;
        zIndex = 1.f;
        filterMode = OGL::linear;

        particlesCount = 10000;
        streamlineLength = 20;
        streamlineSpeed = 1.f;

        lineThickness = 1.f;

        billboardsSize = 32.f;
        billboardsDensity = 0.f;
    }
};

#endif /* DATA_RENDERING_PARAM_H */

