// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/*
 * File:   draw_mode.h
 * Author: gleseach
 *
 * Created on 27 mai 2016, 16:17
 */

#ifndef DRAW_MODE_H
#define DRAW_MODE_H

#include "3d/opengl_enum.h"

namespace OGL
{

    class DrawMode
    {
    public:
        DrawMode();
        virtual ~DrawMode();

        void setDrawType(OGL::DrawType drawType) { _drawType = drawType; }
        void setPrimitive(OGL::Primitive primitive) { _primitive = primitive; }
        void setElementCount(unsigned int count) { _elementCount = count; }
        void setInstanceCount(unsigned int count) { _instanceCount = count; }
        void setDataType(DataType dataType) { _dataType = dataType; }

        virtual void draw() = 0;
        virtual void drawInstanced() = 0;
        virtual void drawTransformedFeedback() = 0;
        virtual void drawSubset(unsigned int offset, unsigned int count) = 0;

    protected:
        OGL::DrawType _drawType;
        OGL::Primitive _primitive;
        DataType _dataType;
        unsigned int _elementCount;
        unsigned int _instanceCount;
    };

} // namespace OGL

#endif /* DRAW_MODE_H */

