// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef EAR_CLIPPING_H
#define EAR_CLIPPING_H

class EarClipping
{
public:
	void run(const std::vector<glm::vec3>& points, std::vector<unsigned int>& outIndices);

private:
	struct p3
	{
		unsigned int prev;
		unsigned int curr;
		unsigned int next;
		bool convex;
		bool ear;

        p3()
        : prev{0u}, curr{0u}, next{0u}, convex{false}, ear{false}
        {}
	};

private:
	int computeOrientation(const std::vector<glm::vec3>& points);
	bool isConvex(const glm::vec3& p0, const glm::vec3& p1, const glm::vec3& p2);
	void updatePoint(const std::vector<glm::vec3>& points, p3& p);

private:
	std::vector<unsigned int> _indices;
	std::vector<p3> _pointsArray;
	std::list<p3> _ears;
};

#endif
