// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ELLIPSE_H
#define ELLIPSE_H

#include "3d/rectangle_base.h"

class Ellipse : public RectangleBase
{
public:
	Ellipse();
	virtual ~Ellipse() override;

	Ellipse* clone() const override;

	//! Create an ellipse with beta point oriented to the north pole.
	void initialize(const glm::vec3& center, float alphaAngle, float betaAngle);

	//! Create an ellipse with three points.
	void initialize(const glm::vec3& center, const glm::vec3& alphaPoint, const glm::vec3& betaPoint);

	const glm::vec3& getCenter() const { return _center; }
	const glm::vec3& getAlphaPoint() const { return _alphaPoint; }
	const glm::vec3& getBetaPoint() const { return _betaPoint; }
	float getAlphaAngle() const { return _alphaAngle; }
	float getBetaAngle() const { return _betaAngle; }


	void setCenter(const glm::vec3& val);
	void setAlphaPoint(const glm::vec3& val);
	void setBetaPoint(const glm::vec3& val);

	// Compute a point on an ellipse centered at north pole.
	static glm::vec3 computePoint(float a, float alpha, float beta);

	// Compute a point on the ellipse defined by all parameters.
	glm::vec3 computePoint(float a);

private:

private:
	glm::vec3 _center;
	glm::vec3 _betaPoint; // y
	glm::vec3 _alphaPoint; // x
	float _alphaAngle;
	float _betaAngle;
	glm::quat _rotationZ;
	glm::quat _rotationGlobe;
};

#endif // ELLIPSE_H
