// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ELLIPSE_MESH_H
#define ELLIPSE_MESH_H

#include "shape_mesh.h"

namespace OGL
{
	class Buffer;
	class VertexArrayObject;
	class DrawMode;
	class Program;
}

class Ellipse;
class TrajectoryColor;

class EllipseMesh : public ShapeMesh
{
public:
	EllipseMesh();
	~EllipseMesh() override;

	void create(OGL::Context* ctx, Ellipse* ellipse);
	void update(Ellipse* ellipse);

	void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
	void drawPicking(OGL::Context* ctx, const glm::mat4& mvp) override;

private:
	void computePoints(Ellipse* e, std::vector<glm::vec3>& points);

private:
	//TrajectoryColor* _renderer;
	//OGL::Program* _program;
	OGL::Buffer* _verticeBuffer;
	OGL::Buffer* _directionBuffer;
	OGL::Buffer* _indicesBuffer;
	OGL::VertexArrayObject* _vao;
	OGL::DrawMode* _drawMode;
};

#endif // ELLIPSE_MESH_H
