// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   Ellipsoid.h
 * Author: gleseach
 *
 * Created on 25 août 2015, 09:55
 */

#ifndef ELLIPSOID_H
#define	ELLIPSOID_H

#include <list>
#include <vector>
#include "3d/geodetic.h"

typedef glm::vec3 Vector3D;

class Ellipsoid
{
public:
    Ellipsoid(double x, double y, double z);
    Ellipsoid(const Vector3D& radii);
    Vector3D CentricSurfaceNormal(const Vector3D& positionOnEllipsoid);
    Vector3D GeodeticSurfaceNormal(const Vector3D& positionOnEllipsoid);
    Vector3D GeodeticSurfaceNormal(const Geodetic3D& geodetic);
    const Vector3D& Radii();
    const Vector3D& RadiiSquared();
    const Vector3D& OneOverRadiiSquared();
    double MinimumRadius();
    double MaximumRadius();
    int Intersections(const Vector3D& origin, const Vector3D& direction, double* result);
    Vector3D ToVector3D(const Geodetic2D& geodetic);
    Vector3D ToVector3D(const Geodetic3D& geodetic);
    void ToGeodetic3D(const std::vector<Vector3D>& positions, std::vector<Geodetic3D>& geodetics);
    void ToGeodetic2D(const std::vector<Vector3D>& positions, std::vector<Geodetic2D>& geodetics);
    Geodetic2D ToGeodetic2D(const Vector3D& positionOnEllipsoid);
    Geodetic3D ToGeodetic3D(const Vector3D& position);
    Vector3D ScaleToGeodeticSurface(const Vector3D& position);
    Vector3D ScaleToGeocentricSurface(const Vector3D& position);
    std::list<Vector3D> ComputeCurve(const Vector3D& start, const Vector3D& stop, double granularity);

private:
    Vector3D _radii;
    Vector3D _radiiSquared;
    Vector3D _radiiToTheFourth;
    Vector3D _oneOverRadiiSquared;
};

#endif	/* ELLIPSOID_H */

