// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   extractor.h
 * Author: gleseach
 *
 * Created on 19 octobre 2016, 16:03
 */

#ifndef EXTRACTOR_H
#define EXTRACTOR_H

#include "core/structs.h"
#include "3d/renderable.h"

namespace OGL
{
    class Context;
    class FrameBuffer;
}

class Image;
class PolygonMesh;
struct Data;
class SceneView;

class Extractor
{
public:
    Extractor();
    ~Extractor();

    void create(OGL::Context* ctx, int width, int height);
    bool extract(OGL::Context* ctx, Renderable2* polygon, SceneView& sceneView, std::map<SEAScope::RenderableId, SEAScope::Extraction>& result, bool discardExtractedData);
    void resize(int width, int height);

    OGL::FrameBuffer* getFBO() const;

private:
    SEAScope::GCP createGCP(const glm::vec3& point, const glm::mat4& mvp, const glm::ivec4& viewport) const;

private:
    OGL::FrameBuffer* _fbo;
};

#endif /* EXTRACTOR_H */

