// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef FRAMEBUFFEROBJECT_H
#define FRAMEBUFFEROBJECT_H

#include "3d/bindable_object.h"
#include "3d/opengl_enum.h"

class Image;

namespace OGL
{

class Texture;
class RenderBuffer;
class Context;

class FrameBuffer : public BindableObject
{
public:
    FrameBuffer();
    FrameBuffer(unsigned int renderTargetCount, bool useDepth);
    virtual ~FrameBuffer() override;

    void setBuffers(unsigned int bufferBit) {_bufferBit = bufferBit;}

    virtual bool allocate(unsigned int width, unsigned int height, DataType dataType) = 0;
    virtual void resize(unsigned int width, unsigned int height) = 0;

    OGL::Texture * getTexture() const {return _texture[0];}
    OGL::Texture * getTexture(unsigned int id) const {return _texture[id];}

    virtual glm::vec3 readPixel(int x, int y) const = 0;
    virtual void readPixelAsync(int x, int y) const = 0;

    virtual void blit(unsigned int width, unsigned int height) = 0;

    virtual void dumpToImage(Image& img) = 0;

protected:
    OGL::Texture** _texture;
    OGL::RenderBuffer* _renderBuffer;
    unsigned int _renderTargetCount;
    unsigned int* _colorAttachement;
    bool _useDepth;
    unsigned int _bufferBit;
    DataType _dataType;
    unsigned int _componentCount;
};

} // namespace OGL

#endif /* FRAMEBUFFEROBJECT_H */

