// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   Geodetic.h
 * Author: gleseach
 *
 * Created on 25 août 2015, 10:58
 */

#ifndef GEODETIC_H
#define	GEODETIC_H

class Geodetic2D;

class Geodetic3D
{
public:
    // In radians !
    Geodetic3D(double longitude, double latitude, double height);
    Geodetic3D(double longitude, double latitude);
    Geodetic3D(const Geodetic2D& geodetic2D);
    Geodetic3D(const Geodetic2D& geodetic2D, double height);
    double Longitude() const;
    double Latitude() const;
    double Height() const;
    bool Equals(const Geodetic3D& other) const;

    friend bool operator ==(const Geodetic3D& left, const Geodetic3D& right);
    friend bool operator !=(const Geodetic3D& left, const Geodetic3D& right);

private:
    double _longitude;
    double _latitude;
    double _height;
};

class Geodetic2D
{
public:
    // In radians !
	Geodetic2D() { _longitude = _latitude = 0.0; }
    Geodetic2D(double longitude, double latitude);
    Geodetic2D(const glm::vec2&);
    Geodetic2D(const Geodetic3D& geodetic3D);
    double Longitude() const;
    double Latitude() const;
    bool EqualsEpsilon(const Geodetic2D& other, double epsilon) const;
    bool Equals(const Geodetic2D& other) const;
    friend bool operator ==(const Geodetic2D& left, const Geodetic2D& right);
    friend bool operator !=(const Geodetic2D& left, const Geodetic2D& right);

    glm::vec2 toVec2() const {return glm::vec2(Longitude(), Latitude());}

private:
    double _longitude;
    double _latitude;
};

#endif	/* GEODETIC_H */
