// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   geometry.h
 * Author: gleseach
 *
 * Created on 25 juillet 2017, 12:07
 */

#ifndef GEOMETRY_H
#define GEOMETRY_H

#include "3d/stdafx.h"

/*! \brief Compute intersection between polyline segments.
 *  \return false if an intersection is found.
 */
bool isPolygonValid(const std::vector<glm::vec3>& points);

/*!
 *  \brief Subdivise triangles until they validate the stopping criteria.
 *  \param[in] positions Vertices positions.
 *  \param[in] indices Triangles indices.
 *  \param[in] granularity Stopping criteria, an angle in radian (minimal size of a triangle).
 *  \param[out] outPositions New array of vertices positions.
 *  \param[out] outIndices New array of triangles indices.
 */
void subdivise(const std::vector<glm::vec3>& positions, const std::vector<unsigned int>& indices, float granularity, std::vector<glm::vec3>& outPositions, std::vector<unsigned int>& outIndices);


void subdiviseContour(const std::vector<glm::vec3>& points, float angleMax, std::vector<glm::vec3>& outPositions);

/*!
 * \brief Triangulate a polyline with the ear clipping algorithm.
 * \param[in] positions Points of the polyline.
 * \param[out] outIndices Indices of the triangles.
 * \warning Need a polyline without segment intersection.
 */
bool triangulateEarClipping(const std::vector<glm::vec3>& points, std::vector<unsigned int>& outIndices);

#endif /* GEOMETRY_H */

