// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   globe.h
 * Author: gleseach
 *
 * Created on 30 janvier 2017, 17:56
 */

#ifndef GLOBE_H
#define GLOBE_H

#include "3d/pickable.h"

namespace OGL
{
    class StaticMesh;
    class Texture;
}

class Globe : public Pickable
{
public:
    Globe(const SEAScope::RenderableId& rId);
    Globe(const Globe& src)
    : Pickable(src), _mesh(src._mesh), _worldTexture(src._worldTexture)
    {
    }
    ~Globe() override;

    void create(OGL::Context* ctx, OGL::Texture* worldMapTexture);

    // Overrided
    void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
    void drawPicking(OGL::Context* ctx, const glm::mat4& mvp) override;

private:
    OGL::StaticMesh* _mesh;
    OGL::Texture* _worldTexture;
};

#endif /* GLOBE_H */

