// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   globe_scene.h
 * Author: gleseach
 *
 * Created on 30 janvier 2017, 18:04
 */

#ifndef GLOBE_SCENE_H
#define GLOBE_SCENE_H

#include "3d/iscene.h"
#include "3d/globe.h"
#include "3d/scene_graph.h"
#include "3d/scene_graph_controller.h"

class Camera;
class Billboard;
class DataSetVisibility;

namespace OGL
{
    class Context;
    class Texture;
    class Shape;
}

namespace SEAScope
{
    struct RenderableId;
    class AppEventsDispatcher;
}

class GlobeScene : public IScene
{
public:
    GlobeScene(SEAScope::AppEventsDispatcher&);
    ~GlobeScene() override;

    void create(OGL::Context* ctx, IWorldVisibility*) override;
    void addWorldToSceneGraph(OGL::Context* ctx, OGL::Texture*) override;

    /*!
     * \brief Compute the lon/lat coordinates of a cartesian coordinate on the globe.
     * \param[in] position A position on the globe.
     * \return The longitude, latitude of the position in degrees.
     */
    glm::vec2 worldPositionToLonLat(const glm::vec3& position) const override;
    float getAltitude(glm::vec3) const override;
    float getZoom(float) const override;

    SceneGraphController* getNodeFactory() override;
    SceneGraph* getSceneGraph();

    bool addShape(OGL::Context*, OGL::Shape*, const SEAScope::RenderableId&, const SEAScope::RenderingCfg&, const Icons&) override;
    bool addGranule(OGL::Context*, DataSet*, const SEAScope::RenderingCfg&, DataSetVisibility*) override;
    bool addTrajectory(OGL::Context*, std::vector<float>&, std::vector<Geodetic2D>&, std::vector<float>&, std::vector<bool>&, const SEAScope::RenderingCfg&, const SEAScope::RenderableId&) override;

protected:
    SceneGraphController _nodeFactory;
    SceneGraph _sceneGraph;

    Globe _globe;
    OGL::StaticMesh* _pointBillboardStaticMesh;
    OGL::Texture* _pointBillboardTexture;
};

#endif /* GLOBE_SCENE_H */
