// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   globe_visibility.h
 * Author: gleseach
 *
 * Created on 22 mars 2017, 16:15
 */

#ifndef GLOBE_VISIBILITY_H
#define GLOBE_VISIBILITY_H

#include "3d/plane.h"
#include "3d/spherical_quad.h"
#include "3d/iworld_visibility.h"

#define VISIBILITY_TOP      2
#define VISIBILITY_BOTTOM   3
#define VISIBILITY_LEFT     0
#define VISIBILITY_RIGHT    1

class GlobeVisibility : public IWorldVisibility
{
public:
    GlobeVisibility();
    ~GlobeVisibility() override;

    bool isVisible(const glm::vec3& p) override;
    void update() override;

    SphericalQuad* getVisibleQuad();
    Plane* getClippingPlane() {return &_zLimit;}

    /*
     * \brief Flag to know if space is visible or not in the viewport.
     * \return true a chunk of space is visible, false only the globe is visible.
     */
    bool isSpaceVisible() {return _spaceVisible;}

private:
    Plane _zLimit;
    SphericalQuad _quad;
    bool _spaceVisible;
};

#endif /* GLOBE_VISIBILITY_H */
