// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   helper.h
 * Author: gleseach
 *
 * Created on 14 juin 2016, 18:14
 */

#ifndef HELPER_H
#define HELPER_H

#include "3d/opengl_enum.h"

class Image;
class Array;
class TypedBuffer;

namespace OGL
{

class Texture;
class Buffer;
class FrameBuffer;
class Context;
class Program;
class ProgramBindHelper;
class DrawMode;
class VertexAttribArray;

class Helper
{
public:
    static OGL::Texture*     createTexture(OGL::Context* ctx, const Image& image);
    static OGL::Buffer*      createBuffer(OGL::Context* ctx, OGL::BufferType type, const Array& array);
    static OGL::FrameBuffer* createFramebuffer(OGL::Context* ctx, unsigned int width, unsigned int height, DataType dataType);
    static OGL::Program*     createProgram(OGL::Context* ctx, const std::string& vertexShaderName, const std::string& fragmentShaderName);
    static OGL::ProgramBindHelper*     createProgramAutoBind(OGL::Context* ctx, const std::string& vertexShaderName, const std::string& fragmentShaderName);
    static OGL::DrawMode*   createDrawModeArrays(OGL::Context* ctx, OGL::Primitive primitive, unsigned int elementCount);
    static OGL::DrawMode*   createDrawModeElements(OGL::Context* ctx, OGL::Primitive primitive, DataType dataType, unsigned int elementCount);
    static OGL::VertexAttribArray* createVertexAttribArray(OGL::Context* ctx, int binding, const TypedBuffer& typedBuffer, unsigned int stride = 0, unsigned int offset = 0);
};

} // namespace OGL

#endif /* HELPER_H */

