// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ICONS_H
#define ICONS_H

#include <unordered_map>
#include <map>

#include "core/icons_enum.h"

namespace OGL
{
    class Context;
    class Texture;
}

class Image;

class Icons
{
    public:
        Icons();
        virtual ~Icons();

        void setTexture(IconType, OGL::Texture*, const std::string& key);
        void setTexture(std::uint16_t iconType, OGL::Texture* texture, const std::string& key);
        void setTexture(OGL::Context* ctx, std::uint16_t iconType, const Image& img, const std::string&);
        void setTexture(IconType iconType, OGL::Context* ctx, const std::string& path, const std::string& key);

        OGL::Texture* getTexture(IconType, bool&) const;
        OGL::Texture* getTexture(std::uint16_t iconType, bool&) const;
        OGL::Texture* getTexture(const std::string& name, bool&) const;

        const std::unordered_map<std::uint16_t, OGL::Texture*, IconTypeHash>& getTextures() const { return _textures; }
        std::string getName(const std::uint16_t& id) const;
        bool getId(const std::string&, std::uint16_t&) const;

protected:
        std::unordered_map<std::uint16_t, OGL::Texture*, IconTypeHash> _textures;
        std::map<const std::string, std::uint16_t> _nameToId;
        std::map<std::uint16_t, std::string> _idToName;
};

#endif // ICONS_H
