// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   ievent_manager.h
 * Author: gleseach
 *
 * Created on 11 juillet 2017, 10:34
 */

#ifndef IEVENT_MANAGER_H
#define IEVENT_MANAGER_H

#include "vendors/_glm.h"

namespace SEAScope
{
	struct RenderableId;
}

class IEventManager
{
public:
    IEventManager() {}
    virtual ~IEventManager();

    //! Event method to capture mouse button event
    virtual bool handleMouseButtonEvent(int /*button*/, int /*state*/, int /*mods*/, const glm::ivec2& /*screenPos*/) {return false;}

    /*
     * \brief Specialized event method.
     * \param[in] x Coordinate of the cursor in width
     * \param[in] y Coordinate of the cursor in height
     * \param[in] onGlobe "cursor on the globe" flag
     * \param[in] pos Cursor coordinates on the globe, undefined if onGlobe is false.
     * \return Event captured or not.
     */
    virtual bool handleMouseMoveEvent(const glm::ivec2& /*screenPos*/, bool /*onGlobe*/, const glm::vec3& /*globePos*/) {return false;}

    virtual void updateAsync() = 0;

	//! Delete the selected object, remove it from the scene
	virtual bool deleteSelected(SEAScope::RenderableId& rId) = 0;
};

#endif /* IEVENT_MANAGER_H */
