// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   iscene.h
 * Author: gleseach
 *
 * Created on 30 janvier 2017, 18:02
 */

#ifndef ISCENE_H
#define ISCENE_H

#include <functional>

#include "core/structs.h"
#include "3d/scene_graph_node.h"
#include "3d/app_enum.h"

namespace OGL
{
    class Context;
    class Shape;
}

namespace SEAScope
{
    struct RenderableId;
    class RenderingCfg;
}

class Camera;
class Geodetic2D;
class IWorldVisibility;
class SceneGraphController;
class DataSetVisibility;
class DataSet;
class Icons;

class IScene
{
public:
    IScene()
    : _sgc{nullptr}, _pickingCallback{nullptr}
    {}

    virtual ~IScene();

    virtual void create(OGL::Context* ctx, IWorldVisibility*) = 0;
    virtual void addWorldToSceneGraph(OGL::Context* ctx, OGL::Texture*) = 0;

    void setSceneGraphController(SceneGraphController* sg) {_sgc = sg;}

    void setPickingUpdateCallback(const std::function<void()>& callback) {_pickingCallback=callback;}

    virtual glm::vec2 worldPositionToLonLat(const glm::vec3& position) const = 0;
    virtual float getAltitude(glm::vec3) const = 0;
    virtual float getZoom(float) const = 0;

    virtual SceneGraphController* getNodeFactory() = 0;
    virtual bool addShape(OGL::Context*, OGL::Shape*, const SEAScope::RenderableId&, const SEAScope::RenderingCfg&, const Icons&) = 0;

    virtual bool addGranule(OGL::Context*, DataSet*, const SEAScope::RenderingCfg&, DataSetVisibility*) = 0;
    virtual bool addTrajectory(OGL::Context*, std::vector<float>&, std::vector<Geodetic2D>&, std::vector<float>&, std::vector<bool>&, const SEAScope::RenderingCfg&, const SEAScope::RenderableId&) = 0;

protected:
    void updatePicking() {_pickingCallback();}

protected:
    SceneGraphController* _sgc;
    std::function<void()> _pickingCallback;
    //float _worldViewportRatio;
};

#endif /* ISCENE_H */
