// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   iworld_visibility.h
 * Author: gleseach
 *
 * Created on 22 mars 2017, 16:14
 */

#ifndef IWORLD_VISIBILITY_H
#define IWORLD_VISIBILITY_H

#include "vendors/_glm.h"

class Camera;

class IWorldVisibility
{
public:
    IWorldVisibility()
    : _camera{nullptr}
    {}

    virtual ~IWorldVisibility();

    //! Set the camera object.
    void setCamera(Camera* cam) {_camera = cam;}
    Camera* getCamera() {return _camera;}

    /*!
     * \brief Compute if the point is visible by the camera or not.
     * \param[in] p A point in the world limit.
     */
    virtual bool isVisible(const glm::vec3& p) = 0;

    /*!
     * \brief To call when the camera moved.
     */
    virtual void update() = 0;

protected:
    Camera* _camera;
};

#endif /* IWORLD_VISIBILITY_H */
