// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   line.h
 * Author: gleseach
 *
 * Created on 23 septembre 2015, 11:30
 */

#ifndef LINE_H
#define	LINE_H

void
line(float x1, float y1, float x2, float y2, //coordinates of the line
        float w, //width/thickness of the line in pixel
        float Cr, float Cg, float Cb, //RGB color components
        float Br, float Bg, float Bb, //color of background when alphablend=false,
        //  Br=alpha of color when alphablend=true
        bool alphablend, //use alpha blend or not
        std::vector<glm::vec3>& vertices,
        std::vector<glm::vec4>& colors);


void line(const glm::vec3& p0, const glm::vec3& p1, //coordinates of the line
        float w, //width/thickness of the line in pixel
        const glm::vec3& c0, //RGB color components
        const glm::vec3& c1, //color of background when alphablend=false,
        //  Br=alpha of color when alphablend=true
        bool alphablend, //use alpha blend or not
        std::vector<glm::vec3>& vertices,
        std::vector<glm::vec4>& colors);

void polyline(
        const std::vector<glm::vec3>& points,
        float w,
        const glm::vec3& color,
        std::vector<glm::vec3>& vertices,
        std::vector<glm::vec4>& colors,
        int type);

void polyline2(const std::vector<glm::vec3>& points, std::vector<glm::vec3>& vertices, std::vector<glm::vec3>& normals, std::vector<float>& miters);
void polylineOnSphere(const std::vector<glm::vec3>& points, std::vector<glm::vec3>& vertices, std::vector<glm::vec3>& normals, std::vector<float>& miters);
void polylineOnSphere2(
        const std::vector<glm::vec3>& points, 
        std::vector<glm::vec3>& vertices,
        std::vector<glm::vec3>& previous,
        std::vector<glm::vec3>& next,
        std::vector<float>& direction);

void polylineOnSphereIndices(
        const std::vector<glm::vec3>& points, 
        std::vector<glm::vec3>& vertices,
        std::vector<glm::vec3>& previous,
        std::vector<glm::vec3>& next,
        std::vector<float>& direction,
        std::vector<unsigned int>& indices);

void polylineOnSphereIndices2(
        const std::vector<glm::vec3>& points, 
        std::vector<glm::vec3>& vertices,
        std::vector<glm::vec2>& direction,
        std::vector<unsigned int>& indices);

void polylineOnSphereIndices2Update(
    const std::vector<glm::vec3>& points, 
    std::vector<glm::vec3>& vertices);

void maskedPolylineOnSphereIndices2(
        const std::vector<glm::vec3>& points,
        std::vector<glm::vec3>& vertices,
        std::vector<glm::vec2>& direction,
        std::vector<unsigned int>& indices,
		std::vector<bool>& mask);
#endif	/* LINE_H */

