// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef MEMORY_MANAGER_H
#define MEMORY_MANAGER_H

#include "3d/singleton.h"

enum MemoryCategory
{
    buffer = 0,
    texture,
    categoryCount
};

class MemoryManager final: public Singleton<MemoryManager>
{
private:
    MemoryManager();
    ~MemoryManager() override;

public:
    friend class Singleton<MemoryManager>;

    void consume(MemoryCategory category, unsigned long long amount);
    void release(MemoryCategory category, unsigned long long  amount);

    unsigned long long getAllocatedMemoryAmount(MemoryCategory category) const;
    unsigned long long getTotalAllocatedMemoryAmount() const;

private:
    unsigned long long _memory[categoryCount];
};

#endif /* MEMORY_MANAGER_H */

