// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   mesh_creator.h
 * Author: gleseach
 *
 * Created on 14 septembre 2015, 11:51
 */

#ifndef MESH_CREATOR_H
#define	MESH_CREATOR_H

#include "3d/opengl_enum.h"
#include "3d/array.h"

typedef std::vector<glm::vec3>      VerticesArray;
typedef std::vector<glm::vec2>      TexCoordArray;
typedef std::vector<unsigned int>   IndicesArray;

namespace OGL
{
    class Texture;
    class Program;
    class StaticMesh;
    class Context;
    class ProgramBindHelper;
}

struct Attribute
{
    unsigned int binding;
    Array array;
};

class MeshCreator
{
    Array vertices;
    Array texcoords;
    Array indices;
    std::vector<Attribute> attributes;
    std::map<unsigned int, OGL::Texture*> textures;
    OGL::DrawType draw_type;
    OGL::Primitive object_type;

public:
    MeshCreator();
    ~MeshCreator();

    void setVertices(const Array& vertices);
    void setTextureCoordinates(const Array& texcoords);
    void setIndices(const Array& indices);
    void setDrawType(OGL::DrawType draw_type, OGL::Primitive object_type);
    void setTexture(OGL::Texture * texture, unsigned int texture_channel);
    void addAttributeArray(int binding, const Array& array);

    OGL::StaticMesh * createStaticMesh(OGL::Context* context);
    OGL::StaticMesh * createStaticMesh(OGL::Context* context, OGL::ProgramBindHelper*);
};

#endif	/* MESH_CREATOR_H */

