// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   opengl_enum.h
 * Author: gleseach
 *
 * Created on 3 septembre 2015, 10:01
 */

#ifndef OPENGL_ENUM_H
#define	OPENGL_ENUM_H

#include "core/data_type.h"

namespace OGL
{

enum ContextType
{
    OpenGL3 = 0, 
    OpenGLES2,
    ContextTypeCount
};

enum Face
{
    front = 0,
    back,
    front_and_back
};

enum RasterizationMode
{
    fill = 0,
    line,
    point
};

enum Capability
{
    blending = 1,
    cull_face = 1 << 1,
    depth_test = 1 << 2,
    rasterizer_discard = 1 << 3,
    stencil_test = 1 << 4,
};

enum BufferBit
{
    color_buffer = 1,
    depth_buffer = 2,
    stencil_buffer = 4
};

enum BlendingFactor
{
    one, 
    one_minus_src_color,
    src_alpha, 
    one_minus_src_alpha
};

enum Func
{
    always,
    equal,
    not_equal,
    less,
    less_or_equal,
    greater,
    greater_or_equal,
    never
};

enum StencilOp
{
    zero,
    keep,
    replace,
    incr,
    incr_wrap,
    decr,
    decr_wrap,
    invert
};
    
enum RenderBufferType
{
    render_buffer_depth,
    render_buffer_stencil,
    render_buffer_depth_stencil
};
    
enum ShaderType
{
    vertex_shader,           
    tess_control_shader,    
    tess_evaluation_shader, 
    geometry_shader,        
    fragment_shader,        
    compute_shader,
    ShaderTypeCount
};

enum TextureType
{
    texture1D,           
    texture2D,    
    TextureTypeCount
};

enum BufferType
{
    array_buffer,
    element_array_buffer,
    copy_read_buffer,
    copy_write_buffer,
    pixel_pack_buffer, // Object to PBO
    pixel_unpack_buffer, // PBO to object
    BufferTypeCount
};

enum BufferUsage
{
    stream_draw = 0, 
    stream_read, 
    stream_copy, 
    static_draw, 
    static_read, 
    static_copy, 
    dynamic_draw, 
    dynamic_read, 
    dynamic_copy,
    BufferUsageCount
};

enum BufferAccess
{
    read_only,
    write_only,
    read_write
};

enum DrawType
{
    draw_arrays,
    draw_elements,
    draw_arrays_instanced,
    draw_elements_instanced,
    DrawTypeCount
};

enum Primitive
{
    points,
    lines,
    line_strip,
    triangles,
    triangle_strip,
    triangle_fan,
    PrimitiveCount
};

enum WrapMode
{
    clamp_to_edge,
    repeat,
    WrapModeCount
};

enum FilterMode
{
    nearest = 0,
    linear,
    nearest_mipmap_nearest,
    nearest_mipmap_linear,
    linear_mipmap_nearest,
    linear_mipmap_linear,
    FilterModeCount
};

enum UniformType
{
    unary_float,
    vec2_float,
    vec3_float,
    vec4_float,
    unary_int,
    vec2_int,
    vec3_int,
    vec4_int,
    unary_uint,
    vec2_uint,
    vec3_uint,
    vec4_uint,
    matrix3x3_float,
    matrix4x4_float,
    sampler1D,
    sampler2D,
    sampler3D,
    unknown_uniform_type
};

enum TransformFeedbackBufferMode
{
    interleaved_attribs,
    separate_attribs
};

enum GPUInfoInteger
{
    max_texture_2d_size,
    msaa_samples
};

enum class GPUInfoFloat
{
    max_anisotropy
};

enum QueryCounterType
{
    samples_passed,
    any_samples_passed,
    any_samples_passed_conservative,
    primitives_generated,
    transform_feedback_primitives_written,
    time_elapsed
};

} // namespace OGL

#endif	/* OPENGL_ENUM_H */
