// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef PARTICLE_GRID_H
#define PARTICLE_GRID_H

namespace OGL
{
    class ProgramBindHelper;
    class Context;
    class Texture;
    class Program;
}

struct DataRenderingParam;
class DataSet;

enum ParticleMode
{
    arrow,
    barb
};

class ParticleGrid
{
public:
    ParticleGrid();
    virtual ~ParticleGrid();

    void create(OGL::Context* ctx, ParticleMode particleMode);

    void bind(OGL::Context* ctx, DataRenderingParam* dataParam);
    void draw(OGL::Context* ctx, DataSet* dataSet);

    static void generateParticlesData(OGL::Context* ctx, DataSet* dataSet);

private:
    OGL::ProgramBindHelper* _particleLocalBarbsProgram;
    OGL::ProgramBindHelper* _particleLocalBarbsFalseColorsProgram;
    OGL::Program* _currentProgram;
    OGL::Texture* _indirectionTexture;
    OGL::Texture* _atlasTexture;
	ParticleMode _mode;
};

#endif /* PARTICLE_GRID_H */
