// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef PARTICLES_DATA_H
#define PARTICLES_DATA_H

#include "data/image.h"

namespace OGL
{
	class Context;
	class Buffer;
	class VertexArrayObject;
	class DrawMode;
	class Texture;
}

class DataSet;

class ParticlesData
{
public:
	ParticlesData();
	~ParticlesData();

	void create(OGL::Context* ctx, DataSet* dataSet);
	void update(OGL::Context* ctx);

	void bind(OGL::Context* ctx);
	void draw(OGL::Context* ctx);

	const glm::vec2& getUVMin() const { return _uvMin; }
	const glm::vec2& getUVRange() const { return _uvRange; }

	float getParticleSize() const { return _particleSize; }
	void setParticleSize(float val);

	glm::vec2 getOffsetUV();
	void setOffsetUV(const glm::vec2& val);
	glm::vec4 getScissorUV() const { return _scissorUV; }
	void setScissorUV(glm::vec4 val) { _scissorUV = val; }
	float getParticleSizePixels() { return _dist; }
	void setParticlesDensity(float value);
    void needUpdate();

private:
	void initBuffers(OGL::Context* ctx, DataSet* dataSet);
	void addPoint(const glm::vec2& point, std::vector<glm::vec2>& vertices, std::vector<glm::vec2>& displacement);
    void updateSize(OGL::Context* ctx);

private:
	OGL::Buffer* _vertices_buffer;
	OGL::Buffer* _texcoords_buffer;
	OGL::Buffer* _displacement_buffer;
	OGL::VertexArrayObject* _vao;
	OGL::DrawMode* _drawMode;

	OGL::Texture* _gcp;
	glm::vec2 _uvMin;
	glm::vec2 _uvRange;

	float _particleSize;
	glm::vec2 _offsetUV;
	glm::vec4 _scissorUV;

	Image _distData;
	DataSet* _dataSet;
	float _dist;
	float _density;
	bool _update;
    std::vector<float> _distX;
    std::vector<float> _distY;
    Image gcpData;

};

#endif // PARTICLES_DATA_H
