// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   pickable.h
 * Author: gleseach
 *
 * Created on 2 novembre 2016, 10:59
 */

#ifndef PICKABLE_H
#define PICKABLE_H

#include "3d/app_enum.h"
#include "3d/renderable.h"
#include "3d/style.h"

namespace OGL
{
    class Shape;
}

class Pickable : public Renderable2
{
public:
    virtual ~Pickable() override;

    virtual void drawPicking(OGL::Context* ctx, const glm::mat4& mvp) = 0;

    void setPickingId(unsigned int id, glm::vec3 color);

    const glm::vec3& pickingColor() const;
    unsigned int pickingId() const;

    glm::vec4 getCurrentStyleColor() const;

    virtual void setStyle(StyleState style);
    Style& getStyle();

    bool isMoveable() const;
    bool isAnchor() const;

    virtual void move(const glm::vec3& /*pos*/) {}

    OGL::Shape* getShape() { return _shape; }

protected:
    Pickable(const SEAScope::RenderableId& rId);
    Pickable(const Pickable& src);

protected:
    glm::vec3 _pickingColor;
    unsigned int _pickingId;
    Style _style;
    OGL::Shape* _shape;
};

#endif /* PICKABLE_H */

