// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   plane.h
 * Author: gleseach
 *
 * Created on 12 octobre 2016, 16:03
 */

#ifndef PLANE_H
#define PLANE_H

enum Halfspace
{
    NEGATIVE = -1,
    ON_PLANE = 0,
    POSITIVE = 1
};

/*!
 * \brief Describe a plane.
 */
class Plane
{
public:

    // Default constructor
    Plane();

    // Contructor with all parameters normal + point.
    Plane(const glm::vec3& n, const glm::vec3& p);

    /*!
     * \brief Compute a plane equation with a normal and the coordinates of a point.
     */
    void setEquation(const glm::vec3& n, const glm::vec3& p);

    /*!
     * \brief Set the equation directly.
     */
    void setEquation(const glm::vec4& e);

    /*!
     * \brief Return the plane equation.
     */
    const glm::vec4& equation() const;

    /*!
     * \brief Return the plane normal.
     */
    const glm::vec3& normal() const;

    /*!
     * \brief Compute if a point is on top, on bottom or on the plane.
     */
    Halfspace classifyPoint(const glm::vec3& pt) const;

private:
	glm::vec4 _equation; //< Plane equation
	glm::vec3 n;
	float d;
};

#endif /* PLANE_H */

