// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef POINT_CREATOR_H
#define POINT_CREATOR_H

#include "3d/shape_creator.h"

namespace OGL
{
	class Context;
}

class PointBuilder;

class PointCreator : public ShapeCreator
{
    public:
        PointCreator();
        ~PointCreator() override;

        void initialize(OGL::Context* ctx) override;
        void setBuilder(ShapeBuilder* builder) override;
        void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
        OGL::Shape* createShape() override;
        void reset() override;
        OGL::Shape* getTemporaryShape() override;
        SEAScope::ShapeType getShapeType() override;

        unsigned int getAnchorCount() const override;
        glm::vec3 getAnchorPos(AnchorType type, unsigned int vertexId) const override;
        void move(AnchorType type, unsigned int vertexId, const glm::vec3& pos) override;
        void move(AnchorType type, unsigned int vertexId) override;
        void pressed(AnchorType type, unsigned int id) override;
        void remove(AnchorType type, unsigned int id) override;
        void setAnchorPool(AnchorPoolNode*) override;

        void addVertexUnderCursor() override;
        void updateCursorPosition(const glm::vec3&, std::uint32_t, std::uint32_t) override;

    protected:
        PointBuilder* _builder;

        bool _positionSet;
        glm::vec3 _position;

        glm::vec3 _currentPos;
};

#endif // POINT_CREATOR_H
