// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef POLYGON_MESH_H
#define POLYGON_MESH_H

#include "3d/app_enum.h"
#include "3d/shape_mesh.h"

namespace OGL
{
	class Context;
	class Buffer;
	class VertexArrayObject;
	class DrawMode;
}

class Polygon;

class PolygonMesh : public ShapeMesh
{
public:
	PolygonMesh();
	~PolygonMesh() override;

	void create(OGL::Context* ctx, const std::vector<glm::vec3>& points, const std::vector<unsigned int>& indices, Polygon* shape);

	void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
	void drawPicking(OGL::Context* ctx, const glm::mat4& mvp) override;

private:
	std::vector<glm::vec3> _points;
	OGL::Buffer* _buffer;
	OGL::Buffer* _indicesBuffer;
	OGL::VertexArrayObject* _vao;
	OGL::DrawMode* _drawMode;
};

#endif // POLYGON_MESH_H
