// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef POLYLINE_BASE_H
#define POLYLINE_BASE_H

#include "3d/shape.h"

class PolylineBase : public OGL::Shape
{
public:
	PolylineBase(SEAScope::ShapeType type);
	virtual ~PolylineBase() override;

	void addPoint(const glm::vec3& point);

	// Move array
	void setPoints(std::vector<glm::vec3>&& points);

	// Copy array
	void setPoints(const std::vector<glm::vec3>& points);

	// Return the number of points in the polyline
	unsigned int getPointCount() const;

	// Return the coordinates of the ith point
	const glm::vec3& getPoint(unsigned int ith) const;

	// Return the points array
	const std::vector<glm::vec3>& getPoints() const { return _points; }

	// update the coordinates of the ith point
	void setPoint(unsigned int ith, const glm::vec3&);

	void setLastPoint(const glm::vec3&);

	// Delete a point
	void deletePoint(unsigned int ith);

	void clear();

protected:
	std::vector<glm::vec3> _points; //< Points array
};

#endif // POLYLINE_BASE_H
