// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef POLYLINE_EDITOR_H
#define POLYLINE_EDITOR_H

#include "3d/shape_editor.h"

class PolylineBuilder;

namespace OGL
{
    class Context;
    class Program;
    class Buffer;
    class VertexArrayObject;
    class DrawMode;
}

class PolylineEditor : public ShapeEditor
{
public:
	PolylineEditor();
	~PolylineEditor() override;

	void initialize(OGL::Context* ctx, Pickable* obj) override;
	void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
	OGL::Shape* createShape() override;
	void setBuilder(ShapeBuilder* builder) override;

	unsigned int getAnchorCount() const override;
	glm::vec3 getAnchorPos(AnchorType type, unsigned int vertexId) const override;
	void move(AnchorType type, unsigned int vertexId, const glm::vec3& pos) override;
	void move(AnchorType type, unsigned int vertexId) override;
	void pressed(AnchorType type, unsigned int id) override;
    void remove(AnchorType type, unsigned int id) override;

    bool shapeIsValid() const override;
    void updateCursorPosition(const glm::vec3&) override;
    void startMove(unsigned int) override;
    void stopMove() override;

	void setAnchorPool(AnchorPoolNode* val) override;

    bool asLinearRing() const;
    bool asLinearRing(bool);

    bool detectSelfIntersections() const;
    bool detectSelfIntersections(bool);
protected:
    Pickable* copyShapeToEdit(Pickable* src);

private:
	glm::vec3 getMiddlePoint(unsigned int id) const;

private:
    OGL::Program* _program;
    OGL::Buffer* _buffer;
    OGL::VertexArrayObject* _vao;
    OGL::DrawMode* _drawMode;
    bool _asLinearRing;
    glm::vec3 _lastPos;
    glm::vec3 _currentPos;

    bool _detectSelfIntersections;
    bool _hasSelfIntersections;
};

#endif // POLYLINE_EDITOR_H
