// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   polyline.h
 * Author: gleseach
 *
 * Created on 30 septembre 2015, 17:45
 */

#ifndef POLYLINE_MESH_H
#define POLYLINE_MESH_H

#include "3d/geodetic.h"
#include "3d/shape_mesh.h"

namespace OGL
{
    class Program;
    class Buffer;
    class VertexArrayObject;
    class Context;
    class DrawMode;
    class Shape;
}

class Array;
class Polyline;

class PolylineMesh : public ShapeMesh
{
public:
    PolylineMesh();
    ~PolylineMesh() override;

    /*
     * \brief Initialize polyline positions
     */
    void initialize(OGL::Context* ctx, const SEAScope::RenderableId& rId, std::vector<Geodetic2D>&& points, OGL::Shape* shape);

    /*
     * \brief Initialize polyline positions and attributes
     */
    void initialize(OGL::Context* ctx, const SEAScope::RenderableId& rId, std::vector<Geodetic2D>&& points, OGL::Shape* shape, Array* attrib);

    void setPolyline(Polyline* p);
    Polyline* getPolyline() const;
    void setPoints(std::vector<Geodetic2D>&& points);
    const std::vector<Geodetic2D>& getPoints() const { return this->_points; }

    void setAttribute(int id, std::vector<float>&& attrib);
    void setMask(std::vector<bool>&& mask);

    void draw();
    void setProgram(OGL::Program* program);
    void setThickness(float value);

    void setHighlighted(bool highlighted) {this->_highlighted = highlighted;}
    bool getHighlighted() const {return this->_highlighted;}

    void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
    void drawPicking(OGL::Context* ctx, const glm::mat4& mvp) override;

private:
    void CreateBufferForSphere(std::vector<glm::vec3>& vertices, std::vector<float>& attrib0, std::vector<bool>& mask);
    void updateData();
    Array duplicate(const Array& data, bool&) const;

private:
    std::vector<Geodetic2D> _points;

    unsigned int _verticeCount;
    std::map<int, std::vector<float>> _attributeMap;
	std::vector<bool> _mask;
    Array* _attributeArray;

    OGL::Program* _program;

    OGL::Buffer* _verticeBuffer;
    OGL::Buffer* _directionBuffer;
    OGL::Buffer* _indicesBuffer;
    OGL::Buffer* _attribBuffer;
    OGL::VertexArrayObject* _vertexArrObj;

    bool _highlighted;

    OGL::Context* _ctx;
    OGL::DrawMode* _drawMode;
};

#endif // POLYLINE_MESH_H
