// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   program_bind_helper.h
 * Author: gleseach
 *
 * Created on 13 juin 2016, 15:30
 */

#ifndef PROGRAM_BIND_HELPER_H
#define PROGRAM_BIND_HELPER_H

namespace OGL
{

class Program;
class Texture;
class Context;

class ProgramBindHelper
{
public:
    ProgramBindHelper();
    virtual ~ProgramBindHelper();

    // Parse the program and bind automaticaly known attrib/sampler.
    void setProgram(OGL::Program* program);

    // Set the texture object to a textureUnit
    void setTexture(int textureUnit, OGL::Texture* texture);

    // Bind program, textures and all related.
    void bind(OGL::Context* ctx);
    void unbind();

    OGL::Program* getProgram() const {return _program;}

    // Accessors
    int getVerticesLocation();
    bool hasTextureCoordinates();
    int getTextureCoordinatesLocation();
    bool hasModelViewProjectionMatrix();
    int getModelViewProjectionMatrixLocation();

private:
    void parse();

protected:
    virtual void parseSpecialized() {}
    virtual void bindSpecialized(OGL::Context* /*ctx*/) {}

protected:
    OGL::Program* _program;
    bool hasPosition;
    int position_location;
    bool hasTexCoord;
    int texCoord_location;
    bool hasMVP;
    int matrix_mvp_location;
    std::vector<std::tuple<int, OGL::Texture*>> _textures;
};

} // namespace OGL

#endif /* PROGRAM_BIND_HELPER_H */
