// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef RECTANGLE_CREATOR_H
#define RECTANGLE_CREATOR_H

#include "3d/shape_creator.h"

namespace OGL
{
	class Context;
	class Program;
	class Buffer;
	class VertexArrayObject;
	class DrawMode;
}

class Rectangle;
class RectangleBasedBuilder;
class EllipseMesh;

class RectangleCreator : public ShapeCreator
{
public:
	RectangleCreator();
	~RectangleCreator() override;

	void initialize(OGL::Context* ctx) override;
	void setBuilder(ShapeBuilder* builder) override;
	void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
	OGL::Shape* createShape() override;
	void reset() override;
	OGL::Shape* getTemporaryShape() override;
	SEAScope::ShapeType getShapeType() override;

    unsigned int getAnchorCount() const override;
    glm::vec3 getAnchorPos(AnchorType type, unsigned int vertexId) const override;
    void move(AnchorType type, unsigned int vertexId, const glm::vec3& pos) override;
    void move(AnchorType type, unsigned int vertexId) override;
    void pressed(AnchorType type, unsigned int id) override;
    void remove(AnchorType type, unsigned int id) override;

    void addVertexUnderCursor() override;
    void updateCursorPosition(const glm::vec3&, std::uint32_t, std::uint32_t) override;

    float getAlpha() const;
    void setAlpha(float);
    float getBeta() const;
    void setBeta(float);

private:
	RectangleBasedBuilder* _builder;
	OGL::Program* _program;
	OGL::Buffer* _buffer;
	OGL::VertexArrayObject* _vao;
	OGL::DrawMode* _drawMode;
	glm::vec3 _lastPos;
	glm::vec3 _currentPos;
	std::vector<glm::vec4> _colors;
	std::vector<glm::vec3> _points;
	Rectangle* _tmpShape;
	int _p0Loc;
    int _p1Loc;
    int _mvpLoc;
    int _colorLoc;
	//bool _allowIntersection;
	//bool _currentPointValid;
	//bool _closed;
	//bool _closedValid;
	std::array<glm::vec3, 4> _corners;
	std::array<glm::vec3, 4> _middles;
	std::array<glm::vec2, 4> _cornersLonLat;
	glm::vec2 _currentLonLat;
	bool _initialized;
	float _alpha;
    float _beta;
    /*
	bool _alphaLocked;
	bool _betaLocked;
    */
	glm::mat4 _model;

	EllipseMesh* _ellipseMesh;
};

#endif // RECTANGLE_CREATOR_H
