// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef RECTANGLE_CREATOR_ONE_CLICK_H
#define RECTANGLE_CREATOR_ONE_CLICK_H

#include "3d/shape_creator.h"

class RectangleBasedBuilder;

class RectangleCreatorOneClick : public ShapeCreator
{
public:
	RectangleCreatorOneClick();
	~RectangleCreatorOneClick() override;

	void initialize(OGL::Context* ctx) override;
	void setBuilder(ShapeBuilder* builder) override;
	void draw(OGL::Context* ctx, const glm::mat4& mvp) override;
	OGL::Shape* createShape() override;
	void reset() override;
	OGL::Shape* getTemporaryShape() override;
	SEAScope::ShapeType getShapeType() override;

    unsigned int getAnchorCount() const override;
    glm::vec3 getAnchorPos(AnchorType type, unsigned int vertexId) const override;
    void move(AnchorType type, unsigned int vertexId, const glm::vec3& pos) override;
    void move(AnchorType type, unsigned int vertexId) override;
    void pressed(AnchorType type, unsigned int id) override;
    void remove(AnchorType type, unsigned int id) override;

    void addVertexUnderCursor() override;
    void updateCursorPosition(const glm::vec3&, std::uint32_t, std::uint32_t) override;

    void setCenterUnderCursor();
private:
	RectangleBasedBuilder* _builder;
	glm::vec3 _center;
    glm::vec3 _currentPosition;
};

#endif // RECTANGLE_CREATOR_ONE_CLICK_H
