// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   renderer_factory.h
 * Author: gleseach
 *
 * Created on 6 décembre 2016, 12:36
 */

#ifndef RENDERER_FACTORY_H
#define RENDERER_FACTORY_H

#include "3d/scene_graph_node.h"

namespace OGL
{
    class Context;
}

class IRenderer2;
class IPickingRenderer;
class IRawRenderer;
class IWorldVisibility;

class RendererFactory
{
public:
    virtual ~RendererFactory();
    void initialize(IWorldVisibility* visibility);

    virtual IRenderer2* createRenderer(OGL::Context* ctx, RendererType renderer, FillType fill) = 0;
    virtual IPickingRenderer* createPickingRenderer(OGL::Context* ctx, PickingRendererType renderer) = 0;
    virtual IRawRenderer* createRawRenderer(OGL::Context* ctx, RawRendererType renderer) = 0;

    IWorldVisibility* getVisibility() const;

protected:
    IWorldVisibility* _visibility;
};

#endif /* RENDERER_FACTORY_H */
