// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   scene_dictionnary.h
 * Author: gleseach
 *
 * Created on 1 décembre 2016, 15:28
 */

#ifndef SCENE_DICTIONNARY_H
#define SCENE_DICTIONNARY_H

#include "core/types.h"
#include "3d/granule_tree.h"

class Renderable2;

namespace SEAScope
{
    struct RenderableId;
}

class SceneDictionnary
{
public:
    SceneDictionnary();
    ~SceneDictionnary();

    void add(Renderable2* obj);
    bool exist(const SEAScope::RenderableId& rId, Renderable2*& obj) const;
    bool exist(const SEAScope::RenderableId& rId) const;
    void remove(Renderable2* obj);
    bool remove(const SEAScope::RenderableId& rId, Renderable2*& renderable);

    void fillListToRemove(const std::set<SEAScope::RenderableId>& rIds, std::list<SEAScope::RenderableId>& toRemove);

    typedef std::list<std::pair<SEAScope::RenderableId, Renderable2*>> ListPair;
    void fillListToRemovePair(const std::set<SEAScope::RenderableId>& rIds, ListPair& existing, ListPair& notFound);

private:
    GranuleTree<Renderable2*> _tree;
};

#endif /* SCENE_DICTIONNARY_H */

