// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef GLOBE_SCENE_RENDERER_H
#define GLOBE_SCENE_RENDERER_H

#include "3d/iscene_renderer.h"
#include "3d/colormap_manager_gpu.h"

class IScene;
class SceneView;
class IPreRenderer;
class IPostRenderer;

namespace OGL
{
    class Context;
}

class SceneRenderer final: public virtual ISceneRenderer
{
    public:
        SceneRenderer();
        virtual ~SceneRenderer() override;

        // ISceneView
        bool initialize(const std::string&, SEAScope::ColormapManager*, const SEAScope::RendererResources* /*, float*/) override;
        void setDisplayCfg(IScene*, const SEAScope::RenderingCfg&) override;
        const std::set<SEAScope::RenderableId>& getToRender() const override;
        void setToRender(const std::set<SEAScope::RenderableId>&) override;
        bool updateStarted(IScene*) override;
        bool updateFinished(IScene*) override;
        OGL::Context* getContext() const override;
        void resize(int, int) override;
        int getMaxTextureSize() const override;
        bool renderSceneView(SceneView*) override;
        void setRasterizationMode(OGL::RasterizationMode) override;
        OGL::ColormapManagerGPU* getColormapManager() override;

        // Draw all objects with their configuration
        void draw(IScene*, const Camera*) override;
        void drawRaw(IScene*, const Camera*) override;

        // Draw only one granule
        void drawOneTrueColors(IScene*, const Camera*, const SEAScope::RenderableId&) override;
        void drawOneRaw(IScene*, const Camera*, const SEAScope::RenderableId&) override;

        void drawWorld(IScene*, const Camera*, const glm::mat4& mvp, FillType type) override;

        void registerPreRenderer(IPreRenderer*) override;
        void unregisterPreRenderer(IPreRenderer*) override;

        void registerPostRenderer(IPostRenderer*) override;
        void unregisterPostRenderer(IPostRenderer*) override;

    protected:
	    std::set<SEAScope::RenderableId> _toRender;
        OGL::Context* _ctx;
        OGL::ColormapManagerGPU _colormapMngr;

        std::vector<IPreRenderer*> _preRenderers;
        std::vector<IPostRenderer*> _postRenderers;

        OGL::RasterizationMode _rasterizationMode;
};

#endif
