// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SCENE_VIEW_H
#define SCENE_VIEW_H

#include <functional>

#include "vendors/_glm.h"
#include "3d/camera.h"  // for Camera
#include "3d/scene_graph_node.h"  // for FillType
#include "3d/globe_visibility.h"
#include "3d/dataset_visibility_globe.h"
#include "core/interfaces/irenderer.h"
#include "core/interfaces/iapp_events_subscriber.h"

class DataSetVisibility;
class DataSet;
class IScene;
class ISceneRenderer;

namespace SEAScope
{
    class AppEvent;
    class AppEventsDispatcher;
}

class SceneView final : public virtual SEAScope::IRenderer,
                        public virtual SEAScope::IAppEventsSubscriber
{
    public:

    SceneView(SEAScope::AppEventsDispatcher&);
    virtual ~SceneView() override;

    // IRenderer
    bool setWindow(SEAScope::IWindow&) override;
    bool renderFrame() override;

    void init(IScene*, ISceneRenderer*, OGL::Texture*);

    IScene* getScene() const;
    ISceneRenderer* getRenderer() const;

    Camera* getCamera();
    const Camera* getConstCamera() const;

    void setViewport(const glm::ivec2&);
    glm::ivec2 getViewport() const;

    float getPixelSize() const;
    float computePixelSize(const glm::ivec2&) const;
    void updatePixelSize();

    bool getLonLat(double x, double y, glm::vec2&, glm::vec3&) const;
    float getAltitude() const;

    IWorldVisibility* getVisibility() {return &(this->_visibility);}
    DataSetVisibilityGlobe* getDataSetVisibility() {return &(this->_dataSetVisibility);}

    bool handleAppEvent(std::type_index, const SEAScope::AppEvent&) override;

    protected:
        SEAScope::AppEventsDispatcher& _appEventsDispatcher;
        Camera _camera;
        glm::ivec2 _viewport;

        IScene* _scene;
        ISceneRenderer* _renderer;

        GlobeVisibility _visibility;
        DataSetVisibilityGlobe _dataSetVisibility;

        float _pixelSize;
};

#endif /* SCENE_CONTROLLER_H */
