// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   screen_quad.h
 * Author: gleseach
 *
 * Created on 4 novembre 2016, 16:00
 */

#ifndef SCREEN_QUAD_H
#define SCREEN_QUAD_H

namespace OGL
{
    class Context;
    class Program;
    class VertexArrayObject;
    class Buffer;
    class DrawMode;
    class StaticMesh;
}

/*!
 * \brief Draw a colored quad billboard.
 */
class ScreenQuad
{
public:
    ScreenQuad();
    ~ScreenQuad();

    void create(OGL::Context* ctx);
    void setSize(unsigned int size);

    void bind(OGL::Context* ctx, const glm::mat4& mvp);
    void draw(OGL::Context* ctx, const glm::mat4& mvp, const glm::vec3& pos, const glm::vec3& color);

    void draw2(const glm::vec3& pos, const glm::vec4& color);

    static OGL::StaticMesh* createMesh(OGL::Context* ctx);

private:
    OGL::Program* _program;
    OGL::Buffer* _displacementBuffer;
    OGL::VertexArrayObject* _vao;
    OGL::DrawMode* _drawMode;
    unsigned int _size;
};

#endif /* SCREEN_QUAD_H */

