// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SELECTION_RECTANGLE_H
#define SELECTION_RECTANGLE_H

namespace OGL
{
	class Context;
	class Buffer;
	class ProgramBindHelper;
	class StaticMesh;
	class VertexArrayObject;
	class DrawMode;
}

class SelectionRectangle
{
public:
	SelectionRectangle();
	~SelectionRectangle();

	void initialize(OGL::Context* ctx, int, int);
    bool initialized() const;

	void setVisible(bool status);
    void setColor(const glm::vec4&);
	void setStartCorner(const glm::ivec2& p);
	void setEndCorner(const glm::ivec2& p);
	void setWindowSize(int, int);
    glm::ivec2 getWindowSize() const;

	void draw(OGL::Context* ctx, const glm::mat4& mvp);

private:
	void update();

private:
	int _width, _height;
	glm::ivec2 _start, _end;
	bool _visible;
	OGL::ProgramBindHelper* _program;
	OGL::VertexArrayObject* _vao;
	OGL::DrawMode* _drawMode;
	OGL::Buffer* _vertices;
	OGL::Buffer* _indices;
	int _colorLoc;
    glm::vec4 _color;
    bool _colorChanged;
};

#endif // SELECTION_RECTANGLE_H
