// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   shader.h
 * Author: gleseach
 *
 * Created on 6 août 2015, 14:27
 */

#ifndef SHADER_H
#define	SHADER_H

#include "3d/globject.h"
#include "3d/opengl_enum.h"

namespace OGL
{

class Shader : public Object
{
public:
    Shader(OGL::ShaderType type, const std::string& name);
    virtual ~Shader() override;

    virtual bool loadFromFile(OGL::Context* context, const std::string& file_name) = 0;
    virtual bool loadFromString(OGL::Context* context, const std::string& source) = 0;

    const std::string& getName() const {return this->_name;}

private:
    virtual bool compile(const std::string& source) = 0;

protected:
    OGL::ShaderType _type;
    std::string _name;
};

} // namespace OGL

#endif	/* SHADER_H */

