// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/*
 * File:   shader_list.h
 * Author: gleseach
 *
 * Created on 3 septembre 2015, 18:21
 */

#ifndef SHADER_LIST_H
#define	SHADER_LIST_H

 // Vertex shaders
#define vertex_uv_grid			"vertex_uv_grid.vert"
#define vertex_pos_tex          "vertex_pos_tex.vert"
#define vertex_pos_mvp_tex      "vertex_pos_mvp_tex.vert"
#define vertex_pos_mvp          "vertex_pos_mvp.vert"
#define vertex_pos_mvp_out_pos  "vertex_pos_mvp_out_pos.vert"
#define vertex_full_screen_tex  "vertex_full_screen_tex.vert"
#define vertex_pos_mvp_col      "vertex_pos_mvp_col.vert"
#define vertex_polyline         "vertex_polyline.vert"
#define vertex_polyline_screenspace "vertex_polyline_screenspace.vert"
#define vertex_raycasting       "vertex_raycasting.vert"
#define vertex_polyline_worldspace   "vertex_polyline_worldspace.vert"
#define vertex_polyline_picking        "vertex_polyline_picking.vert"
#define vertex_particle                "vertex_particle.vert"
#define vertex_physic_particle         "vertex_physic_particle.vert"
#define vertex_physic_streamlineV2     "vertex_physic_streamlineV2.vert"
#define vertex_rendering_spherical_particle "vertex_rendering_spherical_particle.vert"
#define vertex_rendering_planar_particle    "vertex_rendering_planar_particle.vert"
#define vertex_copy_particle           "vertex_copy_particle.vert"
#define vertex_physic_streamline         "vertex_physic_streamline.vert"
#define vertex_rendering_spherical_streamline "vertex_rendering_spherical_streamline.vert"
#define vertex_rendering_spherical_streamline_false_colors "vertex_rendering_spherical_streamline_false_colors.vert"
#define vertex_rendering_planar_streamline    "vertex_rendering_planar_streamline.vert"
#define vertex_rendering_planar_streamline_false_colors    "vertex_rendering_planar_streamline_false_colors.vert"
#define vertex_copy_streamline           "vertex_copy_streamline.vert"
#define vertex_lonlat_globe_mvp          "vertex_lonlat_globe_mvp.vert"
#define vertex_particle_grid             "vertex_particle_grid.vert"
#define vertex_particle                 "vertex_particle.vert"
#define vertex_uv_particle              "vertex_uv_particle.vert"
#define vertex_uv_particle_arrows       "vertex_uv_particle_arrows.vert"
#define vertex_deploy_streamline         "vertex_deploy_streamline.vert"
#define vertex_polyline_screenspace_attribute "vertex_polyline_screenspace_attribute.vert"
#define vertex_polyline_screenspace_attribute_raw "vertex_polyline_screenspace_attribute_raw.vert"
#define vertex_rendering_spherical_streamlineV3 "vertex_rendering_spherical_streamlineV3.vert"
#define vertex_rendering_spherical_streamline_false_colorsV3 "vertex_rendering_spherical_streamline_false_colorsV3.vert"
#define vertex_physic_streamlineV3 "vertex_physic_streamlineV3.vert"
#define vertex_rendering_planar_streamlineV3 "vertex_rendering_planar_streamlineV3.vert"
#define vertex_rendering_planar_streamline_false_colorsV3 "vertex_rendering_planar_streamline_false_colorsV3.vert"
#define vertex_line_tesselate "vertex_line_tesselate.vert"
#define vertex_screenspace_quad "vertex_screenspace_quad.vert"
#define vertex_screenspace_quad_tex "vertex_screenspace_quad_tex.vert"
#define vertex_screenspace_quad_dir_tex_instanced "vertex_screenspace_quad_dir_tex_instanced.vert"
#define vertex_transect_tf "vertex_transect_tf.vert"
#define vertex_physic_uv_streamline "vertex_physic_uv_streamline.vert"
#define vertex_rendering_spherical_uv_streamline "vertex_rendering_spherical_uv_streamline.vert"
#define vertex_rendering_spherical_uv_streamline_false_colors "vertex_rendering_spherical_uv_streamline_false_colors.vert"

// Fragment shaders
#define fragment_uni_color      "fragment_uni_color.frag"
#define fragment_tex_cubemap    "fragment_tex_cubemap.frag"
#define fragment_tex_col_tex_mask_tex_false_colors  "fragment_tex_col_tex_mask_tex_false_colors.frag"
#define fragment_tex_col_tex_mask_tex               "fragment_tex_col_tex_mask_tex.frag"
#define fragment_font           "fragment_font.frag"
#define fragment_operator       "fragment_operator.frag"
#define fragment_polyline       "fragment_polyline.frag"
#define fragment_polyline_screenspace "fragment_polyline_screenspace.frag"
#define fragment_raycasting     "fragment_raycasting.frag"
#define fragment_polyline_worldspace "fragment_polyline_worldspace.frag"
#define fragment_operator_module_sum   "fragment_operator_module_sum.frag"
#define fragment_operator_error        "fragment_operator_error.frag"
#define fragment_apply_palette         "fragment_apply_palette.frag"
#define fragment_operator_module_sum_palette "fragment_operator_module_sum_palette.frag"
#define fragment_raycasting_picking    "fragment_raycasting_picking.frag"
#define fragment_mask_picking_color    "fragment_mask_picking_color.frag"
#define fragment_operator_arrow_field  "fragment_operator_arrow_field.frag"
#define fragment_particle_arrow        "fragment_particle_arrow.frag"
#define fragment_particle_arrow_false_colors        "fragment_particle_arrow_false_colors.frag"
#define fragment_particle_barb         "fragment_particle_barb.frag"
#define fragment_particle_barb_false_colors "fragment_particle_barb_false_colors.frag"
#define fragment_physic_particle       "fragment_physic_particle.frag"
#define fragment_rendering_particle    "fragment_rendering_particle.frag"
#define fragment_copy_particle         "fragment_copy_particle.frag"
#define fragment_physic_streamline       "fragment_physic_streamline.frag"
#define fragment_color_alpha    "fragment_color_alpha.frag"
#define fragment_copy_streamline         "fragment_copy_streamline.frag"
#define fragment_white                   "fragment_white.frag"
#define fragment_polyline_screenspace_false_colors "fragment_polyline_screenspace_false_colors.frag"
#define fragment_polyline_screenspace_raw "fragment_polyline_screenspace_raw.frag"
#define fragment_fxaa "fragment_fxaa.frag"
#define fragment_tex_mask_uni_color "fragment_tex_mask_uni_color.frag"
#define fragment_tex_col_tex_mask_tex_rgb "fragment_tex_col_tex_mask_tex_rgb.frag"
#define fragment_tex_col_tex_mask_tex_rgb_raw "fragment_tex_col_tex_mask_tex_rgb_raw.frag"
#define fragment_raycasting_raw "fragment_raycasting_raw.frag"
#define fragment_tex_col "fragment_tex_col.frag"
#define fragment_tex_color_alpha "fragment_tex_color_alpha.frag"
#define fragment_tex_true_rgba "fragment_tex_true_rgba.frag"

#endif	/* SHADER_LIST_H */
