// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   shader_manager.h
 * Author: gleseach
 *
 * Created on 3 septembre 2015, 09:58
 */

#ifndef SHADER_MANAGER_H
#define	SHADER_MANAGER_H

#include "3d/opengl_enum.h"

namespace OGL
{

class Shader;
class Program;
class Context;
class ProgramBindHelper;

class ShaderManager
{
    std::map<std::string, OGL::Shader*> shaders_id;
    OGL::Context* _ctx;

public:
    ShaderManager(OGL::Context* ctx);
    ~ShaderManager();

    bool addShaderFromFile(const std::string& file_name, const std::string& name,OGL::ShaderType type);
    bool addShaderFromFiles(const std::string& shaderFolder, const std::vector<std::string>& files);
    OGL::Shader * getShader(const std::string& shader_name);
    void clear();

    // Transform feedback

    /*!
     * \brief Transform feedback simple case, only one output data stream.
     * \param[in] vertex_shader Name of the vertex shader.
     * \param[in] output Name of the attrib to save in the vertex shader.
     */
    OGL::Program * createProgramTransformFeedback(const std::string& vertex_shader, const std::string& output);

    /*!
     * \brief Transform feedback with mutiple outputs to save.
     * \param[in] vertex_shader Name of the vertex shader.
     * \param[in] outputs Array of attrib's name to save in the vertex shader.
     * \param[in] bufferMode Save the data in one buffer interleaved ou in multiple buffers.
     */
    OGL::Program * createProgramTransformFeedback(const std::string& vertex_shader, const std::vector<std::string>& outputs, TransformFeedbackBufferMode bufferMode);

    // Classical

    /*!
     * \brief Create a GLSL program with a vertex and a fragment shader stage.
     */
    OGL::Program * createProgram(const std::string& vertex_shader, const std::string& fragment_shader);

    OGL::ProgramBindHelper * createProgramTransformFeedbackAutoBind(const std::string& vertex_shader, const std::string& output);
    OGL::ProgramBindHelper * createProgramAutoBind(const std::string& vertex_shader, const std::string& fragment_shader);
};

} // namespace OGL

#endif	/* SHADER_MANAGER_H */

