// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SHAPE_CREATOR_H
#define SHAPE_CREATOR_H

#include "core/types.h"
#include "3d/anchor_controller.h"

namespace OGL
{
	class Context;
	class Shape;
}

class ShapeBuilder;
struct PipelineParameters;
class AnchorPoolNode;

class ShapeCreator: public virtual AnchorController
{
public:
    ShapeCreator();
	virtual ~ShapeCreator() override;

	virtual void initialize(OGL::Context* ctx) = 0;

	virtual void setBuilder(ShapeBuilder* builder) = 0;
	virtual void draw(OGL::Context* ctx, const glm::mat4& mvp) = 0;
	virtual OGL::Shape* createShape() = 0;
	virtual void reset() = 0;
	virtual OGL::Shape* getTemporaryShape() = 0;
	virtual SEAScope::ShapeType getShapeType() = 0;

    virtual void addVertexUnderCursor() = 0;
    virtual void updateCursorPosition(const glm::vec3&, std::uint32_t, std::uint32_t) = 0;

	PipelineParameters* getParameters() const { return _parameters; }
	void setParameters(PipelineParameters* val) { _parameters = val; }

protected:
	PipelineParameters* _parameters;
};

#endif // SHAPE_CREATOR_H
