// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SHAPE_EDITOR_H
#define SHAPE_EDITOR_H

#include "core/types.h"
#include "3d/app_enum.h"
#include "3d/anchor_controller.h"

class ShapeBuilder;
class Pickable;
class AnchorPoolNode;

namespace OGL
{
	class Shape;
	class Context;
}

class ShapeEditor: public virtual AnchorController
{
public:
    ShapeEditor();
	virtual ~ShapeEditor() override;

	virtual void initialize(OGL::Context* ctx, Pickable* obj) = 0;
	virtual void draw(OGL::Context* ctx, const glm::mat4& mvp) = 0;
	virtual OGL::Shape* createShape() = 0;
	virtual void setBuilder(ShapeBuilder* builder) = 0;
	SEAScope::ShapeType getShapeType();
	OGL::Shape* getShapeInEdition();

    virtual bool shapeIsValid() const = 0;
    virtual void updateCursorPosition(const glm::vec3&) = 0;
    virtual void startMove(unsigned int) = 0;
    virtual void stopMove() = 0;

protected:
	OGL::Context* _ctx;
	ShapeBuilder* _builder;
	Pickable* _shapeToEdit;
};

#endif // SHAPE_EDITOR_H
