// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SINGLETON_H
#define SINGLETON_H

template <typename T>
class Singleton
{
public:
    static T* getInstance()
    {
        if (nullptr == _instance)
        {
            _instance = new T;
        }

        return _instance;
    }

    static void killInstance()
    {
        delete _instance;
        _instance = nullptr;
    }

protected:
    Singleton(){}
    virtual ~Singleton(){}

private:
    static T* _instance;
};

template<typename T> T* Singleton<T>::_instance = nullptr;

#endif /* SINGLETON_H */

