// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   static_mesh.h
 * Author: gleseach
 *
 * Created on 4 septembre 2015, 10:21
 */

#ifndef STATIC_MESH_H
#define	STATIC_MESH_H

#include "3d/opengl_enum.h"

namespace OGL
{

class VertexArrayObject;
class Texture;
class ProgramBindHelper;
class Buffer;
class DrawMode;
class Context;

class StaticMesh
{
public:
    StaticMesh();
    ~StaticMesh();

    void create(OGL::Context* context);
    void create(OGL::Context* context, const std::vector<glm::vec3>& vertices);
    void create(OGL::Context* context, const std::vector<glm::vec3>& vertices, const std::vector<glm::vec2>& texCoords);

    void setVao(OGL::VertexArrayObject* vao);
    void setDrawDescription(OGL::DrawType draw_type, OGL::Primitive object_type, DataType dataType, unsigned int vertice_count);
    void setTextureMap(const std::map<unsigned int, OGL::Texture*>& textures);
    void setProgramHelper(OGL::ProgramBindHelper * program);
    void addBuffer(OGL::Buffer * buffer);

    void bindTextures(OGL::Context* context);

    void draw();

    OGL::ProgramBindHelper* getProgramHelper() const {return this->program;}
    OGL::Texture* getTexture(unsigned int id) const {return textures.find(id)->second;}
    const std::map<unsigned int, OGL::Texture*>& getTextureMap() const {return textures;}
    OGL::Buffer* getBuffer(unsigned int /*id*/) {return buffers[0];}

private:
    VertexArrayObject * vao;
    std::map<unsigned int, OGL::Texture*> textures;
    OGL::ProgramBindHelper * program;
    std::vector<OGL::Buffer*> buffers;
    OGL::DrawMode* _drawMode;
};

}

#endif	/* STATIC_MESH_H */

