// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   sync.h
 * Author: gleseach
 *
 * Created on 8 décembre 2017, 18:28
 */

#include <cstdint>

#ifndef SYNC_H
#define SYNC_H

namespace OGL
{

class Sync
{
public:
    Sync() {}
    virtual ~Sync();

    /*!
     * \brief Create the opengl sync object
     */
    virtual void create() = 0;

    /*!
     * \brief Delete the opengl sync object
     */
    virtual void destroy() = 0;

    /*!
     * \brief Block and wait for the sync object to become signaled.
     * \param[in] timeout Time to block waiting the sync object to become signaled.
     * \return true if the sync object is signaled, false if not signaled.
     */
    virtual bool wait(uint64_t timeout) = 0;
};

} // namespace OGL

#endif /* SYNC_H */
