// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   uv_streamlines.h
 * Author: gleseach
 *
 * Created on 9 octobre 2017, 17:47
 */

#ifndef UV_STREAMLINES_H
#define UV_STREAMLINES_H

namespace OGL
{
    class Context;
    class ProgramBindHelper;
    class DrawMode;
}

class StreamlinesData;
class DataSet;

class UVStreamlines
{
public:
    UVStreamlines();
    ~UVStreamlines();

    void create(OGL::Context* ctx);

    void update(OGL::Context* ctx, StreamlinesData* sd, DataSet* dataSet);

    // swap source and target IDs
    void swapVBO(StreamlinesData* sd);

    void physicPass(OGL::Context* ctx, StreamlinesData* sd, DataSet* dataSet, float rangeMax);

private:
    OGL::ProgramBindHelper* _physicProgram;
    OGL::DrawMode* _drawPhysic;
    int _scaleFactorsLoc;
    int _offsetLoc;
    int _glFactorLoc;
    int _speedLoc;
    int _headIDLoc;
    int _uvTexelStepLoc;
    int _rangeMaxLoc;
    int _seamlessLoc;
    int _uvMinLoc;
    int _uvRangeLoc;
    std::vector<glm::vec3> _particlesBuffer;

};

#endif /* UV_STREAMLINES_H */
