// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   uv_streamlines_color.h
 * Author: gleseach
 *
 * Created on 10 octobre 2017, 17:27
 */

#ifndef UV_STREAMLINES_COLOR_H
#define UV_STREAMLINES_COLOR_H

#include "3d/color_renderer.h"

namespace OGL
{
    class ProgramBindHelper;
    class DrawMode;
}

class DataSet;
class StreamlinesData;
class UVStreamlines;
class IWorldVisibility;

class UVStreamlinesColor : public ColorRenderer
{
public:
    UVStreamlinesColor();
    ~UVStreamlinesColor() override;

    void create(OGL::Context* ctx) override;
    void bind(OGL::Context* ctx, const glm::mat4& mvp, DataRenderingParam* param) override;
    void draw(OGL::Context* ctx, Renderable2* object) override;

private:
    void renderPass(OGL::Context* ctx, StreamlinesData* sd, DataSet* dataSet);

private:
    UVStreamlines* _streamlines;
    DataRenderingParam* _param;
    OGL::DrawMode* _drawRender;
    int _alphaLoc;
    int _streamlineLengthLoc;
    int _sizeLoc;
    int _texelSizeLoc;
    int _ttlLoc;
    int _fadeOutLoc;
    int _fadeInLoc;
    int _currentTimeLoc;
    float _currentTime;
};

#endif /* UV_STREAMLINES_COLOR_H */
