// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef VISIBILITY_GRID_H
#define VISIBILITY_GRID_H

namespace OGL
{
    class StaticMesh;
    class ShaderManager;
    class Program;
    class Context;
};

class VisibilityGrid
{
public:
    VisibilityGrid();
    void create(OGL::Context* ctx, OGL::ShaderManager* shader_mngr, unsigned subdivision);

    // top.xy = topleft, top.zw = topright, same for bottom
    void setBBox(const glm::vec4& top, const glm::vec4& bottom);
    void enable(bool flag);
    void draw(OGL::Context* ctx, const glm::mat4& mvp);
    void setColor(const glm::vec4& color);

private:
    bool _visible;
    glm::vec2 _min;
    glm::vec2 _max;
    OGL::StaticMesh* _mesh;
    OGL::Program* _program;
};

#endif /* VISIBILITY_GRID_H */

