// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ACTION_ID_H
#define ACTION_ID_H

#include <string>

#include "core/types.h"

namespace SEAScope
{

    class ActionId
    {
        public:
            ActionId(const ActorId&, const ActionType&);
            ActionId(const ActionId&);
            virtual ~ActionId();

            bool operator==(const ActionId&) const;

            const ActorId& getActorId() const;
            const ActionType& getActionType() const;
        protected:
            ActorId _actorId;
            ActionType _actionType;
    };

    struct ActionIdHasher
    {
        // https://stackoverflow.com/questions/17016175/c-unordered-map-using-a-custom-class-type-as-the-key
        size_t operator()( const ActionId& k ) const
        {
            // Compute individual hash values for first, second and third
            // http://stackoverflow.com/a/1646913/126995
            size_t res = 17;
            res = res * 31 + std::hash<ActorId>()( k.getActorId() );
            res = res * 31 + std::hash<ActionType>()( k.getActionType() );
            return res;
        }
    };

};

#endif
