// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef BASE_INPUT_EVENTS_SUBSCRIBER_H
#define BASE_INPUT_EVENTS_SUBSCRIBER_H

#include <set>
#include <string>
#include <unordered_map>

#include "core/interfaces/iinput_events_subscriber.h"

namespace SEAScope
{
    class BaseInputEventsSubscriber: public virtual IInputEventsSubscriber
    {
        public:
            BaseInputEventsSubscriber(const std::string&);
            ~BaseInputEventsSubscriber() override;

            void setId(const ActorId&) override;
            const ActorId& getId() const override;

            bool isActive() const override;
            void setActive(bool) override;

            bool getActorLabel(std::string&) const override;
            bool getActionLabel(const ActionType&, std::string&) const override;
            bool getDefaultBindingDefinition(const ActionType&, std::string&) const override;
            const std::set<ActionType> getPossibleActions() const override;
            const std::set<ActionType>& getAllActions() const override;
            bool preemptEvents() const override;

            void defineAction(const ActionType&, const std::string&, const std::string&);

        protected:
            ActorId _actorId;
            std::string _actorLabel;
            bool _active;
            bool _preemptEvents;

            std::set<ActionType> _possibleActions;
            std::set<ActionType> _supportedActions;
            std::unordered_map<ActionType, std::string> _actionLabels;
            std::unordered_map<ActionType, std::string> _defaultBindings;
    };
}

#endif
