// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef COLLECTIONS_CONTROL_H
#define COLLECTIONS_CONTROL_H

#include <map>
#include <vector>
#include "core/types.h"
#include "core/metadata_mediator.h"
#include "core/collections_control_model.h"
#include "ui/icollections_control_view.h"

namespace SEAScope
{
	class CollectionsControl
	{
		public:
			CollectionsControl();
			virtual ~CollectionsControl();

			bool init(MetadataMediator*, ICollectionsControlView*);
			bool setCollectionStateCallback(const std::function<void(const CollectionId&, bool)>&);
			bool setVariableStateCallback(const std::function<void(const CollectionVariable&, bool)>&);

			CollectionsControlModel model;
			ICollectionsControlView* view;
			std::function<void(const CollectionId&, bool)> collectionCallback;
			std::function<void(const CollectionVariable&, bool)> variableCallback;
            std::function<void(const CollectionVariable&)> exclusiveCallback;
	};
};

#endif
