// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef COLLECTIONS_CONTROL_MODEL_H
#define COLLECTIONS_CONTROL_MODEL_H

#include <vector>
#include <map>
#include <set>
#include <functional>
#include "core/types.h"
#include "core/structs.h"
#include "core/metadata_mediator.h"

namespace SEAScope
{
	class CollectionsControlModel
	{
		typedef std::function<void(const std::vector<std::pair<CollectionVariable, CatalogueEntry> >&)> CatalogueControlAddedCallback;
		typedef std::function<void(const std::vector<CollectionVariable>&)> CatalogueControlRemovedCallback;
		typedef std::function<void(const CollectionVariable&, bool)> CatalogueControlSelectionCallback;
		public:
			CollectionsControlModel();
			virtual ~CollectionsControlModel();

			bool init(MetadataMediator*);
			bool addCollections(const std::set<CollectionId>&);
			bool removeCollections(const std::set<CollectionId>&);
			bool select(const CollectionVariable&, bool);
            bool select(const CollectionVariable&, bool, bool);
			bool getSelectedCollections(std::set<CollectionId>&) const;
			bool getSelectedVariables(std::set<CollectionVariable>&) const;
			bool setAddedCallback(const CatalogueControlAddedCallback&);
			bool setRemovedCallback(const CatalogueControlRemovedCallback&);
			bool setSelectionCallback(const CatalogueControlSelectionCallback&);
			size_t countSelectedVars(const CollectionId&) const;
			bool getSelectedVars(const CollectionId&, std::set<VariableId>&) const;
			bool getCollectionVariableByLabel(const std::string&, const std::string&, CollectionVariable&) const;

			MetadataMediator* metadata;
			CatalogueControlAddedCallback addedCallback;
			CatalogueControlRemovedCallback removedCallback;
			CatalogueControlSelectionCallback selectionCallback;
            std::function<void(const CollectionVariable&)> exclusiveSelectionCallback;
			std::map<CollectionVariable, CatalogueEntry> store;
	};
};

#endif
