// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef COMMAND_GRUNT_H
#define COMMAND_GRUNT_H

#include <cstdint>

#include "vendors/_flatbuffers.h"

namespace SEAScope
{
    // Forward declarations
    class Engine;
    class RenderingCfg;

	class CommandGrunt
	{
		public:
			CommandGrunt();
			virtual ~CommandGrunt();
			std::uint8_t* execute( flatbuffers::FlatBufferBuilder&
								 , std::uint8_t*, std::uint64_t&, bool&);

			std::uint8_t* setCurrentDatetime( const API::SetCurrentDatetime*
											, std::uint64_t&, bool&);
            std::uint8_t* getCurrentDatetime( flatbuffers::FlatBufferBuilder&,
                                              std::uint64_t&, bool&);
			std::uint8_t* setRendering( const API::SetRendering*
									  , std::uint64_t&, bool&);
			std::uint8_t* searchGranules( flatbuffers::FlatBufferBuilder&
										, const API::SearchGranules*
										, std::uint64_t&, bool&);
			std::uint8_t* getExtraction( flatbuffers::FlatBufferBuilder&
									   , std::uint64_t&, bool&);
			std::uint8_t* addCollection( flatbuffers::FlatBufferBuilder&
                                       , const API::AddCollection*
									   , std::uint64_t&, bool&);
			std::uint8_t* addGranule( const API::AddGranule*
									, std::uint64_t&, bool&);
			std::uint8_t* addVariable(flatbuffers::FlatBufferBuilder&,
                                      const API::AddVariable*,
									  std::uint64_t&, bool&);
			std::uint8_t* getCatalogues( flatbuffers::FlatBufferBuilder&
									   , std::uint64_t&, bool&);
			std::uint8_t* listVisibleData( flatbuffers::FlatBufferBuilder&
										 , std::uint64_t&, bool&);
            std::uint8_t* getRendering( flatbuffers::FlatBufferBuilder&
                                         , const API::GetRendering*
                                         , std::uint64_t&, bool&);
			std::uint8_t* getVariableIdentifier(flatbuffers::FlatBufferBuilder&,
												const API::GetVariableIdentifier*,
												std::uint64_t&, bool&);
            std::uint8_t* getTimestep(flatbuffers::FlatBufferBuilder&,
									  std::uint64_t&, bool&);
            std::uint8_t* setTimestep(flatbuffers::FlatBufferBuilder&,
                                      const SEAScope::API::SetTimestep*,
										 std::uint64_t&, bool&);
            std::uint8_t* listTimesteps(flatbuffers::FlatBufferBuilder&,
                                        std::uint64_t&, bool&);
            std::uint8_t* getTimespan(flatbuffers::FlatBufferBuilder&,
									  std::uint64_t&, bool&);
            std::uint8_t* setTimespan(flatbuffers::FlatBufferBuilder&,
                                      const SEAScope::API::SetTimespan*,
										 std::uint64_t&, bool&);
            std::uint8_t* listTimespans(flatbuffers::FlatBufferBuilder&,
                                        std::uint64_t&, bool&);
            std::uint8_t* getNearestMode(flatbuffers::FlatBufferBuilder&,
                                         std::uint64_t&, bool&);
            std::uint8_t* setNearestMode(flatbuffers::FlatBufferBuilder&,
                                         const SEAScope::API::SetNearestMode*,
                                         std::uint64_t&, bool&);
            std::uint8_t* getDataAlwaysVisibleMode(flatbuffers::FlatBufferBuilder&,
                                                   std::uint64_t&, bool&);
            std::uint8_t* setDataAlwaysVisibleMode(flatbuffers::FlatBufferBuilder&,
                                                   const SEAScope::API::SetDataAlwaysVisibleMode*,
                                                   std::uint64_t&, bool&);
            std::uint8_t* checkSchemaVersion(flatbuffers::FlatBufferBuilder&,
                                             const SEAScope::API::CheckSchemaVersion*,
                                             std::uint64_t&, bool&);

            void resetCamera(bool&);
            void zoomIn(bool&);
            void zoomOut(bool&);
            void lookAt(const API::LookAt*, bool&);
            void setAltitude(const API::SetAltitude*, bool&);
			void selectVariable(const API::SelectVariable*, bool&);
			void selectVariableByLabel(const API::SelectVariableByLabel*, bool&);

			Engine* engine;
	};
};

#endif
