// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef CONCURRENT_QUEUE_H
#define CONCURRENT_QUEUE_H

#include <deque>
#include <thread>
#include <mutex>
#include <condition_variable>

namespace SEAScope
{
	template <typename T>
	class ConcurrentQueue
	{
		public:
			ConcurrentQueue() = default;

			// Disable copy and assignment
			ConcurrentQueue(const ConcurrentQueue&) = delete;
			ConcurrentQueue& operator=(const ConcurrentQueue&) = delete;

			/**
			 *
			 */
			T pop() {
				std::unique_lock<std::mutex> mlock(this->mutex);
				while
					(this->queue.empty())
				{
					this->cond.wait(mlock);
				}
				auto item = this->queue.front();
				this->queue.pop_front();
				return item;
			}

			/**
			 *
			 */
			void pop(T& item)
			{
				std::unique_lock<std::mutex> mlock(this->mutex);
				while
					(this->queue.empty())
				{
					this->cond.wait(mlock);
				}
				item = this->queue.front();
				this->queue.pop_front();
			}

			/**
			 *
			 */
			void push(const T& item)
			{
				std::unique_lock<std::mutex> mlock(this->mutex);
				this->queue.push_back(item);
				mlock.unlock();
				this->cond.notify_one();
			}

			std::deque<T> queue;
			std::mutex mutex;
			std::condition_variable cond;
	};
};

#endif
