// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef CONTROL_PROTOCOL_HANDLER_H
#define CONTROL_PROTOCOL_HANDLER_H

#include "core/types.h"
#include "core/interfaces/iprotocol_handler.h"

namespace SEAScope
{
    // Forward declarations
    class ThreadsPool;
    class CommandGrunt;

	class ControlProtocolHandler final: public virtual IProtocolHandler
	{
		public:
			ControlProtocolHandler(ThreadsPool*, CommandGrunt*);
			~ControlProtocolHandler() override;

			bool parseStream(socket_t, std::uint8_t*, std::uint64_t, std::uint64_t, std::uint64_t&) override;
			bool sendData(socket_t, std::uint8_t*, std::uint64_t) override;
			bool reset() override;

        protected:
            std::uint64_t _msgSize;
            std::uint64_t _cumulatedRead;
            std::uint8_t* _buffer;

		    ThreadsPool* _workers;
			CommandGrunt* _grunt;
	};
};

#endif
