// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   copy.h
 * Author: gleseach
 *
 * Created on 31 août 2015, 17:22
 */

#ifndef COPY_H
#define	COPY_H

#include "core/log.h"

/* must be included in the header because it contains templates */
#include "vendors/_glm.h"

void memcpy2D(void * src, unsigned int src_origin_x, unsigned int src_origin_y, unsigned src_width, unsigned src_height, unsigned int element_size, 
              void * dst, unsigned int dst_origin_x, unsigned int dst_origin_y, unsigned dst_width, unsigned dst_height, unsigned int area_width, unsigned int area_height);

template<typename T>
void memcpy2D_t(T* src, const glm::ivec2& srcOrigin, const glm::ivec2& srcSize, unsigned int elementCount, 
                T* dst, const glm::ivec2& dstOrigin, const glm::ivec2& dstSize,
                const glm::ivec2& areaSize)
{
    memcpy2D(
        src,
        static_cast<unsigned int>(srcOrigin.x),
        static_cast<unsigned int>(srcOrigin.y),
        static_cast<unsigned int>(srcSize.x),
        static_cast<unsigned int>(srcSize.y),
        sizeof(T) * elementCount,
        dst,
        static_cast<unsigned int>(dstOrigin.x),
        static_cast<unsigned int>(dstOrigin.y),
        static_cast<unsigned int>(dstSize.x),
        static_cast<unsigned int>(dstSize.y),
        static_cast<unsigned int>(areaSize.x),
        static_cast<unsigned int>(areaSize.y)
    );
}

/*
 * \brief Copies a pattern as many times as the buffer size allows
 */
size_t fillBuffer(void* dstBuffer, size_t dstBufferSize, void* srcPattern, size_t srcPatternSize);

#endif	/* COPY_H */

