// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SAFE_DATA_BUCKET_H
#define SAFE_DATA_BUCKET_H

#include <mutex>

#include "core/structs.h"
#include "data/image.h"

namespace SEAScope
{
    class DataBucket
    {
        public:
            DataBucket();
            ~DataBucket();

            // Copy not allowed
            DataBucket(const DataBucket&) = delete;
            DataBucket& operator=(const DataBucket&) = delete;

            Image& getImage();
            const Image& getImage() const;
            const Image* getImagePtr() const;

            DataInfo& getInfo();
            const DataInfo& getInfo() const;
            const DataInfo* getInfoPtr() const;

            void setLoaded(bool);
            bool loaded() const;

            void setMissing(bool);
            bool isMissing() const;

        protected:
            Image _image;
            DataInfo _info;
            bool _loaded;
            bool _missing;

            mutable std::mutex _mutex;
	};
}

#endif
