// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef DATA_MEDIATOR_H
#define DATA_MEDIATOR_H

#include <string>
#include <set>
#include <functional>
#include "core/interfaces/idata_store.h"

namespace OGL
{
    class Shape;
}

namespace SEAScope
{
    // Forward declarations
    class RawToRenderableQueue;
    class IAnnotationsStore;
    class ThreadsPool;
    class SourcesControlModel;

	class DataMediator
	{
		public:
			DataMediator();
			virtual ~DataMediator();

			bool init(IDataStore*, const IAnnotationsStore*, SourcesControlModel*);
            void setWorkers(ThreadsPool*);

            bool getField(const GranuleId, const std::string&, const std::string&, const Image*&, OGL::Shape*&, const DataInfo*&) const;
            bool getFields(const std::string&, const std::set<std::string>&, std::vector<const Image*>&, std::vector<const DataInfo*>&) const;

			bool loadData(const CollectionId&, const GranuleId&, const std::string&, float resolution,const std::set<std::string>&, DataModel, RawToRenderableQueue&);

            GranuleData& getURIDataRef(const std::string&);

        protected:
			SourcesControlModel* _sources;
			ThreadsPool* _workers;
			IDataStore* _store;
            const IAnnotationsStore* _annotations;

            bool _parallelizeFieldsLoading;
	};
};

#endif
