// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef DATA_SCHEDULER_H
#define DATA_SCHEDULER_H

#include <set>
#include <list>
#include <unordered_map>

#include "core/interfaces/ijob_scheduler.h"
#include "core/job.h"

namespace SEAScope
{
    struct OngoingDataLoadingTask
    {
        std::string uri;
        std::set<std::string> fields;
    };

	class DataScheduler final: public virtual IJobScheduler
	{
		public:
			DataScheduler();
			~DataScheduler() override;

			bool addJob(Job&&, std::string&, JobStatus&) override;
			bool removeJob(const std::string&) override;
			bool selectJob(Job&) override;
			bool updateJobStatus(const std::string&, const JobStatus&) override;
			std::size_t countPendingJobs() const override;

        protected:
            void getOngoingFieldsSet(const std::string&, std::set<std::string>&) const;
            void getPendingFieldsSet(const std::string&, std::set<std::string>&) const;

			std::list<DataLoadingJob> _pendingJobs;
            std::unordered_map<std::string, OngoingDataLoadingTask> _ongoingJobs;
	};
};

#endif
