// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef DISPLAY_QUEUE_H
#define DISPLAY_QUEUE_H

#include <set>

#include "core/types.h"
#include "core/concurrent_queue.h"

#include "core/structs.h"
#include "3d/geodetic.h"
#include "core/rendering_cfg.h"

namespace OGL
{
    class Shape;
};

// Forward declarations
class DataSet;

namespace SEAScope
{

    struct DisplayJob
    {
        DisplayJobType jobType;
        DataSet* dataset;
        OGL::Shape* shape;
        std::vector<float> relativeTime;
        std::vector<Geodetic2D> points;
        std::vector<float> fData;
        std::vector<bool> mask;
        RenderingCfg renderingCfg;
        RenderableId rId;
        float resolution;
        std::set<RenderableId> toRender;
        DisplayBatchId batchId;

        DisplayJob()
        : jobType{DisplayJobType::NOOP}, dataset{nullptr}, shape{nullptr}, resolution{0.f}
        {}
    };

	class DisplayQueue: public virtual ConcurrentQueue<DisplayJob>
	{
		public:
			DisplayQueue();
			virtual ~DisplayQueue();

			// Disable copy constructor and assignment
			DisplayQueue(const DisplayQueue&) = delete;
			DisplayQueue& operator=(const DisplayQueue&) = delete;

			bool contains(const RenderableId&, float);
			bool addJob(DisplayJob&);
            bool isLoadDataDeprecated(const DisplayJob&) const;

            DisplayJob& stageFirst();
            void unstage();

            DisplayJob jobOnStage;
            std::set<RenderableId> toRender;
	};
};

#endif
