// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef EVENTHANDLER_H
#define EVENTHANDLER_H

#include <functional>

namespace SEAScope
{
	class EventHandler
	{
		public:
			virtual ~EventHandler();

			virtual bool handleKeyEvent(int,int,int,int){return true;/*override this*/}
			virtual bool handleCharEvent(unsigned int, int){return true;/*override this*/}
			virtual bool handleMouseButtonEvent(int, int, int){return true;/*override this*/}
			virtual bool handleMouseMoveEvent(double, double){return true;/*override this*/}
			virtual bool handleMouseWheelEvent(double,double){return true;/*override this*/}
			virtual bool handleResizeEvent(int, int, int, int){return true;/*override this*/}
			virtual bool handleFocusEvent(int){return true;/*override this*/}
			virtual bool handleIconifyEvent(int){return true;/*override this*/}
			virtual bool handleDropEvent(int,const char**){return true;/*override this*/}
			virtual bool handleCloseEvent(){return false;/*override this*/}
            virtual void setPreemptEventsCallback(const std::function<void(EventHandler*, bool)>&) {}

	};
};

#endif
