// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef GRANULE_METADATA_BLOB_H
#define GRANULE_METADATA_BLOB_H

#include <cstdint>
#include <vector>
#include <string>
#include <tuple>

#include "core/types.h"


namespace SEAScope
{
    // Forward declarations
    struct IDFDescriptor;

    class GranuleMetadata
    {
        public:
            GranuleMetadata();

            void build(std::uint32_t sourceId,
                       std::uint32_t collectionId,
                       std::uint64_t granuleId,
                       DataModel dataModel,
                       timestamp_t timeCoverageStart,
                       timestamp_t timeCoverageEnd,
                       bool hasLatMin,
                       float latMin,
                       bool hasLatMax,
                       float latMax,
                       bool hasLonMin,
                       float lonMin,
                       bool hasLonMax,
                       float lonMax,
                       const std::string& dataId,
                       const std::string& title,
                       const std::string& institution,
                       const std::string& comment,
                       const std::string& fileId,
                       const std::string& productVersion,
                       const std::string& creatorEmail,
                       const std::string& stationId,
                       const std::string& platform,
                       const std::string& sensor,
                       const IDFDescriptor& descriptor);

            void addIDFDescriptor(const IDFDescriptor& descriptor);
            bool selectResolution(std::uint32_t viewportPixelArea,
                                  float viewportSpacingInMeters,
                                  float minScreenEstate,
                                  std::uint32_t& xArity,
                                  std::uint32_t& yArity,
                                  IDFDescriptor& result) const;

            // RO accessors
            std::uint32_t sourceId() const;
            std::uint32_t collectionId() const;
            std::uint64_t granuleId() const;
            DataModel dataModel() const;
            timestamp_t timeCoverageStart() const;
            timestamp_t timeCoverageEnd() const;
            float latMin() const;
            float latMax() const;
            float lonMin() const;
            float lonMax() const;

            std::string dataId() const;
            std::string title() const;
            std::string institution() const;
            std::string comment() const;
            std::string fileId() const;
            std::string productVersion() const;
            std::string creatorEmail() const;
            std::string stationId() const;
            std::string platform() const;
            std::string sensor() const;

            std::uint32_t viewsCount() const;
            bool getView(std::uint32_t, IDFDescriptor&) const;
            bool getViewURI(std::uint32_t, std::string&) const;

            bool has(IDFAttributes) const;

            // RW accessors
            void sourceId(std::uint32_t);
            void collectionId(std::uint32_t);
            void granuleId(std::uint64_t);
            void timeCoverageStart(timestamp_t);
            void timeCoverageEnd(timestamp_t);

            const std::uint8_t* getBlob(std::uint64_t& size) const;
            void setBlob(std::uint8_t* value, std::uint64_t size);

        protected:
            std::vector<std::uint8_t> _blob;
    };
}

#endif
