// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef INPUT_BINDINGS_H
#define INPUT_BINDINGS_H

#include <unordered_map>

#include <string>

#include "core/types.h"
#include "core/action_id.h"
#include "core/input_binding.h"

namespace SEAScope
{
    // Forward declarations
    class InputEvent;
    class InputDevicesState;

    class InputBindings
    {
        public:
            InputBindings();
            virtual ~InputBindings();

            bool registerAction(const ActionId&, const std::string&, Synchronicity);
            void setBinding(const ActionId&, InputBinding&&);
            bool checkAction(const ActionId&, const InputEvent&, const InputDevicesState&) const;
            bool hasValidBinding(const ActionId&) const;
            bool translateBindingDefinition(const std::string&, InputBinding&) const;
            bool getBindingDescription(const ActionId&, std::string&) const;
            virtual bool initKeyboardMapping() = 0;


        protected:
            void getKeyName(int, std::string&) const;
            void getMouseButtonName(int, std::string&) const;

            std::unordered_map<ActionId, Synchronicity, ActionIdHasher> _synchronicities;
            std::unordered_map<ActionId, InputBinding, ActionIdHasher> _bindings;
            std::unordered_map<std::string, int> _keyCodes;

    };
}

#endif
