// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef INPUT_DEVICES_STATE_H
#define INPUT_DEVICES_STATE_H

#include <cstdint>
#include <unordered_map>

namespace SEAScope
{
    class InputDevicesState
    {
        public:
            InputDevicesState();
            virtual ~InputDevicesState();

            double getMouseX() const;
            void setMouseX(double);

            double getMouseY() const;
            void setMouseY(double);

            bool isKeyPressed(int scancode) const;
            void setKeyPressed(int scancode, bool pressed);

            bool isButtonPressed(int button) const;
            void setButtonPressed(int button, bool pressed);

            std::uint16_t getPressedKeysCount() const;
            std::uint16_t getPressedButtonsCount() const;

            const std::array<int, 2>& getPressedButtonPosition(int) const;
            bool hasMouseMovedSinceButtonPress(int button, std::uint32_t pixelsTolerance) const;

            void reset();

        protected:
            double _mouseX;
            double _mouseY;
            std::unordered_map<int, bool> _buttons; // button number
            std::unordered_map<int, bool> _keys; // scancodes
            std::unordered_map<int, std::array<int, 2> > _buttonsPressedPosition;
            std::uint16_t _pressedKeysCount;
            std::uint16_t _pressedButtonsCount;
    };
};

#endif
