// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef INPUT_EVENT_H
#define INPUT_EVENT_H

#include <cstdint>
#include <string>

namespace SEAScope
{
    enum class EventType: std::uint8_t {
        NONE = 0,
        MOUSE_MOVE = 1,
        MOUSE_WHEEL_CHANGE = 2,
        MOUSE_BUTTON_PRESS = 3,
        MOUSE_BUTTON_REPEAT = 4,
        MOUSE_BUTTON_RELEASE = 5,
        MOUSE_BUTTON_CLICK = 6,
        KEY_PRESS = 7,
        KEY_REPEAT = 8,
        KEY_RELEASE = 9,
        MOUSE_BUTTON_STATE_PRESSED = 10,
        MOUSE_BUTTON_STATE_RELEASED = 11,
        KEY_STATE_PRESSED = 12,
        KEY_STATE_RELEASED = 13,
        RESIZE = 14,
        CLOSE = 15,
        ANYTHING = 16
    };

    class InputEvent
    {
        public:
            InputEvent();
            InputEvent(InputEvent&&) noexcept;
            virtual ~InputEvent();

            std::string str() const;

            double getMouseDx() const;
            void setMouseDx(double);

            double getMouseDy() const;
            void setMouseDy(double);

            double getMouseWheelDx() const;
            void setMouseWheelDx(double);

            double getMouseWheelDy() const;
            void setMouseWheelDy(double);

            int getButton() const;
            void setButton(int);

            int getKeyScanCode() const;
            void setKeyScanCode(int);

            int getWidth() const;
            void setWidth(int);

            int getHeight() const;
            void setHeight(int);

            EventType getType() const;
            void setType(EventType);
        private:
            EventType _type;
            double _mouseDx;
            double _mouseDy;
            double _mouseWheelDx;
            double _mouseWheelDy;
            int _button;
            int _keyScanCode;
            int _width;
            int _height;
    };
};

#endif
