// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef INPUT_EVENTS_DISPATCHER_H
#define INPUT_EVENTS_DISPATCHER_H

#include <vector>

#include "core/input_event.h"
#include "core/input_devices_state.h"

namespace SEAScope
{
    /* Forward declarations */
    class IInputEventsSubscriber;
    class InputBindings;

    class InputEventsDispatcher
    {
        public:
            InputEventsDispatcher();
            virtual ~InputEventsDispatcher();

            const InputDevicesState& getInputDevicesState() const;
            const std::vector<IInputEventsSubscriber*>& getSubscribers() const;
            bool addSubscriber(IInputEventsSubscriber*);

            void loadBindings(const std::string&);
            void saveBindings(const std::string&) const;

            virtual void initBindings() = 0;
            virtual const InputBindings& getInputBindings() const = 0;
            virtual bool dispatchEvent(const InputEvent&);

            std::uint32_t getClickPixelsTolerance() const;
            bool setClickPixelsTolerance(std::uint32_t);

        protected:
            virtual InputBindings& getInputBindings() = 0;

            InputDevicesState _inputDevicesState;
            std::vector<IInputEventsSubscriber*> _subscribers;

            std::uint32_t _clickPixelsTolerance;
            /*
            TODO: add sync and async queues
            */
    };
}

#endif
