// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IANNOTATIONS_STORE_H
#define IANNOTATIONS_STORE_H

#include <cstdint>
#include <string>
#include <vector>
#include <tuple>
#include <functional>

#include "core/interfaces/iannotation_buffer.h"
#include "data/annotation.h"
#include "core/structs.h"
#include "core/rendering_cfg.h"

namespace SEAScope
{
    // Forward declarations
    struct RenderableId;

    class IAnnotationsStore
    {
        public:
            virtual ~IAnnotationsStore();

            virtual bool init(const std::string&) = 0;
            virtual bool get(std::uint64_t granuleId, std::uint32_t& collection, Annotation& annotation) const = 0;
            virtual bool set(std::uint64_t granuleId, std::uint32_t collection, const Annotation& annotation) = 0;
            virtual bool set(const std::vector<std::uint64_t>&, const std::vector<std::uint32_t>&, const std::vector<Annotation>&) = 0;
            virtual bool getall(std::vector<std::tuple<std::uint64_t, std::uint32_t, Annotation>>&) const = 0;
            virtual bool applyall(const std::function<void(std::uint64_t, std::uint32_t, Annotation&)>&) const = 0;
            virtual bool remove(const RenderableId&) = 0;
            virtual bool getMaxGranuleId(std::uint64_t&) const = 0;
            virtual bool count(std::size_t&) const = 0;

            virtual bool getMaxCollectionId(std::uint32_t&) const = 0;
            virtual bool getCollection(std::uint32_t, CollectionCfg&) const = 0;
            virtual bool setCollection(std::uint32_t, const CollectionCfg&, const std::vector<RenderingCfg*>&) = 0;
            virtual bool removeCollection(std::uint32_t) = 0;
            virtual bool getAllCollections(std::vector<std::tuple<std::uint32_t, CollectionCfg, std::vector<RenderingCfg>>>& result) const = 0;
            virtual bool applyAllCollections(const std::function<void(std::uint32_t, CollectionCfg&, std::vector<RenderingCfg>&)>&) const = 0;
            virtual bool listCollectionGranules(std::uint32_t, std::vector<std::uint64_t>&) const = 0;
            virtual bool countCollections(std::size_t&) const = 0;

            virtual bool setRendering(const RenderingCfg&) = 0;
    };
}
#endif
