// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IDATA_STORE_H
#define IDATA_STORE_H

#include <set>
#include <string>
#include <vector>
#include <unordered_map>

#include "core/types.h"

// Forward declarations
class Image;

namespace SEAScope
{
    // Forward declarations
    class DataBucket;
    struct DataInfo;

	class IDataStore
	{
		public:
			virtual ~IDataStore();

			virtual GranuleData& getURIDataRef(const std::string& uri) = 0;
			virtual bool removeData(const std::string& uri) = 0;
            virtual bool listFields(const std::string& uri, std::set<std::string>& result) = 0;
            virtual bool getMissingDataBuckets(const std::string&, const std::set<std::string>&, std::unordered_map<std::string, DataBucket*>&) = 0;
            virtual bool getField(const std::string&, const std::string&, const Image*&, const DataInfo*&) const = 0;
            virtual bool getFields(const std::string&, const std::set<std::string>&, std::vector<const Image*>&, std::vector<const DataInfo*>&) const = 0;
	};
};

#endif
