// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IDATASTREAM_SERVICE_CONNECTOR_H
#define IDATASTREAM_SERVICE_CONNECTOR_H

#include <set>
#include <string>

#include "core/types.h"
#include "core/structs.h"

namespace SEAScope
{
    // Forward declarations
    class ILogHandler;
    class IPathHandler;

	class IDatastreamServiceConnector
	{
		public:
			virtual ~IDatastreamServiceConnector();

			virtual bool init(ILogHandler*, IPathHandler*) = 0;
            virtual bool reset() = 0;
			virtual bool loadData(const GranuleId&,
                                  const std::string&,
                                  DataModel,
                                  const std::unordered_map<std::string, DataBucket*>& dataBuckets) const = 0;
			virtual bool canHandle(const std::string&) const = 0;

			/* Required by plugin system */
			static const char* api()
            {
                return "datastream";
            }
	};
};

#endif
