// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IGRANULES_INDEX_H
#define IGRANULES_INDEX_H

#include <set>
#include <tuple>
#include <array>
#include <vector>
#include <string>
#include <cstdint>
#include <functional>

#include "core/types.h"

namespace SEAScope
{
    // Forward declarations;
    struct SearchCriteria;
    class GranuleMetadata;
    class EventsCoverage;

	class IGranulesIndex
	{
		public:
			virtual ~IGranulesIndex();

            virtual bool startBulkOperation() = 0;
            virtual bool stopBulkOperation() = 0;
            virtual bool cancelBulkOperation() = 0;
            virtual bool increaseBulkOpsCount() = 0;
			virtual bool hasGranule(const std::string&) const = 0;
			virtual bool hasGranule(std::uint64_t) const = 0;
			virtual bool add(GranuleMetadata&, bool) = 0;
            virtual bool updateTimeCoverage(std::uint64_t, const timestamp_t, const timestamp_t) = 0;
            virtual bool remove(const std::string&, bool) = 0;
            virtual bool remove(std::uint64_t, bool) = 0;
			virtual std::array<bool, 2> adjacentGranules( const timestamp_t
												, const timestamp_t
												, const std::set<std::uint32_t>&
												, GranuleMetadata&
												, GranuleMetadata&) const = 0;
			virtual bool closestGranuleBound( const timestamp_t
                                    , const timestamp_t
                                    , const timestamp_t
									, const std::set<std::uint32_t>&
									, GranuleMetadata&) const = 0;
			virtual bool getCoverage(const std::set<std::uint32_t>&, EventsCoverage&) const = 0;
			virtual bool getCoverage(std::uint32_t, EventsCoverage&) const = 0;
			virtual bool findGranules(const SearchCriteria&, std::vector<GranuleMetadata>&) const = 0;
            virtual bool setGranuleAddedCallback(const std::function<void(std::uint64_t, std::uint32_t)>&) = 0;
            virtual std::uint64_t generateGranuleId() = 0;
            virtual std::tuple<bool, bool> load(const std::string&) = 0;
            virtual bool save(const std::string&) = 0;
            virtual bool setGranuleIdGenerator(std::uint64_t) = 0;
            virtual bool countGranules(std::size_t&) const = 0;

            virtual bool setCollectionIdGenerator(std::uint32_t) = 0;
            virtual std::uint32_t generateCollectionId() = 0;
            virtual bool addCollection(const std::string&, const std::uint32_t) = 0;
            virtual bool removeCollection(const std::string&) = 0;
            virtual bool removeCollection(std::uint32_t) = 0;
            virtual bool listCollections(std::vector<std::pair<std::uint32_t, std::string> >&) const = 0;
            virtual bool countCollections(std::size_t&) const = 0;

            virtual bool clear() = 0;
	};
};

#endif
