// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IINDEX_SERVICE_CONNECTOR_H
#define IINDEX_SERVICE_CONNECTOR_H

#include <set>
#include <array>
#include "core/types.h"
#include "core/structs.h"
#include "data/events_coverage.h"

namespace SEAScope
{
    // Forward declarations
    class RenderingCfg;
	class IDialogHandler;
    class ILogHandler;
    class IPathHandler;
    class Settings;

	class IIndexServiceConnector
	{
		public:
			virtual ~IIndexServiceConnector();

			virtual bool init(const Settings&, ILogHandler*, IPathHandler*, IDialogHandler*) = 0;
            virtual bool reset() = 0;
			virtual bool searchGranules( const SearchCriteria&
									   , std::vector<GranuleMetadata>&) const = 0;
			virtual std::array<bool, 2> searchPrevNext( timestamp_t
									   , const std::set<CollectionId>&
									   , timestamp_t&, timestamp_t&, timestamp_t) const = 0;
			virtual bool searchClosestBound( timestamp_t
									   , const std::set<CollectionId>&
									   , timestamp_t&
                                       , const timestamp_t
                                       , const timestamp_t) const = 0;
			virtual bool getDataCoverage(const std::set<CollectionId>&, EventsCoverage&) const = 0;
			virtual bool getDataCoverage(const CollectionId&, EventsCoverage&) const = 0;
			virtual bool listCollections(std::set<std::uint32_t>&) const = 0;
			virtual bool getCollectionCfg(std::uint32_t, CollectionCfg&, std::vector<RenderingCfg*>&) const = 0;
            virtual bool saveVariableRenderingCfg(const SEAScope::RenderingCfg*) const = 0;
			virtual bool canHandle(const std::string&) const = 0;

			/* Required by plugin system */
			static const char* api()
            {
                return "index";
            }
	};
};

#endif
