// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IINPUT_EVENTS_SUBSCRIBER_H
#define IINPUT_EVENTS_SUBSCRIBER_H

#include <set>
#include <string>

#include "core/types.h"

namespace SEAScope
{
    // Forward declarations
    class InputEvent;
    class InputDevicesState;

    class IInputEventsSubscriber
    {
        public:
            virtual ~IInputEventsSubscriber();

            virtual void setId(const ActorId&) = 0;
            virtual const ActorId& getId() const = 0;

            virtual bool isActive() const = 0;
            virtual void setActive(bool) = 0;

            virtual bool getActorLabel(std::string&) const = 0;
            virtual bool getActionLabel(const ActionType&, std::string&) const = 0;
            virtual bool getDefaultBindingDefinition(const ActionType&, std::string&) const = 0;
            virtual const std::set<ActionType> getPossibleActions() const = 0;
            virtual const std::set<ActionType>& getAllActions() const = 0;
            virtual bool performAction(const ActionType&, const InputEvent&, const InputDevicesState&) = 0;
            virtual bool preemptEvents() const = 0;
    };
};

#endif
