// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ILOG_HANDLER_H
#define ILOG_HANDLER_H

#include <cstdint>
#include <string>

#include "vendors/_string_view.h"

namespace SEAScope
{
    enum class LogLevel : std::uint8_t
    {
        Debug = 0,
        Info = 1,
        Warn = 2,
        Error = 3,
        Fatal = 4
    };

    class ILogHandler
    {
        public:
            virtual ~ILogHandler();

            virtual bool log(const LogLevel&, const char*, const string_view&) = 0;
            virtual bool isMessageValid(const LogLevel&, const string_view&) = 0;
            virtual LogLevel getLevel() const = 0;
    };
};

#endif
