// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/**
 *
 */

#ifndef IPATH_HANDLER_H
#define IPATH_HANDLER_H

#include <ctime>
#include <cstdint>
#include <sys/stat.h>
#include <string>
#include <vector>
#include <forward_list>
#include <cstdint>

#ifdef _WIN32
	typedef unsigned int mode_t;
#endif

namespace SEAScope
{
	class IPathHandler
	{
		public:
			virtual ~IPathHandler();

			virtual bool dirExists(const std::string&) = 0;
			virtual bool fileExists(const std::string&) = 0;
			virtual bool fileSize(const std::string&, std::int64_t&) = 0;
			virtual bool writeFile(const std::string&) = 0;
			virtual bool createDir(const std::string&, mode_t) = 0;
			virtual bool createDirs(const std::string&, mode_t) = 0;
			virtual void splitPaths(const std::string&, const std::string&, std::forward_list<std::string>&) = 0;
			virtual void joinPaths(const std::vector<std::string>&, std::string&) = 0;
			virtual bool listFiles(const std::string&, std::forward_list<std::string>&) = 0;
            virtual bool listFilesRecursive(const std::string&, std::forward_list<std::string>&) = 0;
			virtual bool deleteFile(const std::string&) = 0;
			virtual char separator() const = 0;
			virtual void fixPlatformPath(std::string& str) = 0;
			virtual bool lastModified(const std::string& path, std::time_t&) = 0;
            virtual bool currentWorkingDir(std::string& path) = 0;
            virtual bool canonicalPath(const std::string&, std::string&) = 0;
            virtual bool homeDir(std::string& path) = 0;
            virtual bool absolutePath(const std::string&, std::string&) = 0;
            virtual bool getParentDir(const std::string&, std::string&) = 0;
	};
};

#endif
