// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ISOCKET_FACTORY_H
#define ISOCKET_FACTORY_H

#include <cstdint>
#include <string>

#include "core/types.h"

namespace SEAScope
{
	class ISocketFactory
	{
		public:
			virtual ~ISocketFactory();

			virtual socket_t create(const std::string&, std::int64_t, bool&) = 0;
			virtual bool nonBlocking(socket_t) = 0;
			virtual bool shutdown(socket_t) = 0;
			virtual bool listening(socket_t) = 0;
			virtual socket_t connect(const std::string&, std::int64_t, bool&) = 0;
            virtual bool resolveHostname(const std::string&, std::string&) = 0;
	};
};

#endif
