// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ISOCKETS_MONITOR_H
#define ISOCKETS_MONITOR_H

#include <cstdint>
#include <functional>

#include "core/interfaces/iprotocol_handler.h"

namespace SEAScope
{
	typedef std::function<IProtocolHandler*(bool&)> ProtocolHandlerFactory;
	struct SocketJob
	{
		SocketJob(bool s, bool t, std::int32_t fd, IProtocolHandler* h,
                  const ProtocolHandlerFactory& hFactory)
		: subscribe(s), terminate(t), fileDescriptor(fd), handler(h),
          handlerFactory(hFactory)
        {
		}

		bool subscribe;
		bool terminate;
		std::int32_t fileDescriptor;
		IProtocolHandler* handler;
		ProtocolHandlerFactory handlerFactory;
	};

	class ISocketsMonitor
	{
		public:
			virtual ~ISocketsMonitor();

			virtual bool init(const std::function<void()>&, int&) = 0;
			virtual bool handleEvent(const SocketJob&) = 0;
			virtual bool triggerEventsProcessing() = 0;
			virtual bool monitorSockets() = 0;
			virtual bool sendData(socket_t, std::uint8_t*, std::uint64_t) = 0;
	};
};

#endif
