// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IUSER_INTERFACE_H
#define IUSER_INTERFACE_H

// Forward declarations
class Icons;

namespace SEAScope
{
    // Forward declarations
    class IWindow;
    class Settings;
    class State;
    class ICollectionsControlView;
    class IRenderingControlView;
    class IAdvancedRenderingControlView;
    class ITimelineView;
    class ISourcesControlView;
    class ICameraControlView;
    class IRenderer;
    class EventHandler;

    class IUserInterface
    {
        public:
            virtual ~IUserInterface();

            virtual bool init(IWindow&, Settings&, State*) = 0;
            virtual void showProcessingOptions(bool) = 0;
            virtual void setIcons(Icons&) = 0;
            virtual ICollectionsControlView* getCollectionsUI() = 0;
            virtual IRenderingControlView* getRenderingUI() = 0;
            virtual IAdvancedRenderingControlView* getAdvancedRenderingUI() = 0;
            virtual ITimelineView* getTimelineUI() = 0;
            virtual ISourcesControlView* getSourcesUI() = 0;
            virtual ICameraControlView* getCameraUI() = 0;
            virtual IRenderer* getRenderer() = 0;
            virtual EventHandler* getEventHandler() = 0;
    };
}

#endif
