// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IWINDOW_H
#define IWINDOW_H

#include <cstdint>
#include "core/settings.h"
#include "core/interfaces/irenderer.h"
#include "core/eventhandler.h"

namespace SEAScope
{
	class IWindow
	{
		public:

            enum class MouseCursor: std::uint8_t
            {
                ARROW = 0,
                CROSSHAIR = 1,
            };

			virtual ~IWindow();

			virtual bool init(const Settings&) = 0;
			virtual bool create( uint16_t,uint16_t) = 0;
			virtual bool renderFrame() = 0;
			virtual bool resize(uint16_t, uint16_t) = 0;
            virtual bool clean() = 0;
            virtual bool getSize(int&, int&) const = 0;
            virtual bool getFrameBufferSize(int&, int&) const = 0;
            virtual bool getCursorPos(double&, double&) const = 0;
            virtual bool setCursor(MouseCursor) = 0;
            virtual void close() = 0;

			virtual bool addRenderer(IRenderer&) = 0;
			virtual bool addHandler(EventHandler&) = 0;

	};
};

#endif
