// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef MEASUREMENTS_H
#define MEASUREMENTS_H

#include <string>
#include <vector>

#include "core/structs.h"
#include "3d/shape.h"

namespace SEAScope
{
    constexpr double PI = 3.141592653589793238462643383279502884;
	constexpr double EARTH_MEAN_RADIUS_IN_METER = 6371008.8;

	bool getOuterRing(const Shape&, std::vector<GCP>&);
	bool haversineDist(double, double, double, double, double&);
	bool vincentyDist(double, double, double, double, double&, double);
	double distance(double, double, double, double);
	double polarTriangleArea(double, double, double, double);
	double computeSignedArea(const std::vector<double>&, const std::vector<double>&);
	double areaFromShape(const SEAScope::Shape&);
	double perimeterFromShape(const SEAScope::Shape&);

    double areaFromShape(const OGL::Shape*);
    double perimeterFromShape(const OGL::Shape*, bool);

    bool formatDistance(double, std::string&);
    bool formatArea(double, std::string&);
    bool formatLongitude(double, std::string&);
    bool formatLatitude(double, std::string&);

    void getShapeMeasurements(OGL::Shape*, double&, double&, double&, double&);
};

#endif
