// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef METADATA_MEDIATOR_H
#define METADATA_MEDIATOR_H

#include <set>
#include <map>
#include <string>
#include <vector>
#include <functional>
#include <unordered_map>

#include "core/types.h"
#include "core/rendering_cfg.h"
#include "data/events_coverage.h"
#include "core/sources_control_model.h"
#include "core/interfaces/imetadata_store.h"
#include "core/interfaces/iindex_service_connector.h"

namespace SEAScope
{
    typedef std::function<bool(RenderingCfg*)> VariableCfgAdded;

	class MetadataMediator
	{
		public:
			MetadataMediator();
			virtual ~MetadataMediator();

			bool init(IMetadataStore*, SourcesControlModel*);
			bool collectionCfgLoaded(const CollectionId&, const CollectionCfg&, const std::vector<RenderingCfg*>&);
			bool getCollectionCfg(const CollectionId&, CollectionCfg&);
			bool getVariableCfg(const CollectionId&, const VariableId&, VariableCfg&);
            bool saveVariableRenderingCfg(const RenderingCfg*);
			bool getFields(const CollectionId&, const std::set<VariableId>&, std::set<std::string>&);
			bool getFields(const CollectionVariable&, std::set<std::string>&);
			bool searchPrevNext(timestamp_t, const std::set<CollectionId>&, timestamp_t&, timestamp_t&, timestamp_t minDelta = 1);
			bool searchClosestBound(timestamp_t, const std::set<CollectionId>&, timestamp_t&, const timestamp_t, const timestamp_t);
			bool searchGranules( timestamp_t, timestamp_t, timestamp_t, float
							   , const std::vector<std::pair<CollectionId, bool> >&
							   , std::map<CollectionId, std::vector<GranuleMetadata> >&);
			bool getDataCoverage(const std::set<CollectionId>&, EventsCoverage&);
			bool getDataCoverage(const CollectionId&, EventsCoverage&);
			bool collectionsBySource(const std::set<CollectionId>&, std::map<SourceId, std::set<CollectionId> >&) const;

            bool registerUserGeneratedIndex(const SourceId&, IIndexServiceConnector*);
            IIndexServiceConnector* getUserGeneratedIndex(const SourceId&, bool&) const;

			IMetadataStore* store;
            VariableCfgAdded variableCfgAdded;
			SourcesControlModel* sources;
            std::unordered_map<SourceId, IIndexServiceConnector*> _userGeneratedIndexes;
            std::unordered_map<std::uint32_t, std::uint32_t> _zindexHistogram;
	};
};

#endif
