// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/**
 *
 */

#ifndef PATH_H
#define PATH_H

#include <ctime>
#include <cstdint>
#include <sys/stat.h>
#include <string>
#include <vector>
#include <forward_list>
#include "core/interfaces/ipath_handler.h"

namespace SEAScope
{
	class Path
	{
		public:
			static bool dirExists(const std::string&);
			static bool fileExists(const std::string&);
			static bool fileSize(const std::string&, std::int64_t&);
			static bool writeFile(const std::string&);
			static bool createDir(const std::string&, mode_t);
			static bool createDirs(const std::string&, mode_t);
			static void splitPaths(const std::string&, const std::string&, std::forward_list<std::string>&);
			static void joinPaths(const std::vector<std::string>&, std::string&);
			static bool listFiles(const std::string&, std::forward_list<std::string>&);
			static bool listFilesRecursive(const std::string&, std::forward_list<std::string>&);
			static bool deleteFile(const std::string&);
			static char separator();
			static void setPathValid(std::string& path);
			static bool lastModified(const std::string& path, std::time_t&);
            static bool currentWorkingDir(std::string&);
            static bool canonicalPath(const std::string& , std::string&);
            static bool homeDir(std::string&);
            static bool absolutePath(const std::string&, std::string&);
            static bool getParentDir(const std::string&, std::string&);

			static IPathHandler* implementation;
	};
};

#endif
