// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef PLUGINS_MANAGER_H
#define PLUGINS_MANAGER_H

#include <set>
#include <string>

#include "core/structs.h"
#include "core/interfaces/iplugins_loader.h"

namespace SEAScope
{
	class PluginsManager
	{
		public:
			PluginsManager();
			virtual ~PluginsManager();

			bool init(IPluginsLoader*);
			bool discoverPlugins(const std::string&, std::set<std::string>&);
			PluginMeta* getPluginMetadata(const std::string&, const std::string&, const std::string&, bool&);
			PluginMeta* getPlugin(const std::string&, const std::string&, const std::string&, bool&);
			bool pluginCreated(const std::string&, const std::string&, const std::string&);
			bool pluginDestroyed(const std::string&, const std::string&, const std::string&);
			bool registerPlugin(const std::string&);
			bool releaseAll();
			PluginMeta& getMeta(const std::string&);

			std::map< std::string, PluginMeta> metadata;
			IPluginsLoader* loader;
	};
};

#endif
