// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef RAW_TO_RENDERABLE_QUEUE_H
#define RAW_TO_RENDERABLE_QUEUE_H

#include <set>
#include <functional>

#include "core/types.h"
#include "core/structs.h"
#include "core/concurrent_queue.h"

namespace SEAScope
{
    struct RawToRenderableJob
    {
        std::set<std::string> fields;
        std::string uri;
        CollectionId collection;
        GranuleId granule;
        float resolution;
    };

	class RawToRenderableQueue: public virtual ConcurrentQueue<RawToRenderableJob>
	{
		public:
			RawToRenderableQueue();
			virtual ~RawToRenderableQueue();

			// Disable copy constructor and assignment
			RawToRenderableQueue(const RawToRenderableQueue&) = delete;
			RawToRenderableQueue& operator=(const RawToRenderableQueue&) = delete;

            bool addJob(const CollectionId& collection, const GranuleId& granule, const std::string& uri, float resolution, const std::set<std::string>&& fields);
            void shutdown();
    };
}

#endif
