// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef RENDERING_CONTROL_H
#define RENDERING_CONTROL_H

#include <set>
#include <map>
#include <string>
#include <functional>
#include "core/rendering_control_model.h"
#include "core/colormap_manager.h"
#include "ui/irendering_control_view.h"
#include "ui/irendering_widget.h"
#include "core/structs.h"
#include "core/advanced_rendering_control.h"

namespace SEAScope
{
	class RenderingControl
	{
		public:
			RenderingControl();
			virtual ~RenderingControl();

			bool init(IRenderingControlView*, const ColormapManager*);
			bool addRendering(const CollectionVariable&);
			bool removeRendering(const CollectionVariable&);
			bool renderedVariables(const CollectionId&, std::set<VariableId>&) const;

			bool getCollectionVariable(const IRenderingWidget*, CollectionVariable&) const;
			RenderingCfg* getRenderingCfg(const IRenderingWidget*, CollectionVariable&) const;
			bool updateCollectionZindex(IRenderingWidget*, IRenderingWidget*, IRenderingWidget*);
			bool updateCollectionOpacity(IRenderingWidget*, float);
			bool updateCollectionVisibility(IRenderingWidget*, bool, bool);
			bool updateCollectionExtrema(IRenderingWidget*, float, bool);
			bool getVariableLabel(const CollectionVariable&, std::string&) const;
			bool getVariableUnits(const CollectionVariable&, std::string&) const;
			bool toggleAdvancedControl(IRenderingWidget*);
			bool updateCounters();
			bool resetRenderingCfg(const RenderableId&);
            void updateAdvancedRenderingAfterRemoval(const RenderingCfg*, IRenderingWidget*);
            bool displayOnTop(const CollectionVariable&);
            IRenderingWidget* getTopWidget() const;
            void exclusive(const CollectionVariable&);

			RenderingControlModel model;
			IRenderingControlView* view;
			const ColormapManager* cmaps;
			AdvancedRenderingControl advanced;
			std::map<CollectionVariable, IRenderingWidget*> widgets;
			std::map<IRenderingWidget*, CollectionVariable> colvars;
			std::function<void(const RenderingCfg*, const RenderingParam&)> callback;
			std::function<bool(const CollectionVariable&, std::string&)> toLabel;
            std::function<bool(const CollectionVariable&, std::string&)> getUnits;
	};
};

#endif
