// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef RENDERING_CONTROL_MODEL_H
#define RENDERING_CONTROL_MODEL_H

#include <map>
#include <vector>
#include <functional>
#include "core/types.h"
#include "core/structs.h"
#include "core/rendering_cfg.h"

namespace SEAScope
{
	class RenderingControlModel
	{
		public:
			RenderingControlModel();
			virtual ~RenderingControlModel();

			RenderingCfg* getCfg(const CollectionId&, const GranuleId&, const VariableId&, bool&) const;
			RenderingCfg* getCollectionCfg(const CollectionId&, const VariableId&, bool&) const;
			RenderingCfg* getGranuleCfg(const GranuleId&, const VariableId&, bool&) const;
			const RenderingCfg* getDefaultCfg(const CollectionId&, const VariableId&) const;
			bool registerCfg(RenderingCfg*);
			bool registerCollectionCfg(RenderingCfg*);
			bool resetCollectionCfg(const CollectionId&, const VariableId&);
			bool registerGranuleCfg(RenderingCfg*);
			bool setCounters(const std::map<CollectionId, uint32_t>& counters);

			void cfgChanged(const RenderingCfg*, const RenderingParam&);

            bool removeCollection(const CollectionId&);
            bool removeVariable(const CollectionId&, const VariableId&);

			void updateDefaultCfg(RenderingCfg*);
            bool listRenderedVariables(const CollectionId&, std::vector<VariableId>&) const;
            std::size_t countRenderedVariables(const CollectionId&) const;

            RenderingCfg* createVolatileRenderingCfg();

			std::map<std::pair<CollectionId, VariableId>, RenderingCfg*> collections;
			std::map<std::pair<GranuleId, VariableId>, RenderingCfg*> granules;
			std::map<CollectionId, uint32_t> counters;
			std::map<std::pair<CollectionId, VariableId>, RenderingCfg*> defaultCfgs;

			std::function<void(const RenderingCfg*, const RenderingParam&)> callback;
			std::function<void()> countersCallback;

        protected:
            std::vector<RenderingCfg*> _volatile;
	};
};

#endif
