// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SEQUENTIAL_BUFFER_H
#define SEQUENTIAL_BUFFER_H

#include <cstdint>

namespace SEAScope
{
	template<std::size_t SIZE>
	class SequentialBuffer final
	{
		public:
			SequentialBuffer(): offset(0), bytesCount(0)
			{
			}

			~SequentialBuffer()
            {
			}

			// No copy
			SequentialBuffer(const SequentialBuffer&) = delete;
			SequentialBuffer& operator = (const SequentialBuffer&) = delete;

			/**
  			 *
			 */
			inline std::uint8_t* getPointer(std::uint64_t& freeBytes) const
			{
				std::uint64_t writeOffset;	//

				writeOffset = (this->offset + this->bytesCount) % SIZE;
				freeBytes = SIZE - writeOffset;
				return const_cast<std::uint8_t*>(&(this->storage.at(writeOffset)));
			}

			/**
			 *
			 */
			inline std::uint64_t updateBytesCount(std::uint64_t added, std::uint64_t removed)
			{
				this->bytesCount = this->bytesCount + (added - removed);
				return this->bytesCount;
			}

			std::array<std::uint8_t, SIZE> storage;
			std::uint64_t offset;
			std::uint64_t bytesCount;
	};
};

#endif
