// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SEQUENTIAL_BUFFER_POOL_H
#define SEQUENTIAL_BUFFER_POOL_H

#include <array>
#include <queue>

#include "core/sequential_buffer.h"

namespace SEAScope
{
	template<std::size_t CAPACITY, std::size_t SIZE>
	class SequentialBufferPool final
	{
		public:
			SequentialBufferPool<CAPACITY, SIZE>()
			{
				for
					(auto& it: this->buffers)
				{
					this->available.push(&it);
				}
			}

			virtual ~SequentialBufferPool<CAPACITY, SIZE>() = default;

			inline SequentialBuffer<SIZE>* acquire(bool& ok)
			{
				SequentialBuffer<SIZE>* result;	//

				result = nullptr;
				if
					(true == available.empty())
				{
					ok = false;
					return result;
				}

				result = this->available.front();
				this->available.pop();
				ok = true;
				return result;
			}

			inline bool release(SequentialBuffer<SIZE>* buffer)
			{
				if
					(nullptr == buffer)
				{
					return false;
				}

				buffer->offset = 0;
				buffer->bytesCount = 0;
				this->available.push(buffer);
				return true;
			}

			std::array<SequentialBuffer<SIZE>, CAPACITY> buffers;
			std::queue<SequentialBuffer<SIZE>*> available;
	};
};

#endif
