// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SETTINGS_H
#define SETTINGS_H

#include <cstdint>
#include <string>

#include "core/timeline_cfg.h"

namespace SEAScope
{
	enum class SettingsKey : uint8_t;
    enum class SettingsValue : uint8_t
    {
        NOT_FOUND = 0,
    };

    class Settings
    {
        public:
            Settings();
            virtual ~Settings();

            Settings& operator=(const Settings&);

			bool init(const std::string&);

			template <typename T>
			bool set(SettingsKey, const T);

			template <typename T>
			const T get(SettingsKey) const;

			std::string glRenderer;
			uint8_t glContextMajor;
			uint8_t glContextMinor;
			std::string pluginsPath;
			std::string colormapsPath;
			std::string operatorsPath;
			std::string dataPath;
            std::string annotationsPath;
            std::string customPath;
			uint16_t windowWidth;
			uint16_t windowHeight;
			bool vsync;
			bool antialiasing;
			bool fullscreen;
			std::string antialiasingType;
			uint8_t antialiasingFactor;
			std::string worldMap;
            TimelineCfg timelineCfg;
			float minGranuleScreenEstate;
            std::string logFilter;
			std::string statePath;
			std::string indexPath;
			bool processorEnabled;
            std::string processorAddress;
            std::int64_t processorPort;
			bool skipIndexCheck;
			double searchRequestMinDelay;
            bool explicitDrawingModeExit;
            std::uint32_t clickPixelsTolerance;
    };
};
#endif
