// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SOCKETS_MANAGER_H
#define SOCKETS_MANAGER_H

#include <cstdint>
#include <string>
#include <thread>
#include <mutex>
#include <map>
#include <functional>
#include "core/types.h"
#include "core/interfaces/isockets_monitor.h"
#include "core/interfaces/isocket_factory.h"

class Server;

namespace SEAScope
{
    class SocketsManager
    {
        public:
            SocketsManager();
            virtual ~SocketsManager();

            bool init(ISocketsMonitor*, ISocketFactory*);
			bool subscribe(socket_t, const std::string&);
			bool unsubscribe(socket_t);
			bool handleEvents();
			bool createServer( const std::string&, std::int64_t
							 , const std::string&);
			bool shutdown();

			bool registerProtocol(const std::string&, const std::function<IProtocolHandler*()>&);
			IProtocolHandler* getProtocolHandler(const std::string&, bool&);

            bool resolveHostname(const std::string&, std::string&);

			ISocketsMonitor* monitor;
			ISocketFactory* factory;
			std::map<std::string, std::function<IProtocolHandler*()> > protocolHandlers;
			std::vector<SocketJob> events;
			std::vector<std::thread> threads;
			std::mutex eventsMutex;
			int signalFd;

#if defined _WIN32 || defined __APPLE__
	private:
		Server * _server;
#endif
    };
};

#endif
