// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SOURCES_MANAGER_H
#define SOURCES_MANAGER_H

#include <map>
#include <string>
#include <forward_list>

#include "core/types.h"
#include "core/structs.h"
#include "core/interfaces/iindex_service_connector.h"
#include "core/interfaces/idatastream_service_connector.h"
#include "core/plugins_manager.h"
#include "core/dynamic.h"

namespace SEAScope
{
    // Forward declarations
    class Settings;

	class SourcesControlModel
	{
		public:
			SourcesControlModel();
			virtual ~SourcesControlModel();

			bool init(PluginsManager&, const Settings&);
            bool reset();
			bool hasSource(const std::string&) const;
			bool addSource(const std::string&, SourceInfo&);
			bool getSource(const std::string&, SourceInfo&) const;

			dynamic<IIndexServiceConnector>* getIndex(SourceId, bool&);
			dynamic<IDatastreamServiceConnector>* getDatastream(SourceId, bool&);

			std::map<SourceId, std::string> uris;
			std::map<std::string, SourceInfo> sources;
			std::uint32_t idGenerator;
			std::forward_list< dynamic<IIndexServiceConnector> > indexPlugins;
			std::forward_list< dynamic<IDatastreamServiceConnector> > datastreamPlugins;
			std::map<SourceId, dynamic<IIndexServiceConnector>* > indices;
			std::map<SourceId, dynamic<IDatastreamServiceConnector>* > datastreams;
	};
};

#endif
