// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef THREADS_POOL_H
#define THREADS_POOL_H

#include <cstdint>
#include <vector>
#include <thread>
#include <mutex>
#include <condition_variable>

#include "core/job.h"
#include "core/interfaces/ijob_scheduler.h"

namespace SEAScope
{
	class ThreadsPool
	{
		public:
			ThreadsPool();
			virtual ~ThreadsPool();

			bool init(std::uint16_t, IJobScheduler*);
			bool submitJob(Job&&, std::string&, JobStatus&);
			bool cancelJob(const std::string&);
			void processJobs();
			bool shutdown();

			std::vector<std::thread> threads;
			std::mutex tasksMutex;
			std::condition_variable condition;
			bool terminationRequested;
			bool terminated;
			IJobScheduler* scheduler;
	};
};

#endif
