// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef TIMELINE_H
#define TIMELINE_H

#include <cstdint>
#include <map>
#include <array>
#include <string>
#include <functional>
#include "core/timeline_cfg.h"
#include "ui/itimeline_view.h"
#include "core/timeline_model.h"

namespace SEAScope
{
    // Forward declarations
    struct TimelineState;

	class Timeline
	{
		public:
			Timeline();
			virtual ~Timeline();

			bool init( ITimelineView*, const TimelineCfg&);
			bool setCurrentChangedCallback(const std::function<void(timestamp_t)>&);
			bool setTimeframeChangedCallback(const std::function<void(timestamp_t, timestamp_t)>&);
			bool getBounds(timestamp_t&, timestamp_t&, timestamp_t&) const;
			bool extractDatetimeFromTimestamp(timestamp_t, std::string&, std::string&, int&, int&, int&) const;
			bool updateOptions();
			bool getTimestepLabel(timestamp_t, std::string&) const;
            bool getTimespanLabel(timestamp_t, timestamp_t, std::string&) const;

            bool getState(TimelineState&) const;
            bool setState(const TimelineState&);

			ITimelineView* view;
			TimelineModel model;
			std::map<std::string, std::array<timestamp_t,2>> timeframes;
			std::map<std::string, timestamp_t> timesteps;
			std::function<void(timestamp_t)> currentCallback;
			std::function<void(timestamp_t, timestamp_t)> timeframeCallback;
	};
};

#endif
