// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef UBYTES_STREAM_H
#define UBYTES_STREAM_H

#include <vector>
#include <cstdint>

struct UbytesStream
{
    UbytesStream(std::vector<std::uint8_t>& data, std::size_t offset)
    : _data(data), _current(offset)
    {
    }

    void read(std::uint8_t* outBuffer, std::size_t length)
    {
        std::size_t i = 0;
        for
            (; (i < length) && (this->_current + i < this->_data.size()); ++i)
        {
            outBuffer[i] = this->_data[this->_current + i];
        }

        this->_current = this->_current + i;
    }

    protected:
        std::vector<std::uint8_t>& _data;
        std::size_t _current;
};

#endif
