// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef UTILS_H
#define UTILS_H

#include <set>
#include <vector>
#include <string>
#include <cstdint>

#include "core/types.h"

namespace SEAScope
{
	int dimStr2pixels(const std::string& dimStr, int ref);
	int posStr2pixels(const std::string& posStr, int ref, int dim);
	std::string& implodeStringSet(const std::set<std::string>&, const std::string&, std::string&);
    void explodeString(const std::string&, const std::string&, std::vector<std::string>&);
    void getColvarLabel(const std::string&, const std::string&, std::string&);
    bool parseDatetime(const std::string&, const std::string&, std::uint16_t&, std::uint8_t&, std::uint8_t&, std::uint8_t&, std::uint8_t&, std::uint8_t&);
    bool parseDate(const std::string&, std::uint16_t&, std::uint8_t&, std::uint8_t&);
    bool parseTime(const std::string&, std::uint8_t&, std::uint8_t&, std::uint8_t&);
    bool getTimestampFromDatetime(const std::string&, timestamp_t&);
    bool iequals(const std::string&, const std::string&);
    std::string& trim(std::string&);
    bool stringEndsWith(const std::string&, const std::string&);
    std::uint8_t getMonthDaysCount(std::uint16_t year, std::uint8_t month);
}
#endif
