// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ANNOTATION_H
#define ANNOTATION_H

#include "3d/shape.h"
#include "core/types.h"
#include "core/data_node.h"

namespace SEAScope
{
    class Annotation
    {
        public:
            Annotation();
            Annotation(Annotation&&);
            virtual ~Annotation();

            bool setShape(OGL::Shape*);
            OGL::Shape* getShape() const;

            const DictNode& getConstProperties() const;
            DictNode& getProperties();
            void assignProperties(DictNode&&);

            bool hasStart() const;
            double getStart() const;
            void setStart(double);

            bool hasEnd() const;
            double getStop() const;
            void setStop(double);

       protected:
            double _start; /* time_coverage_start converted in seconds-EPOCH */
            double _stop; /* time_coverage_end converted in seconds-EPOCH */

            OGL::Shape* _shape;
            DictNode _properties;
    };
}

#endif
