// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/*
 */

#ifndef EVENTS_COVERAGE_H
#define EVENTS_COVERAGE_H

#include <map>
#include <array>
#include <vector>
#include <cstdint>

namespace SEAScope
{
    class EventsCoverage final
    {
        // Don't use unordered_map as keeping the years sorted will allow some
        // optimizations when search for the nearest date with data
		typedef std::map<std::uint16_t, std::array<std::uint32_t, 12> > coverage_t;


        public:
			EventsCoverage()
			{
			}

            EventsCoverage(const EventsCoverage& src)
            {
                this->store.clear();
                this->store.insert(src.store.begin(), src.store.end());
            }

            EventsCoverage& operator=(const EventsCoverage& src)
            {
                this->store.clear();
                this->store.insert(src.store.begin(), src.store.end());
                return *this;
            }

            ~EventsCoverage()
            {}

            void reset();
			bool clone(const EventsCoverage&);
			bool merge(const EventsCoverage&);
			bool intersect(const EventsCoverage&);
			bool getDays(std::uint16_t year, std::uint8_t month,
						 std::vector<std::uint8_t>& values) const;
			bool getMonths(std::uint16_t year, std::vector<std::uint8_t>& values) const;
			bool getYears(std::vector<std::uint16_t>& values) const;
            bool findNearestDays(const std::uint16_t, const std::uint8_t, const std::uint8_t, bool&,
                                 std::uint16_t&, std::uint8_t&, std::uint8_t&, bool&,
                                 std::uint16_t&, std::uint8_t&, std::uint8_t&, bool&) const;
			void display() const;

			coverage_t store;
    };

} /* namespace SEAScope */

#endif /* EVENTS_COVERAGE_H */
