// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef GPU_DATA_BUILDER_H
#define GPU_DATA_BUILDER_H

#include "core/structs.h"

// Forward declarations
class DataSet;
class Geodetic2D;

namespace SEAScope
{
    class GPUDataBuilder
    {
        public:
            GPUDataBuilder();
            virtual ~GPUDataBuilder();

            bool buildDataSet( DataSet* data_set
                             , void* raw_data
                             , const DataInfo& dataInfo
                             , const CollectionCfg& colCfg
                             , const Shape& shape
                             , float resolution
                             , int maxTextureSize);

            bool buildPolyline( void* raw_data
                              , const DataInfo& dataInfo
                              , const CollectionCfg& colCfg
                              , const Shape& shape
                              , std::vector<float>& relativeTime
                              , std::vector<Geodetic2D>& points
							  , std::vector<float>& fData
							  , std::vector<bool>& mask);
    };
};

#endif
