// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SEASCOPE_DEBUG_H
#define SEASCOPE_DEBUG_H

#include "core/log.h"
#include "core/types.h"
#include "core/structs.h"

namespace SEAScope
{
	class Debug final
	{
		public:
			static void displayCollection(const CollectionCfg& colCfg)
			{
					LOG( LogLevel::Debug, "Main"
									, "Collection: {} x:{} y:{} c:{} #v:{}"
									, colCfg.label, colCfg.xSeamless, colCfg.ySeamless, colCfg.mustBeCurrent, colCfg.variables.size());

					/* Display collection tags */
					for
							(const auto& tag : colCfg.tags)
					{
							LOG( LogLevel::Debug, "Main"
											, " #{} {}"
											, tag.first, tag.second);
					}

					/* Display variables info */
					for
							(const auto& it : colCfg.variables)
					{
							LOG( LogLevel::Debug, "Main"
											, " * Variable[{}]: {}"
											, it.first, it.second.label);
							for
									(const auto& field : it.second.fields)
							{
									LOG( LogLevel::Debug, "Main"
													, "   - {}"
													, field);
							}

							for
									(const auto& tag : it.second.tags)
							{
									LOG( LogLevel::Debug, "Main"
													, "   #{} {}"
													, tag.first, tag.second);
							}
					}

			}
	};
};

#endif
