// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef DUMMY_METADATA_STORE_H
#define DUMMY_METADATA_STORE_H

#include <map>
#include <mutex>

#include "core/interfaces/imetadata_store.h"

namespace SEAScope
{
	class DummyMetadataStore final: public virtual IMetadataStore
	{
		public:
			DummyMetadataStore();
			~DummyMetadataStore() override;

            bool hasCollection(const CollectionId&) const override;
			bool listCollections(std::set<CollectionId>&) const override;
			bool getCollectionCfg(const CollectionId&, CollectionCfg&) const override;
			bool setCollectionCfg(const CollectionId&, const CollectionCfg&) override;
            bool deleteCollectionCfg(const CollectionId&) override;
			bool addVariable(const CollectionId&, const VariableId&, const VariableCfg&) override;
            bool removeVariable(const CollectionId&, const VariableId&) override;

            std::map<CollectionId, CollectionCfg> collections;

        protected:
            mutable std::mutex _mutex;
	};
};

#endif
