// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef CURSOR_LOCATION_CHANGED_EVENT_H
#define CURSOR_LOCATION_CHANGED_EVENT_H

#include <cstdint>

#include "vendors/_glm.h"

#include "core/app_event.h"

namespace SEAScope
{
    class CursorLocationChangedEvent: public AppEvent
    {
        public:
            CursorLocationChangedEvent(std::uint32_t, std::uint32_t, float, float, bool, const glm::vec3&);
            virtual ~CursorLocationChangedEvent() override;

            std::uint32_t getScreenX() const;
            std::uint32_t getScreenY() const;
            float getLon() const;
            float getLat() const;
            bool getOnWorld() const;
            const glm::vec3& getHitpoint() const;

        protected:
            std::uint32_t _screenX;
            std::uint32_t _screenY;
            float _lon;
            float _lat;
            glm::vec3 _hitpoint;
            bool _onWorld;
    };
}
#endif
