// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef LOAD_ANNOTATIONS_FROM_FILE_REQUEST_H
#define LOAD_ANNOTATIONS_FROM_FILE_REQUEST_H

#include <map>
#include <set>
#include <string>

#include "core/types.h"
#include "core/app_event.h"

namespace SEAScope
{
    class LoadAnnotationsFromFileRequest: public AppEvent
    {
        public:
            LoadAnnotationsFromFileRequest(const std::string&, const std::map<ShapeType, CollectionId>&, const AnnotationFormat&, double, double, const std::set<CollectionId>&);
            virtual ~LoadAnnotationsFromFileRequest() override;

            const std::string& getPath() const;
            const std::map<ShapeType, CollectionId>& getCollectionsMapping() const;
            const AnnotationFormat& getImportFormat() const;
            double getDefaultStart() const;
            double getDefaultStop() const;
            const std::set<CollectionId>& getSelectedCollections() const;

        protected:
            std::string _path;
            std::map<ShapeType, CollectionId> _mapping;
            AnnotationFormat _importFormat;
            double _defaultStart;
            double _defaultStop;
            const std::set<CollectionId>& _selectedCollections;
    };
}

#endif
