// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SAVE_ANNOTATIONS_FROM_RENDERABLE_IDS_TO_FILE_REQUEST_H
#define SAVE_ANNOTATIONS_FROM_RENDERABLE_IDS_TO_FILE_REQUEST_H

#include <map>
#include <set>
#include <string>

#include "core/types.h"
#include "core/structs.h"
#include "core/app_event.h"

namespace SEAScope
{
    class SaveAnnotationsFromRenderableIdsToFileRequest: public AppEvent
    {
        public:
            SaveAnnotationsFromRenderableIdsToFileRequest(const std::string&, const std::set<RenderableId>&, const std::map<ShapeType, CollectionId>&, const AnnotationFormat&);
            virtual ~SaveAnnotationsFromRenderableIdsToFileRequest() override;

            const std::string& getPath() const;
            const std::map<ShapeType, CollectionId>& getCollectionsMapping() const;
            const AnnotationFormat& getExportFormat() const;
            const std::set<RenderableId>& getRenderableIds() const;

        protected:
            std::string _path;
            const std::set<RenderableId>& _rIds;
            std::map<ShapeType, CollectionId> _mapping;
            AnnotationFormat _exportFormat;
    };
}
#endif
