// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SELECTED_PICKABLE_WITH_INFO_CHANGED_EVENT_H
#define SELECTED_PICKABLE_WITH_INFO_CHANGED_EVENT_H

#include "core/app_event.h"
#include "core/types.h"
#include "core/structs.h"
#include "core/data_node.h"
#include "core/granule_info.h"

class Pickable;

namespace SEAScope
{
    // Forward declarations
    class DictNode;

    class SelectedPickableWithInfoChangedEvent: public AppEvent
    {
        public:
            SelectedPickableWithInfoChangedEvent(Pickable*, const GranuleInfo&, const RenderableType&);
            SelectedPickableWithInfoChangedEvent(Pickable*, const GranuleInfo&, const RenderableType&, DictNode*, double, double);
            virtual ~SelectedPickableWithInfoChangedEvent() override;

            Pickable* getPickable() const;
            const GranuleInfo& getInfo() const;
            const RenderableType& getRenderableType() const;
            DictNode& getProperties();
            double getStart() const;
            double getStop() const;

        protected:
            Pickable* _pickable;
            GranuleInfo _info;
            RenderableType _rType;
            DictNode _properties;
            double _start;
            double _stop;
    };
}
#endif
