// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/*
 * File:   filter_logger.h
 * Author: gleseach
 *
 * Created on 18 septembre 2017, 15:35
 */

#ifndef FILTER_LOGGER_H
#define FILTER_LOGGER_H

#include <unordered_set>

#include "simplelogger.h"

namespace SEAScope
{

    /*!
     * \brief Wrapper around simplelogger to clamp the logger output to a level.
     */
    class FilterLogger final : public SimpleLogger
    {
    public:
        /*!
         * \brief Constructor.
         * \param[in] level The maximum level of verbosity to print (lower = fatal, higher = debug)
         */
        FilterLogger();

        //! Destructor
        ~FilterLogger() override;

        bool isMessageValid(const LogLevel&, const string_view&) override;

        bool init(const std::string&);

        virtual LogLevel getLevel() const override;

    protected:
        LogLevel minLevel;
        std::unordered_set<std::string> allowedLoggers;
    };

};

#endif /* FILTER_LOGGER_H */
