// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef FLATBUFFERS_GRANULES_INDEX_H
#define FLATBUFFERS_GRANULES_INDEX_H

#include <mutex>

#include "core/interfaces/igranules_index.h"

namespace SEAScope
{
	class FlatbuffersGranulesIndex: public virtual IGranulesIndex
	{
		public:
			FlatbuffersGranulesIndex();
			virtual ~FlatbuffersGranulesIndex() override;

			bool hasGranule(const std::string&) const override;
			bool hasGranule(std::uint64_t) const override;
			bool add(GranuleMetadata&, bool) override;
            bool updateTimeCoverage(std::uint64_t, const timestamp_t, const timestamp_t) override;
            bool remove(const std::string&, bool) override;
            bool remove(std::uint64_t, bool) override;
			std::array<bool, 2> adjacentGranules( const timestamp_t
												, const timestamp_t
												, const std::set<std::uint32_t>&
												, GranuleMetadata&
												, GranuleMetadata&) const override;
			bool closestGranuleBound( const timestamp_t
                                    , const timestamp_t
                                    , const timestamp_t
									, const std::set<std::uint32_t>&
									, GranuleMetadata&) const override;
			bool getCoverage(const std::set<std::uint32_t>&, EventsCoverage&) const override;
			bool getCoverage(std::uint32_t, EventsCoverage&) const override;
			bool findGranules(const SearchCriteria&, std::vector<GranuleMetadata>&) const override;
            std::uint64_t generateGranuleId() override;
            bool setGranuleAddedCallback(const std::function<void(std::uint64_t, std::uint32_t)>&) override;
            std::tuple<bool, bool> load(const std::string& path) override;
            bool save(const std::string& path) override;
            bool setGranuleIdGenerator(std::uint64_t) override;
            bool startBulkOperation() override;
            bool stopBulkOperation() override;
            bool cancelBulkOperation() override;
            bool increaseBulkOpsCount() override;
            bool countGranules(std::size_t&) const override;

            bool setCollectionIdGenerator(std::uint32_t) override;
            std::uint32_t generateCollectionId() override;
            bool addCollection(const std::string&, const std::uint32_t) override;
            bool removeCollection(const std::string&) override;
            bool removeCollection(std::uint32_t) override;
            bool listCollections(std::vector<std::pair<std::uint32_t, std::string> >&) const override;
            bool countCollections(std::size_t&) const override;

            bool clear() override;

        protected:
            std::mutex _mapsMutex;
			std::uint64_t _idGenerator;
			std::map<std::uint64_t, GranuleMetadata> _data;
			std::map<std::string, std::uint64_t> _ids;
			std::function<void(std::uint64_t, std::uint32_t)> _granuleAddedCallback;
	};
};

#endif
