// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef GLFW_EVENTS_DISPATCHER_H
#define GLFW_EVENTS_DISPATCHER_H

#include <vector>
#include <functional>

#include "core/eventhandler.h"
#include "core/input_events_dispatcher.h"
#include "glfw_input_bindings.h"

namespace SEAScope
{
    class GLFWEventsDispatcher final: public EventHandler, public virtual InputEventsDispatcher
    {
        public:
            GLFWEventsDispatcher();
            ~GLFWEventsDispatcher() override;

			bool handleKeyEvent(int, int, int, int) override;
			bool handleCharEvent(unsigned int, int) override;
			bool handleMouseButtonEvent(int, int, int) override;
			bool handleMouseMoveEvent(double, double) override;
			bool handleMouseWheelEvent(double, double) override;
			bool handleResizeEvent(int, int, int, int) override;
			bool handleFocusEvent(int) override;
			bool handleIconifyEvent(int) override;
			bool handleDropEvent(int, const char**) override;
			bool handleCloseEvent() override;
            void setPreemptEventsCallback(const std::function<void(EventHandler*, bool)>&) override;

            void initBindings() override;
            const InputBindings& getInputBindings() const override;
            bool dispatchEvent(const InputEvent& event) override;

            void setFrameBufferRatio(double, double);

        protected:
            InputBindings& getInputBindings() override;
            
            GLFWInputBindings _inputBindings;
            std::function<void(EventHandler*, bool)> _preemptEventsCallback;

            double _frameBufferXRatio;
            double _frameBufferYRatio;
    };
};

#endif
