// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef GLFW_WINDOW_H
#define GLFW_WINDOW_H

#include <forward_list>
#include "vendors/_glfw3.h"
#include "core/interfaces/iwindow.h"

struct GLFWwindow; // Beware! the "w" of "window" is not capitalized here!

namespace SEAScope
{
	class GLFWWindow final: public virtual IWindow
	{
		public:
			GLFWWindow();
			~GLFWWindow() override;

			bool init(const Settings&) override;
			bool create( uint16_t, uint16_t) override;
			bool renderFrame() override;
			bool resize(uint16_t, uint16_t) override;
			bool clean() override;
            bool getSize(int&, int&) const override;
            bool getFrameBufferSize(int&, int&) const override;
			bool addRenderer(IRenderer&) override;
			bool addHandler(EventHandler&) override;
            bool getCursorPos(double&, double&) const override;
            bool setCursor(IWindow::MouseCursor) override;
            void close() override;

            void injectKey(int key, int scancode, int action, int mods);
            void injectChar(unsigned int c, int mods);
            void injectMouseButton(int button, int action, int mods);
            void injectMouseMove(double x, double y);
            void injectMouseWheel(double x, double y);
            void injectResize(int width, int height);
            void injectFocus(int focused);
            void injectIconify(int iconified);
			void injectDrop(int,const char**);
			void injectClose();

			bool renderContent();

			std::forward_list<IRenderer*> renderers;
			std::forward_list<IRenderer*>::iterator renderersIt = renderers.before_begin();
			std::forward_list<EventHandler*> handlers;
			std::forward_list<EventHandler*>::iterator handlersIt = handlers.before_begin();
			GLFWwindow* implementation;
			int interval;
			bool fullscreen;
            int frameBufferWidth;
            int frameBufferHeight;
			int windowWidth;
			int windowHeight;
            GLFWcursor* cursor;

        protected:
            EventHandler* _preempter;
            std::size_t _frames;
            double _prevTime;
	};
};

#endif
