// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef INI_SETTINGS_H
#define INI_SETTINGS_H

#include <string>
#include <vector>
#include <unordered_map>
#include "core/interfaces/isettings_store.h"
#include "vendors/_mini.h"


namespace SEAScope
{
	class INISettings final : public virtual ISettingsStore
	{
		public:
			INISettings();
			~INISettings() override;

			bool load(Settings&, std::string&) override;
			bool save(const Settings&, const std::string&) const override;

			bool loadINI(Settings&, const std::string&);
			bool saveINI(const Settings&, const std::string&) const;
			bool splitList(const std::string&, const std::string&, std::vector<std::string>&) const;
			bool loadTimespans(Settings&, const std::string&);
			bool loadTimesteps(Settings&, const std::string&);

        protected:
            bool validate(const std::string&, mINI::INIStructure&);

            std::unordered_map<std::string, std::string> _defaultPathsFields;
            std::unordered_map<std::string, std::string> _defaultRenderingFields;
            std::unordered_map<std::string, std::string> _defaultTimelineFields;
            std::unordered_map<std::string, std::string> _defaultMiscFields;
            std::unordered_map<std::string, std::string> _defaultProcessorFields;
            std::unordered_map<std::string, std::string> _defaultDrawingFields;
	};
};

#endif
