// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef JSON_ANNOTATION_H
#define JSON_ANNOTATION_H

#include <string>

#include "vendors/_picojson.h"
#include "core/interfaces/iannotation_buffer.h"

namespace SEAScope
{
    class JSONAnnotation final : public IAnnotationBuffer
    {
        public:
            JSONAnnotation();
            ~JSONAnnotation() override;

            bool serialize(const Annotation&, std::ostream&) const override;
            bool serialize(const std::vector<Annotation>&, std::ostream&) const override;
            bool deserialize(std::istream&, Annotation&, double, double) const override;
            bool deserialize(std::istream&, std::vector<Annotation>&, double, double) const override;

            bool serializeShapeForSyntool(OGL::Shape* shape, picojson::object& result) const;
            bool toPicoJSON(const Annotation& src, picojson::object& result) const;
            bool serializeNode(const DataNode*, picojson::object&, const std::string&) const;
            bool serializeNode(const DataNode*, picojson::array&) const;

            bool parseSyntoolExport(const picojson::value&, std::vector<Annotation>&) const;
            bool parseSyntoolShape(const picojson::value&, Annotation&) const;
            bool parseGenericProperty(const picojson::value& value, DictNode&, const std::string&) const;
            bool parseGenericProperty(const picojson::value& value, ListNode&) const;

        protected:
            std::string _base64Flag;
    };
}

#endif
