// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef COLLECTIONS_CATALOGUE_H
#define COLLECTIONS_CATALOGUE_H

#include <set>
#include <vector>
#include <string>
#include <cstdint>

#include "core/structs.h"
#include "core/rendering_cfg.h"

namespace SEAScope
{
	class CollectionsCatalogue
	{
		public:
			CollectionsCatalogue();
			virtual ~CollectionsCatalogue();

			bool hasCollection(const std::string&) const;
			bool hasCollection(std::uint32_t) const;
			bool listCollections(std::set<std::uint32_t>&) const;
			bool addCollection(std::uint32_t, const CollectionCfg&&, const std::vector<RenderingCfg*>&&, const std::string&);
			bool getCollection(std::uint32_t, CollectionCfg&, std::vector<RenderingCfg*>&) const;

			std::map<std::string, std::uint32_t> ids;
			std::map<std::uint32_t, std::tuple<CollectionCfg, std::vector<RenderingCfg*> > > database;
	};
};

#endif
