// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IN_MEMORY_DATA_STORE_H
#define IN_MEMORY_DATA_STORE_H

#include <unordered_map>
#include <set>
#include <mutex>
#include <cstdint>

#include "core/interfaces/idata_store.h"

namespace SEAScope
{
	class InMemoryDataStore final : public virtual IDataStore
	{
		public:
			InMemoryDataStore();
			~InMemoryDataStore() override;

			GranuleData& getURIDataRef(const std::string&) override;

            bool removeData(const std::string& uri) override;
            bool listFields(const std::string& uri, std::set<std::string>& result) override;

            bool getMissingDataBuckets(const std::string&, const std::set<std::string>&, std::unordered_map<std::string, DataBucket*>&) override;
            bool getField(const std::string&, const std::string&, const Image*&, const DataInfo*&) const override;
            bool getFields(const std::string&, const std::set<std::string>&, std::vector<const Image*>&, std::vector<const DataInfo*>&) const override;

        protected:
            mutable std::mutex _dataMutex;
			std::unordered_map<std::string, GranuleData> _data;
	};
};

#endif
