// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef USER_GENERATED_H
#define USER_GENERATED_H

#include <vector>
#include <set>

#include "core/types.h"
#include "core/structs.h"
#include "core/interfaces/igranules_index.h"
#include "core/interfaces/iindex_service_connector.h"

namespace SEAScope
{
    // Forward declarations
    class AppEventsDispatcher;
    class IDialogHandler;
    class ILogHandler;
    class IPathHandler;
    class Settings;

	class UserGenerated final : public virtual IIndexServiceConnector
	{
		public:
			UserGenerated(SourceId, AppEventsDispatcher&, IGranulesIndex&);
			~UserGenerated() override;

			virtual bool init(const Settings&, ILogHandler*, IPathHandler*, IDialogHandler*) override;
            virtual bool reset() override;
			virtual bool searchGranules( const SearchCriteria&
									   , std::vector<GranuleMetadata>&) const override;
			virtual std::array<bool, 2> searchPrevNext( timestamp_t
													  , const std::set<CollectionId>&
													  , timestamp_t&
													  , timestamp_t&
													  , timestamp_t) const override;
			virtual bool searchClosestBound( timestamp_t
										   , const std::set<CollectionId>&
										   , timestamp_t&
                                           , const timestamp_t
                                           , const timestamp_t) const override;
			virtual bool getDataCoverage(const std::set<CollectionId>&, EventsCoverage&) const override;
			virtual bool getDataCoverage(const CollectionId&, EventsCoverage&) const override;
            virtual bool listCollections(std::set<std::uint32_t>&) const override;
            virtual bool getCollectionCfg(std::uint32_t, CollectionCfg&, std::vector<RenderingCfg*>&) const override;
            virtual bool saveVariableRenderingCfg(const SEAScope::RenderingCfg*) const override;
			virtual bool canHandle(const std::string&) const override;

            const SourceId& getSourceId() const;
			bool generateRenderableId(const CollectionId&, RenderableId&);
            IGranulesIndex& getIndex();

        protected:
            SourceId _sourceId;
            AppEventsDispatcher& _appEventsDispatcher;
			IGranulesIndex& _index;
	};
};

#endif
