// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NUM_UTILS_H
#define NUM_UTILS_H

#include <cmath>

namespace SEAScope
{
    template<typename T>
    bool floating_point_equality(T a, T b, T absoluteTolerance, T relativeTolerance)
    {
        T diff = static_cast<T>(std::fabs(a - b));
        T maxAB = std::max(static_cast<T>(std::fabs(a)), static_cast<T>(std::fabs(b)));

        return diff <= std::max(absoluteTolerance, relativeTolerance * maxAB);
    }

    template<typename T>
    bool floating_point_equality(T a, T b)
    {
        T absoluteTolerance = std::numeric_limits<T>::epsilon();
        T relativeTolerance = std::numeric_limits<T>::epsilon();
        return floating_point_equality(a, b, absoluteTolerance, relativeTolerance);
    }


    template<typename T>
    bool floating_point_equality(T a, T b, int ulp)
    {
        return std::fabs(a - b) <= std::numeric_limits<T>::epsilon() * std::fabs(a + b) * static_cast<T>(ulp) || std::fabs(a - b) < std::numeric_limits<T>::min();
    }
}

#endif
