// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SQLITE_ANNOTATIONS_H
#define SQLITE_ANNOTATIONS_H

#include <string>
#include <vector>

#include "vendors/_picojson.h"
#include "vendors/_sqlite3.h"

#include "core/interfaces/iannotations_store.h"
#include "core/interfaces/iapp_events_subscriber.h"
#include "json_annotation.h"

namespace SEAScope
{
    // Forward declarations
    class AppEventsDispatcher;

    class SQLiteAnnotations final: public virtual IAnnotationsStore, public virtual IAppEventsSubscriber
    {
        public:
            SQLiteAnnotations(AppEventsDispatcher&);
            ~SQLiteAnnotations() override;

            bool init(const std::string&) override;
            bool get(std::uint64_t granuleId, std::uint32_t& collection, Annotation& annotation) const override;
            bool set(std::uint64_t granuleId, std::uint32_t collection, const Annotation& annotation) override;
            bool set(const std::vector<std::uint64_t>&, const std::vector<std::uint32_t>&, const std::vector<Annotation>&) override;
            bool getall(std::vector<std::tuple<std::uint64_t, std::uint32_t, Annotation>>&) const override;
            bool applyall(const std::function<void(std::uint64_t, std::uint32_t, Annotation&)>&) const override;
            bool remove(const RenderableId&) override;
            bool getMaxGranuleId(std::uint64_t&) const override;
            bool count(std::size_t&) const override;

            bool getMaxCollectionId(std::uint32_t&) const override;
            bool getCollection(std::uint32_t, CollectionCfg&) const override;
            bool setCollection(std::uint32_t, const CollectionCfg&, const std::vector<RenderingCfg*>&) override;
            bool removeCollection(std::uint32_t) override;
            bool getAllCollections(std::vector<std::tuple<std::uint32_t, CollectionCfg, std::vector<RenderingCfg>>>& result) const override;
            bool applyAllCollections(const std::function<void(std::uint32_t, CollectionCfg&, std::vector<RenderingCfg>&)>&) const override;
            bool listCollectionGranules(std::uint32_t, std::vector<std::uint64_t>&) const override;
            bool countCollections(std::size_t&) const override;

            bool setRendering(const RenderingCfg&) override;

            bool handleAppEvent(std::type_index, const AppEvent&) override;

            bool createSQLite(const std::string&) const;
            std::string annotation2string(const Annotation&, bool&) const;
            bool sqlite2Annotation(const std::vector<std::tuple<std::uint64_t, std::uint32_t, std::string>>&,
                                   std::vector<std::tuple<std::uint64_t, std::uint32_t, Annotation>>&) const;
            bool updateCollectionsList();

            bool saveAnnotationProperties(const RenderableId&, const DictNode&);

            static int collectionRetrievedCallback(void* result, int argc, char** argv, char** column);
            static int collectionsRetrievedCallback(void* result, int argc, char** argv, char** column);
            static int variablesRetrievedCallback(void* result, int argc, char** argv, char** column);
            static int annotationRetrievedCallback(void* result, int argc, char** argv, char** column);
            static int annotationsRetrievedCallback(void* result, int argc, char** argv, char** column);
            static int maxCollectionIdRetrievedCallback(void* result, int argc, char** argv, char** column);
            static int variablesWithoutRenderingRetrievedCallback(void* result, int argc, char** argv, char** column);
            static int granuleIdRetrievedCallback(void* result, int argc, char** argv, char** colName);

        protected:
            bool initializeDatabase();

            std::string _path;
            JSONAnnotation _parser;

            AppEventsDispatcher& _appEventsDispatcher;

            sqlite3* _db;
            sqlite3_stmt* _countAnnotationsStmt;
            sqlite3_stmt* _countCollectionsStmt;
    };
}

#endif
