// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SQLITE_STATE_H
#define SQLITE_STATE_H

#include <string>
#include "core/interfaces/istate_store.h"

struct sqlite3;
struct sqlite3_stmt;

namespace SEAScope
{
	class SQLiteState final : public virtual IStateStore
	{
		public:
			SQLiteState();
			~SQLiteState() override;

		    bool setStatePath(const std::string&) override;
			bool load(State&, SceneViewState&, TimelineState&) override;
			bool save(const State&, const SceneViewState&, const TimelineState&) const override;


			static int renderParamsRetrievedCallback(void*,int,char**,char**);
			static int timelineRetrievedCallback(void*,int, char**, char**);

        protected:
            bool prepareStatements();
            bool getLayouts(State&);
            bool getSpacetimeInfo(SceneViewState&, TimelineState&);
            bool saveLayouts(const State&) const;
            bool saveSpacetimeInfo(const SceneViewState&, const TimelineState&) const;

            sqlite3* _db;
            sqlite3_stmt* _getSpacetimeInfoStmt;
            sqlite3_stmt* _upsertSpacetimeInfoStmt;
            sqlite3_stmt* _getLayoutsStmt;
            sqlite3_stmt* _upsertLayoutStmt;

			std::string _statePath;
	};
};

#endif
