// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef DRAWING_CONTROLLER_H
#define DRAWING_CONTROLLER_H

#include "core/base_input_events_subscriber.h"
#include "core/interfaces/iapp_events_subscriber.h"
#include "3d/ipre_renderer.h"
#include "3d/ipost_renderer.h"
#include "core/structs.h"

/*
#include "3d/ipre_renderer.h"
#include "3d/selection_rectangle.h"
*/


// Forward declarations
class ShapeEditor;
class ShapeCreator;
class SceneView;
class ShapeMesh;
class ShapeAnchor;
class AnchorPoolNode;
class Icons;

namespace OGL
{
    class Context;
    class Shape;
}

namespace SEAScope
{
    // Forward declarations
    class AppEventsDispatcher;
    struct RenderableId;

    enum class DrawingControllerActions: ActionType {
        CANCEL_AND_LEAVE_DRAWING,
        COMMIT_DRAWING,
        COMMIT_DRAWING_ALT,
        ADD_VERTEX,
        INCREASE_RECTANGLE_ALPHA,
        DECREASE_RECTANGLE_ALPHA,
        INCREASE_RECTANGLE_BETA,
        DECREASE_RECTANGLE_BETA,
        SET_RECTANGLE_CENTER,
        START_MOVING_ANCHOR,
        MOVE_ANCHOR,
        STOP_MOVING_ANCHOR,
        ADD_ANCHOR,
        DELETE_ANCHOR,
        DELETE_ANCHOR_ALT,
        CANCEL_EDIT,
        COMMIT_EDIT,
        COMMIT_EDIT_ALT,
        TOGGLE_POINT_CREATION,
        TOGGLE_POLYLINE_CREATION,
        TOGGLE_POLYGON_CREATION,
    };

    class DrawingController final : public virtual BaseInputEventsSubscriber,
                                    public virtual IAppEventsSubscriber,
                                    public virtual IPreRenderer,
                                    public virtual IPostRenderer
    {
        public:
            DrawingController(AppEventsDispatcher&, SceneView&, const Icons&);
            ~DrawingController() override;

            bool isActive() const override;
            bool performAction(const ActionType&, const InputEvent&, const InputDevicesState&) override;

            bool preRender(OGL::Context*, SceneView*) override;

            bool postRender(OGL::Context*, SceneView*) override;

            bool handleAppEvent(std::type_index, const AppEvent&) override;

            void updateMeasurements(const OGL::Shape*, const ShapeType&);

            void possibleActionsUpdated();

            bool getExplicitDrawingModeExit() const;
            bool setExplicitDrawingModeExit(bool);

        protected:
            ShapeMesh* getShapeMesh(const RenderableId&, bool&);

            void startEdition(const RenderableId&);
            void stopEdition(bool);
            void clearEdition();
            void startCreation(const CollectionVariable&, const ShapeType&);
            void stopCreation();

            AppEventsDispatcher& _appEventsDispatcher;
            AnchorPoolNode* _anchors;
            ShapeEditor* _editor;
            ShapeCreator* _creator;
            SceneView& _sceneView;
            CollectionVariable _collectionVariable;
            RenderableId _editedRenderableId;
            OGL::Shape* _editedShape;
            ShapeAnchor* _hoveredAnchor;
            bool _explicitDrawingModeExit;
    };
};

#endif
