// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef EXTRACTION_CONTROLLER_H
#define EXTRACTION_CONTROLLER_H

#include <set>
#include <map>
#include <string>
#include <functional>
#include <unordered_map>

#include "core/base_input_events_subscriber.h"
#include "3d/extractor.h"
#include "3d/transect.h"
#include "3d/iscene.h"
#include "3d/scene_graph_controller.h"

class SceneView;
class Renderable2;
class ShapeMesh;

namespace SEAScope
{
    struct Extraction;
    struct RenderableId;
    class Engine;

    enum class ExtractionControllerActions: ActionType {
        RESIZE_FRAMEBUFFER,
    };

    class ExtractionController final : public virtual BaseInputEventsSubscriber
    {
        public:
            ExtractionController();
            ~ExtractionController() override;

            bool isActive() const override;
            bool performAction(const ActionType&, const InputEvent&, const InputDevicesState&) override;

            bool extractData(SceneView&, Renderable2*, std::map<RenderableId, Extraction>&, bool);
            bool transectData(SceneView&, ShapeMesh*, std::map<RenderableId, Extraction>&, bool);
            bool extractFromRenderableId(RenderableId, Engine*, std::map<RenderableId, Extraction>&, std::map<RenderableId, std::string>&, bool);

            bool listIntersectedGranules(const RenderableId&, Engine*, std::vector<std::string>&);
            bool makePersistentExtraction(const RenderableId&, Engine*);
            const std::map<RenderableId, Extraction>& getExtractedData() const;
            const std::map<RenderableId, std::string>& getExtractedLabels() const;

			static bool clearExtraction(std::map<RenderableId, Extraction>&, std::map<RenderableId, std::string>&);
            static bool initExtraction(Engine&, std::map<RenderableId, Extraction>&, std::map<RenderableId, std::string>&);

        protected:
            Extractor _extractor;
            Transect _transect;

            std::map<RenderableId, Extraction> _extractedData;
            std::map<RenderableId, std::string> _extractedLabels;
    };
};

#endif
