// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SCENE_VIEW_CONTROLLER_BASE_H
#define SCENE_VIEW_CONTROLLER_BASE_H

#include "core/base_input_events_subscriber.h"
#include "core/interfaces/iapp_events_subscriber.h"

class SceneView;

namespace SEAScope
{

class AppEventsDispatcher;
struct SceneViewState;

class SceneViewControllerBase : public BaseInputEventsSubscriber,
								public IAppEventsSubscriber
{
public:
	SceneViewControllerBase(const std::string& name, AppEventsDispatcher& appEventsDispatcher);
	virtual ~SceneViewControllerBase() override;

	virtual void resetCamera() = 0;
	virtual void zoomIn() = 0;
	virtual void zoomOut() = 0;
	virtual void lookAt(float lon, float lat) = 0;
	virtual void setZoom(float altitude) = 0;
    virtual void setState(const SceneViewState&) = 0;
    virtual void getState(SceneViewState&) const = 0;

	SceneView* getSceneView();
	void setSceneView(SceneView* sceneView);


protected:
	AppEventsDispatcher& _appEventsDispatcher;
	SceneView* _sceneView;
};

} // namespace SEAScope

#endif // SCENE_VIEW_CONTROLLER_BASE_H
