// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef WINDOW_CONTROLLER_H
#define WINDOW_CONTROLLER_H

#include <set>
#include <string>
#include <unordered_map>

#include "core/base_input_events_subscriber.h"
#include "core/interfaces/iapp_events_subscriber.h"

namespace SEAScope
{
    // Forward declaration
    class Application;
    class IWindow;
    class AppEvent;
    class AppEventsDispatcher;

    enum class WindowControllerActions: ActionType {
        QUIT,
    };

    class WindowController final : public virtual BaseInputEventsSubscriber, public virtual IAppEventsSubscriber
    {
        public:
            WindowController(AppEventsDispatcher&);
            ~WindowController() override;

            bool isActive() const override;
            bool performAction(const ActionType&, const InputEvent&, const InputDevicesState&) override;

            bool handleAppEvent(std::type_index, const AppEvent&) override;

            void setWindow(IWindow*);

        protected:
            AppEventsDispatcher& _appEventsDispatcher;
            IWindow* _window;
    };
};

#endif
