// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IADVANCED_RENDERING_CONTROL_VIEW_H
#define IADVANCED_RENDERING_CONTROL_VIEW_H

#include <functional>
#include <cstdint>
#include <string>
#include <array>

#include "core/rendering_cfg.h"
#include "core/colormap_manager.h"

namespace SEAScope
{
	class IAdvancedRenderingControlView
	{
		public:
			virtual ~IAdvancedRenderingControlView();

			virtual bool init(const ColormapManager*) = 0;
			virtual bool setScaleChangedCallback(const std::function<void(float,float,bool)>&) = 0;
			virtual bool setColormapChangedCallback(const std::function<void(const std::string&)>&) = 0;
			virtual bool setColorChangedCallback(const std::function<void(const std::array<std::uint8_t, 3>&)>&) = 0;
			virtual bool setSaveConfigCallback(const std::function<void()>&) = 0;
			virtual bool setResetCallback(const std::function<void()>&) = 0;
			virtual bool setFilterModeCallback(const std::function<void(const std::string&)>&) = 0;
            virtual bool setStreamlinesCallback(const std::function<void(std::uint32_t, std::uint32_t, float)>&) = 0;
			virtual void setBillboardsSizeCallback(const std::function<void(float)>&) = 0;
			virtual void setBillboardsDensityCallback(const std::function<void(float)>&) = 0;
            virtual bool setLineThicknessCallback(const std::function<void(float)>&) = 0;
			virtual void setMarkerChangedCallback(const std::function<void(const std::string& name)>&) = 0;

			virtual bool update(RenderingCfg*, const std::string&) = 0;
			virtual bool update(RenderingCfg*) = 0;
            virtual void show() = 0;
            virtual void hide() = 0;
	};
};

#endif
