// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ICOLLECTIONS_CONTROL_VIEW_H
#define ICOLLECTIONS_CONTROL_VIEW_H

#include <vector>
#include <functional>

#include "core/types.h"
#include "core/structs.h"

namespace SEAScope
{
	class ICollectionsControlView
	{
		public:
			virtual ~ICollectionsControlView();

			virtual bool addVariables(const std::vector<std::pair<CollectionVariable, CatalogueEntry> >&) = 0;
			virtual bool removeVariables(const std::vector<CollectionVariable>&) = 0;
			virtual bool selectVariable(const CollectionVariable&, bool) = 0;
			virtual bool setSelectionCallback(const std::function<void(const CollectionVariable&, bool, bool)>&) = 0;
            virtual bool allowDeletion(const CollectionVariable&, bool) = 0;
	};
};

#endif
