// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ICOMPONENT_H
#define ICOMPONENT_H

#include <functional>
#include <string>

namespace SEAScope
{
	class IComponent
	{
		public:
			virtual ~IComponent();

			virtual bool isResizable() const = 0;
			virtual bool isMovable() const = 0;
			virtual bool setPosition(int x, int y) = 0;
			virtual bool setSize(int width, int height) = 0;
			virtual bool setVisible(bool visible) = 0;
			virtual bool visible() const = 0;
			virtual bool requestReflow() = 0;
			virtual bool setComponentId(const std::string&) = 0;
			virtual const std::string& getComponentId() const = 0;
			virtual bool setResizeEventEmitter(const std::function<void(IComponent*)>&) = 0;
			virtual int getWidth() const = 0;
			virtual int getHeight() const = 0;
			virtual int getWidthOffset() const = 0;
			virtual int getHeightOffset() const = 0;
	};
};

#endif
