// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IRENDERING_CONTROL_VIEW_H
#define IRENDERING_CONTROL_VIEW_H

#include <vector>
#include <functional>
#include "ui/irendering_widget.h"

namespace SEAScope
{
	class IRenderingControlView
	{
		public:
			virtual ~IRenderingControlView();

			virtual IRenderingWidget* createRendering() = 0;
			virtual bool addRendering(IRenderingWidget*) = 0;
			virtual bool removeRendering(IRenderingWidget*) = 0;
			virtual bool setOrderCallback(const std::function<bool(IRenderingWidget*,IRenderingWidget*,IRenderingWidget*)>&) = 0;
			virtual bool setOpacityCallback(const std::function<bool(IRenderingWidget*, float)>&) = 0;
			virtual bool setVisibilityCallback(const std::function<bool(IRenderingWidget*, bool, bool)>&) = 0;
			virtual bool update() = 0;
			virtual bool setMinCallback(const std::function<void(IRenderingWidget*, float)>&) = 0;
			virtual bool setMaxCallback(const std::function<void(IRenderingWidget*, float)>&) = 0;
	};
};
#endif
