// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IRENDERING_WIDGET_H
#define IRENDERING_WIDGET_H

#include <string>
#include <functional>
#include "core/types.h"
#include "core/rendering_cfg.h"

namespace SEAScope
{
	class IRenderingWidget
	{
		public:
			virtual ~IRenderingWidget();

			virtual bool update(RenderingCfg*) = 0;
			virtual bool update(const std::string&, const std::string&, RenderingCfg*) = 0;
			virtual bool setOpacityCallback(const std::function<bool(IRenderingWidget*, float)>&) = 0;
			virtual bool setVisibilityCallback(const std::function<void(IRenderingWidget*, bool, bool)>&) = 0;
			virtual bool setSettingsCallback(const std::function<bool(IRenderingWidget*)>&) = 0;
			virtual bool setColormapGetter(const std::function<bool(const std::string&, Colormap&)>&) = 0;
			virtual bool setMinCallback(const std::function<void(IRenderingWidget*, float)>&) = 0;
			virtual bool setMaxCallback(const std::function<void(IRenderingWidget*, float)>&) = 0;
			virtual bool setCounter(uint32_t) = 0;
			virtual bool setSettingsState(bool) = 0;
            virtual bool setOpacityScrollStep(float) = 0;
	};
};

#endif
