// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ITIMELINE_VIEW_H
#define ITIMELINE_VIEW_H

#include <cstdint>
#include <vector>
#include <string>
#include <functional>
#include "core/types.h"
#include "data/events_coverage.h"

namespace SEAScope
{
	class ITimelineView
	{
		public:
			virtual ~ITimelineView();

			virtual bool init( const std::vector<std::string>& timeframes
							 , const std::vector<std::string>& timeframesTooltips
							 , const std::vector<std::string>& timesteps
							 , const std::vector<std::string>& timestepsTooltips
							 , std::uint16_t minYear
							 , std::uint16_t maxYear
                             , bool hoverEnabled) = 0;
			virtual bool setCurrentChangedCallback(const std::function<void(const std::string&, const std::string&)>& callback) = 0;
			virtual bool setTimeframeChangedCallback(const std::function<void(const std::string&)>& callback) = 0;
			virtual bool setTimestepChangedCallback(const std::function<void(const std::string&)>& callback) = 0;
			virtual bool setPrevCallback(const std::function<void()>& callback) = 0;
			virtual bool setNextCallback(const std::function<void()>& callback) = 0;
            virtual bool setYearScrollCallback(const std::function<void(float)>& callback) = 0;
            virtual bool setMonthScrollCallback(const std::function<void(float)>& callback) = 0;
            virtual bool setDayScrollCallback(const std::function<void(float)>& callback) = 0;
			virtual bool setOptionsChangedCallback(const std::function<void()>&) = 0;
			virtual bool setCurrent(const std::string& date, const std::string& time, int, int, int) = 0;
			virtual bool setVisibleBounds(timestamp_t, timestamp_t) = 0;
			virtual bool setTimeframe(const std::string& label, const std::string& description) = 0;
			virtual bool setTimestep(const std::string& label, const std::string& description) = 0;
			virtual bool setHintsSource(const EventsCoverage&) = 0;
			virtual bool setMainCoverage(const EventsCoverage&) = 0;
			virtual bool setExtraCoverage(const EventsCoverage&) = 0;
			virtual bool setNearestMode(bool) = 0;
			virtual bool nearestMode() const = 0;
			virtual void setMonth(std::uint16_t, std::uint8_t) = 0;
			virtual void setYear(std::uint16_t) = 0;
            virtual bool getExtendedView() const = 0;
            virtual void setExtendedView(bool, bool) = 0;
	};
};

#endif
