// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_ADD_PROPERTY_FORM_H
#define NANOGUI_ADD_PROPERTY_FORM_H

#include <set>
#include <vector>
#include <string>
#include <functional>

#include "vendors/_nanogui.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiTextField;

    class NanoguiAddPropertyForm: public virtual nanogui::Widget
    {
        public:
            NanoguiAddPropertyForm(nanogui::Widget*, const std::set<std::string>&);
            virtual ~NanoguiAddPropertyForm() override;

            bool getName(std::string&) const;
            bool getType(std::string&) const;

            void reset();

            bool setValidateCallback(const std::function<bool(const std::string&, const std::string&)>&);
            bool setOkCallback(const std::function<void()>&);
            bool setNokCallback(const std::function<void()>&);

        protected:
            std::vector<nanogui::Button*> _buttons;
            NanoguiTextField* _nameField;
            nanogui::Button* _okButton;
            nanogui::Button* _nokButton;

            std::function<bool(const std::string&, const std::string&)> _validateCallback;
            std::function<void()> _okCallback;
            std::function<void()> _nokCallback;
    };
}

#endif
