// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_ADVANCED_RENDERING_CONTROL_H
#define NANOGUI_ADVANCED_RENDERING_CONTROL_H

#include <nanogui/widget.h>
#include <nanogui/window.h>

#include "ui/iadvanced_rendering_control_view.h"
#include "ui/nanogui/nanogui_hcolorbar.h"
#include "ui/nanogui/nanogui_anchored.h"
#include "ui/nanogui/nanogui_numberbox.h"
#include "ui/nanogui/nanogui_colorbar_list.h"
#include "ui/nanogui/nanogui_checkbox.h"

namespace nanogui
{
    // Forward declarations
    class Button;
    class Label;
    class Slider;
}

namespace SEAScope
{
	class NanoguiTexturedButton;
	class NanoguiTexturePanel;


	class NanoguiAdvancedRenderingControl final: public virtual nanogui::Widget, public virtual IAdvancedRenderingControlView
	{
		public:
			NanoguiAdvancedRenderingControl(nanogui::Widget* parent);
			~NanoguiAdvancedRenderingControl() override;

			void draw(NVGcontext*) override;
			bool scrollEvent(const Eigen::Vector2i&, const Eigen::Vector2f&) override;

			void resize(const Eigen::Vector2i&);

			bool init(const ColormapManager*) override;
			bool setScaleChangedCallback(const std::function<void(float, float, bool)>&) override;
			bool setColormapChangedCallback(const std::function<void(const std::string&)>&) override;
			bool setColorChangedCallback(const std::function<void(const std::array<std::uint8_t, 3>&)>&) override;
			bool setSaveConfigCallback(const std::function<void()>&) override;
			bool setResetCallback(const std::function<void()>&) override;
			bool setFilterModeCallback(const std::function<void(const std::string&)>&) override;
            bool setStreamlinesCallback(const std::function<void(std::uint32_t, std::uint32_t, float)>&) override;
			void setBillboardsSizeCallback(const std::function<void(float)>&) override;
            void setBillboardsDensityCallback(const std::function<void(float)>&) override;
            bool setLineThicknessCallback(const std::function<void(float)>&) override;
			void setMarkerChangedCallback(const std::function<void(const std::string& name)>&) override;

			bool update(RenderingCfg*, const std::string&) override;
			bool update(RenderingCfg*) override;
            void show() override;
            void hide() override;

			void changeExtremum(bool, float);
            bool setActivationCallback(const std::function<void(bool)>&);

            void buildPalette(nanogui::Widget*);
            void buildSettingsPanel(nanogui::Widget*);
			void initBillboards(nanogui::Widget* parent);
			void showIconWindow(bool b);

	protected:
			std::function<void(float, float, bool)> _scaleCallback;
			std::function<void(const std::string&)> _cmapCallback;
			std::function<void(const std::array<std::uint8_t, 3>&)> _colorCallback;
			std::function<void()> _resetCallback;
			std::function<void()> _saveCallback;
			std::function<void(const std::string&)> _filterModeCallback;
            std::function<void(bool)> _activationCallback;
            std::function<void(std::uint32_t, std::uint32_t, float)> _streamlinesCallback;
            std::function<void(float)> _billboardsSizeCallback;
            std::function<void(float)> _billboardsDensityCallback;
            std::function<void(float)> _lineThicknessCallback;
			std::function<void(const std::string& name)> _markerChangedCallback;

			const ColormapManager* _cmapManager;
			std::string _label;
            std::string _labelBuffer;
			nanogui::Button* _saveButton;
			nanogui::Button* _resetButton;
			nanogui::Button* _nearestBtn;
			nanogui::Button* _bilinearBtn;
			NanoguiNumberBox* _minField;
			NanoguiNumberBox* _maxField;
			NanoguiCheckbox* _logScale;
			nanogui::Widget* _palette;
			nanogui::Widget* _scale;
			NanoguiHColorbar* _colorbar;
			nanogui::Label* _tick1;
			nanogui::Label* _tick2;
			nanogui::Label* _tick3;
			nanogui::Label* _tick4;
			NanoguiAnchored* _helper;
	        NanoguiColorbarList* _colorChooser;
			uint16_t _cursorY;
			std::string _cursorTxt;
            bool _reflowNeeded;

            nanogui::Widget* _leftSide;
            nanogui::Widget* _rightSide;

            nanogui::Widget* _filteringWidget;

            nanogui::Widget* _colorscaleWidget;

            nanogui::Widget* _streamlinesWidget;
            NanoguiNumberBox* _particlesField;
            NanoguiNumberBox* _lengthField;
            NanoguiNumberBox* _speedField;

            nanogui::Widget* _linesWidget;
            NanoguiNumberBox* _lineThicknessField;

            nanogui::Widget* _leftSpacer;

			nanogui::Widget* _billboardsWidget;
			NanoguiNumberBox* _billboardsSizeField;
			NanoguiNumberBox* _billboardsDensityField;
            nanogui::Widget* _billboardsDensityContainer;

			SEAScope::NanoguiTexturedButton* _tbutton;
			SEAScope::NanoguiTexturePanel* _texPanel;
            nanogui::Widget* _markerContainer;
            SEAScope::RenderableId _current;
	};
};

#endif
