// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_ANCHORED_H
#define NANOGUI_ANCHORED_H

#include <nanogui/widget.h>

namespace SEAScope
{
	class NanoguiAnchored final: public virtual nanogui::Widget
	{
		public:
			NanoguiAnchored(nanogui::Widget*, nanogui::Widget*);
			~NanoguiAnchored() override;

			void draw(NVGcontext*) override;
			bool mouseEnterEvent(const Eigen::Vector2i&, bool) override;
			bool mouseButtonEvent(const Eigen::Vector2i&, int, bool, int) override;
			bool mouseDragEvent(const Eigen::Vector2i&, const Eigen::Vector2i&, int, int) override;
			bool scrollEvent(const Eigen::Vector2i&, const Eigen::Vector2f&) override;
			bool setMouseEnterCallback(const std::function<void(bool)>&);

			void refreshRelativePlacement();

			nanogui::Widget* anchor;
			bool layoutUpdateRequested;
			bool autoSize;
			uint16_t offset;

            bool useDirectionalOffset;
            uint16_t leftOffset;
            uint16_t topOffset;
            uint16_t rightOffset;
            uint16_t bottomOffset;

			uint8_t placement;
			std::function<void(bool)> callback;

			static NanoguiAnchored* create(nanogui::Widget*);
			static uint8_t ON_LEFT;
			static uint8_t ON_RIGHT;
			static uint8_t ON_TOP;
			static uint8_t ON_BOTTOM;
	};
};

#endif
