// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_ANNOTATION_COLLECTION_SELECTOR_H
#define NANOGUI_ANNOTATION_COLLECTION_SELECTOR_H

#include <string>
#include <vector>
#include <functional>

#include "vendors/_nanogui.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiIcon;
    class NanoguiComboBox;

    class NanoguiAnnotationCollectionSelector: public virtual nanogui::Widget
    {
        public:
            NanoguiAnnotationCollectionSelector(nanogui::Widget*, int, const std::string&);
            virtual ~NanoguiAnnotationCollectionSelector() override;

            void draw(NVGcontext*) override;
            void setChangeCallback(const std::function<void(bool, const std::string&)>&);
            void setDropDownCallback(const std::function<void()>&);
            void setAddCallback(const std::function<void()>&);

            void setChoices(const std::vector<std::string>&);
            void choose(const std::string&);
            const std::string& getChoice() const;

            void allowNewCollection(bool);

            void useDefault();
            void useCustom();

            void hideDropDown();

            void enable();
            void disable();

        protected:
            NanoguiIcon* _icon;
            NanoguiComboBox* _combobox;
            nanogui::Button* _addButton;
            nanogui::Label* _useLabel;
            nanogui::Button* _defaultButton;
            nanogui::Button* _customButton;
            nanogui::Label* _collectionLabel;

            std::function<void(bool, const std::string&)> _changeCallback;
            std::function<void()> _dropdownCallback;
            std::function<void()> _addCallback;

            bool _reflowNeeded;
            std::vector<nanogui::Button*> _modeButtons;
            std::string _defaultValue;
    };
}

#endif
