// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_ANNOTATION_DIALOG_H
#define NANOGUI_ANNOTATION_DIALOG_H

#include <vector>
#include <functional>
#include <nanogui/widget.h>
#include <nanogui/opengl.h>

#include "core/structs.h"

#include "ui/nanogui/nanogui_image_view.h"

namespace SEAScope
{
    // Forward declarations
    class DictNode;
    class GranuleInfo;
    class NanoguiCheckbox;
    class NanoguiAnchored;
    class NanoguiAttribute;
    class NanoguiPathSelector;
    class NanoguiTimeCoverageEditor;
    class NanoguiPropertiesEditor;

    class NanoguiAnnotationDialog final: public virtual nanogui::Widget
    {
        public:
            NanoguiAnnotationDialog(nanogui::Widget*);
            ~NanoguiAnnotationDialog() override;

            bool keyboardEvent(int, int, int, int) override;

            bool enable(const RenderableId&, const GranuleInfo&, DictNode&&, double, double);
            bool disable();

            void draw(NVGcontext*) override;

            void enableEdition(bool, bool);

            bool setSavePropertiesCallback(const std::function<void(const RenderableId&, const DictNode&)>&);
            bool setSaveCallback(const std::function<void(const RenderableId&, const std::string&, const AnnotationFormat&, bool)>&);
            bool setUpdateTimeCoverageCallback(const std::function<void(const RenderableId&, double, double)>&);
            void updateSpatialCoverage(double, double, double, double);
            void updateTemporalCoverage(double, double);

            NanoguiAttribute* collectionAttr;
            NanoguiAttribute* variableAttr;
            NanoguiAttribute* spatialCoverageAttr;
            NanoguiPropertiesEditor* _editor;
            nanogui::Widget* btns;
            nanogui::Button* extractBtn;
            nanogui::Button* delBtn;
            nanogui::Button* editBtn;
            nanogui::Button* saveBtn;
			std::function<void(const RenderableId&)> extractCallback;
            std::function<void(const RenderableId&)> deleteCallback;
            std::function<void(const RenderableId&, bool)> editCallback;
			RenderableId target;

        protected:
            void initializeSavePanel(nanogui::Widget*);

            NanoguiTimeCoverageEditor* _timeCoverageEditor;

            std::vector<nanogui::Button*> _exportFormatButtons;
            NanoguiAnchored* _saveWidget;
            NanoguiCheckbox* _includeGranulesListCheckbox;
            NanoguiPathSelector* _pathSelector;
            nanogui::Button* _saveDataButton;

            std::function<void(const RenderableId&, const DictNode&)> _savePropertiesCallback;
            std::function<void(const RenderableId&, double, double)> _updateTimeCoverageCallback;
            std::function<void(const RenderableId&, const std::string&, const AnnotationFormat&, bool)> _saveCallback;
    };
};

#endif
