// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_ANNOTATIONS_COLLECTION_CREATOR_H
#define NANOGUI_ANNOTATIONS_COLLECTION_CREATOR_H

#include <functional>

#include "vendors/_nanogui.h"
#include "core/types.h"

namespace nanogui
{
    // Forward declarations
    class TextBox;
    class Button;
}

namespace SEAScope
{
    // Forward declarations
    class NanoguiButtonBar;
    class NanoguiTextField;

    class NanoguiAnnotationsCollectionCreator: public virtual nanogui::Widget
    {
        public:
            NanoguiAnnotationsCollectionCreator(nanogui::Widget*);
            virtual ~NanoguiAnnotationsCollectionCreator() override;

            void reset();
            void resize(const Eigen::Vector2i&);
            void setValidateCallback(const std::function<void(const ShapeType&, const std::string&, const std::string&)>&);
            void setCancelCallback(const std::function<void()>&);
            bool selectCollectionType(const std::string&);

        protected:
            NanoguiButtonBar* _collectionType;
            NanoguiTextField* _collectionLabel;
            NanoguiTextField* _variableLabel;
            nanogui::Button* _okButton;
            nanogui::Button* _nokButton;
            nanogui::Widget* _topBlock;
            nanogui::Widget* _bottomBlock;

            std::function<void(const ShapeType&, const std::string&, const std::string&)> _validateCallback;
            std::function<void()> _cancelCallback;
    };
}
#endif
