// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_APP_SETTINGS_EDITOR_H
#define NANOGUI_APP_SETTINGS_EDITOR_H

#include <functional>

#include "vendors/_nanogui.h"
#include "core/settings.h"
#include "ui/nanogui/nanogui_scrollable_with_title.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiPathSelector;
    class NanoguiTextField;
    class NanoguiCheckbox;
    class NanoguiNumberBox;
    class NanoguiList;
    class NanoguiTimespanEditor;
    class NanoguiTimestepEditor;

    class NanoguiAppSettingsEditor final: public virtual NanoguiScrollableWithTitle
    {
        public:
            NanoguiAppSettingsEditor(nanogui::Widget* parent);
            ~NanoguiAppSettingsEditor() override;

            void draw(NVGcontext*) override;

            void loadSettings(const Settings&);
            void setChangeCallback(const std::function<void(const Settings&)>&);

        protected:

            NanoguiTimespanEditor* buildTimespanEditor();
            NanoguiTimestepEditor* buildTimestepEditor();

            void updateColormapsPath(const std::string&);
            void updateDataPath(const std::string&);
            void updateStatePath(const std::string&);
            void updateIndexPath(const std::string&);
            void updateAnnotationsPath(const std::string&);
            void updateCustomPath(const std::string&);

            void updateFullscreen(bool);
            void updateWindowed(bool);
            void updateWindowWidth(double);
            void updateWindowHeight(double);
            void updateVSync(bool);
            void updateAntialiasing(bool);
            void updateMSAASampling(double);
            void updateMinScreenPortion(double);

            void updateMinYear(double);
            void updateMaxYear(double);
            bool updateTimespan(const std::string&, const std::string&, double, double, const std::string&, bool);
            bool updateTimestep(const std::string&, const std::string&, double, const std::string&, bool);
            void updateNearest(bool);
            void updateAdvancedScrolling(bool);
            bool updateInitialDatetime(const std::string&);

            void updateDrawingAutoExit(bool);
            void updateDrawingManualExit(bool);
            void updateClickTolerance(double);

            void updateProcessorEnabled(bool);
            bool updateProcessorAddress(const std::string&);
            void updateProcessorPort(double);

            void updateIndexCheck(bool);
            void updateMinSearchDelay(double);
            void updateBackgroundPath(const std::string&);

            void checkRestartRequirement();

            void sendChangeNotification();

            void hideDropdownsWithHiddenAnchors();

            // Paths
            NanoguiPathSelector* _colormapsPath;
            NanoguiPathSelector* _dataPath;
            NanoguiPathSelector* _statePath;
            NanoguiPathSelector* _indexPath;
            NanoguiPathSelector* _annotationsPath;
            NanoguiPathSelector* _customPath;

            // Rendering
            NanoguiCheckbox* _fullscreenCheck;
            NanoguiCheckbox* _windowedCheck;
            NanoguiNumberBox* _windowWidth;
            NanoguiNumberBox* _windowHeight;
            NanoguiCheckbox* _vsyncCheck;
            NanoguiCheckbox* _antialiasingCheck;
            NanoguiNumberBox* _msaaSampling;
            NanoguiNumberBox* _minScreenPortion;

            // Timeline
            NanoguiNumberBox* _minYear;
            NanoguiNumberBox* _maxYear;
            NanoguiList* _timesteps;
            NanoguiList* _timespans;
            NanoguiCheckbox* _nearestCheck;
            NanoguiCheckbox* _advancedScrollingCheck;
            NanoguiTextField* _initialDatetime;

            // Drawing
            NanoguiCheckbox* _drawingAutoExit;
            NanoguiCheckbox* _drawingManualExit;
            NanoguiNumberBox* _clickTolerance;

            // Processor
            NanoguiCheckbox* _processorCheck;
            NanoguiTextField* _processorAddress;
            NanoguiNumberBox* _processorPort;

            // Misc
            NanoguiCheckbox* _indexCheck;
            NanoguiNumberBox* _minSearchDelay;
            NanoguiPathSelector* _backgroundPath;

            std::function<void(const Settings&)> _changeCallback;
            Settings _settings;
            Settings _editedSettings;

            bool _muted;
            nanogui::Label* _restartNeededMessage;
            std::vector<NanoguiTimespanEditor*> _timespanEditorsToRemove;
            std::vector<NanoguiTimestepEditor*> _timestepEditorsToRemove;
    };
}

#endif
