// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_BLUEPRINT_H
#define NANOGUI_BLUEPRINT_H

#include <string>

#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_resize_handle.h"
#include "ui/icomponent.h"
#include "ui/iblueprint.h"

namespace SEAScope {

	class NanoguiBlueprint final: public virtual nanogui::Widget, public virtual IBlueprint {

		public:
			NanoguiBlueprint( nanogui::Screen *parent, bool, bool);
			~NanoguiBlueprint() override;

			bool setMinDimensions(const std::string&, const std::string&) override;
			bool getLayout(std::string&, std::string&, std::string&, std::string&) const override;
			bool setLayout(const std::string&, const std::string&, const std::string&, const std::string&) override;
			bool resizeAuto() override;
			bool setTarget(IComponent*) override;
			IComponent* getTarget() const override;

			void draw(NVGcontext *ctx) override;
			bool mouseDragEvent(const Eigen::Vector2i &p, const Eigen::Vector2i &rel, int button, int modifiers) override;

			void setVisible(bool visible);


			bool startEditing();
			bool stopEditing();
			bool applyToTarget();
			bool updateSize(const Eigen::Vector2i&);

			nanogui::Screen* screen;
			IComponent* target;
			bool targetIsVisible;
	        NanoguiResizeHandle* resizeHandle;
			bool movable;
			std::string minWidth;
			std::string minHeight;
			std::string storedWidth;
			std::string storedHeight;
			std::string storedX;
			std::string storedY;
			bool editing;
			bool edited;
	};

};

#endif
