// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_BOOL_FIELD_EDITOR_H
#define NANOGUI_BOOL_FIELD_EDITOR_H

#include <string>
#include <functional>

#include "vendors/_nanogui.h"


namespace SEAScope
{
    // Forward declarations
    class NanoguiCheckbox;

    class NanoguiBoolFieldEditor: public virtual nanogui::Widget
    {
        public:
            NanoguiBoolFieldEditor(nanogui::Widget*);
            virtual ~NanoguiBoolFieldEditor() override;

            void draw(NVGcontext*) override;

            void setCategory(const std::string&);
            const std::string& getCategory() const;

            void setName(const std::string&);
            const std::string& getName() const;

            void setValue(bool);
            bool getValue() const;

            void setOriginalValue(bool);
            bool getOriginalValue() const;

            void setBackgroundColor(const nanogui::Color&);
            const nanogui::Color& getBackgroundColor() const;

            bool activateEdition(bool);

            bool setDeleteCallback(const std::function<void(const std::string&, const std::string&)>&);

        protected:
            nanogui::Color _backgroundColor;
            nanogui::Label* _nameLabel;
            NanoguiCheckbox* _valueCheckBox;
            nanogui::Button* _resetButton;
            nanogui::Button* _deleteButton;

            std::function<void(const std::string&, const std::string&)> _deleteCallback;

            std::string _category;
            bool _originalValue;
            bool _editing;
            bool _reflowNeeded;
    };
}

#endif
