// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_BUTTON_BAR_H
#define NANOGUI_BUTTON_BAR_H

#include <vector>
#include <string>
#include <nanogui/widget.h>
#include <nanogui/button.h>

namespace SEAScope
{
	class NanoguiButtonBar final: public virtual nanogui::Widget
	{
		public:
			NanoguiButtonBar(nanogui::Widget*);
			~NanoguiButtonBar() override;

			bool init( const std::vector<std::string>& values
					 , const std::vector<std::string>& tooltips
					 , uint8_t fontSize
                     , const Eigen::Vector2i& buttonSize);
            const std::string selectedValue() const;
            bool selectValue(const std::string& value);

            void selectionChanged();
            bool setSelectionChangedCallback(const std::function<void(const std::string&)>& callback);

            std::vector<nanogui::Button*> buttons;
            std::function<void(const std::string&)> selectionChangedCallback = nullptr;
	};
};

#endif
