// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_CALENDAR_H
#define NANOGUI_CALENDAR_H

#include <cstdint>
#include <functional>

#include <nanogui/widget.h>
#include "ui/nanogui/nanogui_flat_button.h"

namespace SEAScope
{
	class NanoguiCalendar final: public virtual nanogui::Widget
	{
		public:
			NanoguiCalendar(nanogui::Widget*);
			~NanoguiCalendar() override;

			bool setYear(std::uint16_t);
			bool setMonthFgColor(std::uint8_t, nanogui::Color);
			bool setMonthBgColor(std::uint8_t, nanogui::Color);
			bool select();
			bool unselect();
			bool selectMonth(std::uint8_t);
			bool unselectMonth();
			bool enableMonth(std::uint8_t);
			bool disableMonth(std::uint8_t);
			bool reset();
			bool setYearBgColor(nanogui::Color);
			bool setYearFgColor(nanogui::Color);

			std::function<void(std::uint16_t, std::uint8_t)> monthCallback;
			std::function<void(std::uint16_t)> yearCallback;
			NanoguiFlatButton* yearBtn;
			std::uint16_t year;
			std::array<NanoguiFlatButton*, 12> months;
			std::array<nanogui::Color, 12> backgrounds;
			std::array<nanogui::Color, 12> textColors;
			std::size_t selectedMonth;
			nanogui::Color yearBackground;
			nanogui::Color yearForeground;
	};
};

#endif
