// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_CALENDARS_TIMELINE_H
#define NANOGUI_CALENDARS_TIMELINE_H

#include <cstdint>
#include <functional>
#include <nanogui/widget.h>
#include <nanogui/button.h>
#include <nanogui/opengl.h>

#include "core/types.h"
#include "data/events_coverage.h"
#include "ui/nanogui/nanogui_calendar.h"

namespace SEAScope
{
	class NanoguiCalendarsTimeline final: public virtual nanogui::Widget
	{
		public:
			NanoguiCalendarsTimeline(nanogui::Widget*);
			~NanoguiCalendarsTimeline() override;

			void setSize(const Eigen::Vector2i&);
			void setWidth(int);
			void setFixedSize(const Eigen::Vector2i&);
			void setFixedWidth(int);
			void draw(NVGcontext*) override;

			bool scrollEvent(const Eigen::Vector2i&, const Eigen::Vector2f&) override;

			bool setMainCoverage(const EventsCoverage&);
			bool setExtraCoverage(const EventsCoverage&);
			bool setCurrent(std::uint16_t year, std::uint8_t month, std::uint8_t day, bool);
			bool setCentralYear(std::uint16_t);
            bool isYearVisible(const std::uint16_t) const;
			bool updateCalendars();
			bool updateNavButtons();
			bool resize(int);
			bool nextPage();
			bool prevPage();
			std::uint16_t adaptYear(std::uint16_t) const;
			bool setVisibilityBounds(timestamp_t, timestamp_t);
			bool setBounds(std::uint16_t, std::uint16_t);

			const int minGap;
			std::uint16_t minYear;
			std::uint16_t maxYear;

			std::uint16_t currentYear;
			std::uint8_t currentMonth;
			std::uint8_t currentDay;

			std::size_t visibleCount;
			std::uint16_t centralYear;
			std::vector<NanoguiCalendar*> calendarsPool;
			EventsCoverage mainCoverage;
			EventsCoverage extraCoverage;
			nanogui::Widget* calendarsPanel;
			nanogui::Button* prevButton;
			nanogui::Button* nextButton;
			std::function<void(std::uint16_t)> yearCallback;
			std::function<void(std::uint16_t, std::uint8_t)> monthCallback;
            std::function<void(bool)> currentYearVisibilityChanged;
			std::function<void(float)> scrollCallback;
			timestamp_t pastOffset;
			timestamp_t futureOffset;
            bool hoverEnabled;
	};
};

#endif
