// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_CAMERA_CONTROL_H
#define NANOGUI_CAMERA_CONTROL_H

#include <functional>
#include <nanogui/widget.h>
#include <nanogui/label.h>
#include <nanogui/opengl.h>
#include "ui/nanogui/nanogui_component.h"
#include "ui/nanogui/nanogui_scale.h"
#include "core/interfaces/iapp_events_subscriber.h"
#include "ui/icamera_control_view.h"

// Forward declarations
class SceneView;

namespace SEAScope
{
    // Forward declarations
    class AppEvent;
    class AppEventsDispatcher;
    class InputEventsDispatcher;

	class NanoguiCameraControl final: public virtual NanoguiComponent, public virtual IAppEventsSubscriber, public virtual ICameraControlView
	{
		public:
			NanoguiCameraControl(nanogui::Widget*, AppEventsDispatcher&, const InputEventsDispatcher&);
			~NanoguiCameraControl() override;

			bool setSize(int, int) override;

			void draw(NVGcontext*) override;

            bool handleAppEvent(std::type_index, const AppEvent&) override;

            void setLocation(bool, float, float) override;
            void update(const SceneView*) override;

        protected:
            AppEventsDispatcher& _appEventsDispatcher;
            const InputEventsDispatcher& _inputEventsDispatcher;

            nanogui::Label* _location;
            nanogui::Label* _altitude;
			NanoguiScale* _scale;

	};
};

#endif
