// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_CLICKABLE_LABEL_H
#define NANOGUI_CLICKABLE_LABEL_H

#include <chrono>
#include <functional>

#include "ui/nanogui/nanogui_painted_label.h"

namespace SEAScope
{
    class NanoguiClickableLabel final: public virtual NanoguiPaintedLabel
    {
        public:
            NanoguiClickableLabel(nanogui::Widget*);
            ~NanoguiClickableLabel() override;

            bool mouseButtonEvent(const Eigen::Vector2i&, int, bool, int) override;
            bool mouseEnterEvent(const Eigen::Vector2i&, bool) override;
            bool focusEvent(bool) override;

            void toggleSelection();
            void setSelected(bool, bool);
            void updateBackgroundColor();
            bool setChangeCallback(const std::function<void(const std::string&, bool, float)>&);

        protected:
            bool _selected;
            std::chrono::time_point<std::chrono::system_clock> _lastChange;
            std::function<void(const std::string&, bool, float)> _changeCallback;
    };
}

#endif
