// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_COLLECTIONS_CONTROL_H
#define NANOGUI_COLLECTIONS_CONTROL_H

#include <map>
#include <vector>
#include <nanogui/widget.h>
#include <nanogui/opengl.h>

#include "ui/icollections_control_view.h"
#include "ui/nanogui/nanogui_scrollable_with_title.h"
#include "core/types.h"
#include "core/structs.h"
#include "ui/nanogui/nanogui_checkbox.h"

namespace SEAScope
{
	class NanoguiCollectionsControl final: public virtual NanoguiScrollableWithTitle, public virtual ICollectionsControlView
	{
		public:
			NanoguiCollectionsControl(nanogui::Widget*);
			~NanoguiCollectionsControl() override;

			bool addVariables(const std::vector<std::pair<CollectionVariable, CatalogueEntry> >&) override;
			bool removeVariables(const std::vector<CollectionVariable>&) override;
			bool selectVariable(const CollectionVariable&, bool) override;
			bool setSelectionCallback(const std::function<void(const CollectionVariable&, bool, bool)>&) override;
            bool setListChangedCallback(const std::function<void()>&);
            bool setCollectionRemovedCallback(const std::function<void(const CollectionId&)>&);
            bool allowDeletion(const CollectionVariable&, bool) override;

            static bool compareCollections(const nanogui::Widget*, const nanogui::Widget*);
			const std::string& getCategory(const CollectionVariable&, const CatalogueEntry&) const;


			std::function<void()> _listChangedCallback;
            std::function<void(const CollectionId&)> _collectionRemovedCallback;
			std::function<void(const CollectionVariable&, bool, bool)> selectionCallback;
			std::map<std::string, nanogui::Widget*> categories;
			std::map<CollectionVariable, NanoguiCheckbox*> variables;
	};
};

#endif
