// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_COLORBAR_H
#define NANOGUI_COLORBAR_H

#include <functional>
#include <nanogui/widget.h>
#include "core/types.h"

namespace SEAScope
{
	class NanoguiColorbar final: public virtual nanogui::Widget
	{
		public:
			NanoguiColorbar(nanogui::Widget*);
			~NanoguiColorbar() override;

			Eigen::Vector2i preferredSize(NVGcontext*) const override;
			void draw(NVGcontext*) override;
			bool mouseButtonEvent(const Eigen::Vector2i&,int,bool,int) override;
			bool mouseDragEvent(const Eigen::Vector2i &, const Eigen::Vector2i &,int, int) override;
			bool mouseMotionEvent(const Eigen::Vector2i &, const Eigen::Vector2i &, int, int) override;

			bool isMouseOver() const;
			bool setClickCallback(const std::function<void(const std::string&, const Colormap&)>&);
			bool setMouseMoveCallback(const std::function<void(bool, float)>&);
			bool setDragEndCallback(const std::function<void(bool, float)>&);

			Colormap cmap;
			std::string label;
			std::function<void(const std::string&, const Colormap&)> clickCallback = nullptr;
			std::function<void(bool, float)> mouseMoveCallback = nullptr;
			std::function<void(bool, float)> dragEndCallback = nullptr;
			bool dragging = false;
			bool dragTargetIsMax = false;
	};
};

#endif
