// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_COMBOBOX_H
#define NANOGUI_COMBOBOX_H

#include <functional>

#include "vendors/_nanogui.h"

// Forward declarations
struct NVGcontext;

namespace SEAScope
{
    // Forward declarations
    class NanoguiAnchored;
    class NanoguiList;
    class NanoguiClickableLabel;

    class NanoguiComboBox: public virtual nanogui::Widget
    {
        public:
            NanoguiComboBox(nanogui::Widget*);
            virtual ~NanoguiComboBox() override;

            bool mouseButtonEvent(const Eigen::Vector2i&, int, bool, int) override;
            void draw(NVGcontext*) override;

            void enable();
            void disable();

            void show();
            void hide();

            void toggleDropDown();
            void showDropDown();
            void hideDropDown();

            void setFixedSize(const Eigen::Vector2i&);
            void setItems(const std::vector<std::string>&);

            bool setSelectedValue(const std::string&);
            const std::string& getSelectedValue() const;

            bool setDropDownCallback(const std::function<void()>&);
            bool setChangeCallback(const std::function<void(const std::string&)>&);

        protected:
            std::string _selectedValue;
            nanogui::Label* _selectedLabel;
            nanogui::Button* _dropdownButton;
            NanoguiAnchored* _dropdownContainer;
            NanoguiList* _itemsList;
            std::vector<std::string> _items;
            std::vector<NanoguiClickableLabel*> _widgets;
            std::function<void()> _dropdownCallback;
            std::function<void(const std::string&)> _changeCallback;
    };
}
#endif
