// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_COMPONENT_H
#define NANOGUI_COMPONENT_H

#include "vendors/_nanogui.h"
#include "ui/icomponent.h"

namespace SEAScope
{
	class NanoguiComponent: public virtual IComponent, public virtual nanogui::Widget
	{
		public:
			NanoguiComponent(nanogui::Widget*);
			virtual ~NanoguiComponent() override;

			bool isResizable() const override;
			bool isMovable() const override;
			bool setPosition(int, int) override;
			bool setSize(int, int) override;
			bool setVisible(bool) override;
			bool visible() const override;
			bool requestReflow() override;
			bool setComponentId(const std::string&) override;
			const std::string& getComponentId() const override;
			bool setResizeEventEmitter(const std::function<void(IComponent*)>&) override;
			int getWidth() const override;
			int getHeight() const override;
			int getWidthOffset() const override;
			int getHeightOffset() const override;

			bool reflowNeeded;
			std::string componentId;
			std::function<void(IComponent*)> resizeEventEmitter;
	};
};

#endif
