// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_CONTEXTUAL_MENU_H
#define NANOGUI_CONTEXTUAL_MENU_H

#include <functional>
#include <nanogui/widget.h>
#include <nanogui/label.h>
#include <nanogui/button.h>

#include "core/structs.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiDraggablePane;
    class NanoguiAnnotationDialog;
    class NanoguiGranuleDialog;
    class NanoguiPathSelector;
    class DictNode;
    class GranuleInfo;

    class NanoguiContextualMenu
    {
        public:
            NanoguiContextualMenu();
            virtual ~NanoguiContextualMenu();

            bool init(nanogui::Widget*);
            bool display(const Eigen::Vector2i&);
			bool adaptPosition(const Eigen::Vector2i&, Eigen::Vector2i&) const;
            bool setTarget(const RenderableId&, const RenderableType&, const GranuleInfo&, DictNode&&, double, double);
            void hide();

            NanoguiAnnotationDialog* getAnnotationDialog() const;
            NanoguiGranuleDialog* getGranuleDialog() const;

            bool setAnnotationSaveCallback(const std::function<void(const RenderableId&, const std::string&, const AnnotationFormat&, bool)>&);
            bool setCloseCallback(const std::function<void()>&);

        protected:
            NanoguiDraggablePane* _window;
			nanogui::Label* _title;
			nanogui::Button* _closeBtn;
            NanoguiAnnotationDialog* _annotationDialog;
            NanoguiGranuleDialog* _granuleDialog;
            std::function<void()> _closeCallback;
            RenderableId _renderableId;
            RenderableType _renderableType;
    };
};

#endif
