// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_CORNER_ANCHOR_H
#define NANOGUI_CORNER_ANCHOR_H

#include <cstdint>
#include <functional>
#include <vector>
#include "vendors/_nanogui.h"

namespace SEAScope
{
	enum class NanoguiCorner : std::uint8_t {
		TOP_LEFT = 0,
		TOP_RIGHT = 1,
		BOTTOM_LEFT = 2,
		BOTTOM_RIGHT= 3,
	};

	class NanoguiCornerAnchor final: public virtual nanogui::Widget
	{
		public:
			NanoguiCornerAnchor(nanogui::Widget*);
			~NanoguiCornerAnchor() override;

			void draw(NVGcontext*) override;
			bool mouseButtonEvent(const Eigen::Vector2i&, int, bool, int) override;

			void setSize(const Eigen::Vector2i&);
			void setFixedSize(const Eigen::Vector2i&);

			bool setCorner(const NanoguiCorner&);
			bool update();
			bool updateRadii();
			bool updatePosition();
			bool nextCorner();

			NanoguiCorner corner;
			std::uint16_t topLeftRadius;
			std::uint16_t topRightRadius;
			std::uint16_t bottomLeftRadius;
			std::uint16_t bottomRightRadius;
			std::vector<NanoguiCorner> validCorners;

			std::function<void(int, int, int, int)> updateCallback;
			std::function<void()> clickCallback;
	};
};

#endif

