// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_DATEBOX_H
#define NANOGUI_DATEBOX_H

#include <cstdint>
#include <array>
#include <vector>
#include <nanogui/button.h>
#include "ui/nanogui/nanogui_textfield.h"
#include "ui/nanogui/nanogui_anchored.h"

namespace SEAScope
{
	class NanoguiDateBox: public virtual NanoguiTextField
	{
		public:
			NanoguiDateBox(nanogui::Widget*, std::uint16_t year, std::uint8_t month, std::uint8_t day);
			virtual ~NanoguiDateBox() override;

			enum class HelperType : std::uint8_t {
				NONE = 0,
				YEARS = 1,
				MONTHS = 2,
				DAYS = 3
			};

			bool focusEvent(bool) override;
			bool keyboardEvent(int,int,int,int) override;
			bool keyboardCharacterEvent(unsigned int codepoint) override;
			bool mouseButtonEvent(const Eigen::Vector2i&,int, bool,int) override;
            void draw(NVGcontext*) override;
			virtual void setValue(const std::string&);

			virtual bool checkValidity(const std::string& input);
			bool checkDate(const std::string& input);
			std::size_t getHelperButtons(std::size_t, std::vector<nanogui::Button*>&);
			bool updateHelpers(std::size_t pos, bool visible);

			bool setYearsGrabber(const std::function<bool(std::vector<std::uint16_t>&, std::vector<std::uint16_t>&)>&);
			bool setMonthsGrabber(const std::function<bool(std::vector<std::uint8_t>&, std::vector<std::uint8_t>&, std::uint16_t)>&);
			bool setDaysGrabber(const std::function<bool(std::vector<std::uint8_t>&, std::vector<std::uint8_t>&, std::uint16_t, std::uint8_t)>&);

			bool displayYears();
			bool displayMonths();
			bool displayDays();
			bool showHelpers(HelperType);

			bool setYear(std::uint16_t year, bool commit = true);
			bool setMonth(std::uint8_t month, bool commit = true);
			bool setDay(std::uint8_t day, bool commit = true);

			bool getYear(std::uint16_t& year);
			bool getMonth(std::uint8_t& month);
			bool getDay(std::uint8_t& day);

			bool commitValue(const std::string&);
			bool commitValue();

			static std::array<std::uint8_t, 12> daysPerMonth;
			static std::uint8_t INVALID_MONTH;
			static std::uint8_t INVALID_DAY;

			NanoguiAnchored* helper;
			bool mouseOverHelper = false;

			std::function<bool(std::vector<std::uint16_t>&, std::vector<std::uint16_t>&)> grabAvailableYears;
			std::function<bool(std::vector<std::uint8_t>&, std::vector<std::uint8_t>&, std::uint16_t)> grabAvailableMonths;
			std::function<bool(std::vector<std::uint8_t>&, std::vector<std::uint8_t>&, std::uint16_t, std::uint8_t)> grabAvailableDays;
			std::vector<std::uint16_t> availableYears;
			std::vector<std::uint8_t> availableMonths;
			std::vector<std::uint8_t> availableDays;
			std::vector<std::uint16_t> hlYears;
			std::vector<std::uint8_t> hlMonths;
			std::vector<std::uint8_t> hlDays;

			std::uint16_t yearTemp;
			std::uint8_t monthTemp;
			std::uint8_t dayTemp;

			std::string defaultYearCompletion = "-01-01";
			std::string defaultMonthCompletion = "-01";
			std::string defaultDayCompletion = "";
	};
};

#endif
