// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_DAYS_LIST_H
#define NANOGUI_DAYS_LIST_H

#include <array>
#include <functional>
#include <nanogui/widget.h>

#include "core/types.h"
#include "ui/nanogui/nanogui_flat_button.h"
#include "data/events_coverage.h"

namespace SEAScope
{
	class NanoguiDaysList final: public virtual nanogui::Widget
	{
		public:
			NanoguiDaysList(nanogui::Widget*);
			~NanoguiDaysList() override;

			void setSize(const Eigen::Vector2i&);
			void setWidth(int);
			void setFixedSize(const Eigen::Vector2i&);
			void setFixedWidth(int);
			bool scrollEvent(const Eigen::Vector2i&, const Eigen::Vector2f&) override;
			void draw(NVGcontext* ctx) override;

			bool selectDay(std::uint8_t);
			bool setMonth(std::uint16_t, std::uint8_t);
			bool setMainCoverage(const EventsCoverage&);
			bool setExtraCoverage(const EventsCoverage&);
			bool update();
			bool setVisibilityBounds(timestamp_t, timestamp_t);
			bool resize(int);

			std::uint16_t year;
			std::uint8_t month;
			std::array<NanoguiFlatButton*, 31> buttons;
			std::array<nanogui::Color, 31> backgrounds;
			std::array<nanogui::Color, 31> textColors;
			std::function<void(std::uint8_t)> dayCallback;
			std::function<void(float)> scrollCallback;
			std::uint8_t daysInMonth;
			timestamp_t pastOffset;
			timestamp_t futureOffset;
            bool hoverEnabled;
			EventsCoverage mainCoverage;
			EventsCoverage extraCoverage;
	};
};

#endif
