// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_DICT_FIELD_EDITOR_H
#define NANOGUI_DICT_FIELD_EDITOR_H

#include <string>
#include <functional>

#include "vendors/_nanogui.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiAddPropertyForm;

    class NanoguiDictFieldEditor: public virtual nanogui::Widget
    {
        public:
            NanoguiDictFieldEditor(nanogui::Widget*);
            virtual ~NanoguiDictFieldEditor() override;

            void draw(NVGcontext*) override;

            void setName(const std::string&);
            const std::string& getName() const;

            void setCategory(const std::string&);
            const std::string& getCategory() const;

            void setBackgroundColor(const nanogui::Color&);
            const nanogui::Color& getBackgroundColor() const;

            bool activateEdition(bool);

            bool setDeleteCallback(const std::function<void(const std::string&, const std::string&)>&);
            bool setChildUnicityCallback(const std::function<bool(const std::string&, const std::string&)>&);
            bool setAddPropertyOkCallback(const std::function<void(const std::string&, const std::string&, const std::string&)>&);
            bool setLayoutChangeCallback(const std::function<void()>&);

        protected:
            bool checkChildUnicity(const std::string&);

            nanogui::Color _backgroundColor;
            nanogui::Label* _nameLabel;
            //nanogui::Button* _addButton;
            nanogui::Button* _deleteButton;
            NanoguiAddPropertyForm* _addPropertyForm;

            std::function<void(const std::string&, const std::string&)> _deleteCallback;
            std::function<bool(const std::string&, const std::string&)> _childUnicityCallback;
            std::function<void(const std::string&, const std::string&, const std::string&)> _addPropertyOkCallback;
            std::function<void()> _layoutChangeCallback;

            std::string _category;
            bool _editing;
            bool _reflowNeeded;
    };
}

#endif
