// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_DRAGGABLE_PANE_H
#define NANOGUI_DRAGGABLE_PANE_H

#include <cstdint>
#include <functional>
#include <nanogui/widget.h>
#include <nanogui/opengl.h>

namespace SEAScope
{
	class NanoguiDraggablePane final: public virtual nanogui::Widget
	{
		public:
			NanoguiDraggablePane(nanogui::Widget*);
			~NanoguiDraggablePane() override;

			bool display(const Eigen::Vector2i&);
			bool hide();
            bool updateLayout();

			void draw(NVGcontext*) override;
			bool mouseButtonEvent(const Eigen::Vector2i&,int,bool,int) override;
            bool mouseMotionEvent(const Eigen::Vector2i&, const Eigen::Vector2i&, int, int) override;
            bool keyboardEvent(int,int,int,int) override;
            bool keyboardCharacterEvent(unsigned int) override;
            bool scrollEvent(const Eigen::Vector2i&, const Eigen::Vector2f&) override;

            bool reflowNeeded;
            bool dragging;
            nanogui::Color backgroundColor;
            nanogui::Color headerBackgroundColor;
            std::uint32_t headerHeight;
            std::function<bool(int,int,int,int)> keyCallback;
            std::function<bool(unsigned int)> charCallback;
	};
};

#endif
