// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_EXTRACTION_DIALOG_H
#define NANOGUI_EXTRACTION_DIALOG_H

#include <map>
#include <vector>
#include <functional>
#include <nanogui/opengl.h>
#include <nanogui/widget.h>
#include <nanogui/button.h>

#include "core/structs.h"
#include "ui/nanogui/nanogui_scrollable_with_title.h"

namespace SEAScope
{
	class NanoguiExtractionDialog final: public virtual NanoguiScrollableWithTitle
	{
		public:
			NanoguiExtractionDialog(nanogui::Widget*);
			~NanoguiExtractionDialog() override;

			bool setExtraction(const std::map<RenderableId, Extraction>&, const std::map<RenderableId, std::string>&, bool);

            nanogui::Button* pngButton;
            std::function<void()> pngCallback;

            nanogui::Button* pickleButton;
            std::function<void()> pickleCallback;

            void showProcessingOptions();
            void hideProcessingOptions();

        protected:
			nanogui::Widget* _processorOptions;
            nanogui::Widget* _extractedItems;
	};
};

#endif
