// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_GRANULE_DIALOG_H
#define NANOGUI_GRANULE_DIALOG_H

#include <nanogui/widget.h>
#include <nanogui/label.h>
#include <nanogui/opengl.h>

#include "core/structs.h"
#include "core/granule_info.h"
#include "ui/nanogui/nanogui_attribute.h"

namespace SEAScope
{
    // Forward declarations
    class GranuleInfo;

    class NanoguiGranuleDialog final: public virtual nanogui::Widget
    {
        public:
            NanoguiGranuleDialog(nanogui::Widget*);
            ~NanoguiGranuleDialog() override;

            bool enable(const RenderableId&, const GranuleInfo&);
            bool disable();

            void draw(NVGcontext*) override;

            nanogui::Widget* btns;
            NanoguiAttribute* collectionAttr;
            NanoguiAttribute* granuleAttr;
            NanoguiAttribute* variableAttr;
			NanoguiAttribute* timeCoverageAttr;
			NanoguiAttribute* spatialCoverageAttr;
			std::function<void(const RenderableId&)> infoCallback;
            std::function<void(const RenderableId&)> centerTimeCallback;
            std::function<void(const RenderableId&)> extractCallback;
			RenderableId target;
    };
};

#endif
