// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_GRANULE_SHOWCASE_H
#define NANOGUI_GRANULE_SHOWCASE_H

#include <functional>

#include <nanogui/widget.h>
#include <nanogui/label.h>

#include "core/structs.h"
#include "ui/nanogui/nanogui_attribute.h"
#include "ui/nanogui/nanogui_scrollable_with_title.h"

namespace SEAScope
{
    // Forward declarations
    class GranuleInfo;

	class NanoguiGranuleShowcase final: public virtual NanoguiScrollableWithTitle
	{
		public:
			NanoguiGranuleShowcase(nanogui::Widget* parent);
			~NanoguiGranuleShowcase() override;

			void draw(NVGcontext*) override;

			nanogui::Label* addAttribute(nanogui::Widget*, const std::string&);
			bool displayInfo(const GranuleInfo&);

			nanogui::Label* uri;
			nanogui::Label* collectionLbl;
			nanogui::Widget* collectionTags;
			nanogui::Label* variableLbl;
			nanogui::Label* variableUnitsLbl;
			nanogui::Widget* variableTags;
			NanoguiAttribute* dataId;
			NanoguiAttribute* dataModel;
			NanoguiAttribute* start;
			NanoguiAttribute* stop;
			NanoguiAttribute* granuleName;
			NanoguiAttribute* institution;
			NanoguiAttribute* comment;
			NanoguiAttribute* fileId;
			NanoguiAttribute* productVersion;
			NanoguiAttribute* latMin;
			NanoguiAttribute* latMax;
			NanoguiAttribute* lonMin;
			NanoguiAttribute* lonMax;
			NanoguiAttribute* creatorEmail;
			NanoguiAttribute* station;
			NanoguiAttribute* platform;
			NanoguiAttribute* sensor;
			nanogui::Widget* resContainer;
			nanogui::Widget* datesContainer;
			std::function<void(std::uint16_t, std::uint8_t)> monthCallback;
			std::function<void(std::uint16_t)> yearCallback;
	};
};

#endif
