// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_HCOLORBAR_H
#define NANOGUI_HCOLORBAR_H

#include <cstdint>
#include <array>
#include <functional>
#include <nanogui/widget.h>
#include <nanogui/layout.h>
#include <nanogui/opengl.h>
#include "core/types.h"

namespace SEAScope
{
	class NanoguiHColorbar final: public virtual nanogui::Widget
	{
		public:
			NanoguiHColorbar(nanogui::Widget*);
			~NanoguiHColorbar() override;

			Eigen::Vector2i preferredSize(NVGcontext*) const override;
			void draw(NVGcontext*) override;
			bool mouseButtonEvent(const Eigen::Vector2i&,int,bool,int) override;
			bool mouseDragEvent(const Eigen::Vector2i &, const Eigen::Vector2i &,int, int) override;
			bool mouseMotionEvent(const Eigen::Vector2i &, const Eigen::Vector2i &, int, int) override;

			void drawColormap(NVGcontext*);
			bool isMouseOver() const;
			bool setClickCallback(const std::function<void(const std::string&, const Colormap&, int, float, bool)>&);
			bool setMouseMoveCallback(const std::function<void(bool, float)>&);
			bool setDragEndCallback(const std::function<void(bool, float)>&);
			bool setColormap(const Colormap&);
			bool setColor(const std::array<std::uint8_t, 3>&);

            void setSize(const Eigen::Vector2i&);
            void setFixedSize(const Eigen::Vector2i&);

			Colormap cmap;
			std::array<std::uint8_t, 3> color;
			nanogui::Orientation orientation;
			std::string label;
			std::function<void(const std::string&, const Colormap&, int, float, bool)> clickCallback = nullptr;
			std::function<void(bool, float)> mouseMoveCallback = nullptr;
			std::function<void(bool, float)> dragEndCallback = nullptr;
			bool dragging = false;
			bool dragTargetIsMax = false;
            int textureId;
            bool textureNeeded;
	};
};

#endif
