// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_IMAGE_FIELD_EDITOR_H
#define NANOGUI_IMAGE_FIELD_EDITOR_H

#include <string>
#include <functional>

#include "vendors/_nanogui.h"
#include "data/base64_image.h"

class Image;

namespace SEAScope
{
    class NanoguiImageFieldEditor: public virtual nanogui::Widget
    {
        public:
            NanoguiImageFieldEditor(nanogui::Widget*);
            virtual ~NanoguiImageFieldEditor() override;

            void draw(NVGcontext*) override;

            void setName(const std::string&);
            const std::string& getName() const;

            void setCategory(const std::string&);
            const std::string& getCategory() const;

            void setValue(const std::string&);
            const std::string& getValue() const;

            void setOriginalValue(const std::string&);
            const std::string& getOriginalValue() const;

            void setBackgroundColor(const nanogui::Color&);
            const nanogui::Color& getBackgroundColor() const;

            bool activateEdition(bool);

            bool setDeleteCallback(const std::function<void(const std::string&, const std::string&)>&);
            bool setResetCallback(const std::function<void()>&);

            bool setViewImageCallback(const std::function<void(const Image&)>&);
            bool setLoadImageCallback(const std::function<void()>&);
            const Image& loadFromFile(const std::string&);
            const Image& getConstImage() const;

        protected:
            nanogui::Color _backgroundColor;
            nanogui::Label* _nameLabel;
            nanogui::Button* _resetButton;
            nanogui::Button* _loadImageButton;
            nanogui::Button* _deleteButton;
            Base64Image _base64Image;

            std::function<void(const Image&)> _viewImageCallback;
            std::function<void()> _loadImageCallback;
            std::function<void(const std::string&, const std::string&)> _deleteCallback;
            std::function<void()> _resetCallback;

            std::string _category;
            std::string _value;
            std::string _originalValue;
            bool _editing;
            bool _reflowNeeded;
    };
}

#endif
