// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_MAIN_MENU_H
#define NANOGUI_MAIN_MENU_H

#include <nanogui/widget.h>
#include <nanogui/opengl.h>

namespace SEAScope
{
    // Forward declarations
    class NanoguiAboutPane;
    class NanoguiGranuleShowcase;
    class NanoguiExtractionDialog;
    class NanoguiSourcesControl;
    class NanoguiCollectionsControl;
    class NanoguiAppSettingsEditor;

	class NanoguiMainMenu final: public virtual nanogui::Widget
	{
		public:
			NanoguiMainMenu(nanogui::Widget*);
			~NanoguiMainMenu() override;

			void draw(NVGcontext*) override;
			bool scrollEvent(const Eigen::Vector2i&, const Eigen::Vector2f&) override;
			void update();

            NanoguiAboutPane* getAboutPane() const;
            NanoguiAppSettingsEditor* getSettingsPane() const;
            NanoguiGranuleShowcase* getGranuleInfo() const;
            NanoguiExtractionDialog* getExtractionDialog() const;
            NanoguiSourcesControl* getSourcesControl() const;
            NanoguiCollectionsControl* getCollectionsControl() const;

        protected:
            NanoguiAboutPane* _aboutPane;
            NanoguiAppSettingsEditor* _settingsPane;
			NanoguiGranuleShowcase* _granuleInfo;
			NanoguiExtractionDialog* _extractionDialog;
			NanoguiSourcesControl* _sourcesControl;
			NanoguiCollectionsControl* _collectionsControl;
			bool _reflowNeeded;
	};
};

#endif
