// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_NUMBERBOX_H
#define NANOGUI_NUMBERBOX_H

#include <cstdint>

#include "vendors/_nanogui.h"
#include "ui/nanogui/nanogui_textfield.h"

namespace SEAScope
{
	class NanoguiNumberBox final: public virtual NanoguiTextField
	{
		public:
			NanoguiNumberBox(nanogui::Widget*);
			~NanoguiNumberBox() override;

			void draw(NVGcontext*) override;
			bool scrollEvent(const Eigen::Vector2i &p, const Eigen::Vector2f &rel) override;
            bool keyboardEvent(int key, int scancode, int action, int modifiers) override;
            bool focusEvent(bool focused) override;

            double value() const;
            void setValue(double);

            void setCallback(const std::function<void(double)>&);

            bool setDelta(double);
            double getDelta() const;

            bool setRequiresKeyboardValidation(bool);
            bool getRequiresKeyboardValidation() const;

            bool setValidationFramesDelay(std::uint8_t);
            std::uint8_t getValidationFramesDelay() const;

            bool setValidMin(double);
            bool setValidMax(double);

        protected:
			double _delta;
            bool _requiresKeyboardValidation;
            bool _hasPendingChanges;
            bool _useDelayedCommit;
            std::uint8_t _validationFramesDelay;    // up to 256 frames <=> 4.266 seconds at 60 fps
            std::uint8_t _elapsedFrames;
            double _validMin;
            double _validMax;
	};
};

#endif
