// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_PATH_SELECTOR_H
#define NANOGUI_PATH_SELECTOR_H

#include <string>
#include <vector>
#include <functional>

#include "vendors/_nanogui.h"
#include "ui/idrop_target.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiList;
    class NanoguiAnchored;
    class NanoguiTextField;
    class NanoguiClickableLabel;

    enum class PathSelectorType
    {
        get_file,
        save_file,
        get_folder
    };

    class NanoguiPathSelector : public virtual nanogui::Widget, public virtual IDropTarget
    {
    public:
        NanoguiPathSelector(nanogui::Widget*, PathSelectorType type);
        virtual ~NanoguiPathSelector() override;

        void draw(NVGcontext*) override;

        bool dropEvent(const std::vector<std::string>&) override;

        void resize(const Eigen::Vector2i&);

        void enable();
        void disable();

        void show();
        void hide();

        void setPlaceholder(const std::string&);

        const std::string& getPath() const;
        bool setPath(const std::string&);
        bool setPath(const std::string&, bool);

        void toggleDropDown();
        void hideDropDown();
        void showDropDown();

        void updateExplorer(const std::string&);

        void hintValidPath();
        void hintInvalidPath();

        bool setChangeCallback(const std::function<void(const std::string&)>&);
        bool setDropDownCallback(const std::function<void()>&);

        void setType(PathSelectorType type);

    protected:
        std::string _parentPath;
        std::string _exploredPath;
        NanoguiTextField* _inputText;
        NanoguiAnchored* _dropdownContainer;
        NanoguiList* _itemsList;
        nanogui::Button* _browseButton;
        nanogui::Button* _selectButton;
        nanogui::Button* _parentDirButton;
        std::vector<NanoguiClickableLabel*> _widgets;
        std::function<void(const std::string&)> _changeCallback;
        std::function<void()> _dropdownCallback;
        bool _updateRequested;
        std::string _selected;
        PathSelectorType _type;
    };
}

#endif
