// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_PROPERTIES_EDITOR_H
#define NANOGUI_PROPERTIES_EDITOR_H

#include <unordered_map>
#include <string>
#include <vector>
#include <functional>

#include "vendors/_nanogui.h"
#include "core/data_node.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiAddPropertyForm;
    class NanoguiBoolFieldEditor;
    class NanoguiNumFieldEditor;
    class NanoguiTextFieldEditor;
    class NanoguiImageFieldEditor;
    class NanoguiDictFieldEditor;
    class NanoguiPathSelector;
    class NanoguiImageView;
    class NanoguiAnchored;
    class NanoguiList;

    class NanoguiPropertiesEditor: public virtual nanogui::Widget
    {
        public:
            NanoguiPropertiesEditor(nanogui::Widget*);
            virtual ~NanoguiPropertiesEditor() override;

            void draw(NVGcontext*) override;

            void activateEdition(bool);
            void update();
            void resize(const Eigen::Vector2i&);
            void setProperties(DictNode&&);
            bool getProperties(DictNode&);
            void hideImageViewer();
            void setPreventImageViewer(bool);

            bool addProperty(const std::string&, const std::string&, const std::string&);
            bool checkPropertyUnicity(const std::string&, const std::string&);
            bool setSaveCallback(const std::function<void(const DictNode&)>&);

        protected:
            void initImageViewer();
            void updateSubEditors(const DictNode*, std::size_t&, std::size_t&, std::size_t&, std::size_t&, std::size_t&, const std::string&);
            bool widgetsSorter(const nanogui::Widget*, const nanogui::Widget*) const;
            std::size_t getFieldIndex(const std::string&, const std::string&) const;

            NanoguiBoolFieldEditor* getBoolEditor(std::size_t, const std::string&);
            NanoguiNumFieldEditor* getNumEditor(std::size_t, const std::string&);
            NanoguiTextFieldEditor* getTextEditor(std::size_t, const std::string&);
            NanoguiImageFieldEditor* getImageEditor(std::size_t, const std::string&);
            NanoguiDictFieldEditor* getDictEditor(std::size_t, const std::string&);

            DictNode _properties;

            nanogui::Button* _editButton;
            nanogui::Button* _saveButton;
            nanogui::Button* _cancelButton;
            NanoguiList* _itemsList;

            std::vector<NanoguiBoolFieldEditor*> _boolEditors;
            std::vector<NanoguiNumFieldEditor*> _numEditors;
            std::vector<NanoguiTextFieldEditor*> _textEditors;
            std::vector<NanoguiImageFieldEditor*> _imageEditors;
            std::vector<NanoguiDictFieldEditor*> _dictEditors;

            std::unordered_map<const nanogui::Widget*, std::uint32_t> _sortedWidgets;
            std::unordered_map<const nanogui::Widget*, bool> _widgetIsContainer;

            nanogui::Widget* _pathContainer;
            nanogui::Label* _pathField;
            NanoguiPathSelector* _pathSelector;
            nanogui::Button* _pathOkButton;
            nanogui::Button* _pathNokButton;

            NanoguiImageFieldEditor* _pathSelectorTarget;

            NanoguiAnchored* _imageContainer;
            NanoguiImageView* _imageView;

            NanoguiAddPropertyForm* _addPropertyForm;

            bool _reflowNeeded;
            bool _preventImageViewer;

            std::function<void(const DictNode&)> _saveCallback;
    };
}

#endif
