// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_RENDERER_H
#define NANOGUI_RENDERER_H

#include <vector>
#include <string>

#include "vendors/_nanogui.h"

#include "core/interfaces/irenderer.h"
#include "core/eventhandler.h"
#include "ui/nanogui/nanogui_icons.h"

namespace SEAScope
{
	/**
	 *
	 */
	class NanoguiRenderer final: public virtual nanogui::Screen, public virtual IRenderer, public virtual EventHandler
	{
		public:
			NanoguiRenderer();
			~NanoguiRenderer() override;

			bool setWindow(IWindow&) override;
			bool renderFrame() override;
            void setIcons(const Icons& icons);
			NanoguiIcons& getIcons();

			bool handleKeyEvent(int,int,int,int) override;
			bool handleCharEvent(unsigned int, int) override;
			bool handleMouseButtonEvent(int, int, int) override;
			bool handleMouseMoveEvent(double, double) override;
			bool handleMouseWheelEvent(double,double) override;
			bool handleDropEvent(int,const char**) override;
			bool handleResizeEvent(int,int, int, int) override;

            bool dropEvent(const std::vector<std::string>&) override;

            void drawWidgetsWithCustomizedTooltips();

			void setResizeCallback(const std::function<void(int,int)>& callback){this->resizeCallback=callback;}
			std::function<void(int,int)> resizeCallback = nullptr;

	private:
		NanoguiIcons _icons;
	};
};

#endif
