// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_RENDERING_CONTROL_H
#define NANOGUI_RENDERING_CONTROL_H

#include <vector>
#include <map>

#include <nanogui/widget.h>
#include <nanogui/opengl.h>

#include "core/types.h"
#include "ui/irendering_control_view.h"
#include "ui/nanogui/nanogui_list.h"
#include "ui/nanogui/nanogui_sorted_list.h"

namespace SEAScope
{
	class NanoguiRenderingControl final: public virtual nanogui::Widget, public IRenderingControlView
	{
		public:
			NanoguiRenderingControl(nanogui::Widget*);
			~NanoguiRenderingControl() override;

			void draw(NVGcontext*) override;

			IRenderingWidget* createRendering() override;
			bool addRendering(IRenderingWidget*) override;
			bool removeRendering(IRenderingWidget*) override;
			bool setOrderCallback(const std::function<bool(IRenderingWidget*,IRenderingWidget*,IRenderingWidget*)>&) override;
			bool setOpacityCallback(const std::function<bool(IRenderingWidget*, float)>&) override;
			bool setVisibilityCallback(const std::function<bool(IRenderingWidget*, bool, bool)>&) override;
			bool update() override;
			bool setMinCallback(const std::function<void(IRenderingWidget*, float)>&) override;
			bool setMaxCallback(const std::function<void(IRenderingWidget*, float)>&) override;

			static bool compareRenderers(const nanogui::Widget*, const nanogui::Widget*);

            bool reflowNeeded;
			NanoguiList* scroller;
			NanoguiSortedList* renderings;
			std::function<bool(IRenderingWidget*, IRenderingWidget*, IRenderingWidget*)> orderCallback;
			std::function<bool(IRenderingWidget*, float)> opacityCallback;
			std::function<bool(IRenderingWidget*, bool, bool)> visibilityCallback;
			std::function<void(IRenderingWidget*, float)> minCallback;
			std::function<void(IRenderingWidget*, float)> maxCallback;
	};
};

#endif
