// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_RENDERING_WIDGET_H
#define NANOGUI_RENDERING_WIDGET_H

#include <functional>
#include <nanogui/widget.h>
#include <nanogui/opengl.h>
#include <nanogui/label.h>
#include <nanogui/button.h>

#include "core/rendering_cfg.h"
#include "ui/irendering_widget.h"
#include "ui/nanogui/nanogui_gauge.h"
#include "ui/nanogui/nanogui_hcolorbar.h"
#include "ui/nanogui/nanogui_num_label.h"
#include "ui/nanogui/nanogui_fixed_size_label.h"
#include "ui/nanogui/nanogui_checkbox.h"

namespace SEAScope
{
	class NanoguiRenderingWidget final: public virtual IRenderingWidget, public virtual nanogui::Widget
	{
		public:
			NanoguiRenderingWidget(nanogui::Widget*);
			~NanoguiRenderingWidget() override;

			void draw(NVGcontext* ctx) override;

			bool update(RenderingCfg*) override;
			bool update(const std::string&, const std::string&, RenderingCfg*) override;
			bool setOpacityCallback(const std::function<bool(IRenderingWidget*, float)>&) override;
			bool setVisibilityCallback(const std::function<void(IRenderingWidget*, bool, bool)>&) override;
			bool setSettingsCallback(const std::function<bool(IRenderingWidget*)>&) override;
			bool setColormapGetter(const std::function<bool(const std::string&, Colormap&)>&) override;
			bool setMinCallback(const std::function<void(IRenderingWidget*, float)>&) override;
			bool setMaxCallback(const std::function<void(IRenderingWidget*, float)>&) override;
			bool setCounter(uint32_t count) override;
			bool setSettingsState(bool) override;
            bool setOpacityScrollStep(float) override;

			bool reflowNeeded;
			float zindex;
			NanoguiFixedSizeLabel* label;
			NanoguiNumLabel* minLabel;
			NanoguiNumLabel* maxLabel;
			NanoguiHColorbar* colormap;
			nanogui::Label* logLabel;
            nanogui::Label* unitsLabel;
			nanogui::Label* countLabel;
			NanoguiGauge* opacityGauge;
			nanogui::Label* opacityLabel;
			NanoguiCheckbox* visibilityCb;
			nanogui::Button* settingsButton;
            nanogui::Widget* _leftSpacer;
			std::function<bool(IRenderingWidget*, float)> opacityCallback;
			std::function<void(IRenderingWidget*, bool, bool)> visibilityCallback;
            std::function<bool(const std::string&, Colormap&)> cmapGetter;
			std::function<bool(IRenderingWidget*)> settingsCallback;
			std::function<void(IRenderingWidget*, float)> minCallback;
			std::function<void(IRenderingWidget*, float)> maxCallback;
            bool expanded;
	};
};

#endif
