// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUi_SEASCOPE_SIDE_MENU_H
#define NANOGUi_SEASCOPE_SIDE_MENU_H

#include <vector>

#include "vendors/_nanogui.h"
#include "ui/nanogui/nanogui_side_menu.h"

#include "core/app_events_dispatcher.h"

#include "ui/nanogui/nanogui_side_menu.h"
#include "ui/nanogui/nanogui_main_menu.h"
#include "ui/nanogui/nanogui_about_pane.h"
#include "ui/nanogui/nanogui_granule_showcase.h"
#include "ui/nanogui/nanogui_extraction_dialog.h"
#include "ui/nanogui/nanogui_collections_control.h"
#include "ui/nanogui/nanogui_app_settings_editor.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiMainMenu;

    class NanoguiSEAScopeSideMenu final: public virtual NanoguiSideMenu
    {
        public:
            NanoguiSEAScopeSideMenu(nanogui::Widget*, AppEventsDispatcher&);
            virtual ~NanoguiSEAScopeSideMenu() override;

            void resize(int width, int height);
            void displayGranuleInfo(const GranuleInfo& granuleInfo);
            void displayCatalogue();
            void displayExtraction(const std::map<RenderableId, Extraction>& extractionData,
                                   const std::map<RenderableId, std::string>& labels);
            NanoguiMainMenu* getMainMenu() const;

        protected:
            NanoguiMainMenu* _mainMenu;
            nanogui::Button* _catalogueButton;
            nanogui::Button* _granuleInfoButton;
            nanogui::Button* _extractedDataButton;
            nanogui::Button* _settingsButton;
            nanogui::Button* _aboutButton;
            std::vector<nanogui::Button*> _tabButtons;
    };
}

#endif
