// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_SORTED_LIST_H
#define NANOGUI_SORTED_LIST_H

#include <tuple>
#include <functional>
#include <nanogui/widget.h>
#include <nanogui/opengl.h>
#include "core/types.h"

namespace SEAScope
{
	class NanoguiSortedList final: public virtual nanogui::Widget
	{
		public:
			NanoguiSortedList(nanogui::Widget*);
			~NanoguiSortedList() override;

			void draw(NVGcontext*) override;
			bool mouseButtonEvent(const Eigen::Vector2i&, int, bool, int) override;
			bool mouseMotionEvent(const Eigen::Vector2i&, const Eigen::Vector2i&, int, int) override;
			bool mouseEnterEvent(const Eigen::Vector2i&, bool) override;

			bool update();
			bool sortWidgets();
			std::tuple<nanogui::Widget*, int> childAt(const Eigen::Vector2i&) const;
			bool mouseUpEvent(const Eigen::Vector2i&, int, int);
			bool mouseDownEvent(const Eigen::Vector2i&, int, int);

            void setBackgroundFillColor(nanogui::Color);
            void setBackgroundStrokeColor(nanogui::Color);

            bool reflowNeeded;
			std::function<bool(nanogui::Widget*, nanogui::Widget*, nanogui::Widget*)> dragCallback;
			std::function<bool(const nanogui::Widget*, const nanogui::Widget*)> widgetsSorter;
			nanogui::Widget* draggedChild;
			nanogui::Widget* hoveredChild;
			Eigen::Vector2i draggedOffset;
			Eigen::Vector2i cursorPosition;
			int draggedIndex;
			int hoveredIndex;

        protected:
            nanogui::Color _backgroundFillColor;
            nanogui::Color _backgroundStrokeColor;
	};
};

#endif
