// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_SOURCES_CONTROL_H
#define NANOGUI_SOURCES_CONTROL_H

#include <map>

#include <nanogui/widget.h>
#include <nanogui/opengl.h>
#include <nanogui/textbox.h>
#include <nanogui/button.h>

#include "ui/isources_control_view.h"
#include "ui/nanogui/nanogui_list.h"
#include "ui/nanogui/nanogui_source_widget.h"

namespace SEAScope
{
	class NanoguiSourcesControl final: public virtual nanogui::Widget, public virtual ISourcesControlView
	{
		public:
			NanoguiSourcesControl(nanogui::Widget*);
			~NanoguiSourcesControl() override;

			ISourceWidget* createSource() override;
			bool addSource(ISourceWidget*) override;
			bool removeSource(ISourceWidget*) override;
			bool setAddCallback(const std::function<void(const std::string&)>&) override;
			bool setInfoCallback(const std::function<void(const std::string&)>&) override;
			bool setSelectionCallback(const std::function<void(const std::string&, bool)>&) override;

			void draw(NVGcontext*) override;

			void setFixedSize(const Eigen::Vector2i&);

			static bool compareSources(const nanogui::Widget*, const nanogui::Widget*);

			bool reflowNeeded;
			nanogui::TextBox* uriField;
			nanogui::Button* addButton;
			NanoguiList* list;
			std::function<void(const std::string&)> addCallback;
			std::function<void(const std::string&)> infoCallback;
			std::function<void(const std::string&, bool)> selectionCallback;
	};
};

#endif
