// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_TEXT_FIELD_EDITOR_H
#define NANOGUI_TEXT_FIELD_EDITOR_H

#include <string>
#include <functional>

#include "vendors/_nanogui.h"


namespace SEAScope
{
    // Forward declarations
    class NanoguiTextField;

    class NanoguiTextFieldEditor: public virtual nanogui::Widget
    {
        public:
            NanoguiTextFieldEditor(nanogui::Widget*);
            virtual ~NanoguiTextFieldEditor() override;

            void draw(NVGcontext*) override;

            void setName(const std::string&);
            const std::string& getName() const;

            void setCategory(const std::string&);
            const std::string& getCategory() const;

            void setValue(const std::string&);
            const std::string& getValue() const;

            void setOriginalValue(const std::string&);
            const std::string& getOriginalValue() const;

            void setBackgroundColor(const nanogui::Color&);
            const nanogui::Color& getBackgroundColor() const;

            bool activateEdition(bool);

            bool setDeleteCallback(const std::function<void(const std::string&, const std::string&)>&);

        protected:
            nanogui::Color _backgroundColor;
            nanogui::Label* _nameLabel;
            NanoguiTextField* _valueTextField;
            nanogui::Label* _valueLabel;
            nanogui::Button* _resetButton;
            nanogui::Button* _deleteButton;

            std::function<void(const std::string&, const std::string&)> _deleteCallback;

            std::string _category;
            std::string _originalValue;
            bool _editing;
            bool _reflowNeeded;
    };
}

#endif
