// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_TEXTURED_BUTTON_H
#define NANOGUI_TEXTURED_BUTTON_H

#include "vendors/_nanogui.h"

namespace SEAScope
{

class NanoguiTexturedButton : public nanogui::Widget
{
public:
    NanoguiTexturedButton(nanogui::Widget* parent);
    ~NanoguiTexturedButton() override;

    /// Compute the preferred size of the widget
    nanogui::Vector2i preferredSize(NVGcontext* ctx) const override;

    /// Draw the widget (and all child widgets)
    void draw(NVGcontext* ctx) override;

    void setTheme(nanogui::Theme* theme) override;
    void setIcon(int);

    bool mouseButtonEvent(const nanogui::Vector2i& p, int button, bool down, int modifiers) override;

    void setCallback(const std::function<void()>& func);

private:
    int _icon;

    /// Whether or not this Button is currently pushed.
    bool mPushed;

    /// The background color of this Button.
    nanogui::Color mBackgroundColor;

    /// The callback issued for all types of buttons.
    std::function<void()> _callback;
};

} // namespace NanoguiTexturedButton

#endif // NANOGUI_TEXTURED_BUTTON_H
