// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_TIMELINE_H
#define NANOGUI_TIMELINE_H

#include <vector>
#include <string>
#include "data/events_coverage.h"
#include "ui/itimeline_view.h"
#include "ui/nanogui/nanogui_component.h"
#include "ui/nanogui/nanogui_datetimebox.h"
#include "ui/nanogui/nanogui_button_bar.h"
#include "ui/nanogui/nanogui_calendars_timeline.h"
#include "ui/nanogui/nanogui_days_list.h"
#include "ui/nanogui/nanogui_months_list.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiTimestepSettings;

	class NanoguiTimeline final: public virtual NanoguiComponent, public virtual ITimelineView
	{
		public:
			NanoguiTimeline(nanogui::Widget* parent);
			~NanoguiTimeline() override;

			bool init( const std::vector<std::string>& timeframes
					 , const std::vector<std::string>& timeframesTooltips
				     , const std::vector<std::string>& timesteps
					 , const std::vector<std::string>& timestepsTooltips
					 , std::uint16_t minYear
					 , std::uint16_t maxYear
                     , bool hoverEnabled) override;

			bool setCurrentChangedCallback(const std::function<void(const std::string&, const std::string&)>&) override;
			bool setTimeframeChangedCallback(const std::function<void(const std::string&)>&) override;
			bool setTimestepChangedCallback(const std::function<void(const std::string&)>&) override;
			bool setPrevCallback(const std::function<void()>&) override;
			bool setNextCallback(const std::function<void()>&) override;
			bool setYearScrollCallback(const std::function<void(float)>&) override;
			bool setMonthScrollCallback(const std::function<void(float)>&) override;
			bool setDayScrollCallback(const std::function<void(float)>&) override;
			bool setCurrent(const std::string&, const std::string&, int, int, int) override;
			bool setVisibleBounds(timestamp_t, timestamp_t) override;
			bool setTimeframe(const std::string& label, const std::string& description) override;
			bool setTimestep(const std::string& label, const std::string& description) override;
			bool setHintsSource(const EventsCoverage&) override;
			bool setMainCoverage(const EventsCoverage&) override;
			bool setExtraCoverage(const EventsCoverage&) override;
			bool setNearestMode(bool) override;
			bool nearestMode() const override;
			bool setOptionsChangedCallback(const std::function<void()>&) override;
			void setMonth(std::uint16_t, std::uint8_t) override;
			void setYear(std::uint16_t) override;

            bool getExtendedView() const override;
            void setExtendedView(bool, bool) override;

			bool mouseButtonEvent(const Eigen::Vector2i&, int, bool, int) override;
			bool scrollEvent(const Eigen::Vector2i&, const Eigen::Vector2f&) override;

			bool setPosition(int x, int y) override;
			bool setSize(int width, int height) override;
			int getHeightOffset() const override;

            void draw(NVGcontext* ctx) override;

			bool updateViews(bool);

			std::function<void(const std::string&, const std::string&)> currentChangedCallback;
			std::function<void(const std::string&)> timeframeChangedCallback;
			std::function<void()> prevCallback;
			std::function<void()> nextCallback;
			std::function<void()> optionsChangedCallback;
			std::function<void()> modeChangedCallback;
			NanoguiDatetimeBox* datebox;
			NanoguiButtonBar* timeframes;
			NanoguiCalendarsTimeline* calendarsView;
			NanoguiMonthsList* monthsList;
			NanoguiDaysList* daysList;
			nanogui::Button* nearestButton;
            nanogui::Button* recenterViewButton;

            EventsCoverage hintsSource;
            EventsCoverage mainCoverage;
            EventsCoverage extraCoverage;
			std::uint16_t year;
			std::uint8_t month;
			std::uint8_t day;
			timestamp_t visiblePast;
			timestamp_t visibleFuture;
			std::string changeOrigin;

            protected:
                nanogui::Button* _timestepSettingsButton;
                nanogui::Label* _timestepLabel;
                NanoguiTimestepSettings* _timestepSettings;
                nanogui::Button* _viewModeButton;
	};
};

#endif
