// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_TIMESPAN_EDITOR_H
#define NANOGUI_TIMESPAN_EDITOR_H

#include <functional>
#include <string>

#include "core/types.h"
#include "vendors/_nanogui.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiTextField;
    class NanoguiCheckbox;
    class NanoguiNumberBox;

    class NanoguiTimespanEditor: public virtual nanogui::Widget
    {
        public:
            NanoguiTimespanEditor(nanogui::Widget*);
            virtual ~NanoguiTimespanEditor() override;

            void draw(NVGcontext*) override;

            void set(const std::string&, const std::string&, const timestamp_t&, const timestamp_t&, bool);

            void enableEdition();
            void disableEdition();

            void checkValidity();
            const std::string& getIdentifier() const;

            void setSaveCallback(const std::function<void(const std::string&, double, double, const std::string&, bool)>&);
            void setDeleteCallback(const std::function<void()>&);
            void setDefaultCallback(const std::function<void()>&);
            void checkDefault();
            void uncheckDefault();

        protected:
            NanoguiTextField* _label;
            NanoguiTextField* _description;
            NanoguiNumberBox* _past;
            NanoguiNumberBox* _future;
            NanoguiCheckbox* _default;
            nanogui::Button* _okButton;
            nanogui::Button* _deleteButton;

            std::string _identifier;

            std::function<void(const std::string&, double, double, const std::string&, bool)> _saveCallback;
            std::function<void()> _deleteCallback;
            std::function<void()> _defaultCallback;
    };
}

#endif
