// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_TOOLBAR_H
#define NANOGUI_TOOLBAR_H

#include <map>
#include <string>
#include "core/interfaces/iapp_events_subscriber.h"
#include "core/types.h"

namespace nanogui
{
    // Forward declarations
    class Widget;
    class Button;
    class Label;
}

namespace SEAScope
{
    // Forward declarations
    class AppEventsDispatcher;
    class NanoguiDraggablePane;
    class NanoguiAnnotationsCollectionCreator;
    class NanoguiAnchored;
    class NanoguiComboBox;
    class NanoguiPathSelector;
    class InputEventsDispatcher;
    class NanoguiAnnotationCollectionSelector;

    class NanoguiToolbar: public virtual IAppEventsSubscriber
    {
        public:
            NanoguiToolbar(AppEventsDispatcher&, const InputEventsDispatcher&);
            virtual ~NanoguiToolbar() override;

            bool init(nanogui::Widget*);
            bool expand();
            bool reduce();
            void showAt(const Eigen::Vector2i&);

            bool handleAppEvent(std::type_index, const AppEvent&) override;

            void updatePushButtonColors();
        protected:

            void initLoadDataWidgets(nanogui::Widget*);
            void initSaveDataWidgets(nanogui::Widget*);
            void initCollectionsCreator(nanogui::Widget*);
            void initPathSelector(nanogui::Widget*);

            void updateCollectionsChoice();
            void updateLoadSaveButtons();

            bool getSelectedCollectionVariable(CollectionVariable&, const RenderMethod&, const std::string&);
            void buildCollectionsMapping(std::map<ShapeType, CollectionId>&);

            AppEventsDispatcher& _appEventsDispatcher;
            const InputEventsDispatcher& _inputEventsDispatcher;

            NanoguiDraggablePane* _window;
            nanogui::Widget* _headerbar;
            nanogui::Widget* _preButtonsSpacer;
            nanogui::Widget* _postButtonsSpacer;
            nanogui::Button* _reduceButton;
            nanogui::Button* _expandButton;

            nanogui::Widget* _buttons;
            nanogui::Widget* _settings;
            nanogui::Widget* _instructions;

            nanogui::Widget* _pathBlock;
            nanogui::Label* _pathSelectorLabel;
            NanoguiPathSelector* _pathSelector;

            nanogui::Widget* _loadButtonsBlock;
            nanogui::Button* _loadDataButton;

            nanogui::Widget* _saveButtonsBlock;
            nanogui::Button* _saveDataButton;

            nanogui::Button* _polylineButton;
            nanogui::Button* _polygonButton;
            nanogui::Button* _pointButton;

            nanogui::Button* _importButton;
            nanogui::Button* _exportButton;

            nanogui::Button* _zoomInButton;
            nanogui::Button* _zoomOutButton;
            nanogui::Button* _cameraResetButton;
            nanogui::Button* _lockCameraAxisButton;
            nanogui::Button* _rasterizationButton;

            NanoguiAnchored* _rasterizationDropdown;
            nanogui::Button* _fullModeButton;
            nanogui::Button* _wireframeModeButton;
            nanogui::Button* _dotModeButton;

            nanogui::Widget* _ioFormatBlock;
            nanogui::Widget* _collectionsBlock;
            NanoguiAnnotationCollectionSelector* _pointSelector;
            NanoguiAnnotationCollectionSelector* _polylineSelector;
            NanoguiAnnotationCollectionSelector* _polygonSelector;

            nanogui::Widget* _titleBlock;
            nanogui::Label* _drawingCollectionsTitle;
            nanogui::Label* _loadCollectionsTitle;
            nanogui::Label* _saveCollectionsTitle;

            nanogui::Label* _instructionsLabel;

            nanogui::Theme* _defaultTheme;
            nanogui::Theme* _pushedTheme;

            std::string _defaultCollectionComboBoxValue;

            NanoguiAnnotationsCollectionCreator* _collectionCreator;
            std::map<RenderMethod, std::map<CollectionVariable, std::string>> _variablesByRenderMethod;
            std::vector<nanogui::Button*> _drawingButtons;
            std::vector<nanogui::Button*> _rasterizationButtons;
            std::vector<nanogui::Button*> _ioFormatButtons;
    };
}

#endif
