// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_UI_H
#define NANOGUI_UI_H

#include <map>
#include <string>
#include <functional>
#include "core/interfaces/iapp_events_subscriber.h"
#include "core/interfaces/iuser_interface.h"
#include "ui/icomponent.h"
#include "ui/nanogui/nanogui_blueprint.h"
#include "ui/nanogui/nanogui_blueprint_factory.h"
#include "ui/nanogui/nanogui_renderer.h"
#include "ui/nanogui/nanogui_settings.h"

namespace SEAScope
{
    // Forward declarations
    class State;
    class IWindow;
    class AppEvent;
    class AppEventsDispatcher;
    class InputEventsDispatcher;
    class NanoguiRenderingMenu;
    class NanoguiFloatingLabel;
    class Settings;
    class NanoguiTimeline;
    class NanoguiAdvancedRenderingControl;
    class NanoguiRenderingControl;
    class NanoguiCameraControl;
    class NanoguiToolbar;
    class NanoguiSEAScopeSideMenu;
    class ContextualMenu;

	class NanoguiUI: public virtual IAppEventsSubscriber, public virtual IUserInterface
	{
		public:
			NanoguiUI(AppEventsDispatcher&, InputEventsDispatcher&);
			virtual ~NanoguiUI() override;

            bool handleAppEvent(std::type_index, const AppEvent&) override;

            // IUserInterface
            bool init(IWindow&, Settings&, State*) override;
            void showProcessingOptions(bool) override;
            void setIcons(Icons&) override;
            ICollectionsControlView* getCollectionsUI() override;
            IRenderingControlView* getRenderingUI() override;
            IAdvancedRenderingControlView* getAdvancedRenderingUI() override;
            ITimelineView* getTimelineUI() override;
            ISourcesControlView* getSourcesUI() override;
            ICameraControlView* getCameraUI() override;
            IRenderer* getRenderer() override;
            EventHandler* getEventHandler() override;


			bool resize(int, int);
			bool addComponent( IComponent* component, const std::string& label
							 , const std::string& minWidth, const std::string& minHeight
							 , const std::string& defaultX, const std::string& defaultY
							 , bool reflow);
			bool setSaveLayoutCallback(const std::function<void()>&);
			IBlueprint* getBlueprint(const IComponent*, bool&);
            bool trackMouse(IComponent*);

            NanoguiRenderingMenu* getRenderingMenu() const;
            NanoguiFloatingLabel* getFloatingLabel() const;

            bool setResizeCallback(const std::function<void(int, int)>&);

            bool setComponentResizeCallback(const std::function<void(const std::string&, int, int)>&);
            const std::function<void(const std::string&, int, int)> getComponentResizeCallback() const;

        protected:
            AppEventsDispatcher& _appEventsDispatcher;
            InputEventsDispatcher& _inputEventsDispatcher;
			NanoguiSettings _settings;
			NanoguiRenderer _renderer;
			State* _state;
            IWindow* _window;
            NanoguiRenderingMenu* _renderingMenu;
            NanoguiFloatingLabel* _floatingLabel;
			std::function<void()> _saveLayoutCallback;
			std::function<void(int,int)> _resizeCallback;
			std::function<void(const std::string, int, int)> _componentResizeCallback;

            //
	        NanoguiBlueprintFactory _blueprintFactory;	//
            NanoguiTimeline* _timelineUI;	//
            NanoguiCameraControl* _cameraUI;
            NanoguiToolbar* _toolbarUI;
            NanoguiSEAScopeSideMenu* _sideMenu;
            ContextualMenu* _ctxMenu;
	};
};

#endif
