// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IMPRECISE_SCROLL_FIX_H
#define IMPRECISE_SCROLL_FIX_H

#import <Foundation/Foundation.h>
#import <Cocoa/Cocoa.h>

/// Fix imprecise scrolling deltas by wrapping an NSEvent and modifying the
/// returned scrollDeltaX/Y to be either -1, 0, or 1, when
/// hasPreciseScrollingDeltas is NO.
/// When hasPreciseScrollingDeltas is NO, the event's scrollDeltaX/Y will have a
/// scale of 0.1 instead of 1 (so a single tick on the scroll wheel will be
/// +/- 0.1).
@interface FixScrollNSEventProxy : NSObject

@property (nonatomic, nonnull, retain) NSEvent *realEvent;

- (instancetype _Nonnull)initWithEvent:(NSEvent * _Nonnull)event;

@end

#endif
