// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#pragma once

namespace SEAScope
{
	class IProtocolHandler;
	class SocketsManager;
}

class SocketData;

class Server
{
public:
	Server();
	~Server();

	void initialize(const std::string& address, std::int64_t port, SEAScope::SocketsManager*);
	bool run();
	void stop();

protected:
	bool acceptConnection(SocketData& s);
	bool readSocket(SocketData& s);
	bool closeSocket(SocketData& s);

private:
	std::string _address;
	std::int64_t _port;
	bool _stopThread;
	std::mutex _mutex;
	SocketData* _listener;
	std::vector<SEAScope::IProtocolHandler*> _protocolHandler;
	SEAScope::SocketsManager* _socketManager;
    int _kq;
};
