// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#import "platform/imprecise_scroll_fix.h"

@implementation FixScrollNSEventProxy

@synthesize realEvent;

- (instancetype)initWithEvent:(NSEvent *)event
{
    self = [super init];
    if (self) {
        self->realEvent = event;
    }
    return self;
}

- (id)forwardingTargetForSelector:(SEL)aSelector
{
    return self.realEvent;
}

- (CGFloat)scrollingDeltaX
{
    CGFloat delta = self.realEvent.scrollingDeltaX;

    if
        (NO == self.realEvent.hasPreciseScrollingDeltas)
    {
        return (delta > 0) - (delta < 0);
    }
    return delta;
}

- (CGFloat)scrollingDeltaY
{
    CGFloat delta = self.realEvent.scrollingDeltaY;

    if
        (NO == self.realEvent.hasPreciseScrollingDeltas)
    {
        return (delta > 0) - (delta < 0);
    }
    return delta;
}

@end
