// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <iostream>
#include <thread>
#include <chrono>
#include <unistd.h>
#include <netdb.h>
#include <fcntl.h>
#include "core/log.h"
#include "platform/sockets_monitor_impl.h"

#include "core/perfs.h"
#include "core/sequential_buffer.h"

using namespace SEAScope;

/**
 *
 */
SocketsMonitorImpl::SocketsMonitorImpl(): signalHandler(nullptr), terminationRequested(false), epfd(-1), read_pipe(-1), write_pipe(-1)
{
};

/**
 *
 */
SocketsMonitorImpl::~SocketsMonitorImpl()
{
};

/**
 *
 */
bool SocketsMonitorImpl::handleEvent(const SocketJob& event)
{
	LOG(LogLevel::Debug, "Unix sockets monitor", "handle event");
	if
		(event.terminate)
	{
		this->terminationRequested = true;
	}
	else if
		(event.subscribe)
	{
		return this->subscribe(event.fileDescriptor, event.handler, event.handlerFactory);
	}
	else
	{
		// Unsubscribe
		this->unsubscribe(event.fileDescriptor);
	}
	return false;
};

/**
 *
 */
bool SocketsMonitorImpl::subscribe(socket_t /*fd*/, IProtocolHandler* /*handler*/, const ProtocolHandlerFactory& /*handlerFactory*/)
{
	return true;
};

/**
 *
 */
bool SocketsMonitorImpl::unsubscribe(socket_t /*fd*/)
{
	return true;
};

/**
 *
 */
bool SocketsMonitorImpl::init(const std::function<void()>& /*signalHandler*/, int& /*eventWakeFd*/)
{
    return true;
};

/**
 *
 */
bool SocketsMonitorImpl::monitorSockets()
{
	return false;
};

/**
 *
 */
bool SocketsMonitorImpl::triggerEventsProcessing()
{
	return true;
};

/**
 *
 */
bool SocketsMonitorImpl::sendData(socket_t sock, std::uint8_t* buffer, std::uint64_t dataSize)
{
	std::uint64_t remaining;	//
	std::int64_t sent;	//

	// Send data
	remaining = dataSize;
	while
		(0 < remaining)
	{
		sent = send(sock, buffer, remaining, 0);
		if
			(-1 == sent)
		{
			if
				((errno == EWOULDBLOCK) || (errno == EAGAIN))
			{
				continue;
			}

			/* Something wrong happened */
			break;
		}

		/* Prepare next write */
		remaining -= static_cast<std::uint64_t>(sent);
		buffer += sent;
	}

	return 0 == remaining;
};
