// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"

#include "3d/OGL3/opengl.h"
#include "3d/OGL3/draw_modeGL3.h"
#include "3d/OGL3/opengl_error.h"
#include "3d/OGL3/enum_translation.h"

namespace OGL3
{
   
void DrawModeGL3::draw()
{
    GLenum gl_primitive = primitiveToGLType(_primitive);
    
    if (OGL::draw_arrays == _drawType)
    {
        GL_CHECK(glDrawArrays(gl_primitive, 0, static_cast<GLsizei>(_elementCount)));
    }
    else if (OGL::draw_elements == _drawType)
    {
        GLenum gl_data_type = dataTypeToGLDataType(_dataType);
        GL_CHECK(glDrawElements(gl_primitive, static_cast<GLsizei>(_elementCount), gl_data_type, nullptr));
    }

}

void DrawModeGL3::drawInstanced()
{
    GLenum gl_primitive = primitiveToGLType(_primitive);

    if (OGL::draw_elements_instanced == _drawType)
    {
        GLenum gl_data_type = dataTypeToGLDataType(_dataType);
        GL_CHECK(glDrawElementsInstanced(gl_primitive, static_cast<GLsizei>(_elementCount), gl_data_type, nullptr, static_cast<GLsizei>(_instanceCount)));
    }
    else if (OGL::draw_arrays_instanced == _drawType)
    {
        GL_CHECK(glDrawArraysInstanced(gl_primitive, 0, static_cast<GLsizei>(_elementCount), static_cast<GLsizei>(_instanceCount)));
    }
}

void DrawModeGL3::drawSubset(unsigned int offset, unsigned int count)
{
    GLenum gl_primitive = primitiveToGLType(_primitive);
    
    if (OGL::draw_arrays == _drawType)
    {
        GL_CHECK(glDrawArrays(gl_primitive, static_cast<GLint>(offset), static_cast<GLsizei>(count)));
    }
}

void DrawModeGL3::drawTransformedFeedback()
{
    glBeginTransformFeedback(GL_POINTS);
    
    draw();
    
    glEndTransformFeedback();
}
    
} // namespace OGL3
