// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"

#include "3d/OGL3/opengl.h"
#include "3d/OGL3/opengl_error.h"

#include "core/log.h"


namespace OGL3
{

void checkOpenglError(const char* file, int line)
{
    GLenum error;
    while((error = glGetError()) != GL_NO_ERROR)
    {
        std::string strError;

        switch(error) 
        {
        case GL_INVALID_OPERATION:      
            strError="INVALID_OPERATION";      
            break;
        case GL_INVALID_ENUM:           
            strError="INVALID_ENUM";           
            break;
        case GL_INVALID_VALUE:          
            strError="INVALID_VALUE";          
            break;
        case GL_OUT_OF_MEMORY:          
            strError="OUT_OF_MEMORY";          
            break;
        case GL_INVALID_FRAMEBUFFER_OPERATION:  
            strError="INVALID_FRAMEBUFFER_OPERATION";  
            break;
        default:
            strError = std::to_string(static_cast<int>(error));
        }
        
        LOG(SEAScope::LogLevel::Error, "OpenGL Debug",
                           "[{}: {}] {}", file, line, strError);
    }
}

void DebugCallBack(unsigned int source, unsigned int type, unsigned int id, unsigned int severity, int length,const char *message, void* userParam)
{
	DebugCallBack(source, type, id, severity, length, message, const_cast<const void*>(userParam));
}

void DebugCallBack(unsigned int /*source*/, unsigned int /*type*/, unsigned int id, unsigned int /*severity*/, int /*length*/,const char *message, const void* /*userParam*/)
{
    std::stringstream ss;
    
    ss << id << ":" << message;
    LOG(SEAScope::LogLevel::Debug, "OpenGL Debug", ss.str());
    
    // Clear error
    glGetError();
}

} // namespace OGL
