// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/OGL3/queryGL3.h"

#include "3d/OGL3/opengl.h"

namespace
{
    GLenum enumCounterTypeToGLEnum(OGL::QueryCounterType type)
    {
        GLenum glEnum(0);
        
        switch(type)
        {
        case OGL::QueryCounterType::samples_passed:
            glEnum = GL_SAMPLES_PASSED;
            break;
        case OGL::QueryCounterType::any_samples_passed:
            glEnum = GL_ANY_SAMPLES_PASSED;
            break;
        case OGL::QueryCounterType::primitives_generated:
            glEnum = GL_PRIMITIVES_GENERATED;
            break;
        case OGL::QueryCounterType::transform_feedback_primitives_written:
            glEnum = GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN;
            break;
        case OGL::QueryCounterType::time_elapsed:
            glEnum = GL_TIME_ELAPSED;
            break;
        case OGL::QueryCounterType::any_samples_passed_conservative:
            // OpenGL 4.3
            break;
        }
        
        return glEnum;
    }
}

namespace OGL3
{

Query::Query()
:   OGL::Query(),
    _currentCounterType(0)
{
}

Query::~Query()
{
}

void Query::create(OGL::Context*)
{
    glGenQueries(1, &handle);
}

void Query::destroy()
{
    glDeleteQueries(1, &handle);
    handle = 0;
}

void Query::begin(OGL::QueryCounterType type)
{
    _currentCounterType = enumCounterTypeToGLEnum(type);
    glBeginQuery(_currentCounterType, handle);
}

void Query::end()
{
    glEndQuery(_currentCounterType);
}

unsigned int Query::getResult()
{
    unsigned int result = 0;
    glGetQueryObjectuiv(handle, GL_QUERY_RESULT, &result);
    return result;
}

} // namespace OGL3
