// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"

#include "3d/OGL3/opengl.h"
#include "3d/OGL3/renderbufferGL3.h"
#include "3d/OGL3/opengl_error.h"

namespace
{
    GLenum renderBufferTypeToGLenum(OGL::RenderBufferType type)
    {
        GLenum val = GL_DEPTH_COMPONENT16;
        
        switch(type)
        {
        case OGL::render_buffer_depth:
            val = GL_DEPTH_COMPONENT16;
            break;
        case OGL::render_buffer_stencil:
            val = GL_STENCIL_INDEX8;
            break;
        case OGL::render_buffer_depth_stencil:
            val = GL_DEPTH24_STENCIL8;
            break;
        }
        
        return val;
    }
}

namespace OGL3
{

RenderBufferGL3::RenderBufferGL3(OGL::RenderBufferType type)
:   OGL::RenderBuffer(type)
{
}
    
RenderBufferGL3::~RenderBufferGL3()
{
    destroy();
}
    
void RenderBufferGL3::create(OGL::Context* /*context*/)
{
    if (0 == this->handle)
    {
        GL_CHECK(glGenRenderbuffers(1, &this->handle));
    }
}

void RenderBufferGL3::destroy()
{
    if (0 != this->handle)
    {
        GL_CHECK(glDeleteRenderbuffers(1, &this->handle));
        this->handle = 0;
    }
}

void RenderBufferGL3::bind() const
{
    GL_CHECK(glBindRenderbuffer(GL_RENDERBUFFER, this->handle));
}

void RenderBufferGL3::unbind() const
{
    GL_CHECK(glBindRenderbuffer(GL_RENDERBUFFER, 0));
}

void RenderBufferGL3::allocate(unsigned int width, unsigned int height)
{
    // Bind the current fbo object
    bind();

    GLenum glType = renderBufferTypeToGLenum(_type);
    
    GL_CHECK(glRenderbufferStorage(GL_RENDERBUFFER,
                                   glType,
                                   static_cast<GLsizei>(width),
                                   static_cast<GLsizei>(height)));

    unbind();
}

void RenderBufferGL3::resize(unsigned int width, unsigned int height)
{
    allocate(width, height);
}

} // namespace OGL3
