// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"

#include "3d/OGL3/opengl.h"
#include "3d/OGL3/syncGL3.h"

namespace OGL3
{

Sync::Sync()
:   OGL::Sync(),
    _handle(nullptr)
{
}

Sync::~Sync()
{
    destroy();
}

void Sync::create()
{
    // Insert a sync flag in the opengl rendering pipeline
    _handle = glFenceSync(GL_SYNC_GPU_COMMANDS_COMPLETE, 0);
}

void Sync::destroy()
{
    // Delete the opengl object
    glDeleteSync(_handle);
    _handle = nullptr;
}

bool Sync::wait(uint64_t timeout)
{
    GLenum result = glClientWaitSync(_handle, GL_SYNC_FLUSH_COMMANDS_BIT, timeout);
    return (GL_ALREADY_SIGNALED == result || GL_CONDITION_SATISFIED == result);
}

}
