// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"

#include "3d/OGL3/utilsGL3.h"
#include "3d/OGL3/contextGL3.h"
#include "3d/OGL3/opengl.h"
#include "3d/mesh_creator.h"
#include "3d/array.h"
#include "3d/meta.h"

#include "core/log.h"


namespace OGL3
{

OGL::StaticMesh* UtilsGL3::createFullscreenVao(OGL::Context* ctx)
{   
    std::vector<glm::vec3> vertices(3);
    
    MeshCreator mc;
    mc.setDrawType(OGL::draw_arrays, OGL::triangles);
    mc.setVertices(makeArray(vertices));
    
    return mc.createStaticMesh(ctx);  
}

void UtilsGL3::printInfo()
{
    const GLubyte* str;
    
    str = glGetString(GL_VENDOR);
    LOG(SEAScope::LogLevel::Info, "OGL3::Utils", "Vendor: {}", reinterpret_cast<const char*>(str));

    str = glGetString(GL_RENDERER);
    LOG(SEAScope::LogLevel::Info, "OGL3::Utils", "Renderer: {}", reinterpret_cast<const char*>(str));

    str = glGetString(GL_VERSION);
    LOG(SEAScope::LogLevel::Info, "OGL3::Utils", "OpenGL version: {}", reinterpret_cast<const char*>(str));

    str = glGetString(GL_SHADING_LANGUAGE_VERSION);
    LOG(SEAScope::LogLevel::Info, "OGL3::Utils", "GLSL version: {}", reinterpret_cast<const char*>(str));

    int value;
    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &value);
    
    LOG(SEAScope::LogLevel::Info, "OGL3::Utils", "Texture 2D max size: {}", value);

    int NumberOfExtensions;
    std::stringstream ss;
    
    glGetIntegerv(GL_NUM_EXTENSIONS, &NumberOfExtensions);
    
    for(int i=0; i<NumberOfExtensions; i++) 
    {
        ss << glGetStringi(GL_EXTENSIONS, static_cast<GLuint>(i)) << std::endl;
    }
    
    LOG(SEAScope::LogLevel::Info, "OGL3::Utils", "Number of extensions: {} \n {}", NumberOfExtensions, ss.str());
}

} // namespace OGL3
