// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   vertex_array_object.cpp
 * Author: gleseach
 * 
 * Created on 4 septembre 2015, 10:34
 */

#include "3d/stdafx.h"

#include "3d/OGL3/opengl.h"
#include "3d/OGL3/vertex_array_objectGL3.h"
#include "3d/OGL3/opengl_error.h"
#include "3d/OGL3/vertex_attrib_arrayGL3.h"

namespace OGL3
{
    
VertexArrayObjectGL3::VertexArrayObjectGL3()
:   OGL::VertexArrayObject()
{
}

VertexArrayObjectGL3::~VertexArrayObjectGL3()
{
    destroy();
}

void VertexArrayObjectGL3::create(OGL::Context* /*context*/)
{
    if (0 == this->handle)
    {
        GL_CHECK(glGenVertexArrays(1, &this->handle));
    }
}

void VertexArrayObjectGL3::destroy()
{
    for (OGL::BindableObject* obj : this->toDelete)
    {
        delete obj;
		obj = nullptr;
    }

    this->pipeline.clear();
    this->toDelete.clear();
        
    if (0 != this->handle)
    {       
        GL_CHECK(glDeleteVertexArrays(1, &this->handle));
        this->handle = 0;
    }
}

void VertexArrayObjectGL3::bind() const
{
    GL_CHECK(glBindVertexArray(this->handle));
}

void VertexArrayObjectGL3::unbind() const
{
    GL_CHECK(glBindVertexArray(0));
}

} // namespace OGL3
