// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   vertex_attrib_array.cpp
 * Author: gleseach
 * 
 * Created on 4 septembre 2015, 10:57
 */

#include "3d/stdafx.h"

#include "3d/OGL3/opengl.h"
#include "3d/OGL3/vertex_attrib_arrayGL3.h"
#include "3d/OGL3/opengl_error.h"
#include "3d/OGL3/bufferGL3.h"
#include "3d/OGL3/enum_translation.h"

namespace OGL3
{

VertexAttribArrayGL3::VertexAttribArrayGL3(unsigned int idIn, unsigned int component_countIn, OGL::Buffer * bufferIn, DataType dataTypeIn, unsigned int strideIn, unsigned int offsetIn)
: OGL::VertexAttribArray(idIn, component_countIn, bufferIn, dataTypeIn, strideIn, offsetIn)
{
}

VertexAttribArrayGL3::~VertexAttribArrayGL3()
{
    destroy();
}

void VertexAttribArrayGL3::create(OGL::Context* /*context*/)
{
}

void VertexAttribArrayGL3::destroy()
{
}

void VertexAttribArrayGL3::bind() const
{
    buffer->bind();
    GL_CHECK(glEnableVertexAttribArray(this->id));
    GLenum glDataType = dataTypeToGLDataType(this->dataType);
    glVertexAttribPointer(this->id, static_cast<GLint>(this->component_count), glDataType, GL_FALSE, static_cast<GLsizei>(this->stride), reinterpret_cast<const void*>(offset));
}

void VertexAttribArrayGL3::unbind() const
{
    buffer->unbind();
}

} // namespace OGL3
