// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/barbs_color.h"

#include "3d/core_objects.h"
#include "3d/program_bind_helper.h"
#include "3d/renderable.h"
#include "3d/particle_grid.h"
#include "3d/data_set.h"

BarbsColor::BarbsColor()
:   ColorRenderer(),
    _particles(nullptr)
{
}

BarbsColor::~BarbsColor()
{
    delete _particles;
	_particles = nullptr;
}

void BarbsColor::create(OGL::Context* ctx)
{
    _particles = new ParticleGrid();
    _particles->create(ctx, ParticleMode::barb);
}

void BarbsColor::bind(OGL::Context* ctx, const glm::mat4& /*mvp*/, DataRenderingParam* param)
{
    ctx->setDepthMask(false);

    _particles->bind(ctx, param);
}

void BarbsColor::draw(OGL::Context* ctx, Renderable2* object)
{
    _particles->draw(ctx, static_cast<DataSet*>(object));
}
