// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/basic_mesh.h"

#include "3d/mesh_creator.h"
#include "3d/meta.h"
#include "3d/shader_list.h"
#include "3d/program_bind_helper.h"
#include "3d/program.h"
#include "3d/helper.h"

OGL::StaticMesh * createDisk(OGL::Context* ctx, float /*radius*/, unsigned int /*subdivision*/)
{
    MeshCreator mc;
    std::vector<glm::vec3> vertices;

    for (unsigned int i = 0; i < 360; ++i)
    {
        float theta = glm::radians(static_cast<float>(i));
        float thetab = glm::radians(static_cast<float>(i+1));

        vertices.push_back(glm::normalize(glm::vec3(0.f, (cosf(theta) - sinf(theta)), (sinf(theta) + cosf(theta)))));
        vertices.push_back(glm::normalize(glm::vec3(0.f, (cosf(thetab) - sinf(thetab)), (sinf(thetab) + cosf(thetab)))));
        vertices.push_back(glm::vec3(0.f, 0.f, 0.f));
    }

    mc.setVertices(makeArray(vertices));
    mc.setDrawType(OGL::draw_arrays, OGL::triangles);
    OGL::ProgramBindHelper* p = OGL::Helper::createProgramAutoBind(ctx, vertex_pos_mvp, fragment_white);
    return mc.createStaticMesh(ctx, p);
}
