// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/billboard.h"

#include "3d/point.h"
#include "3d/context.h"
#include "3d/static_mesh.h"
#include "3d/screen_quad.h"

Billboard::Billboard()
: ShapeMesh(SEAScope::RenderableId()),
  _mesh(nullptr),
  _texture(nullptr),
  _position({0, 0, 0})
{
	_flags.set(RenderableFlags::editable, true);
	_flags.set(RenderableFlags::erasable, true);
}

Billboard::~Billboard()
{
    this->_mesh = nullptr;  // not owning
    this->_texture = nullptr;   // not owning
}

void Billboard::initialize(Point* point)
{
    if
        (nullptr != this->_shape)
    {
        return;
    }

    this->_shape = point;
    if
        (nullptr != point)
    {
        this->setPosition(point->getPosition());
    }
}

void Billboard::draw(OGL::Context* ctx, const glm::mat4& /*mvp*/)
{
    if
        (nullptr == this->_mesh)
    {
        return;
    }

    if
        (false == this->_visible)
    {
        return;
    }

    if
        (nullptr != this->_texture)
    {
        ctx->activeTextureUnit(0, this->_texture);
    }

    this->_mesh->draw();
}

void Billboard::drawPicking(OGL::Context* /*ctx*/, const glm::mat4& /*mvp*/)
{
    if
        (nullptr == this->_mesh)
    {
        return;
    }

    if
        (false == this->_visible)
    {
        return;
    }

    this->_mesh->draw();
}

void Billboard::setPosition(const glm::vec3& position)
{
    this->_position = position;

    Point* point = dynamic_cast<Point*>(this->_shape);
    if
        (nullptr != point)
    {
        point->setPosition(position);
    }
}

const glm::vec3 Billboard::getPosition() const
{
    return this->_position;
}

void Billboard::setMesh(OGL::StaticMesh* mesh)
{
    this->_mesh = mesh;
}

void Billboard::setTexture(OGL::Texture* texture)
{
    this->_texture = texture;
}
