// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/billboard_pool_renderer.h"

#include "3d/program_bind_helper.h"
#include "3d/core_objects.h"
#include "3d/data_rendering_param.h"
#include "3d/renderable.h"

BillboardPoolRenderer::BillboardPoolRenderer()
:   TextureRenderer(),
    _sizeLoc(-1)
{
}

BillboardPoolRenderer::~BillboardPoolRenderer()
{
}

void BillboardPoolRenderer::create(OGL::Context* ctx)
{
    TextureRenderer::createProgram(ctx, vertex_screenspace_quad_dir_tex_instanced, fragment_tex_true_rgba);

    OGL::Program* p = _program->getProgram();
    this->_sizeLoc = p->getUniformLocation("size");
}

void BillboardPoolRenderer::bind(OGL::Context* ctx, const glm::mat4& mvp, DataRenderingParam* param)
{
    TextureRenderer::bind(ctx, mvp, param);

    OGL::Program* p = _program->getProgram();

    float ps = ctx->getPixelSizeUnit();
    p->setUniform(this->_sizeLoc, ps * (param->billboardsSize / 2.f));
}

void BillboardPoolRenderer::draw(OGL::Context* ctx, Renderable2* object)
{
    object->draw(ctx, glm::mat4());
}
