// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/camera.h"

#include "3d/frustum.h"

Camera::Camera()
{
}

Camera::~Camera()
{
}

void Camera::lookAt(const glm::vec3& eye, const glm::vec3& center, const glm::vec3& up)
{
    this->_view = glm::lookAt(eye, center, up);
    this->_frustum.lookAt(_view);
}

void Camera::perspective(float fovy, float aspect, float near, float far)
{
    float b, t, l, r;

    float scale = static_cast<float>(tan(glm::radians(fovy) * 0.5f) * near);
    r = aspect * scale;
    l = -r;
    t = scale;
    b = -t;

    glm::mat4& M = this->_projection;

    // set OpenGL perspective projection matrix
    M[0][0] = 2 * near / (r - l);
    M[0][1] = 0;
    M[0][2] = 0;
    M[0][3] = 0;

    M[1][0] = 0;
    M[1][1] = 2 * near / (t - b);
    M[1][2] = 0;
    M[1][3] = 0;

    M[2][0] = (r + l) / (r - l);
    M[2][1] = (t + b) / (t - b);
    M[2][2] = -(far + near) / (far - near);
    M[2][3] = -1;

    M[3][0] = 0;
    M[3][1] = 0;
    M[3][2] = -2 * far * near / (far - near);
    M[3][3] = 0;

    this->_frustum.setPerspective(fovy, aspect, near, far);
}

void Camera::ortho(float left, float right, float bottom, float top)
{
    this->_projection = glm::ortho(left, right, bottom, top);
}

void Camera::setViewMatrix(const glm::mat4& view)
{
    this->_view = view;
}

glm::vec3 Camera::getWorldPosition() const
{
    glm::mat3 rotMat(this->_view);
    glm::vec3 p(this->_view[3]);
    p = -p * rotMat;
    return p;
}

void Camera::setModelMatrix(const glm::mat4& model)
{
    this->_model = model;
}
