// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   chrono.cpp
 * Author: gleseach
 * 
 * Created on 8 mars 2016, 11:35
 */

#include "3d/stdafx.h"
#include "3d/chrono.h"

void Chrono::start()
{
    _origin = std::chrono::system_clock::now();
}

void Chrono::reset()
{
    start();
}

unsigned int Chrono::elapsed(TimeUnit timeUnit) const
{
    auto currentTime = std::chrono::system_clock::now();

    unsigned int result = 0;

    if (millisecond == timeUnit)
    {
        result = static_cast<unsigned int>(std::chrono::duration_cast<std::chrono::milliseconds>(currentTime-_origin).count());
    }
    else if (second == timeUnit)
    {
        result = static_cast<unsigned int>(std::chrono::duration_cast<std::chrono::seconds>(currentTime-_origin).count());
    }
    else if (nanosecond == timeUnit)
    {
        result = static_cast<unsigned int>(std::chrono::duration_cast<std::chrono::nanoseconds>(currentTime-_origin).count());
    }

    return result;
}

float Chrono::elapsedf(TimeUnit timeUnit) const
{
    auto currentTime = std::chrono::system_clock::now();

    float result = static_cast<float>(std::chrono::duration_cast<std::chrono::nanoseconds>(currentTime-_origin).count());

    if (second == timeUnit)
    {
        result *= 0.000000001f;
    }
    else if (millisecond == timeUnit)
    {
        result *= 0.000001f;
    }

    return result;
}
