// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/color_renderer.h"

#include "3d/core_objects.h"
#include "3d/program_bind_helper.h"
#include "3d/renderable.h"
#include "3d/data_rendering_param.h"

ColorRenderer::ColorRenderer()
:   IRenderer2(),
    _program(nullptr),
    _mvpLoc(-1),
    _colorLoc(-1)
{
}

ColorRenderer::~ColorRenderer()
{
    delete _program;
    _program = nullptr;
}

void ColorRenderer::createProgram(OGL::Context* ctx, const std::string& vertexShader, const std::string& fragmentShader)
{
    _program = OGL::Helper::createProgramAutoBind(ctx, vertexShader, fragmentShader);

    OGL::Program* p = _program->getProgram();

    _mvpLoc = p->getUniformLocation("mvp");
    _colorLoc = p->getUniformLocation("color");
}

void ColorRenderer::bind(OGL::Context* ctx, const glm::mat4& mvp, DataRenderingParam* param)
{
    ctx->setBlendFunc(OGL::src_alpha, OGL::one_minus_src_alpha);

    ctx->setFlag(OGL::blending, true);

    ctx->setDepthMask(false);

    _program->bind(ctx);

    OGL::Program* p = _program->getProgram();

    p->setUniform(_mvpLoc, mvp);
    p->setUniform(_colorLoc, glm::vec4(param->color, param->alpha));
}
