// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/context.h"

#include "3d/pipeline_parameters.h"
#include "3d/shader_manager.h"

namespace OGL
{

Context::Context()
{
    for (size_t i = 0; i < _textureUnit.size(); ++i)
    {
        _textureUnit[i].first = false;
        _textureUnit[i].second = nullptr;
    }

    _pipelineParams = new PipelineParameters;
    _shaderMngr = new OGL::ShaderManager(this);
    _pixelSize = 0.f;
}

Context::~Context()
{
    delete _pipelineParams;
	this->_pipelineParams = nullptr;

    delete _shaderMngr;
	this->_shaderMngr = nullptr;

	// Renderer resources are owned by another class, no delete.
	this->_resources = nullptr;
}

OGL::Texture* Context::getCurrentTextureBind(unsigned int textureUnit)
{
    return _textureUnit[textureUnit].second;
}

void Context::setShaderManager(OGL::ShaderManager* shaderMngr)
{
    _shaderMngr = shaderMngr;
}

OGL::ShaderManager* Context::getShaderManager() const
{
    return _shaderMngr;
}

PipelineParameters* Context::pipelineParameters()
{
    return _pipelineParams;
}

} // namespace OGL
