// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   cube.cpp
 * Author: gleseach
 * 
 * Created on 14 octobre 2015, 11:56
 */

#include "3d/stdafx.h"
#include "3d/cube.h"


void createCube(VerticeArray& vertices, std::vector<unsigned int>& indices)
{
    vertices.reserve(8);
    indices.reserve(36);
    
    const float s = 1.f;
    
    // Bottom
    vertices.push_back(glm::vec3(-s, -s, -s));
    vertices.push_back(glm::vec3( s, -s, -s));
    vertices.push_back(glm::vec3( s, -s,  s));
    vertices.push_back(glm::vec3(-s, -s,  s));
    
    // Top
    vertices.push_back(glm::vec3(-s,  s, -s));
    vertices.push_back(glm::vec3( s,  s, -s));
    vertices.push_back(glm::vec3( s,  s,  s));
    vertices.push_back(glm::vec3(-s,  s,  s));
    
    // Bottom
    indices.push_back(0);
    indices.push_back(2);
    indices.push_back(1);
    
    indices.push_back(0);
    indices.push_back(3);
    indices.push_back(2);
    
    // back
    indices.push_back(4);
    indices.push_back(1);
    indices.push_back(5);
    
    indices.push_back(4);
    indices.push_back(0);
    indices.push_back(1);
    
    // front
    indices.push_back(3);
    indices.push_back(6);
    indices.push_back(2);
    
    indices.push_back(3);
    indices.push_back(7);
    indices.push_back(6);
    
    // left
    indices.push_back(4);
    indices.push_back(3);
    indices.push_back(0);
    
    indices.push_back(4);
    indices.push_back(7);
    indices.push_back(3);
    
    // right
    indices.push_back(1);
    indices.push_back(6);
    indices.push_back(5);
    
    indices.push_back(1);
    indices.push_back(2);
    indices.push_back(6);
    
    // Top
    indices.push_back(5);
    indices.push_back(7);
    indices.push_back(4);
    
    indices.push_back(5);
    indices.push_back(6);
    indices.push_back(7);    
}

void createCube(VerticeArray& vertices, std::vector<unsigned char>& indices)
{
    vertices.reserve(8);
    indices.reserve(36);
    
    const float s = 1.f;
    
    // Bottom
    vertices.push_back(glm::vec3(-s, -s, -s));
    vertices.push_back(glm::vec3( s, -s, -s));
    vertices.push_back(glm::vec3( s, -s,  s));
    vertices.push_back(glm::vec3(-s, -s,  s));
    
    // Top
    vertices.push_back(glm::vec3(-s,  s, -s));
    vertices.push_back(glm::vec3( s,  s, -s));
    vertices.push_back(glm::vec3( s,  s,  s));
    vertices.push_back(glm::vec3(-s,  s,  s));
    
    // Bottom
    indices.push_back(0);
    indices.push_back(2);
    indices.push_back(1);
    
    indices.push_back(0);
    indices.push_back(3);
    indices.push_back(2);
    
    // back
    indices.push_back(4);
    indices.push_back(1);
    indices.push_back(5);
    
    indices.push_back(4);
    indices.push_back(0);
    indices.push_back(1);
    
    // front
    indices.push_back(3);
    indices.push_back(6);
    indices.push_back(2);
    
    indices.push_back(3);
    indices.push_back(7);
    indices.push_back(6);
    
    // left
    indices.push_back(4);
    indices.push_back(3);
    indices.push_back(0);
    
    indices.push_back(4);
    indices.push_back(7);
    indices.push_back(3);
    
    // right
    indices.push_back(1);
    indices.push_back(6);
    indices.push_back(5);
    
    indices.push_back(1);
    indices.push_back(2);
    indices.push_back(6);
    
    // Top
    indices.push_back(5);
    indices.push_back(7);
    indices.push_back(4);
    
    indices.push_back(5);
    indices.push_back(6);
    indices.push_back(7);
}
