// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <algorithm>

#include "3d/datasets_allocator.h"
#include "3d/data_set_tiled.h"

using namespace SEAScope;

/**
 *
 */
DatasetsAllocator::DatasetsAllocator()
{
}

/**
 *
 */
DatasetsAllocator::~DatasetsAllocator()
{
    for
        (auto entry : this->entries)
    {
        delete entry;
        entry = nullptr;
    }
    this->entries.clear();
}

/**
 *
 */
DataSet* DatasetsAllocator::get(const RenderableId& rId, bool tiled)
{
    return DatasetsAllocator::getInstance()->_get(rId, tiled);
}

/**
 *
 */
bool DatasetsAllocator::release(DataSet* dataset)
{
    return DatasetsAllocator::getInstance()->_release(dataset);
}

/**
 *
 */
DataSet* DatasetsAllocator::_get(const RenderableId& rId, bool tiled)
{
    std::lock_guard<std::mutex> lock(this->entriesMutex);
    if
        (true == tiled)
    {
        this->entries.emplace_back(new DataSetTiled(rId));
    }
    else
    {
        this->entries.emplace_back(new DataSet(rId));
    }
    return this->entries.back();
}

/**
 *
 */
bool DatasetsAllocator::_release(DataSet* dataset)
{
    if
        (nullptr == dataset)
    {
        return false;
    }

    std::lock_guard<std::mutex> lock(this->entriesMutex);
    auto found = std::find(this->entries.begin(), this->entries.end(), dataset);
    if
        (this->entries.end() != found)
    {
        delete *found;
        *found = nullptr;
        this->entries.erase(found);
        return true;
    }
    return false;
}
