// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/ellipse_builder.h"

#include "3d/ellipse.h"
#include "3d/pipeline_parameters.h"
#include "3d/math_func.h"

OGL::Shape* EllipseBuilder::build()
{
	Ellipse* e = new Ellipse;

	const int ellipseWidth = 200;
	const int ellipseHeigh = 100;

    if
        (nullptr == this->_parameters)
    {
        return nullptr;
    }
	glm::ivec2 center(this->_parameters->viewport.x / 2, this->_parameters->viewport.y / 2);

	glm::vec3 hitCenter;
	if (true == getPickingPosition(_parameters->viewport, center.x, center.y, hitCenter, _parameters->view, _parameters->projection))
	{
	}

	glm::vec3 hitW;
	if (true == getPickingPosition(_parameters->viewport, center.x + ellipseWidth / 2, center.y, hitW, _parameters->view, _parameters->projection))
	{
	}

	glm::vec3 hitH;
	if (true == getPickingPosition(_parameters->viewport, center.x, center.y + ellipseHeigh / 2, hitH, _parameters->view, _parameters->projection))
	{
	}

	float angleW = static_cast<float>(AngleBetweenNormalized(hitCenter, hitW));
	float angleH = static_cast<float>(AngleBetweenNormalized(hitCenter, hitH));

	const float minAngleW = 0.0002f;
	const float minAngleH = 0.0001f;

	angleW = isnan(angleW) ? minAngleW : fmax(minAngleW, angleW);
	angleH = isnan(angleH) ? minAngleH : fmax(minAngleH, angleH);

	e->initialize(this->_center, angleW, angleH);

	return e;
}

SEAScope::ShapeType EllipseBuilder::getShapeType()
{
	return SEAScope::ELLIPSE;
}
