// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/false_colors_renderer.h"

#include "3d/core_objects.h"
#include "3d/data_rendering_param.h"
#include "3d/shader_manager.h"
#include "3d/program_bind_helper.h"

FalseColorsRenderer::FalseColorsRenderer()
:   IRenderer2(),
    _program(nullptr),
    _mvpLoc(-1),
    _rangeLoc(-1),
    _logScaleLoc(-1),
    _alphaLoc(-1)
{
}

FalseColorsRenderer::~FalseColorsRenderer()
{
    delete _program;
	_program = nullptr;
}

void FalseColorsRenderer::createProgram(OGL::Context* ctx, const std::string& vertexShader, const std::string& fragmentShader)
{
    _program = ctx->getShaderManager()->createProgramAutoBind(vertexShader, fragmentShader);

    OGL::Program* p = _program->getProgram();

    _mvpLoc = p->getUniformLocation("mvp");
    _rangeLoc = p->getUniformLocation("range");
    _logScaleLoc = p->getUniformLocation("logscale");
    _alphaLoc = p->getUniformLocation("alpha");
}

void FalseColorsRenderer::bind(OGL::Context* ctx, const glm::mat4& mvp, DataRenderingParam* param)
{
    ctx->setBlendFunc(OGL::src_alpha, OGL::one_minus_src_alpha);
    ctx->setFlag(OGL::blending, true);
    ctx->setDepthMask(false);

    _program->bind(ctx);

    OGL::Program* p = _program->getProgram();

    p->setUniform(_mvpLoc, mvp);
    p->setUniform(_rangeLoc, param->range);
    p->setUniform(_logScaleLoc, param->log ? 1.f : 0.f);
    p->setUniform(_alphaLoc, param->alpha);

    ctx->activeTextureUnit(2, param->colormap);
}
