// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/file.h"

#include "core/log.h"


bool File::readAll(const std::string& file_name, std::string& data)
{
    FILE *fp = fopen(file_name.c_str(), "rb");

    if (nullptr == fp)
    {
        LOG(SEAScope::LogLevel::Error, "File", "Error opening file {} : {}", file_name, strerror(errno));
        return false;
    }

    fseek(fp, 0, SEEK_END);
    std::int64_t pos = ftell(fp);
    rewind(fp);

    if (pos > 0)
    {
        data.resize(static_cast<std::size_t>(pos));
        std::size_t size = fread(reinterpret_cast<void*>(const_cast<char*>(data.data())), 1, static_cast<std::size_t>(pos), fp);

        if (size != static_cast<std::size_t>(pos))
        {
            data.resize(size);
        }
    }

    fclose(fp);

    return true;
}
