// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/globe.h"

#include "3d/app_enum.h"
#include "3d/static_mesh.h"
#include "3d/mesh_creator.h"
#include "3d/cube.h"
#include "3d/texture.h"
#include "3d/meta.h"
#include "3d/context.h"

Globe::Globe(const SEAScope::RenderableId& rId)
:   Pickable(rId),
    _mesh(nullptr),
    _worldTexture(nullptr)
{
}

Globe::~Globe()
{
    delete _mesh;
	_mesh = nullptr;
    delete _worldTexture;
	_worldTexture = nullptr;
}

void Globe::create(OGL::Context* ctx, OGL::Texture* worldMapTexture)
{
    VerticeArray vertices;
    IndiceArray indices;

    // Cube
    createCube(vertices, indices);

    MeshCreator mc;
    mc.setVertices(makeArray(vertices));
    mc.setIndices(makeArray(indices));
    mc.setDrawType(OGL::draw_elements, OGL::triangles);

    _mesh = mc.createStaticMesh(ctx);

    _worldTexture = worldMapTexture;
}

void Globe::draw(OGL::Context* ctx, const glm::mat4& /*mvp*/)
{
    ctx->activeTextureUnit(0, _worldTexture);

    _mesh->draw();

    ctx->deactiveTextureUnit(0);
}

void Globe::drawPicking(OGL::Context* /*ctx*/, const glm::mat4& /*mvp*/)
{
    _mesh->draw();
}
