// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/globe_renderer.h"

#include "3d/core_objects.h"
#include "3d/data_rendering_param.h"
#include "3d/shader_manager.h"
#include "3d/program_bind_helper.h"
#include "3d/pipeline_parameters.h"
#include "3d/renderable.h"

GlobeRenderer::GlobeRenderer()
: IRenderer2{},
  _program{nullptr},
  _mvpLoc{-1},
  _eyePositionLoc{-1},
  _modelLoc{-1},
  _projectionLoc{-1},
  _viewLoc{-1},
  _invModelLoc{-1}
{
}

GlobeRenderer::~GlobeRenderer()
{
    delete _program;
    _program = nullptr;
}

void GlobeRenderer::create(OGL::Context* ctx)
{
    _program = OGL::Helper::createProgramAutoBind(ctx, vertex_raycasting, fragment_raycasting);

    OGL::Program* p = _program->getProgram();

    _mvpLoc = p->getUniformLocation("mvp");
    _eyePositionLoc = p->getUniformLocation("eyePosition");
    _modelLoc = p->getUniformLocation("model");
    _projectionLoc = p->getUniformLocation("projection");
    _viewLoc = p->getUniformLocation("view");
    _invModelLoc = p->getUniformLocation("inv_model");
}

void GlobeRenderer::bind(OGL::Context* ctx, const glm::mat4& /*mvp*/, DataRenderingParam* /*param*/)
{
    ctx->setBlendFunc(OGL::src_alpha, OGL::one_minus_src_alpha);
    ctx->setFlag(OGL::blending, false);
    ctx->setDepthMask(true);

    _program->bind(ctx);

    OGL::Program* p = _program->getProgram();

    const PipelineParameters* pParam = ctx->pipelineParameters();

    p->setUniform(_mvpLoc, pParam->viewProjection);
    p->setUniform(_eyePositionLoc, pParam->eyePosition);
    p->setUniform(_modelLoc, pParam->model);
    p->setUniform(_projectionLoc, pParam->projection);
    p->setUniform(_viewLoc, pParam->view);
    p->setUniform(_invModelLoc, pParam->inverseModel);
}

void GlobeRenderer::draw(OGL::Context* ctx, Renderable2* object)
{
    object->draw(ctx, glm::mat4());
}
