// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/globe_visibility.h"

#include "3d/math_func.h"
#include "3d/camera.h"
#include "3d/frustum.h"

GlobeVisibility::GlobeVisibility()
:   IWorldVisibility()
{
    _spaceVisible = false;
}

GlobeVisibility::~GlobeVisibility()
{
}

void GlobeVisibility::update()
{

    glm::vec3 cameraPos = _camera->getWorldPosition();

    // Compute the inverse model matrix to move the result.
    const glm::mat4 invModelMatrix = glm::inverse(_camera->getModelMatrix());

    // Intersection test between top left frustum ray and the sphere
    // Frustum* frustum = _camera->getFrustum();
    // glm::vec3 tl = frustum->getTopLeftRay();
    // glm::vec3 hitTL;
    //bool resultTopLeft = intersectUnitarySphereWorldCentered(tl, cameraPos, hitTL);

    // If the ray intersect, compute the intersection of each frustum corner ray
    //if (resultTopLeft)
    //{
    //    glm::vec3 tr = frustum->getTopRightRay();
    //    glm::vec3 bl = frustum->getBottomLeftRay();
    //    glm::vec3 br = frustum->getBottomRightRay();
    //
    //    glm::vec3 hitTR, hitBL, hitBR;

    //    intersectUnitarySphereWorldCentered(tr, cameraPos, hitTR);
    //    intersectUnitarySphereWorldCentered(bl, cameraPos, hitBL);
    //    intersectUnitarySphereWorldCentered(br, cameraPos, hitBR);
    //    
    //    _quad->setCornerPosition(0, invModelMatrix * glm::vec4(hitTL, 1.f));
    //    _quad->setCornerPosition(1, invModelMatrix * glm::vec4(hitBR, 1.f));
    //    _quad->setCornerPosition(2, invModelMatrix * glm::vec4(hitTR, 1.f));
    //    _quad->setCornerPosition(3, invModelMatrix * glm::vec4(hitBL, 1.f));
    //    
    //    // Update the visibility plane according to the new camera position.
    //    _zLimit->setEquation(glm::normalize(cameraPos), _quad->getCornerPosition(0));
    //    
    //    _spaceVisible = false;
    //}
    //else
    {
        // No intersection found, we are zoom out, compute the "half globe" area
        const glm::vec3 globePos(0.f, 0.f, 0.f);
        const glm::vec3 northPole(0.f, 0.f, 1.f);
        glm::vec3 hit;

        // Trace a ray between the camera position and the northpole
        intersect(glm::normalize(northPole - cameraPos), cameraPos, globePos, 1.f, hit);

        // The visibility limit is at half way between the ray intersection and the north pole.
        glm::vec3 limit = glm::normalize((northPole + hit) / 2.f);

        // Apply the rotation camera to our result.
        limit =  invModelMatrix * glm::vec4(limit, 1.f);
        cameraPos = invModelMatrix * glm::vec4(cameraPos, 1.f);

        // Update the visibility plane according to the new camera position.
        _zLimit.setEquation(glm::normalize(cameraPos), limit);

        // Compute angle between the camera to sphere center line and the sphere tangent from the camera
        float alpha = acosf(1.f / glm::length(cameraPos));

        // Compute 4 points left, right, top, bottom
        float angles[4] = 
        {
            alpha,
            -alpha,
            alpha,
            -alpha,
        };

        glm::vec3 axis[4] =
        {
            glm::vec3(0.f, 0.f, 1.f),
            glm::vec3(0.f, 0.f, 1.f),
            glm::vec3(0.f, 1.f, 0.f),
            glm::vec3(0.f, 1.f, 0.f)
        };

        glm::quat q[4];
        glm::vec3 origin(1.f, 0.f, 0.f);

        glm::vec3 points[4];

        for (unsigned int i = 0; i < 4; ++i)
        {
            q[i] = glm::rotate(q[i], angles[i], axis[i]);
            points[i] = origin * q[i];

            points[i] = glm::vec4(points[i], 1.f) * _camera->getModelMatrix();

            _quad.setCornerPosition(i, points[i]);
         }
        
        _spaceVisible = true;
    }
}

bool GlobeVisibility::isVisible(const glm::vec3& p)
{
    return _zLimit.classifyPoint(p) == Halfspace::POSITIVE;
}

SphericalQuad* GlobeVisibility::getVisibleQuad()
{
    return &_quad;
}
