// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"

#include "3d/opengl_enum.h"

#include "3d/helper.h"
#include "3d/context.h"
#include "3d/buffer.h"
#include "3d/framebuffer.h"
#include "3d/texture.h"
#include "3d/array.h"
#include "3d/shader_manager.h"
#include "3d/draw_mode.h"
#include "3d/typed_buffer.h"

namespace OGL
{

OGL::Buffer* Helper::createBuffer(OGL::Context* ctx, OGL::BufferType type, const Array& array)
{
    OGL::Buffer* buffer = ctx->createBuffer(type);
    buffer->create(ctx);
    buffer->allocate(array);
    return buffer;
}

OGL::FrameBuffer* Helper::createFramebuffer(OGL::Context* ctx, unsigned int width, unsigned int height, DataType dataType)
{
    OGL::FrameBuffer* fb = ctx->createFramebuffer();
    fb->create(ctx);
    fb->allocate(width, height, dataType);
    return fb;
}

OGL::Texture* Helper::createTexture(OGL::Context* ctx, const Image& image)
{
    OGL::Texture* t = ctx->createTexture(OGL::texture2D);
    t->create(ctx);
    t->allocate2D(image);
    return t;
}

OGL::Program* Helper::createProgram(OGL::Context* ctx, const std::string& vertexShaderName, const std::string& fragmentShaderName)
{
    OGL::Program* p = ctx->getShaderManager()->createProgram(vertexShaderName, fragmentShaderName);
    return p;
}

OGL::ProgramBindHelper* Helper::createProgramAutoBind(OGL::Context* ctx, const std::string& vertexShaderName, const std::string& fragmentShaderName)
{
    OGL::ProgramBindHelper* p = ctx->getShaderManager()->createProgramAutoBind(vertexShaderName, fragmentShaderName);
    return p;
}

OGL::DrawMode* Helper::createDrawModeArrays(OGL::Context* ctx, OGL::Primitive primitive, unsigned int elementCount)
{
    OGL::DrawMode* d = ctx->createDrawMode();
    d->setPrimitive(primitive);
    d->setDrawType(OGL::draw_arrays);
    d->setElementCount(elementCount);
    return d;
}

OGL::DrawMode* Helper::createDrawModeElements(OGL::Context* ctx, OGL::Primitive primitive, DataType dataType, unsigned int elementCount)
{
    OGL::DrawMode* d = ctx->createDrawMode();
    d->setPrimitive(primitive);
    d->setDrawType(OGL::draw_elements);
    d->setElementCount(elementCount);
    d->setDataType(dataType);
    return d;
}

OGL::VertexAttribArray* Helper::createVertexAttribArray(OGL::Context* ctx, int binding, const TypedBuffer& typedBuffer, unsigned int stride, unsigned int offset)
{
    return ctx->createVertexAttribArray(static_cast<unsigned int>(binding), typedBuffer, stride, offset);
}

} // namespace OGL
