// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/memory_manager.h"

#include "core/log.h"


MemoryManager::MemoryManager()
{
    memset(_memory, 0, sizeof(unsigned long long) * categoryCount);
}

MemoryManager::~MemoryManager()
{
}

void MemoryManager::consume(MemoryCategory category, unsigned long long amount)
{
    _memory[category] += amount;
}

void MemoryManager::release(MemoryCategory category, unsigned long long amount)
{
    if (amount > _memory[category])
    {
        LOG(SEAScope::LogLevel::Error, "MemoryManager", "Can't release more memory than the amount allocated.");
    }
    else
    {
        _memory[category] -= amount;
    }
}

unsigned long long MemoryManager::getAllocatedMemoryAmount(MemoryCategory category) const
{
    return _memory[category];
}

unsigned long long MemoryManager::getTotalAllocatedMemoryAmount() const
{
    unsigned long long total = 0;

    for (unsigned long long amount : _memory)
    {
        total += amount;
    }

    return total;
}

