// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/picking_impl.h"

#include "3d/core_objects.h"
#include "3d/renderable.h"

PickingImpl::PickingImpl()
: IPickingRenderer{},
  _program{nullptr},
  _mvp{-1},
  _color{-1}
{
}

PickingImpl::~PickingImpl()
{
    delete _program;
    _program = nullptr;
}

void PickingImpl::create(OGL::Context* /*ctx*/)
{
}

void PickingImpl::createProgram(OGL::Context* ctx, const char* vertexShader, const char* fragmentShader)
{
    _program = OGL::Helper::createProgram(ctx, vertexShader, fragmentShader);

    _mvp = _program->getUniformLocation("mvp");
    _color = _program->getUniformLocation("color");
}

void PickingImpl::bind(OGL::Context* ctx, const glm::mat4& mvp, const DataRenderingParam*)
{
    ctx->setDepthMask(false);

    _program->bind();

    _program->setUniform(_mvp, mvp);
}

void PickingImpl::draw(OGL::Context* ctx, Renderable2* object)
{
    object->draw(ctx, glm::mat4());
}
