// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/plane.h"

Plane::Plane()
{
}

Plane::Plane(const glm::vec3& nIn, const glm::vec3& pIn)
{
    setEquation(nIn, pIn);
}

Halfspace Plane::classifyPoint(const glm::vec3& pt) const
{
    float res;
    res = glm::dot(n, pt) + d;

    if (res < 0)
        return NEGATIVE;
    if (res > 0)
        return POSITIVE;

    return ON_PLANE;
}

const glm::vec4& Plane::equation() const
{
    return _equation;
}

void Plane::setEquation(const glm::vec4& e)
{
    _equation = e;
    n = glm::vec3(e.x, e.y, e.z);
    d = e.w;
}

void Plane::setEquation(const glm::vec3& nIn, const glm::vec3& pIn)
{
    _equation = glm::vec4(nIn, -(glm::dot(nIn, pIn)));
    this->n = nIn;
    d = _equation.w;
}

const glm::vec3& Plane::normal() const
{
    return n;
}
