// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/point_editor.h"

#include "3d/point.h"
#include "3d/point_builder.h"

#include "3d/core_objects.h"
#include "3d/polyline_mesh.h"
#include "3d/geodetic.h"
#include "3d/ellipsoid.h"
#include "3d/geodetic.h"
#include "3d/spherical_math.h"
#include "3d/math_func.h"
#include "3d/geometry.h"
#include "3d/polyline_base.h"
#include "3d/scene_graph_node.h"
#include "3d/shape_anchor.h"
#include "3d/anchor_pool_node.h"

#include <iostream>

PointEditor::PointEditor()
: ShapeEditor()
{
}

PointEditor::~PointEditor()
{
}

void PointEditor::initialize(OGL::Context* /*ctx*/, Pickable* obj)
{
	this->_shapeToEdit = obj;
}

void PointEditor::draw(OGL::Context* /*ctx*/, const glm::mat4& /*mvp*/)
{
}

OGL::Shape* PointEditor::createShape()
{
	Point* p = static_cast<Point*>(this->_shapeToEdit->getShape());

	static_cast<PointBuilder*>(this->_builder)->setPosition(p->getPosition());
	return _builder->build();
}

void PointEditor::setBuilder(ShapeBuilder* builder)
{
    // takes ownership (delete done in ShapeEditor destructor)
	this->_builder = builder;
}

unsigned int PointEditor::getAnchorCount() const
{
    return 1;
}

glm::vec3 PointEditor::getAnchorPos(AnchorType /*type*/, unsigned int /*vertexId*/) const
{
	Point* p = static_cast<Point*>(this->_shapeToEdit->getShape());

	return p->getPosition();
}

void PointEditor::move(AnchorType type, unsigned int /*vertexId*/, const glm::vec3& pos)
{
	if (AnchorType::PointMoveable == type)
	{
		Point* p = static_cast<Point*>(this->_shapeToEdit->getShape());
        p->setPosition(pos);
	}
}

void PointEditor::move(AnchorType type, unsigned int vertexId)
{
    this->move(type, vertexId, this->_currentPos);
}

void PointEditor::pressed(AnchorType /*type*/, unsigned int /*id*/)
{
}

void PointEditor::setAnchorPool(AnchorPoolNode* val)
{
	ShapeEditor::setAnchorPool(val);

	this->_anchorPool->clear();

    // Add a single anchor
    this->_anchorPool->addAnchor(this, AnchorType::PointMoveable, 0u);
}

/**
 *
 */
void PointEditor::updateCursorPosition(const glm::vec3& position)
{
    this->_currentPos = position;
}

/**
 *
 */
void PointEditor::startMove(unsigned int)
{

}

/**
 *
 */
void PointEditor::stopMove()
{

}

/**
 *
 */
void PointEditor::remove(AnchorType /*type*/, unsigned int /*id*/)
{
}

/**
 *
 */
bool PointEditor::shapeIsValid() const
{
    return true;
}
