// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/polygon_color.h"

#include "3d/core_objects.h"
#include "3d/renderable.h"
#include "3d/data_rendering_param.h"
#include "3d/program_bind_helper.h"
#include "3d/polygon_mesh.h"

PolygonColor::PolygonColor()
:   ColorRenderer(),
    _param(nullptr)
{
}

PolygonColor::~PolygonColor()
{
    _param = nullptr;
}

void PolygonColor::create(OGL::Context* ctx)
{
    createProgram(ctx, vertex_pos_mvp, fragment_uni_color);
}

void PolygonColor::bind(OGL::Context* ctx, const glm::mat4& mvp, DataRenderingParam* param)
{
    ColorRenderer::bind(ctx, mvp, param);

    ctx->setFlag(OGL::blending, true);

    _param = param;
}

void PolygonColor::draw(OGL::Context* ctx, Renderable2* object)
{
    PolygonMesh* p = dynamic_cast<PolygonMesh*>(object);

    if (nullptr != p)
    {
		glm::vec4 color;

		if (p->getStyle().getState() == StyleState::normal && false == p->getStyle().isHighlighted())
		{
			color = glm::vec4(_param->color, _param->alpha);
		}
		else
		{
			color = p->getCurrentStyleColor();
			color.a = _param->alpha;
		}

		_program->getProgram()->setUniform(_colorLoc, color);
    }

	object->draw(ctx, glm::mat4());
}
