// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/polygon_picking.h"

#include "3d/core_objects.h"
#include "3d/pickable.h"

PolygonPicking::PolygonPicking()
:   PickingImpl()
{
}

PolygonPicking::~PolygonPicking()
{
}

void PolygonPicking::create(OGL::Context* ctx)
{
    createProgram(ctx, vertex_pos_mvp, fragment_uni_color);
}

void PolygonPicking::bind(OGL::Context* ctx, const glm::mat4& mvp, const DataRenderingParam* param)
{
    PickingImpl::bind(ctx, mvp, param);
}

void PolygonPicking::draw(OGL::Context* ctx, Renderable2* object)
{
    Pickable* p = static_cast<Pickable*>(object);
    _program->setUniform(_color, glm::vec4(p->pickingColor(), 1.f));

    p->drawPicking(ctx, glm::mat4());
}
