// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/polyline_base.h"


PolylineBase::PolylineBase(SEAScope::ShapeType type)
: OGL::Shape(type)
{
}

PolylineBase::~PolylineBase()
{
	clear();
}

void PolylineBase::addPoint(const glm::vec3& point)
{
	_points.push_back(point);
}

const glm::vec3& PolylineBase::getPoint(unsigned int ith) const
{
	if (ith < _points.size())
    {
		return _points[ith];
    }
	else
    {
		throw std::out_of_range("Position parameter is out of range");
    }
}

unsigned int PolylineBase::getPointCount() const
{
	return static_cast<unsigned int>(_points.size());
}

void PolylineBase::setPoint(unsigned int ith, const glm::vec3& point)
{
	if (ith < _points.size())
	{
		_points[ith] = point;
	}
	else
    {
		throw std::out_of_range("Position parameter is out of range");
    }
}

void PolylineBase::setLastPoint(const glm::vec3& p)
{
	if (false == _points.empty())
	{
		_points[_points.size() - 1] = p;
	}
}

void PolylineBase::setPoints(const std::vector<glm::vec3>& points)
{
	_points = points;
}

void PolylineBase::setPoints(std::vector<glm::vec3>&& points)
{
	_points = std::move(points);
}

void PolylineBase::deletePoint(unsigned int ith)
{
	if (ith < _points.size())
	{
		_points.erase(_points.begin() + ith);
	}
	else
		throw std::out_of_range("Position parameter is out of range");
}

void PolylineBase::clear()
{
	_points.clear();
}
