// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/polyline_builder.h"

#include "3d/shapes_allocator.h"
#include "3d/polyline.h"

PolylineBuilder::~PolylineBuilder()
{
}

void PolylineBuilder::setPoints(std::vector<glm::vec3>&& points)
{
	_points = std::move(points);
}

SEAScope::ShapeType PolylineBuilder::getShapeType()
{
	return SEAScope::POLYLINE;
}

OGL::Shape* PolylineBuilder::build()
{
	Polyline* p = nullptr;

	if (1 < _points.size())
	{
		p = ShapesAllocator::getShape<Polyline>();
		p->setPoints(std::move(_points));
	}

	return p;
}
