// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/polyline_custom_color.h"

#include "3d/core_objects.h"
#include "3d/shader_manager.h"
#include "3d/program_bind_helper.h"
#include "3d/renderable.h"
#include "3d/data_rendering_param.h"
#include "3d/polyline_mesh.h"

PolylineCustomColor::PolylineCustomColor()
: ColorRenderer{},
  _thicknessLoc{-1},
  _viewportLoc{-1},
  _param{nullptr}
{
}

PolylineCustomColor::~PolylineCustomColor()
{
}

void PolylineCustomColor::create(OGL::Context* ctx)
{
	createProgram(ctx, vertex_polyline_screenspace, fragment_polyline_screenspace);

	OGL::Program* p = _program->getProgram();

	_thicknessLoc = p->getUniformLocation("thickness");
	_viewportLoc = p->getUniformLocation("viewport");
}

void PolylineCustomColor::bind(OGL::Context* ctx, const glm::mat4& mvp, DataRenderingParam* param)
{
	ColorRenderer::bind(ctx, mvp, param);

	OGL::Program* p = _program->getProgram();

	p->setUniform(_thicknessLoc, param->lineThickness);

	glm::ivec4 v = ctx->getViewportSize();
	p->setUniform(_viewportLoc, glm::vec2(v.z, v.w));

	_param = param;
}

void PolylineCustomColor::draw(OGL::Context* ctx, Renderable2* object)
{
	PolylineMesh* polyline = static_cast<PolylineMesh*>(object);

	glm::vec4 color;
	Style& style = polyline->getStyle();

	if (style.getState() == StyleState::normal && false == style.isHighlighted())
	{
		color = glm::vec4(_param->color, _param->alpha);
	}
	else
	{
	color = polyline->getCurrentStyleColor();
	color.a = _param->alpha;
	}

	_program->getProgram()->setUniform(_colorLoc, color);

	object->draw(ctx, glm::mat4());
}
