// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/raster_false_colors.h"

#include "3d/core_objects.h"
#include "3d/program_bind_helper.h"
#include "3d/data_set.h"

RasterFalseColors::RasterFalseColors()
:   FalseColorsRenderer()
{
}

RasterFalseColors::~RasterFalseColors()
{
}

void RasterFalseColors::create(OGL::Context* ctx)
{
    createProgram(ctx, vertex_pos_mvp_tex, fragment_tex_col_tex_mask_tex_false_colors);
}

void RasterFalseColors::draw(OGL::Context* ctx, Renderable2* object)
{
    DataSet* dataSet = static_cast<DataSet*>(object);

    OGL::Program* p = _program->getProgram();

    glm::vec4 scaleFactors;
    glm::vec4 offsets;
    std::uint32_t channels;
    float glFactor;

    channels = static_cast<std::uint32_t>(dataSet->getDimensionCount());
    glFactor = dataSet->getScaleFactorGL();

    auto _scaleFactors = dataSet->getScaleFactor();
    auto _offsets = dataSet->getOffset();
    for
        (std::uint32_t i=0; i < channels; ++i)
    {
        scaleFactors[static_cast<int>(i)] = _scaleFactors[i];
        offsets[static_cast<int>(i)] = _offsets[i];
    }
    p->setUniformByName("scaleFactor", scaleFactors);
    p->setUniformByName("offset", offsets);
    p->setUniformByName("glFactor", glFactor);
    p->setUniformByName("channels", channels);

    dataSet->draw(ctx, glm::mat4());
}
