// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/rectangle_base.h"

#include "core/log.h"

RectangleBase::RectangleBase(SEAScope::ShapeType type)
: OGL::Shape(type)
{
}

RectangleBase::~RectangleBase()
{
}

glm::vec3 RectangleBase::getCorner(size_t i) const
{
	glm::vec3 result(0.f, 0.f, 0.f);

	if (i < _corners.size())
	{
		result = _corners[i];
	}
	else
	{
		LOG(SEAScope::LogLevel::Error, "RectangleBase", "invalid corner id");
	}

	return result;
}

const std::array<glm::vec3, 4>& RectangleBase::getCorners() const
{
	return _corners;
}
