// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/rectangle_creator_one_click.h"

#include "3d/shape.h"
#include "3d/rectangle_based_builder.h"

#include "3d/core_objects.h"
#include "3d/geodetic.h"
#include "3d/ellipsoid.h"
#include "3d/geodetic.h"
#include "3d/spherical_math.h"
#include "3d/math_func.h"
#include "3d/geometry.h"

#include "3d/shapes_allocator.h"
#include "3d/rectangle.h"
#include "3d/unit_sphere.h"
#include "3d/ellipse_mesh.h"
#include "3d/ellipse.h"

RectangleCreatorOneClick::RectangleCreatorOneClick()
: _builder(nullptr)
{
}

RectangleCreatorOneClick::~RectangleCreatorOneClick()
{
}

void RectangleCreatorOneClick::initialize(OGL::Context* /*ctx*/)
{
}

void RectangleCreatorOneClick::setBuilder(ShapeBuilder* builder)
{
	_builder = dynamic_cast<RectangleBasedBuilder*>(builder);
}

void RectangleCreatorOneClick::draw(OGL::Context* /*ctx*/, const glm::mat4& /*mvp*/)
{
}

OGL::Shape* RectangleCreatorOneClick::createShape()
{
	OGL::Shape* shape = nullptr;

	if (this->_builder)
	{
		this->_builder->setParameters(this->_parameters);
		this->_builder->setCenter(this->_center);

		shape = this->_builder->build();
	}

	return shape;
}

void RectangleCreatorOneClick::reset()
{
}

OGL::Shape* RectangleCreatorOneClick::getTemporaryShape()
{
	return nullptr;
}

SEAScope::ShapeType RectangleCreatorOneClick::getShapeType()
{
	return _builder->getShapeType();
}

void RectangleCreatorOneClick::addVertexUnderCursor()
{
}

void RectangleCreatorOneClick::updateCursorPosition(const glm::vec3& position, std::uint32_t, std::uint32_t)
{
    this->_currentPosition = position;
}

void RectangleCreatorOneClick::setCenterUnderCursor()
{
    this->_center = this->_currentPosition;
}

/**
 *
 */
unsigned int RectangleCreatorOneClick::getAnchorCount() const
{
    return 0;
}

/**
 *
 */
glm::vec3 RectangleCreatorOneClick::getAnchorPos(AnchorType, unsigned int) const
{
    return {0, 0, 0};
}

/**
 *
 */
void RectangleCreatorOneClick::move(AnchorType, unsigned int, const glm::vec3&)
{
}

/**
 *
 */
void RectangleCreatorOneClick::move(AnchorType, unsigned int)
{
}

/**
 *
 */
void RectangleCreatorOneClick::pressed(AnchorType, unsigned int)
{
}

/**
 *
 */
void RectangleCreatorOneClick::remove(AnchorType, unsigned int)
{
}
