// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/rectangle_editor.h"

#include "3d/rectangle_builder.h"

#include "3d/core_objects.h"
#include "3d/geodetic.h"
#include "3d/ellipsoid.h"
#include "3d/geodetic.h"
#include "3d/spherical_math.h"
#include "3d/math_func.h"
#include "3d/geometry.h"

#include "3d/rectangle_base.h"
#include "3d/pickable.h"

RectangleEditor::RectangleEditor()
: _program(nullptr), _buffer(nullptr), _vao(nullptr), _drawMode(nullptr)
{
}

RectangleEditor::~RectangleEditor()
{
	delete _buffer;
	_buffer = nullptr;
	delete _program;
	_program = nullptr;
	delete _vao;
	_vao = nullptr;
	delete _drawMode;
	_drawMode = nullptr;
}

void RectangleEditor::initialize(OGL::Context* ctx, Pickable* obj)
{
	_shapeToEdit = obj;

	_program = OGL::Helper::createProgram(ctx, vertex_line_tesselate, fragment_uni_color);

	_program->bind();
	_program->setUniformByName("color", glm::vec4(1.f, 0.f, 0.f, 1.f));
	_program->unbind();

	const int segmentCount = 20;
	std::vector<float> percents;
	percents.reserve(segmentCount);

	for (int i = 0; i < segmentCount; ++i)
	{
		percents.push_back(static_cast<float>(i) / (segmentCount - 1));
	}

	_buffer = OGL::Helper::createBuffer(ctx, OGL::array_buffer, makeArray(percents));
	OGL::VertexAttribArray* vaa = ctx->createVertexAttribArray(0, 1, _buffer, DataType::float32);

	_vao = ctx->createVertexArrayObject();
	_vao->create(ctx);
	_vao->addObject(vaa);
	_vao->fill();

	_drawMode = OGL::Helper::createDrawModeArrays(ctx, OGL::line_strip, segmentCount);
}

void RectangleEditor::draw(OGL::Context*, const glm::mat4& mvp)
{
	RectangleBase* p = static_cast<RectangleBase*>(_shapeToEdit->getShape());

	auto points = p->getCorners();

	_program->bind();

	_program->setUniformByName("mvp", mvp);
	_program->setUniformByName("color", green4f);

	_vao->bind();

	for (unsigned int i = 0; i < 4; ++i)
	{
		_program->setUniformByName("p0", points[i % 4]);
		_program->setUniformByName("p1", points[(i + 1) % 4]);

		_drawMode->draw();
	}

	_vao->unbind();
}

OGL::Shape* RectangleEditor::createShape()
{
	/*PolylineBase* p = static_cast<PolylineBase*>(_shapeToEdit->getShape());

	std::vector<glm::vec3> points = p->getPoints();
	static_cast<PolylineBuilder*>(_builder)->setPoints(std::move(points));
	return _builder->build();*/
	return nullptr;
}

void RectangleEditor::setBuilder(ShapeBuilder* builder)
{
	// takes ownership
	this->_builder = builder;
}

unsigned int RectangleEditor::getAnchorCount() const
{
	return 0;
}

glm::vec3 RectangleEditor::getAnchorPos(AnchorType /*type*/, unsigned int /*vertexId*/) const
{
	return this->_dummyAnchorPos;
}

void RectangleEditor::move(AnchorType /*type*/, unsigned int /*vertexId*/, const glm::vec3& /*pos*/)
{

}

/**
 *
 */
void RectangleEditor::move(AnchorType type, unsigned int vertexId)
{
    this->move(type, vertexId, this->_currentPos);
}

/**
 *
 */
void RectangleEditor::remove(AnchorType /*type*/, unsigned int /*vertexId*/)
{

}

/**
 *
 */
void RectangleEditor::updateCursorPosition(const glm::vec3&)
{

}

/**
 *
 */
void RectangleEditor::startMove(unsigned int)
{

}

/**
 *
 */
void RectangleEditor::stopMove()
{

}
