// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/rgb_raw.h"

#include "3d/core_objects.h"
#include "3d/program_bind_helper.h"
#include "3d/data_rendering_param.h"
#include "3d/renderable.h"
#include "3d/data_set.h"

RGBRaw::RGBRaw()
: RawImpl{},
  _mvpLoc{-1},
  _scaleFactorLoc{-1},
  _offsetLoc{-1},
  _scaleFactorGLLoc{-1}
{
}

RGBRaw::~RGBRaw()
{
}

void RGBRaw::create(OGL::Context* ctx)
{
    createProgram(ctx, vertex_pos_mvp_tex, fragment_tex_col_tex_mask_tex_rgb_raw);

    OGL::Program* p = _program->getProgram();

    _mvpLoc = p->getUniformLocation("mvp");
    _scaleFactorLoc = p->getUniformLocation("scaleFactor");
    _offsetLoc = p->getUniformLocation("offset");
    _scaleFactorGLLoc = p->getUniformLocation("multiplicator");
}

void RGBRaw::bind(OGL::Context* ctx, const glm::mat4& mvp, const DataRenderingParam* /*param*/)
{
    ctx->setBlendFunc(OGL::src_alpha, OGL::one_minus_src_alpha);
    ctx->setFlag(OGL::blending, true);
    ctx->setDepthMask(false);

    _program->bind(ctx);

    OGL::Program* p = _program->getProgram();

    p->setUniform(_mvpLoc, mvp);
}

void RGBRaw::draw(OGL::Context* ctx, Renderable2* object)
{
    DataSet* dataSet = static_cast<DataSet*>(object);

    OGL::Program* p = _program->getProgram();

    p->setUniform(_scaleFactorGLLoc, dataSet->getScaleFactorGL());

    float *fValuePtr = dataSet->getScaleFactor();

    glm::vec3 sFactor = glm::vec3(fValuePtr[0], fValuePtr[1], fValuePtr[2]);

    p->setUniform(_scaleFactorLoc, sFactor);

    fValuePtr = dataSet->getOffset();

    glm::vec3 offset = glm::vec3(fValuePtr[0], fValuePtr[1], fValuePtr[2]);

    p->setUniform(_offsetLoc, offset);

    dataSet->draw(ctx, glm::mat4());
}
