// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/scene_dictionnary.h"

#include "3d/renderable.h"
#include "3d/granule_tree.h"

SceneDictionnary::SceneDictionnary()
{
}

SceneDictionnary::~SceneDictionnary()
{
}

void SceneDictionnary::add(Renderable2* obj)
{
    _tree.add(obj->renderableId(), obj);
}

void SceneDictionnary::remove(Renderable2* obj)
{
    Renderable2* renderable;
    _tree.remove(obj->renderableId(), renderable);
}

bool SceneDictionnary::remove(const SEAScope::RenderableId& rId, Renderable2*& renderable)
{
    return _tree.remove(rId, renderable);
}

bool SceneDictionnary::exist(const SEAScope::RenderableId& rId) const
{
    return _tree.exist(rId);
}

bool SceneDictionnary::exist(const SEAScope::RenderableId& rId, Renderable2*& obj) const
{
    if (_tree.exist(rId, obj))
    {
        return true;
    }

    return false;
}

void SceneDictionnary::fillListToRemove(const std::set<SEAScope::RenderableId>& rIds, std::list<SEAScope::RenderableId>& toRemove)
{
    _tree.populate(rIds, toRemove);
}

void SceneDictionnary::fillListToRemovePair(const std::set<SEAScope::RenderableId>& rIds, ListPair& existing, ListPair& notFound)
{
    _tree.populatePair(rIds, existing, notFound);
}
