// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/screen_quad_color.h"

#include "3d/core_objects.h"
#include "3d/shape_anchor.h"
#include "3d/screen_quad.h"

ScreenQuadColor::ScreenQuadColor()
: _sq(nullptr)
{
}

ScreenQuadColor::~ScreenQuadColor()
{
    delete this->_sq;
    this->_sq = nullptr;
}

void ScreenQuadColor::create(OGL::Context* ctx)
{
    _sq = new ScreenQuad();
    _sq->create(ctx);
}

void ScreenQuadColor::bind(OGL::Context* ctx, const glm::mat4& mvp, DataRenderingParam* /*param*/)
{
    _sq->bind(ctx, mvp);
}

void ScreenQuadColor::draw(OGL::Context* /*ctx*/, Renderable2* object)
{
    ShapeAnchor* anchor = static_cast<ShapeAnchor*>(object);

    _sq->draw2(anchor->getPosition(), anchor->getCurrentStyleColor());
}

void ScreenQuadColor::draw(OGL::Context* /*ctx*/, Renderable2* object, const glm::vec4& color)
{
    ShapeAnchor* anchor = static_cast<ShapeAnchor*>(object);

    _sq->draw2(anchor->getPosition(), color);
}

