// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/screen_quad_picking.h"

#include "3d/core_objects.h"
#include "3d/program_bind_helper.h"

#include "3d/billboard.h"
#include "3d/math_values.h"
#include "3d/default_value.h"

ScreenQuadPicking::ScreenQuadPicking()
:	PickingImpl(),
	_sizeLoc(-1),
	_viewportLoc(-1),
	_positionLoc(-1)
{
}

ScreenQuadPicking::~ScreenQuadPicking()
{
}

void ScreenQuadPicking::create(OGL::Context* ctx)
{
	createProgram(ctx, vertex_screenspace_quad, fragment_uni_color);

	OGL::Program* p = _program;

	this->_sizeLoc = p->getUniformLocation("arrowTileSize");
	this->_viewportLoc = p->getUniformLocation("viewport");
	this->_positionLoc = p->getUniformLocation("position");
}

void ScreenQuadPicking::bind(OGL::Context* ctx, const glm::mat4& mvp, const DataRenderingParam* param)
{
	PickingImpl::bind(ctx, mvp, param);

	OGL::Program* p = _program;
	glm::ivec4 viewport = ctx->getViewportSize();

	constexpr float halfDiagonal = (SEAScope::DefaultValue::anchorEdgeSize * SEAScope::Math::fsqrt2) * 0.5f;

	p->setUniform(this->_sizeLoc, halfDiagonal);
	p->setUniform(this->_viewportLoc, glm::vec2(static_cast<float>(viewport.z), static_cast<float>(viewport.w)));
}

void ScreenQuadPicking::draw(OGL::Context* ctx, Renderable2* object)
{
    Billboard* billboard = static_cast<Billboard*>(object);
    glm::vec3 billboardPos = billboard->getPosition();

	_program->setUniform(this->_positionLoc, billboardPos);
	_program->setUniform(_color, glm::vec4(billboard->pickingColor(), 1.f));

    billboard->drawPicking(ctx, glm::mat4());
}
