// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/shape_anchor.h"

#include "3d/screen_quad.h"
#include "3d/static_mesh.h"
#include "3d/anchor_controller.h"

#include "core/structs.h"

ShapeAnchor::ShapeAnchor()
: Billboard{}, _type{AnchorType::AnchorTypeCount}, _controller{nullptr},
  _id{0u}
{
    this->_flags.set(anchor, true);
}

ShapeAnchor::~ShapeAnchor()
{
    this->_controller = nullptr;  // this is just a pointer copy, the anchor does not own the controller
}

void ShapeAnchor::initialize(AnchorType type, AnchorController* controller, OGL::StaticMesh* sm, unsigned int id)
{
	this->_type = type;
	this->_controller = controller;
	this->_id = id;

    this->setMesh(sm);
}

const glm::vec3 ShapeAnchor::getPosition() const
{
	return this->_controller->getAnchorPos(this->_type, this->_id);
}

void ShapeAnchor::setPosition(const glm::vec3& pos)
{
	this->_controller->move(this->_type, this->_id, pos);
}

void ShapeAnchor::pressed()
{
	this->_controller->pressed(this->_type, this->_id);
}

const AnchorType& ShapeAnchor::getType() const
{
    return this->_type;
}
