// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/shape_editor_factory.h"

// Editors
#include "3d/polyline_editor.h"
#include "3d/rectangle_editor.h"
#include "3d/ellipse_editor.h"
#include "3d/point_editor.h"

// Builders
#include "3d/polyline_builder.h"
#include "3d/polygon_builder.h"
#include "3d/ellipse_builder.h"
#include "3d/point_builder.h"

#include "core/log.h"

ShapeEditor* ShapeEditorFactory::create(SEAScope::ShapeType type)
{
	ShapeEditor* shapeEditor = nullptr;

	switch (type)
	{
	case SEAScope::ShapeType::POLYLINE:
	{
		shapeEditor = new PolylineEditor;
        static_cast<PolylineEditor*>(shapeEditor)->asLinearRing(false);
		shapeEditor->setBuilder(new PolylineBuilder);
	}
	break;

	case SEAScope::ShapeType::POLYGON:
	{
		shapeEditor = new PolylineEditor;
        static_cast<PolylineEditor*>(shapeEditor)->asLinearRing(true);
        static_cast<PolylineEditor*>(shapeEditor)->detectSelfIntersections(true);
		shapeEditor->setBuilder(new PolygonBuilder);
	}
	break;

	case SEAScope::ShapeType::CIRCLE:
	{
		/* Not implemented yet */
	}
	break;

	case SEAScope::ShapeType::RECTANGLE:
	{
		/* Not implemented yet */
	}
	break;


	case SEAScope::ShapeType::ELLIPSE:
	case SEAScope::ShapeType::ELLIPSE2:
	{
		shapeEditor = new EllipseEditor;
		shapeEditor->setBuilder(new EllipseBuilder);
	}
	break;

    case SEAScope::ShapeType::POINT:
    {
        /* Not implemented yet */
        shapeEditor = new PointEditor;
        shapeEditor->setBuilder(new PointBuilder);
    }
    break;

    case SEAScope::ShapeType::UNKNOWN_SHAPE_TYPE:
    {
        LOG(SEAScope::LogLevel::Error, "ShapeEditorFactory",
                           "Create method called for a shape type that is not supported");
	}
    break;

    }

	return shapeEditor;
}
