// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/shapes_allocator.h"

ShapesAllocator::ShapesAllocator()
{
}

ShapesAllocator::~ShapesAllocator()
{
    for
        (auto shape : this->_shapes)
    {
        delete shape;
        shape = nullptr;
    }
    this->_shapes.clear();

    for
        (auto mesh : this->_meshes)
    {
        delete mesh;
        mesh = nullptr;
    }
    this->_meshes.clear();
}

/**
 *
 */
bool ShapesAllocator::deleteShape(OGL::Shape* shape)
{
    std::lock_guard<std::mutex> scopeLock { this->_shapesMutex };

    auto found = std::find(this->_shapes.begin(),
                           this->_shapes.end(),
                           shape);
    if
        (this->_shapes.end() != found)
    {
        delete *found;
        *found = nullptr;
        this->_shapes.erase(found);
        return true;
    }
    return false;

}

/**
 *
 */
bool ShapesAllocator::release(OGL::Shape* shape)
{
    return ShapesAllocator::getInstance()->deleteShape(shape);
}

/**
 *
 */
bool ShapesAllocator::deleteMesh(ShapeMesh* mesh)
{
    std::lock_guard<std::mutex> scopeLock { this->_meshesMutex };

    auto found = std::find(this->_meshes.begin(),
                           this->_meshes.end(),
                           mesh);
    if
        (this->_meshes.end() != found)
    {
        delete *found;
        *found = nullptr;
        this->_meshes.erase(found);
        return true;
    }
    return false;
}

/**
 *
 */
bool ShapesAllocator::release(ShapeMesh* mesh)
{
    return ShapesAllocator::getInstance()->deleteMesh(mesh);
}
