// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/spherical_quad.h"

#include "3d/spherical_math.h"

SphericalQuad::SphericalQuad()
{
}

SphericalQuad::~SphericalQuad()
{
}

void SphericalQuad::setCornerPosition(unsigned int id, const glm::vec3& pos)
{
    _corners[id] = pos;
}

const glm::vec3& SphericalQuad::getCornerPosition(unsigned int id)
{
    return _corners[id];
}

glm::vec3 SphericalQuad::getRandomPoint()
{
    float f0 = static_cast<float>(rand()) / static_cast<float>(RAND_MAX);
    float f1 = static_cast<float>(rand()) / static_cast<float>(RAND_MAX);

    glm::vec3 a = slerp(_corners[0], _corners[2], f0);
    glm::vec3 c = slerp(_corners[3], _corners[1], f0);
    glm::vec3 e = slerp(a, c, f1);

    return e;
}

glm::vec3 SphericalQuad::getRandomGaussianPoint()
{
    std::random_device rd{};
    std::mt19937 gen{rd()};

    // values near the mean are the most likely
    // standard deviation affects the dispersion of generated values from the mean
    std::normal_distribution<> d{0.5, 0.15};

    float f0 = static_cast<float>(d(gen));
    float f1 = static_cast<float>(d(gen));

    glm::vec3 a = slerp(_corners[0], _corners[2], f0);
    glm::vec3 c = slerp(_corners[3], _corners[1], f0);
    glm::vec3 e = slerp(a, c, f1);

    return e;
}
