// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/texture_loader.h"

// File reader
#include <stb_image.h>

// OpenGL
#include "3d/texture.h"
#include "3d/context.h"
#include "core/data_type.h"
#include "core/log.h"
#include "data/image.h"

#include "vendors/_glad.h"
#include "vendors/_nanovg.h"

OGL::Texture* TextureLoader::loadFromFile(OGL::Context* ctx, const std::string& filePath)
{
    int w, h, c;

    unsigned char* data = stbi_load(filePath.c_str(), &w, &h, &c, 0);

    if 
        (nullptr == data)
    {
        LOG(SEAScope::LogLevel::Error, "TextureLoader",
                           "An error occurred during texture loading : {}, {}", stbi_failure_reason(), filePath);
        return nullptr;
    }

    ImageDesc desc;
    desc.width = static_cast<unsigned int>(w);
    desc.height = static_cast<unsigned int>(h);
    desc.channels = static_cast<unsigned int>(c);
    desc.dataType = DataType::unsigned_byte;

    Image img;
    img.setData(desc, data);
    OGL::Texture* tex = ctx->createTexture(OGL::texture2D);
    tex->create(ctx);
    tex->allocate2D(img);

    return tex;
}


int TextureLoader::loadFromFile(NVGcontext* nvgCtx, const std::string& filePath)
{
    return nvgCreateImage(nvgCtx, filePath.c_str(), 0);
}
