// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/unit_sphere.h"
#include "3d/math_func.h"


/**
 *
 */
static Vector3D GeodeticSurfaceNormal(const Geodetic3D& geodetic)
{
    double cosLatitude = cos(geodetic.Latitude());

    return Vector3D(
        cosLatitude * cos(geodetic.Longitude()),
        cosLatitude * sin(geodetic.Longitude()),
        sin(geodetic.Latitude()));
}


/**
 *
 */
Geodetic2D UnitSphere::ToGeodetic2D(const Vector3D& positionOnEllipsoid)
{
    Vector3D n = glm::normalize(positionOnEllipsoid);
    float azimuthAngle = atan2(n.y, n.x);
    float elevationAngle = asin(n.z / glm::length(n));
    return Geodetic2D(static_cast<double>(azimuthAngle),
                      static_cast<double>(elevationAngle));
}

/**
 *
 */
Vector3D UnitSphere::ToVector3D(const Geodetic2D& geodetic)
{
    return ToVector3D(Geodetic3D(geodetic.Longitude(), geodetic.Latitude(), 0.0));
}

/**
 *
 */
Vector3D UnitSphere::ToVector3D(const Geodetic3D& geodetic)
{
    Vector3D n = GeodeticSurfaceNormal(geodetic);
    return n * (1.0f + static_cast<float>(geodetic.Height()));
}
