// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   vertex_array_object.cpp
 * Author: gleseach
 * 
 * Created on 4 septembre 2015, 10:34
 */

#include "3d/stdafx.h"
#include "3d/vertex_array_object.h"

#include "3d/vertex_attrib_array.h"

namespace OGL
{

VertexArrayObject::VertexArrayObject()
{
}

VertexArrayObject::~VertexArrayObject()
{
    this->pipeline.clear();

    for
        (auto it : toDelete)
    {
        delete it;
        it = nullptr;
    }
    toDelete.clear();
}

void VertexArrayObject::addObject(BindableObject* object)
{
    this->pipeline.push_back(object);

    if (dynamic_cast<OGL::VertexAttribArray*>(object))
    {
        this->toDelete.push_back(object);
    }
}

void VertexArrayObject::fill()
{
    bind();

    for (BindableObject* obj : this->pipeline)
    {
        obj->bind();
    }

    unbind();
}


} // namespace OGL
