// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "core/base_input_events_subscriber.h"

using namespace SEAScope;


/**
 *
 */
BaseInputEventsSubscriber::BaseInputEventsSubscriber(const std::string& actorLabel)
: _actorId(0),
  _actorLabel(actorLabel),
  _active(false),
  _preemptEvents(false)
{

}

/**
 *
 */
BaseInputEventsSubscriber::~BaseInputEventsSubscriber()
{

}

/**
 *
 */
void BaseInputEventsSubscriber::setId(const ActorId& actorId)
{
    this->_actorId = actorId;
}

/**
 *
 */
const ActorId& BaseInputEventsSubscriber::getId() const
{
    return this->_actorId;
}

/**
 *
 */
bool BaseInputEventsSubscriber::isActive() const
{
    return this->_active;
}

/**
 *
 */
void BaseInputEventsSubscriber::setActive(bool value)
{
    this->_active = value;
}

/**
 *
 */
const std::set<ActionType>& BaseInputEventsSubscriber::getAllActions() const
{
    // For this one we can return a const ref as it will not change over time
    return this->_supportedActions;
}


/**
 *
 */
const std::set<ActionType> BaseInputEventsSubscriber::getPossibleActions() const
{
    // Return a copy to avoid concurrency issues for this one because its
    // content will change over time
    return this->_possibleActions;
}

/**
 *
 */
bool BaseInputEventsSubscriber::getActionLabel(const ActionType& actionType, std::string& result) const
{
    auto found = this->_actionLabels.find(actionType);
    if
        (this->_actionLabels.end() == found)
    {
        return false;
    }

    result = found->second;
    return true;
}

/**
 *
 */
bool BaseInputEventsSubscriber::getDefaultBindingDefinition(const ActionType& actionType, std::string& result) const
{
    auto found = this->_defaultBindings.find(actionType);
    if
        (this->_defaultBindings.end() == found)
    {
        return false;
    }

    result = found->second;
    return true;
}

/**
 *
 */
bool BaseInputEventsSubscriber::preemptEvents() const
{
    return this->_preemptEvents;
}

/**
 *
 */
bool BaseInputEventsSubscriber::getActorLabel(std::string& result) const
{
    result = this->_actorLabel;
    return true;
}


/**
 *
 */
void BaseInputEventsSubscriber::defineAction(const ActionType& actionType,
                                             const std::string& actionLabel,
                                             const std::string& actionBinding)
{
    this->_supportedActions.insert(actionType);
    this->_actionLabels.emplace(actionType, actionLabel);
    this->_defaultBindings.emplace(actionType, actionBinding);
}
