// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <fstream>

#include "core/log.h"
#include "core/colormap_manager.h"

using namespace SEAScope;

/**
 *
 */
ColormapManager::ColormapManager()
{

};

/**
 *
 */
ColormapManager::~ColormapManager()
{

};

/**
 *
 */
bool ColormapManager::loadColormap(const std::string& path)
{
	int r;	//
	int g;	//
	int b;	//
	std::ifstream f;	//
	Colormap cmap;	//
	std::string base;	//
	std::string label;	//

	f.open(path.c_str(), std::fstream::binary);
	if
		(false == f.is_open())
	{
		LOG(LogLevel::Error, "ColormapManager", "Could not read colormap from {}", path);
		return false;
	}

	base = path.substr(path.find_last_of("/\\") + 1);
	label = base.substr(0, base.find_last_of('.'));
	while
		(false == f.eof())
	{
		f >> r >> g >> b;

		if
			(false == f.eof())
		{
			cmap.push_back({{ static_cast<uint8_t>(r), static_cast<uint8_t>(g), static_cast<uint8_t>(b) }});
		}
	}
	f.close();

	this->store.insert(std::make_pair(label, cmap));

	return true;
};

/**
 *
 */
bool ColormapManager::getColormap(const std::string& label, Colormap& result) const
{
	auto found = this->store.find(label);
	if
		(this->store.end() == found)
	{
		return false;
	}

	result = found->second;
	return true;
};

/**
 *
 */
const std::map<std::string, Colormap>& ColormapManager::getColormaps() const
{
	return this->store;
};
