// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <cstring>

#include "vendors/_glm.h"
#include "core/copy.h"

void memcpy2D(void* src, unsigned int src_origin_x, unsigned int src_origin_y, unsigned int src_width, unsigned int src_height, unsigned int element_size, 
              void* dst, unsigned int dst_origin_x, unsigned int dst_origin_y, unsigned int dst_width, unsigned int dst_height,
              unsigned int area_width, unsigned area_height)
{
    const unsigned int src_row_size = src_width * element_size;
    const unsigned int dst_row_size = dst_width * element_size;
    const unsigned int area_row_size = area_width * element_size;

    if (src_origin_x + area_width > src_width)
    {
        std::abort();
    }
    if (src_origin_y + area_height > src_height)
    {
        std::abort();
    }
    if (dst_origin_x + area_width > dst_width)
    {
        std::abort();
    }
    if (dst_origin_y + area_height > dst_height)
    {
        std::abort();
    }

    char* src_origin = static_cast<char*>(src) + ((src_origin_y * src_width + src_origin_x) * element_size);
    char* dst_origin = static_cast<char*>(dst) + ((dst_origin_y * dst_width + dst_origin_x) * element_size);

    for (unsigned int y = 0; y < area_height; ++y)
    {
        memcpy(dst_origin + dst_row_size * y, src_origin + src_row_size * y, area_row_size);
    }
}

size_t fillBuffer(void* dstBuffer, size_t dstBufferSize, void* srcPattern, size_t srcPatternSize)
{
    unsigned char* dst = static_cast<unsigned char*>(dstBuffer);

    size_t remaining = dstBufferSize;
    size_t size = srcPatternSize;
    size_t count = 0;

    while (remaining > 0)
    {
        if (remaining < srcPatternSize)
        {
            size = remaining;
        }

        memcpy(dst, srcPattern, size);
        dst += size;
        remaining -= size;
        ++count;
    }

    return count;
}

