// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.

#include "core/data_bucket.h"

using namespace SEAScope;

/**
 *
 */
DataBucket::DataBucket()
: _loaded{false}, _missing{false}
{
}

/**
 *
 */
DataBucket::~DataBucket()
{
}

Image& DataBucket::getImage()
{
    return this->_image;
}

/**
 *
 */
DataInfo& DataBucket::getInfo()
{
    return this->_info;
}

/**
 *
 */
const Image& DataBucket::getImage() const
{
    return this->_image;
}

/**
 *
 */
const Image* DataBucket::getImagePtr() const
{
    return &(this->_image);
}

/**
 *
 */
const DataInfo& DataBucket::getInfo() const
{
    return this->_info;
}

/**
 *
 */
const DataInfo* DataBucket::getInfoPtr() const
{
    return &(this->_info);
}

/**
 *
 */
void DataBucket::setLoaded(bool value)
{
    std::lock_guard<std::mutex> lock{this->_mutex};
    this->_loaded = value;
}

/**
 *
 */
bool DataBucket::loaded() const
{
    std::lock_guard<std::mutex> lock{this->_mutex};
    return this->_loaded;
}

/**
 *
 */
void DataBucket::setMissing(bool value)
{
    std::lock_guard<std::mutex> lock{this->_mutex};
    this->_missing = value;
}

/**
 *
 */
bool DataBucket::isMissing() const
{
    std::lock_guard<std::mutex> lock{this->_mutex};
    return this->_missing;
}
