// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "core/data_node.h"

#include "core/log.h"

using namespace SEAScope;

/**
 *
 */
DataNode::~DataNode()
{
}

/**
 *
 */
EmptyNode::~EmptyNode()
{
}

/**
 *
 */
DictNode::~DictNode()
{
    for
        (auto& it : this->children)
    {
        delete it.second;
        it.second = nullptr;
    }
    this->children.clear();
}

/**
 *
 */
ListNode::~ListNode()
{
    for
        (auto& it : this->children)
    {
        delete it;
        it = nullptr;
    }
    this->children.clear();
}

/**
 *
 */
BoolNode::~BoolNode()
{
}

/**
 *
 */
NumberNode::~NumberNode()
{
}

/**
 *
 */
TextNode::~TextNode()
{
}

/**
 *
 */
ImageNode::~ImageNode()
{
}

bool SEAScope::testDataNode(DictNode& result)
{
    ListNode* list = new ListNode();

    list->children.emplace_back(new BoolNode(false));
    list->children.emplace_back(new NumberNode(1.5));
    list->children.emplace_back(new TextNode("fzefze"));

    result.children.emplace(std::make_pair("text", new TextNode("qksjberkfsbkr")));
    result.children.emplace(std::make_pair("scalar", new NumberNode(5.0)));
    result.children.emplace(std::make_pair("example,", list));
    return false;
}

void SEAScope::printDataNode(const DataNode& node)
{
    switch
        (node.getType())
    {
        case NodeType::empty:
            {
                LOG(LogLevel::Debug, "DataNodeTest", "Null value");
            }
            break;
        case NodeType::boolean:
            {
                const BoolNode& boolNode = static_cast<const BoolNode&>(node);
                LOG(LogLevel::Debug, "DataNodeTest", "Bool value: {}", boolNode.value);
            }
            break;
        case NodeType::number:
            {
                const NumberNode& numberNode = static_cast<const NumberNode&>(node);
                LOG(LogLevel::Debug, "DataNodeTest", "Number value: {}", numberNode.value);
            }
            break;
        case NodeType::text:
            {
                const TextNode& textNode = static_cast<const TextNode&>(node);
                LOG(LogLevel::Debug, "DataNodeTest", "Text value: {}", textNode.value);
            }
            break;
        case NodeType::image:
            {
                const ImageNode& imageNode = static_cast<const ImageNode&>(node);
                LOG(LogLevel::Debug, "DataNodeTest", "Image value: {}", imageNode.value);
            }
            break;
        case NodeType::list:
            {
                const ListNode& listNode = static_cast<const ListNode&>(node);
                LOG(LogLevel::Debug, "DataNodeTest", "List");
                for
                    (const auto& it : listNode.children)
                {
                    printDataNode(*it);
                }
            }
            break;
        case NodeType::dict:
            {
                const DictNode& dictNode = static_cast<const DictNode&>(node);
                LOG(LogLevel::Debug, "DataNodeTest", "Dict");
                for
                    (const auto& it : dictNode.children)
                {
                    LOG(LogLevel::Debug, "DataNodeTest", "*** {} ***", it.first);
                    printDataNode(*it.second);
                }
            }
            break;
    }
}
