// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "core/input_binding.h"
#include "core/input_event.h"

using namespace SEAScope;

/**
 *
 */
InputBinding::InputBinding()
{

}

/**
 *
 */
InputBinding::InputBinding(InputBinding&& src) noexcept
{
    this->_events = std::move(src._events);
}

/**
 *
 */
InputBinding::~InputBinding()
{

}

/**
 *
 */
void InputBinding::addEvent(InputEvent&& event)
{
    this->_events.push_back(std::move(event));
}

/**
 *
 */
const std::vector<InputEvent>& InputBinding::getEvents() const
{
    return this->_events;
}

/**
 *
 */
bool InputBinding::isValid() const
{
    return (0 < this->_events.size());
}

/**
 *
 */
void InputBinding::clear()
{
    this->_events.clear();
}
