// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <algorithm>

#include "core/path.h"
#include "core/utils.h"

using namespace SEAScope;

IPathHandler* Path::implementation = nullptr;

/**
 * Check if a file exists.
 */
bool Path::fileExists(const std::string& path)
{
	/* TODO: assert implementation */
	return Path::implementation->fileExists(path);
};

/**
 *
 */
bool Path::fileSize(const std::string& path, std::int64_t& size)
{
	return Path::implementation->fileSize(path, size);
};

/**
 *
 */
bool Path::writeFile(const std::string& path)
{
	return Path::implementation->writeFile(path);
};

/**
 * Check if a directory exists.
 */
bool Path::dirExists(const std::string& path)
{
	/* TODO: assert implementation */
	return Path::implementation->dirExists(path);
}

/**
 * Create directory
 */
bool Path::createDir(const std::string& path, mode_t mode)
{
	/* TODO: assert implementation */
	return Path::implementation->createDir(path, mode);
};


/**
 * Create directory tree.
 */
bool Path::createDirs(const std::string& path, mode_t mode)
{
	/* TODO: assert implementation */
	return Path::implementation->createDirs(path, mode);
};

/**
 *
 */
void Path::splitPaths(const std::string& paths, const std::string& delim, std::forward_list<std::string>& result)
{
	return Path::implementation->splitPaths(paths, delim, result);
};

/**
 *
 */
void Path::joinPaths(const std::vector<std::string>& paths, std::string& result)
{
	return Path::implementation->joinPaths(paths, result);
}

/**
 *
 */
bool Path::listFiles(const std::string& path, std::forward_list<std::string>& result)
{
	return Path::implementation->listFiles(path, result);
};

/**
 *
 */
bool Path::listFilesRecursive(const std::string& path, std::forward_list<std::string>& result)
{
	return Path::implementation->listFilesRecursive(path, result);
};

/**
 *
 */
bool Path::deleteFile(const std::string& path)
{
	return Path::implementation->deleteFile(path);
};

/**
 *
 */
char Path::separator()
{
	return Path::implementation->separator();
}

/**
 *
 */
void Path::setPathValid(std::string& str)
{
	Path::implementation->fixPlatformPath(str);
}

/**
 *
 */
bool Path::lastModified(const std::string& path, std::time_t& result)
{
	return Path::implementation->lastModified(path, result);
};

/**
 *
 */
bool Path::currentWorkingDir(std::string& result)
{
    return Path::implementation->currentWorkingDir(result);
}

/**
 *
 */
bool Path::canonicalPath(const std::string& inputPath, std::string& outputPath)
{
    return Path::implementation->canonicalPath(inputPath, outputPath);
}

/**
 *
 */
bool Path::homeDir(std::string& outputPath)
{
    return Path::implementation->homeDir(outputPath);
}

/**
 *
 */
bool Path::absolutePath(const std::string& path, std::string& absolutePath)
{
    return Path::implementation->absolutePath(path, absolutePath);
}

/**
 *
 */
bool Path::getParentDir(const std::string& inputPath, std::string& outputPath)
{
    return Path::implementation->getParentDir(inputPath, outputPath);
}
