// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "core/path_tools.h"
#include "core/path.h"

std::string extension(const std::string& path)
{
    std::string result; //

    int i = static_cast<int>(path.length()) - 1;
    for (; i >= 0; --i)
    {
        std::size_t index = static_cast<std::size_t>(i);

        if ('/' == path[index] || '\\' == path[index])
        {
            break;
        }
        else if ('.' == path[index])
        {
            result = path.substr(index + 1, std::string::npos);
            break;
        }
    }

    return result;
}

std::string filename(const std::string& path)
{
    std::string result; //

    int i = static_cast<int>(path.length()) - 1;
    for (; i >= 0; --i)
    {
        std::size_t index = static_cast<std::size_t>(i);

        if ('/' == path[index] || '\\' == path[index])
        {
            result = path.substr(index + 1, std::string::npos);
            break;
        }
    }

    if
        (false == result.empty())
    {
        return result;
    }
    else
    {
        return path;
    }
}

void addEndingSeparator(std::string& path)
{
    if
        (false == path.empty())
    {
        auto it = path.rbegin();

        if
            ('/' != *it && '\\' != *it)
        {
            path += SEAScope::Path::separator();
        }
    }
}
