// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.

#include "core/raw_to_renderable_queue.h"

using namespace SEAScope;

/**
 *
 */
RawToRenderableQueue::RawToRenderableQueue()
{
}

/**
 *
 */
RawToRenderableQueue::~RawToRenderableQueue()
{
}

/**
 *
 */
bool RawToRenderableQueue::addJob(const CollectionId& collection, const GranuleId& granule, const std::string& uri, float resolution, const std::set<std::string>&& fields)
{
    RawToRenderableJob j;   //

    j.collection = collection;
    j.granule = granule;
    j.uri = uri;
    j.fields = fields;
    j.resolution = resolution;

	std::unique_lock<std::mutex> mlock(this->mutex);
	this->queue.push_back(j);
	mlock.unlock();
	this->cond.notify_one();

    return true;
}

/**
 *
 */
void RawToRenderableQueue::shutdown()
{
    RawToRenderableJob j;   //

    j.collection = {0, 0};
    j.granule = {0, 0};
    j.uri = "";
    j.fields = {};
    j.resolution = -1.f;

	std::unique_lock<std::mutex> mlock(this->mutex);
	this->queue.clear();
    this->queue.push_back(j);
	mlock.unlock();
	this->cond.notify_one();
}
