// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "events/collections_update_requested_event.h"

using namespace SEAScope;

CollectionsUpdateRequestedEvent::CollectionsUpdateRequestedEvent(const std::set<CollectionId>& toAdd, const std::set<CollectionId>& toRemove, const std::set<CollectionId>& toUpdate)
: _toAdd(toAdd), _toRemove(toRemove), _toUpdate(toUpdate)
{

}

CollectionsUpdateRequestedEvent::~CollectionsUpdateRequestedEvent()
{
    this->_toAdd.clear();
    this->_toRemove.clear();
    this->_toUpdate.clear();
}

const std::set<CollectionId>& CollectionsUpdateRequestedEvent::getToAdd() const
{
    return _toAdd;
}

const std::set<CollectionId>& CollectionsUpdateRequestedEvent::getToRemove() const
{
    return this->_toRemove;
}

const std::set<CollectionId>& CollectionsUpdateRequestedEvent::getToUpdate() const
{
    return this->_toUpdate;
}
