// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "events/cursor_location_changed_event.h"

using namespace SEAScope;

CursorLocationChangedEvent::CursorLocationChangedEvent(std::uint32_t screenX, std::uint32_t screenY, float lon, float lat, bool onWorld, const glm::vec3& hitpoint)
: _screenX(screenX),
  _screenY(screenY),
  _lon(lon),
  _lat(lat),
  _hitpoint(hitpoint),
  _onWorld(onWorld)
{

}

CursorLocationChangedEvent::~CursorLocationChangedEvent()
{

}

std::uint32_t CursorLocationChangedEvent::getScreenX() const
{
    return this->_screenX;
}

std::uint32_t CursorLocationChangedEvent::getScreenY() const
{
    return this->_screenY;
}

float CursorLocationChangedEvent::getLon() const
{
    return this->_lon;
}

float CursorLocationChangedEvent::getLat() const
{
    return this->_lat;
}

bool CursorLocationChangedEvent::getOnWorld() const
{
    return this->_onWorld;
}

const glm::vec3& CursorLocationChangedEvent::getHitpoint() const
{
    return this->_hitpoint;
}
