// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "events/load_annotations_from_file_request.h"

using namespace SEAScope;

/**
 *
 */
LoadAnnotationsFromFileRequest::LoadAnnotationsFromFileRequest(const std::string& path, const std::map<ShapeType, CollectionId>& mapping, const AnnotationFormat& importFormat, double defaultStart, double defaultStop, const std::set<CollectionId>& selectedCollections)
: _path(path),
  _mapping(mapping),
  _importFormat(importFormat),
  _defaultStart(defaultStart),
  _defaultStop(defaultStop),
  _selectedCollections(selectedCollections)
{

}

/** 
 *
 */
LoadAnnotationsFromFileRequest::~LoadAnnotationsFromFileRequest()
{

}

/**
 *
 */
const std::string& LoadAnnotationsFromFileRequest::getPath() const
{
    return this->_path;
}

/**
 *
 */
const std::map<ShapeType, CollectionId>& LoadAnnotationsFromFileRequest::getCollectionsMapping() const
{
    return this->_mapping;
}

/**
 *
 */
const AnnotationFormat& LoadAnnotationsFromFileRequest::getImportFormat() const
{
    return this->_importFormat;
}

/**
 *
 */
double LoadAnnotationsFromFileRequest::getDefaultStart() const
{
    return this->_defaultStart;
}

/**
 *
 */
double LoadAnnotationsFromFileRequest::getDefaultStop() const
{
    return this->_defaultStop;
}

/**
 *
 */
const std::set<CollectionId>& LoadAnnotationsFromFileRequest::getSelectedCollections() const
{
    return this->_selectedCollections;
}
