// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "events/save_annotations_from_renderable_ids_to_file_request.h"

using namespace SEAScope;

/**
 *
 */
SaveAnnotationsFromRenderableIdsToFileRequest::SaveAnnotationsFromRenderableIdsToFileRequest(const std::string& path, const std::set<RenderableId>& rIds, const std::map<ShapeType, CollectionId>& mapping, const AnnotationFormat& exportFormat)
: _path(path),
  _rIds(rIds),
  _mapping(mapping),
  _exportFormat(exportFormat)
{

}

/**
 *
 */
SaveAnnotationsFromRenderableIdsToFileRequest::~SaveAnnotationsFromRenderableIdsToFileRequest()
{

}

/**
 *
 */
const std::string& SaveAnnotationsFromRenderableIdsToFileRequest::getPath() const
{
    return this->_path;
}

/**
 *
 */
const std::map<ShapeType, CollectionId>& SaveAnnotationsFromRenderableIdsToFileRequest::getCollectionsMapping() const
{
    return this->_mapping;
}

/**
 *
 */
const AnnotationFormat& SaveAnnotationsFromRenderableIdsToFileRequest::getExportFormat() const
{
    return this->_exportFormat;
}

/**
 *
 */
const std::set<RenderableId>& SaveAnnotationsFromRenderableIdsToFileRequest::getRenderableIds() const
{
    return this->_rIds;
}
