// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "events/save_one_annotation_to_file_request.h"
#include "core/data_node.h"

using namespace SEAScope;

/**
 *
 */
SaveOneAnnotationToFileRequest::SaveOneAnnotationToFileRequest(const RenderableId& rId, const std::string& path, const AnnotationFormat& exportFormat, const DictNode* volatileProperties)
: _renderableId(rId),
  _path(path),
  _exportFormat(exportFormat),
  _volatileProperties(volatileProperties)
{

}

/**
 *
 */
SaveOneAnnotationToFileRequest::~SaveOneAnnotationToFileRequest()
{
    // Volatile properties are not allocated by this class, but their lifetime
    // should end once the event has been processed
    delete this->_volatileProperties;
}

/**
 *
 */
const RenderableId& SaveOneAnnotationToFileRequest::getRenderableId() const
{
    return this->_renderableId;
}

/**
 *
 */
const std::string& SaveOneAnnotationToFileRequest::getPath() const
{
    return this->_path;
}

/**
 *
 */
const AnnotationFormat& SaveOneAnnotationToFileRequest::getExportFormat() const
{
    return this->_exportFormat;
}

/**
 *
 */
const DictNode* SaveOneAnnotationToFileRequest::getVolatileProperties() const
{
    return this->_volatileProperties;
}
