// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "events/selected_pickable_with_info_changed_event.h"

using namespace SEAScope;

SelectedPickableWithInfoChangedEvent::SelectedPickableWithInfoChangedEvent(Pickable* pickable, const GranuleInfo& info, const RenderableType& rType)
: _pickable(pickable), _info(info), _rType(rType), _start(0.0), _stop(0.0)
{

}

SelectedPickableWithInfoChangedEvent::SelectedPickableWithInfoChangedEvent(Pickable* pickable, const GranuleInfo& info, const RenderableType& rType, DictNode* properties, double start, double stop)
: _pickable(pickable), _info(info), _rType(rType), _properties(std::move(*properties)), _start(start), _stop(stop)
{
}

SelectedPickableWithInfoChangedEvent::~SelectedPickableWithInfoChangedEvent()
{

}

Pickable* SelectedPickableWithInfoChangedEvent::getPickable() const
{
    return this->_pickable;
}

const GranuleInfo& SelectedPickableWithInfoChangedEvent::getInfo() const
{
    return this->_info;
}

const RenderableType& SelectedPickableWithInfoChangedEvent::getRenderableType() const
{
    return this->_rType;
}

DictNode& SelectedPickableWithInfoChangedEvent::getProperties()
{
    return this->_properties;
}

double SelectedPickableWithInfoChangedEvent::getStart() const
{
    return this->_start;
}

double SelectedPickableWithInfoChangedEvent::getStop() const
{
    return this->_stop;
}
