// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "local/collections_catalogue.h"

using namespace SEAScope;

/**
 *
 */
CollectionsCatalogue::CollectionsCatalogue()
{

};

/**
 *
 */
CollectionsCatalogue::~CollectionsCatalogue()
{
    for
        (auto collection : this->database)
    {
        for
            (auto it: std::get<1>(collection.second))
        {
            delete it;
            it = nullptr;
        }
        std::get<1>(collection.second).clear();
    }
    this->database.clear();
    this->ids.clear();
};

/**
 *
 */
bool CollectionsCatalogue::listCollections(std::set<std::uint32_t>& result) const
{
	result.clear();
	for
		(auto& it : this->database)
	{
		result.insert(it.first);
	}
	return true;
};

/**
 *
 */
bool CollectionsCatalogue::hasCollection(const std::string& path) const
{
	auto found = this->ids.find(path);
	return this->ids.end() != found;
};

/**
 *
 */
bool CollectionsCatalogue::hasCollection(std::uint32_t colId) const
{
	auto found = this->database.find(colId);
	return this->database.end() != found;
};

/**
 *
 */
bool CollectionsCatalogue::addCollection(std::uint32_t colId, const CollectionCfg&& cfg, const std::vector<RenderingCfg*>&& renderings, const std::string& path)
{
	this->ids[path] = colId;
	this->database[colId] = std::make_pair(cfg, renderings);
	return true;
};

/**
 *
 */
bool CollectionsCatalogue::getCollection(std::uint32_t colId, CollectionCfg& cfg, std::vector<RenderingCfg*>& renderings) const
{
	auto found = this->database.find(colId);
	if
		(this->database.end() == found)
	{
		return false;
	}

	std::tie(cfg, renderings) = found->second;
	return true;
};
