// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <iostream>
#include "simplelogger.h"

using namespace SEAScope;

/**
 *
 */
SimpleLogger::SimpleLogger()
{

};

/**
 *
 */
SimpleLogger::~SimpleLogger()
{

};

/**
 *
 */
bool SimpleLogger::log(const LogLevel& level, const char* logger, const string_view& message)
{
	if
		(LogLevel::Debug == level
			|| LogLevel::Info == level)
	{
		std::cout << "[" << logger << "] " << message.data() << std::endl;
		return true;
	}
	else if
		(LogLevel::Warn == level
			|| LogLevel::Error == level
			|| LogLevel::Fatal == level)
	{
		std::cerr << "\033[1;31m[" << logger << "] " << message.data() << "\033[0m" << std::endl;
		return true;
	}

	return false;
};

/**
 *
 */
bool SimpleLogger::isMessageValid(const LogLevel&, const string_view&)
{
	return true;
}

LogLevel SimpleLogger::getLevel() const
{
	return LogLevel::Debug;
}
