// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "core/log.h"
#include "sink_eventhandler.h"

using namespace SEAScope;

SinkEventHandler::SinkEventHandler(): EventHandler()
{

};

SinkEventHandler::~SinkEventHandler()
{

};

bool SinkEventHandler::handleKeyEvent(int,int,int,int)
{
    LOG(LogLevel::Warn, "Sink event handler", "Key event not captured");
	return false;
};

bool SinkEventHandler::handleCharEvent(unsigned int, int)
{
    LOG(LogLevel::Warn, "Sink event handler", "Char event not captured");
	return false;
};

bool SinkEventHandler::handleMouseButtonEvent(int, int, int)
{
    LOG(LogLevel::Warn, "Sink event handler", "Mouse button event not captured");
	return false;
};

bool SinkEventHandler::handleMouseMoveEvent(double, double)
{
	/* Silenced, too verbose, uncomment only if needed */
	return false;
};

bool SinkEventHandler::handleMouseWheelEvent(double,double)
{
    LOG(LogLevel::Warn, "Sink event handler", "Mouse wheel event not captured");
	return false;
};

bool SinkEventHandler::handleResizeEvent(int, int, int, int)
{
    LOG(LogLevel::Warn, "Sink event handler", "Resize event not captured");
	return false;
};

bool SinkEventHandler::handleFocusEvent(int)
{
    LOG(LogLevel::Warn, "Sink event handler", "Focus event not captured");
	return false;
};

bool SinkEventHandler::handleIconifyEvent(int)
{
    LOG(LogLevel::Warn, "Sink event handler", "Iconify event not captured");
	return false;
};

bool SinkEventHandler::handleDropEvent(int,const char**)
{
    LOG(LogLevel::Warn, "Sink event handler", "Drop event not captured");
	return false;
};
