// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "tools/scene_view_controller_base.h"

#include "core/app_events_dispatcher.h"
#include "3d/default_value.h"


namespace SEAScope
{

SceneViewControllerBase::SceneViewControllerBase(const std::string& name, AppEventsDispatcher& appEventsDispatcher)
:	BaseInputEventsSubscriber(name),
	_appEventsDispatcher(appEventsDispatcher),
	_sceneView(nullptr)
{
}

SceneViewControllerBase::~SceneViewControllerBase()
{
}

SceneView* SceneViewControllerBase::getSceneView()
{
	return _sceneView;
}

void SceneViewControllerBase::setSceneView(SceneView* sceneView)
{
	_sceneView = sceneView;
	this->setZoom(GLOBE_ZOOM_DEFAULT);
}

} // namespace SEAScope

