// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "tools/timeline_controller.h"


#include "core/log.h"
#include "core/input_event.h"
#include "core/input_devices_state.h"
#include "core/interfaces/iwindow.h"
#include "core/app_events_dispatcher.h"
#include "events/shift_to_prev_time_request.h"
#include "events/shift_to_next_time_request.h"

using namespace SEAScope;

/**
 *
 */
TimelineController::TimelineController(AppEventsDispatcher& appEventsDispatcher)
: BaseInputEventsSubscriber("Timeline controller"),
  _appEventsDispatcher(appEventsDispatcher)
{
    // Configure actions

    // - Quit application
    this->defineAction(static_cast<ActionType>(TimelineControllerActions::SHIFT_PAST),
                       "Next time step", "}LEFT");  /* keypress: <- */
    this->defineAction(static_cast<ActionType>(TimelineControllerActions::SHIFT_FUTURE),
                       "Next time step", "}RIGHT");  /* keypress: -> */

    this->_possibleActions = {static_cast<ActionType>(TimelineControllerActions::SHIFT_PAST),
                              static_cast<ActionType>(TimelineControllerActions::SHIFT_FUTURE)};
}

/**
 *
 */
TimelineController::~TimelineController()
{
}

/**
 *
 */
bool TimelineController::isActive() const
{
    return this->_active;
}

/**
 *
 */
bool TimelineController::performAction(const ActionType& actionType,
                                       const InputEvent& /*event*/,
                                       const InputDevicesState& /*devicesState*/)
{
    if
        (static_cast<ActionType>(TimelineControllerActions::SHIFT_PAST) == actionType)
    {
        LOG(LogLevel::Debug, "TimelineController", "Shift past");
        ShiftToPrevTimeRequest outEvent {};
        this->_appEventsDispatcher.dispatch<ShiftToPrevTimeRequest>(outEvent);
        return true;
    }
    else if
        (static_cast<ActionType>(TimelineControllerActions::SHIFT_FUTURE) == actionType)
    {
        LOG(LogLevel::Debug, "TimelineController", "Shift future");
        ShiftToNextTimeRequest outEvent {};
        this->_appEventsDispatcher.dispatch<ShiftToNextTimeRequest>(outEvent);
        return true;
    }

    return false;
}
