// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <sstream>

#include "ui/nanogui/nanogui_about_pane.h"
#include "ui/nanogui/nanogui_selectable_label.h"

#if defined __linux__ || defined __APPLE__
	#include "seascope_version.h"
#endif

using namespace SEAScope;

/**
 *
 */
NanoguiAboutPane::NanoguiAboutPane(nanogui::Widget* parent)
: nanogui::Widget(parent), NanoguiScrollableWithTitle(parent)
{
    nanogui::Widget* wrap {nullptr};
    nanogui::Widget* spacer {nullptr};
    nanogui::Label* lbl {nullptr};

    this->header->setFixedHeight(10);
    this->footer->setFixedHeight(5);

    this->scrollable->content->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                                nanogui::Alignment::Minimum,
                                                                8, 0));

    wrap = new nanogui::Widget(this->scrollable->content);
	wrap->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										   , nanogui::Alignment::Minimum
										   , 0, 0));
	wrap->setFixedWidth(304);
	wrap->setWidth(304);

    new nanogui::Label(wrap, "SEAScope is a 3D visualisation & analysis application", "sans-bold");
    new nanogui::Label(wrap, "for satellite, in-situ and numerical model data.", "sans-bold");

    spacer = new nanogui::Widget(wrap);
    spacer->setFixedHeight(3);

    new nanogui::Label(wrap, "Copyright (C) 2014-2025 OceanDataLab", "sans-bold");
    new nanogui::Label(wrap, "License LGPLv3+", "sans-bold");

    spacer = new nanogui::Widget(wrap);
    spacer->setFixedHeight(3);

    new nanogui::Label(wrap, "New features and improvements have been sponsored", "sans-bold");
    new nanogui::Label(wrap, "by BPI FRANCE and ESA SEOM", "sans-bold");

    spacer = new nanogui::Widget(wrap);
    spacer->setFixedHeight(30);

    lbl = new nanogui::Label(wrap, "Getting help", "sans-bold");
    lbl->setFontSize(20);

    new nanogui::Label(wrap, "Please consult documentation, FAQ and tutorials", "sans-bold");
    new nanogui::Label(wrap, "on the SEAScope website, they may provide some", "sans-bold");
    new nanogui::Label(wrap, "useful information :).", "sans-bold");
    NanoguiSelectableLabel* tbx = new NanoguiSelectableLabel(wrap);
    tbx->setValue("https://seascope.oceandatalab.com");
    tbx->setFixedWidth(304);
    tbx->setFixedHeight(30);
    tbx->setFontSize(18);
    tbx->setAlignment(nanogui::TextBox::Alignment::Center);

    spacer = new nanogui::Widget(wrap);
    spacer->setFixedHeight(10);

    new nanogui::Label(wrap, "You can also ask questions on the forum:", "sans-bold");

    tbx = new NanoguiSelectableLabel(wrap);
    tbx->setValue("https://forum.oceandatalab.com/seascope");
    tbx->setFixedWidth(304);
    tbx->setFixedHeight(30);
    tbx->setFontSize(18);
    tbx->setAlignment(nanogui::TextBox::Alignment::Center);

    spacer = new nanogui::Widget(wrap);
    spacer->setFixedHeight(10);

    new nanogui::Label(wrap, "If your question cannot be disclosed publicly, you can", "sans-bold");
    new nanogui::Label(wrap, "contact us be email at:", "sans-bold");

    tbx = new NanoguiSelectableLabel(wrap);
    tbx->setValue("seascope@oceandatalab.com");
    tbx->setFixedWidth(304);
    tbx->setFixedHeight(30);
    tbx->setFontSize(18);
    tbx->setAlignment(nanogui::TextBox::Alignment::Center);


    #if defined __linux__ || defined __APPLE__
    spacer = new nanogui::Widget(wrap);
    spacer->setFixedHeight(30);

    lbl = new nanogui::Label(wrap, "Build information", "sans-bold");
    lbl->setFontSize(20);
    lbl->setFixedHeight(30);

    for
        (const auto& it : SEAScope::getVersion())
    {
        if
            (  "date" != it.first
            && "ref" != it.first
            && "glm_version" != it.first
            && "glibc_version" != it.first
            && "stdc++_version" != it.first)
        {
            continue;
        }
        std::ostringstream buffer {""};
        buffer << it.first << ": " << it.second << "\n";
        new nanogui::Label(wrap, buffer.str(), "sans-bold");
    }
    #endif
}

/**
 *
 */
NanoguiAboutPane::~NanoguiAboutPane()
{
}
