// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "ui/nanogui/nanogui_annotations_collection_creator.h"

#include "ui/nanogui/nanogui_button_bar.h"
#include "ui/nanogui/nanogui_textfield.h"
#include "core/log.h"

using namespace SEAScope;

/**
 *
 */
NanoguiAnnotationsCollectionCreator::NanoguiAnnotationsCollectionCreator(nanogui::Widget* parent)
: nanogui::Widget(parent),
  _collectionType(nullptr),
  _collectionLabel(nullptr),
  _variableLabel(nullptr),
  _okButton(nullptr),
  _nokButton(nullptr),
  _topBlock(nullptr),
  _bottomBlock(nullptr),
  _validateCallback(nullptr),
  _cancelCallback(nullptr)
{
    Widget* spacer(nullptr);

    this->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                           nanogui::Alignment::Minimum,
                                           0, 0));

    this->_topBlock = new nanogui::Widget(this);
    this->_topBlock->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                      nanogui::Alignment::Middle,
                                                      0, 0));
    new nanogui::Label(this->_topBlock, "New collection", "sans-bold", 16);

    nanogui::Widget* shapeTypeBlock = new nanogui::Widget(this);
    shapeTypeBlock->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                     nanogui::Alignment::Middle,
                                                     3, 2));
    new nanogui::Label(shapeTypeBlock, "Annotations type");
    this->_collectionType = new NanoguiButtonBar(shapeTypeBlock);
    this->_collectionType->init({"point", "polyline", "polygon"},
                                {"Create a point collection", "Create a polyline collection", "Create a polygon collection"},
                                16, Eigen::Vector2i(60, 18));
    this->_collectionType->setSelectionChangedCallback([this](const std::string& value) {
        if
            (0 < value.size() * this->_collectionLabel->value().size() * this->_variableLabel->value().size())
        {
            this->_okButton->setEnabled(true);
        }
        else
        {
            this->_okButton->setEnabled(false);
        }
    });

    nanogui::Widget* labelBlock(nullptr);
    labelBlock = new nanogui::Widget(this);
    labelBlock->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                 nanogui::Alignment::Minimum,
                                                 3, 2));
    spacer = new nanogui::Widget(labelBlock);
    spacer->setFixedSize(Eigen::Vector2i(7, 24));

    new nanogui::Label(labelBlock, "Collection label");

    this->_collectionLabel = new NanoguiTextField(labelBlock);
    this->_collectionLabel->setAlignment(nanogui::TextBox::Alignment::Center);
    this->_collectionLabel->setFontSize(18);
    this->_collectionLabel->setFixedSize(Eigen::Vector2i(100, 20));
    this->_collectionLabel->setPlaceholder("Please provide a label");
    this->_collectionLabel->setCallback([this] (const std::string& value) -> bool{
        if
            (0 < value.size() * this->_collectionType->selectedValue().size() * this->_variableLabel->value().size())
        {
            this->_okButton->setEnabled(true);
        }
        else
        {
            this->_okButton->setEnabled(false);
        }
        return true;
    });

    labelBlock = new nanogui::Widget(this);
    labelBlock->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                 nanogui::Alignment::Minimum,
                                                 3, 2));
    spacer = new nanogui::Widget(labelBlock);
    spacer->setFixedSize(Eigen::Vector2i(16, 24));

    new nanogui::Label(labelBlock, "Variable label");

    this->_variableLabel = new NanoguiTextField(labelBlock);
    this->_variableLabel->setAlignment(nanogui::TextBox::Alignment::Center);
    this->_variableLabel->setFontSize(18);
    this->_variableLabel->setFixedSize(Eigen::Vector2i(100, 20));
    this->_variableLabel->setPlaceholder("Please provide a label");
    this->_variableLabel->setCallback([this] (const std::string& value) -> bool{
        if
            (0 < value.size() * this->_collectionType->selectedValue().size() * this->_collectionLabel->value().size())
        {
            this->_okButton->setEnabled(true);
        }
        else
        {
            this->_okButton->setEnabled(false);
        }
        return true;
    });

    this->_bottomBlock = new nanogui::Widget(this);
    this->_bottomBlock->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                  nanogui::Alignment::Maximum,
                                                  0, 0));
    nanogui::Widget* buttonsBlock = new nanogui::Widget(this->_bottomBlock);
    buttonsBlock->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                   nanogui::Alignment::Middle,
                                                   1, 2));

    this->_okButton = new nanogui::Button(buttonsBlock, "Create");
    this->_okButton->setFixedSize(Eigen::Vector2i(60, 18));
    this->_okButton->setFontSize(16);
    this->_okButton->setEnabled(false);
    this->_okButton->setCallback([this] () {
        if
            (nullptr == this->_validateCallback)
        {
            return;
        }

        std::string collectionLabel = this->_collectionLabel->value();
        std::string variableLabel = this->_variableLabel->value();
        std::string typeName = this->_collectionType->selectedValue();
        ShapeType shapeType(ShapeType::UNKNOWN_SHAPE_TYPE);
        if
            ("polyline" == typeName)
        {
            shapeType = ShapeType::POLYLINE;
        }
        else if
            ("polygon" == typeName)
        {
            shapeType = ShapeType::POLYGON;
        }
        else if
            ("point" == typeName)
        {
            shapeType = ShapeType::POINT;
        }
        else
        {
            LOG(LogLevel::Error, "NanoguiAnnotationsCollectionCreator",
                     "Selected shape type is not supported");
            return;
        }

        this->_validateCallback(shapeType, collectionLabel, variableLabel);
    });

    this->_nokButton = new nanogui::Button(buttonsBlock, "Cancel");
    this->_nokButton->setFixedSize(Eigen::Vector2i(60, 18));
    this->_nokButton->setFontSize(16);
    this->_nokButton->setCallback([this] () {
        if
            (nullptr != this->_cancelCallback)
        {
            this->_cancelCallback();
        }
    });
}

/**
 *
 */
NanoguiAnnotationsCollectionCreator::~NanoguiAnnotationsCollectionCreator()
{

}

/**
 *
 */
void NanoguiAnnotationsCollectionCreator::resize(const Eigen::Vector2i& size)
{
    this->setFixedSize(size);

    this->_collectionLabel->setFixedWidth(size.x() - 100);
    this->_variableLabel->setFixedWidth(size.x() - 100);
    this->_topBlock->setFixedWidth(size.x());
    this->_bottomBlock->setFixedWidth(size.x());
}

/**
 *
 */
void NanoguiAnnotationsCollectionCreator::setValidateCallback(const std::function<void(const ShapeType&, const std::string&, const std::string&)>& callback)
{
    this->_validateCallback = callback;
}

/**
 *
 */
void NanoguiAnnotationsCollectionCreator::setCancelCallback(const std::function<void()>& callback)
{
    this->_cancelCallback = callback;
}

/**
 *
 */
void NanoguiAnnotationsCollectionCreator::reset()
{
    this->_collectionLabel->setValue("");
    this->_variableLabel->setValue("");
    this->_collectionType->selectValue("");
    this->_okButton->setEnabled(false);
}

/**
 *
 */
bool NanoguiAnnotationsCollectionCreator::selectCollectionType(const std::string& collectionType)
{
    bool found(false);

    found = this->_collectionType->selectValue(collectionType);
    return found;
}
