// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"
#include "ui/nanogui/nanogui_attribute.h"

using namespace SEAScope;

/**
 *
 */
NanoguiAttribute::NanoguiAttribute(nanogui::Widget* parent)
: nanogui::Widget(parent), nameLbl(nullptr), valueLbl(nullptr)
{
	nanogui::BoxLayout* layout;	//
	nanogui::Widget* wrapper;	//
	nanogui::Widget* spacer;	//

	layout = new nanogui::BoxLayout( nanogui::Orientation::Vertical
								   , nanogui::Alignment::Minimum
								   , 0, 0);
	this->setLayout(layout);

	this->nameLbl = new nanogui::Label(this, "", "sans-bold");
	this->nameLbl->setFixedWidth(220);
	this->nameLbl->setColor(nanogui::Color(180,180,180,150));

	layout = new nanogui::BoxLayout( nanogui::Orientation::Horizontal
								   , nanogui::Alignment::Middle
								   , 0, 0);

	wrapper = new nanogui::Widget(this);
	wrapper->setLayout(layout);
	spacer = new Widget(wrapper);
	spacer->setFixedWidth(10);

	this->valueLbl = new nanogui::Label(wrapper, "", "sans-bold");
	this->valueLbl->setFixedWidth(205);
}

/**
 *
 */
NanoguiAttribute::~NanoguiAttribute()
{};

/**
 *
 */
bool NanoguiAttribute::setName(const std::string& value)
{
	if
		(this->nameLbl->caption() != value)
	{
		this->nameLbl->setCaption(value);
		return true;
	}
	return false;
};

/**
 *
 */
bool NanoguiAttribute::setValue(const std::string& value)
{
	if
		(this->valueLbl->caption() != value)
	{
		this->valueLbl->setCaption(value);
		return true;
	}
	return false;
};
