// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_blueprint_editor.h"
#include "ui/nanogui/nanogui_blueprint_liveedit.h"
#include "ui/nanogui/nanogui_icon.h"
#include "core/log.h"

using namespace SEAScope;

/**
 *
 */
NanoguiBlueprintEditor::NanoguiBlueprintEditor(nanogui::Widget* parent, NanoguiBlueprint* blueprintIn): nanogui::Widget(parent), blueprint(blueprintIn)
{
	nanogui::PopupButton* editBtn;	//
	NanoguiBlueprintLiveedit* live;	//
	NanoguiIcon* ico;	//
	std::ostringstream buffer;	//

	this->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
										   , nanogui::Alignment::Middle
										   , 0, 3));

	ico = new NanoguiIcon(this, ENTYPO_ICON_COPY);
	ico->setFixedSize(Eigen::Vector2i(20,20));
	if
		(true == blueprint->target->isMovable())
	{
		ico->setFontSize(18);
		ico->setTooltip("Component can be moved using live edit");
		ico->setColor(nanogui::Color(230, 230, 230, 230));
	}
	else
	{
		ico->setFontSize(14);
		ico->setTooltip("Component has fixed position");
		ico->setColor(this->mTheme->mDisabledTextColor);
	}

	ico = new NanoguiIcon(this, ENTYPO_ICON_RESIZE_FULL_SCREEN);
	ico->setFixedSize(Eigen::Vector2i(20,20));
	if
		(true == blueprint->target->isResizable())
	{
		ico->setFontSize(18);
		buffer << "Component can be resized using live edit (min dimensions: " << this->blueprint->minWidth << " x " << this->blueprint->minHeight;
		ico->setTooltip(buffer.str());
		buffer.str("");
		ico->setColor(nanogui::Color(230, 230, 230, 230));
	}
	else
	{
		ico->setFontSize(14);
		ico->setTooltip("Component has fixed size");
		ico->setColor(this->mTheme->mDisabledTextColor);
	}


	editBtn = new nanogui::PopupButton(this, "Live edit");
	editBtn->setFixedSize(Eigen::Vector2i(50,20));
	editBtn->setFontSize(14);
	editBtn->setFlags(nanogui::Button::ToggleButton);
	editBtn->setEnabled( (true == blueprint->target->isMovable()) || (true == blueprint->target->isResizable()));

	auto popup = editBtn->popup();
	popup->setFixedSize(Eigen::Vector2i(157, 132));
	popup->setVisible(false);
	live = new NanoguiBlueprintLiveedit(popup, this->blueprint);
	editBtn->setChangeCallback([this, live] (bool active){
		bool modified;	//
		if
			(true == active)
		{
			this->blueprint->startEditing();
		}
		else
		{
			modified = this->blueprint->stopEditing();
			if
				(true == modified)
			{
				live->commit();
				this->blueprint->applyToTarget();
				if
					(nullptr != this->callback)
				{
					this->callback();
				}
			}
		}});

	this->blueprint->setVisible(false);
};

/**
 *
 */
NanoguiBlueprintEditor::~NanoguiBlueprintEditor()
{
};

/**
 *
 */
bool NanoguiBlueprintEditor::setCallback(const std::function<void()>& callbackIn)
{
	this->callback = callbackIn;
	return true;
};
