// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <ostream>

#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_blueprint_liveedit.h"

using namespace SEAScope;

/**
 *
 */
NanoguiBlueprintLiveedit::NanoguiBlueprintLiveedit(nanogui::Widget* parent, NanoguiBlueprint* blueprintIn): nanogui::Widget(parent), blueprint(blueprintIn)
{
	nanogui::Widget* grid;	//
	nanogui::Widget* msg;	//

	this->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										   , nanogui::Alignment::Middle
										   , 0, 10));

	msg = new nanogui::Widget(this);
	msg->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										  , nanogui::Alignment::Middle
										  , 0, 0));
	new nanogui::Label(msg, "Choose how live edit");
	new nanogui::Label(msg, "results will be converted");

	grid = new nanogui::Widget(this);
	grid->setLayout( new nanogui::GridLayout( nanogui::Orientation::Horizontal, 5
											, nanogui::Alignment::Middle
											, 0, 2));
	new nanogui::Label(grid, "Width");
	this->widthUnits.push_back(new nanogui::Button(grid, "px"));
	this->widthUnits.push_back(new nanogui::Button(grid, "%"));

	this->widthUnits[0]->setButtonGroup(this->widthUnits);
	this->widthUnits[0]->setFlags(nanogui::Button::RadioButton);
	this->widthUnits[0]->setFixedSize(Eigen::Vector2i(20, 20));
	this->widthUnits[0]->setFontSize(14);
	this->widthUnits[0]->setPushed(true);

	this->widthUnits[1]->setButtonGroup(this->widthUnits);
	this->widthUnits[1]->setFlags(nanogui::Button::RadioButton);
	this->widthUnits[1]->setFixedSize(Eigen::Vector2i(20, 20));
	this->widthUnits[1]->setFontSize(14);

	new nanogui::Label(grid, "");
	new nanogui::Label(grid, "");

	new nanogui::Label(grid, "Height");
	this->heightUnits.push_back(new nanogui::Button(grid, "px"));
	this->heightUnits.push_back(new nanogui::Button(grid, "%"));

	this->heightUnits[0]->setButtonGroup(this->heightUnits);
	this->heightUnits[0]->setFlags(nanogui::Button::RadioButton);
	this->heightUnits[0]->setFixedSize(Eigen::Vector2i(20, 20));
	this->heightUnits[0]->setFontSize(14);
	this->heightUnits[0]->setPushed(true);

	this->heightUnits[1]->setButtonGroup(this->heightUnits);
	this->heightUnits[1]->setFlags(nanogui::Button::RadioButton);
	this->heightUnits[1]->setFixedSize(Eigen::Vector2i(20, 20));
	this->heightUnits[1]->setFontSize(14);

	new nanogui::Label(grid, "");
	new nanogui::Label(grid, "");

	new nanogui::Label(grid, "H offset");
	this->hOffsetUnits.push_back(new nanogui::Button(grid, "px"));
	this->hOffsetUnits.push_back(new nanogui::Button(grid, "%"));

	this->hOffsetUnits[0]->setButtonGroup(this->hOffsetUnits);
	this->hOffsetUnits[0]->setFlags(nanogui::Button::RadioButton);
	this->hOffsetUnits[0]->setFixedSize(Eigen::Vector2i(20, 20));
	this->hOffsetUnits[0]->setFontSize(14);
	this->hOffsetUnits[0]->setPushed(true);

	this->hOffsetUnits[1]->setButtonGroup(this->hOffsetUnits);
	this->hOffsetUnits[1]->setFlags(nanogui::Button::RadioButton);
	this->hOffsetUnits[1]->setFixedSize(Eigen::Vector2i(20, 20));
	this->hOffsetUnits[1]->setFontSize(14);

	this->hOffsetOrigin.push_back(new nanogui::Button(grid, "left"));
	this->hOffsetOrigin.push_back(new nanogui::Button(grid, "right"));

	this->hOffsetOrigin[0]->setButtonGroup(this->hOffsetOrigin);
	this->hOffsetOrigin[0]->setFlags(nanogui::Button::RadioButton);
	this->hOffsetOrigin[0]->setFixedSize(Eigen::Vector2i(25,20));
	this->hOffsetOrigin[0]->setFontSize(14);
	this->hOffsetOrigin[0]->setPushed(true);

	this->hOffsetOrigin[1]->setButtonGroup(this->hOffsetOrigin);
	this->hOffsetOrigin[1]->setFlags(nanogui::Button::RadioButton);
	this->hOffsetOrigin[1]->setFixedSize(Eigen::Vector2i(40,20));
	this->hOffsetOrigin[1]->setFontSize(14);

	new nanogui::Label(grid, "V offset");
	this->vOffsetUnits.push_back(new nanogui::Button(grid, "px"));
	this->vOffsetUnits.push_back(new nanogui::Button(grid, "%"));

	this->vOffsetUnits[0]->setButtonGroup(this->vOffsetUnits);
	this->vOffsetUnits[0]->setFlags(nanogui::Button::RadioButton);
	this->vOffsetUnits[0]->setFixedSize(Eigen::Vector2i(20, 20));
	this->vOffsetUnits[0]->setFontSize(14);
	this->vOffsetUnits[0]->setPushed(true);

	this->vOffsetUnits[1]->setButtonGroup(this->vOffsetUnits);
	this->vOffsetUnits[1]->setFlags(nanogui::Button::RadioButton);
	this->vOffsetUnits[1]->setFixedSize(Eigen::Vector2i(20,20));
	this->vOffsetUnits[1]->setFontSize(14);

	this->vOffsetOrigin.push_back(new nanogui::Button(grid, "top"));
	this->vOffsetOrigin.push_back(new nanogui::Button(grid, "bottom"));

	this->vOffsetOrigin[0]->setButtonGroup(this->vOffsetOrigin);
	this->vOffsetOrigin[0]->setFlags(nanogui::Button::RadioButton);
	this->vOffsetOrigin[0]->setFixedSize(Eigen::Vector2i(25,20));
	this->vOffsetOrigin[0]->setFontSize(14);
	this->vOffsetOrigin[0]->setPushed(true);

	this->vOffsetOrigin[1]->setButtonGroup(this->vOffsetOrigin);
	this->vOffsetOrigin[1]->setFlags(nanogui::Button::RadioButton);
	this->vOffsetOrigin[1]->setFixedSize(Eigen::Vector2i(40,20));
	this->vOffsetOrigin[1]->setFontSize(14);
};

/**
 *
 */
NanoguiBlueprintLiveedit::~NanoguiBlueprintLiveedit()
{

};

/**
 *
 */
const std::string NanoguiBlueprintLiveedit::getWidth() const
{
	std::ostringstream result;	//
	int percent;	//

	if
		(true == this->widthUnits[0]->pushed())
	{
		// Pixels
		result << this->blueprint->size().x() << "px";
	}
	else
	{
		// Percentage
		percent = (100 * this->blueprint->size().x()) / this->blueprint->parent()->size().x();
		result << percent << "%";
	}
	return result.str();
};

/**
 *
 */
const std::string NanoguiBlueprintLiveedit::getHeight() const
{
	std::ostringstream result;	//
	int percent;	//

	if
		(true == this->heightUnits[0]->pushed())
	{
		// Pixels
		result << this->blueprint->size().y() << "px";
	}
	else
	{
		// Percentage
		percent = (100 * this->blueprint->size().y()) / this->blueprint->parent()->size().y();
		result << percent << "%";
	}
	return result.str();
};

/**
 *
 */
const std::string NanoguiBlueprintLiveedit::getX() const
{
	std::stringstream result;	//
	int delta;	//
	int percent;	//

	if
		(true == this->hOffsetOrigin[0]->pushed())
	{
		// From left
		if
			(true == this->hOffsetUnits[0]->pushed())
		{
			// Pixels
			result << this->blueprint->position().x() << "px";
		}
		else
		{
			// Percentage
			percent = (100 * this->blueprint->position().x()) / this->blueprint->parent()->size().x();
			result << percent << "%";
		}
	}
	else
	{
		// From right
		delta = this->blueprint->position().x() + this->blueprint->size().x();
		delta = delta - this->blueprint->parent()->size().x();
		delta = -1 * delta; /* positive value */
		if
			(true == this->hOffsetUnits[0]->pushed())
		{
			// Pixels
			result << "-" << delta << "px";
		}
		else
		{
			// Percentage
			percent = (100 * delta) / this->blueprint->parent()->size().x();
			result << "-" << percent << "%";
		}
	}

	return result.str();
};

/**
 *
 */
const std::string NanoguiBlueprintLiveedit::getY() const
{
	std::stringstream result;	//
	int delta;	//
	int percent;	//

	if
		(true == this->vOffsetOrigin[0]->pushed())
	{
		// From left
		if
			(true == this->vOffsetUnits[0]->pushed())
		{
			// Pixels
			result << this->blueprint->position().y() << "px";
		}
		else
		{
			// Percentage
			percent = (100 * this->blueprint->position().y()) / this->blueprint->parent()->size().y();
			result << percent << "%";
		}
	}
	else
	{
		// From right
		delta = this->blueprint->position().y() + this->blueprint->size().y();
		delta = delta - this->blueprint->parent()->size().y();
		delta = -1 * delta; /* positive value */
		if
			(true == this->vOffsetUnits[0]->pushed())
		{
			// Pixels
			result << "-" << delta << "px";
		}
		else
		{
			// Percentage
			percent = (100 * delta) / this->blueprint->parent()->size().y();
			result << "-" << percent << "%";
		}
	}

	return result.str();
};

/**
 *
 */
bool NanoguiBlueprintLiveedit::commit()
{
	if
		(nullptr == this->blueprint)
	{
		return false;
	}
	this->blueprint->storedWidth = this->getWidth();
	this->blueprint->storedHeight = this->getHeight();
	this->blueprint->storedX = this->getX();
	this->blueprint->storedY = this->getY();
	return true;
};
