// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_calendar.h"

using namespace SEAScope;

/**
 *
 */
NanoguiCalendar::NanoguiCalendar(nanogui::Widget* parent): nanogui::Widget(parent), monthCallback(nullptr), yearCallback(nullptr), yearBtn(nullptr), year(0), selectedMonth(0)
{
	nanogui::Widget* semester1;	//
	nanogui::Widget* semester2;	//
	nanogui::Widget* monthsWidget;	//
	nanogui::Widget* spacer;	//
	NanoguiFlatButton* btn;	//
	std::uint32_t btnWidth;	//
	std::uint32_t btnHeight;	//

	btnWidth = 12;
	btnHeight = 5;

	this->yearBackground = nanogui::Color(0,0,0,0);
	this->yearForeground = nanogui::Color(250,250,250,255);

	this->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										   , nanogui::Alignment::Fill
										   , 0, 0));
	monthsWidget = new nanogui::Widget(this);
	monthsWidget->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
											 , nanogui::Alignment::Middle
											 , 0, 0));
	semester1 = new nanogui::Widget(monthsWidget);
	semester1->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
											   , nanogui::Alignment::Middle
											   , 0, 0));
	spacer = new nanogui::Widget(this);
	spacer->setFixedHeight(1);

	semester2 = new nanogui::Widget(this);
	semester2->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
											   , nanogui::Alignment::Middle
											   , 0, 0));

	/* January */
	btn = new NanoguiFlatButton(semester1);
	btn->setTooltip("January");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 1);
		}
	});
	this->months[0] = btn;
	this->backgrounds[0] = nanogui::Color(250,250,250,255);
	this->textColors[0] = nanogui::Color(30,30,30,255);

	/* February */
	btn = new NanoguiFlatButton(semester1);
	btn->setTooltip("February");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 2);
		}
	});
	this->months[1] = btn;
	this->backgrounds[1] = nanogui::Color(250,250,250,255);
	this->textColors[1] = nanogui::Color(30,30,30,255);

	/* March */
	btn = new NanoguiFlatButton(semester1);
	btn->setTooltip("March");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 3);
		}
	});
	this->months[2] = btn;
	this->backgrounds[2] = nanogui::Color(250,250,250,255);
	this->textColors[2] = nanogui::Color(30,30,30,255);

	/* April */
	btn = new NanoguiFlatButton(semester1);
	btn->setTooltip("April");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 4);
		}
	});
	this->months[3] = btn;
	this->backgrounds[3] = nanogui::Color(250,250,250,255);
	this->textColors[3] = nanogui::Color(30,30,30,255);

	/* May */
	btn = new NanoguiFlatButton(semester1);
	btn->setTooltip("May");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 5);
		}
	});
	this->months[4] = btn;
	this->backgrounds[4] = nanogui::Color(250,250,250,255);
	this->textColors[4] = nanogui::Color(30,30,30,255);

	/* June */
	btn = new NanoguiFlatButton(semester1);
	btn->setTooltip("June");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 6);
		}
	});
	this->months[5] = btn;
	this->backgrounds[5] = nanogui::Color(250,250,250,255);
	this->textColors[5] = nanogui::Color(30,30,30,255);

	/* July */
	btn = new NanoguiFlatButton(semester2);
	btn->setTooltip("July");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 7);
		}
	});
	this->months[6] = btn;
	this->backgrounds[6] = nanogui::Color(250,250,250,255);
	this->textColors[6] = nanogui::Color(30,30,30,255);

	/* August */
	btn = new NanoguiFlatButton(semester2);
	btn->setTooltip("August");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 8);
		}
	});
	this->months[7] = btn;
	this->backgrounds[7] = nanogui::Color(250,250,250,255);
	this->textColors[7] = nanogui::Color(30,30,30,255);

	/* September */
	btn = new NanoguiFlatButton(semester2);
	btn->setTooltip("September");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 9);
		}
	});
	this->months[8] = btn;
	this->backgrounds[8] = nanogui::Color(250,250,250,255);
	this->textColors[8] = nanogui::Color(30,30,30,255);

	/* October */
	btn = new NanoguiFlatButton(semester2);
	btn->setTooltip("October");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 10);
		}
	});
	this->months[9] = btn;
	this->backgrounds[9] = nanogui::Color(250,250,250,255);
	this->textColors[9] = nanogui::Color(30,30,30,255);

	/* November */
	btn = new NanoguiFlatButton(semester2);
	btn->setTooltip("November");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 11);
		}
	});
	this->months[10] = btn;
	this->backgrounds[10] = nanogui::Color(250,250,250,255);
	this->textColors[10] = nanogui::Color(30,30,30,255);

	/* December */
	btn = new NanoguiFlatButton(semester2);
	btn->setTooltip("December");
	btn->setFixedSize(Eigen::Vector2i(btnWidth, btnHeight));
	btn->setCallback([this]() {
		if
			(nullptr != this->monthCallback)
		{
			this->monthCallback(this->year, 12);
		}
	});
	this->months[11] = btn;
	this->backgrounds[11] = nanogui::Color(250,250,250,255);
	this->textColors[11] = nanogui::Color(30,30,30,255);

	spacer = new nanogui::Widget(this);
	spacer->setFixedHeight(2);

	this->yearBtn = new NanoguiFlatButton(this);
	this->yearBtn->setBackgroundColor(this->yearBackground);
	this->yearBtn->setTextColor(this->yearForeground);
	this->yearBtn->mFontFace = "sans-bold";
	this->yearBtn->setFontSize(18);
	this->yearBtn->setFixedWidth(72);
	this->yearBtn->setFixedHeight(20);
	this->yearBtn->setCallback([this]() {
		if
			(nullptr != this->yearCallback)
		{
			this->yearCallback(this->year);
		}	
	});

	this->reset();
};

/**
 *
 */
NanoguiCalendar::~NanoguiCalendar()
{
};

/**
 *
 */
bool NanoguiCalendar::setYear(std::uint16_t yearIn)
{
	std::ostringstream buffer;	//

	if
		(yearIn == this->year)
	{
		return false;
	}

	buffer << yearIn;
	this->yearBtn->setCaption(buffer.str());
	this->year = yearIn;
	return true;
};

/**
 *
 */
bool NanoguiCalendar::setYearBgColor(nanogui::Color color)
{
	if
		(color != this->yearBackground)
	{
		this->yearBackground = color;
		this->yearBtn->setBackgroundColor(color);
		return true;
	}
	return false;
};

/**
 *
 */
bool NanoguiCalendar::setYearFgColor(nanogui::Color color)
{
	if
		(color != this->yearForeground)
	{
		this->yearForeground = color;
		this->yearBtn->setTextColor(color);
		return true;
	}
	return false;
};

/**
 *
 */
bool NanoguiCalendar::setMonthFgColor(std::uint8_t month, nanogui::Color color)
{
	if
		(month < 1 || month > 12)
	{
		return false;
	}

	this->textColors[month-1u] = color;
	this->months[month-1u]->setTextColor(color);
	return true;
};

/**
 *
 */
bool NanoguiCalendar::setMonthBgColor(std::uint8_t month, nanogui::Color color)
{
	if
		(month < 1 || month > 12)
	{
		return false;
	}

	this->backgrounds[month-1u] = color;
	this->months[month-1u]->setBackgroundColor(color);
	return true;
};

/**
 *
 */
bool NanoguiCalendar::select()
{
	this->yearBtn->setBorderColor(nanogui::Color(250,250,250,255));
	return true;
};

/**
 *
 */
bool NanoguiCalendar::unselect()
{
	this->yearBtn->setBorderColor(nanogui::Color(0,0,0,0));
	return true;
};

/**
 *
 */
bool NanoguiCalendar::selectMonth(std::uint8_t month)
{
	NanoguiFlatButton* btn;	//
	if
		(  month < 1
		|| month > 12)
	{
		return false;
	}

	this->unselectMonth();
	this->selectedMonth = month;
	btn = this->months[month - 1u];
	btn->setBackgroundColor(nanogui::Color(30, 150, 255, 255));
	btn->mFontFace = "sans-bold";
	btn->setTextColor(nanogui::Color(250,250,250,255));
	return true;
};

/**
 *
 */
bool NanoguiCalendar::unselectMonth()
{
	NanoguiFlatButton* btn;	//

	if
		(0 < this->selectedMonth)
	{
		btn = this->months[this->selectedMonth - 1];
		btn->setBackgroundColor(this->backgrounds[this->selectedMonth - 1]);
		btn->setTextColor(this->textColors[this->selectedMonth - 1]);
		btn->mFontFace = "sans";
		this->selectedMonth = 0;
		return true;
	}
	return false;
};

/**
 *
 */
bool NanoguiCalendar::reset()
{
	size_t i;	//
	for
		(i=0; i < 12; ++i)
	{
		this->months[i]->setBackgroundColor(this->backgrounds[i]);
		this->months[i]->setTextColor(this->textColors[i]);
		this->months[i]->mFontFace = "sans";
	}
	return true;
};

/**
 *
 */
bool NanoguiCalendar::enableMonth(std::uint8_t month)
{
	if
		(  month < 1 
		|| month > 12
		|| true == this->months[month - 1u]->enabled())
	{
		return false;
	}

	this->months[month - 1u]->setEnabled(true);
	return true;
};

/**
 *
 */
bool NanoguiCalendar::disableMonth(std::uint8_t month)
{
	if
		( month < 1 
		|| month > 12
		|| false == this->months[month - 1u]->enabled())
	{
		return false;
	}

	this->months[month - 1u]->setEnabled(false);
	return true;
};
