// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "ui/nanogui/nanogui_checkbox.h"

using namespace SEAScope;

NanoguiCheckbox::NanoguiCheckbox(nanogui::Widget* parent, const std::string& caption)
: nanogui::CheckBox(parent, caption),
  _editable(true)
{
	this->outlineColor = nanogui::Color(150,150,150,150);
    this->keyPressed = 0;
};

NanoguiCheckbox::~NanoguiCheckbox()
{
};

void NanoguiCheckbox::draw(NVGcontext* ctx)
{
	nanogui::CheckBox::draw(ctx);

    if
        (true == this->_editable)
    {
        /* Draw the outline of the box */
        nvgBeginPath(ctx);
        nvgRoundedRect(ctx, static_cast<float>(mPos.x() + 1), static_cast<float>(mPos.y() + 1), static_cast<float>(mSize.y() - 2),
                       static_cast<float>(mSize.y() - 2), 3);
        nvgStrokeColor(ctx, this->outlineColor);
        nvgStroke(ctx);
    }
};

/**
 *
 */
bool NanoguiCheckbox::mouseButtonEvent(const Eigen::Vector2i &p, int button, bool down, int modifiers)
{
	bool ok(false);	//

    if
        (true == this->_editable)
    {
        this->keyPressed = modifiers;
        ok = nanogui::CheckBox::mouseButtonEvent(p, button, down, modifiers);
    }

	return ok;
};

/**
 *
 */
void NanoguiCheckbox::setEditable(bool editable)
{
    this->_editable = editable;
}

/**
 *
 */
bool NanoguiCheckbox::editable() const
{
    return this->_editable;
}
