// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "core/log.h"
#include "ui/nanogui/nanogui_hcolorbar.h"
#include "ui/nanogui/nanogui_colorbar_list.h"

using namespace SEAScope;

/**
 *
 */
NanoguiColorbarList::NanoguiColorbarList(nanogui::Widget* parent):
 nanogui::Widget(parent), colormaps(nullptr), colorwheel(nullptr),
 cmapCallback(nullptr), colorCallback(nullptr)
{
	this->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										   , nanogui::Alignment::Middle
										   , 0, 0));
	this->colorwheel = new nanogui::ColorWheel(this);
	this->colorwheel->setFixedSize(Eigen::Vector2i(100, 100));
	this->colorwheel->setCallback([this](const nanogui::Color& value) {
		std::array<std::uint8_t, 3> color;	//

		if
			(nullptr != this->colorCallback)
		{
			color[0] = static_cast<std::uint8_t>(255 * value.r());
			color[1] = static_cast<std::uint8_t>(255 * value.g());
			color[2] = static_cast<std::uint8_t>(255 * value.b());
			this->colorCallback(color);
		}
	});

	this->colormaps = new NanoguiList(this);
	this->colormaps->setFixedWidth(130);
	this->colormaps->content->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
															   , nanogui::Alignment::Middle
															   , 0, 3));
};

/**
 *
 */
NanoguiColorbarList::~NanoguiColorbarList()
{
};

/**
 *
 */
bool NanoguiColorbarList::init(const std::map<std::string, Colormap>& cmaps)
{
	NanoguiHColorbar* cb;	//

	std::map<std::string, Colormap>::const_iterator  it = cmaps.begin();
	for
		(; it != cmaps.end(); ++it)
	{
		cb = new NanoguiHColorbar(this->colormaps->content);
		cb->setFixedSize(Eigen::Vector2i(100, 20));
		cb->orientation = nanogui::Orientation::Horizontal;
		cb->label = it->first;
		cb->cmap = it->second;
		cb->setTooltip(it->first);
		cb->setClickCallback([this](const std::string& label, const Colormap& cmap, int, float, bool) {
			if
				(nullptr != this->cmapCallback)
			{
				this->cmapCallback(label, cmap);
			}
		});
	}

	cb = nullptr;

	return false;
};

/**
 *
 */
bool NanoguiColorbarList::setColormapCallback(const std::function<void(const std::string&, const Colormap&)>& callback)
{
	this->cmapCallback = callback;
	return true;
};

/**
 *
 */
bool NanoguiColorbarList::setColorCallback(const std::function<void(const std::array<std::uint8_t, 3>&)>& callback)
{
	this->colorCallback = callback;
	return true;
};
