// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "ui/nanogui/nanogui_component.h"

using namespace SEAScope;

/**
 *
 */
NanoguiComponent::NanoguiComponent(nanogui::Widget* parent)
: nanogui::Widget(parent), reflowNeeded(true), componentId("")
, resizeEventEmitter(nullptr)
{
};

/**
 *
 */
NanoguiComponent::~NanoguiComponent()
{
};

/**
 *
 */
bool NanoguiComponent::isResizable() const
{
	return true;
};

/**
 *
 */
bool NanoguiComponent::isMovable() const
{
	return true;
};

/**
 *
 */
bool NanoguiComponent::setPosition(int x, int y)
{
	nanogui::Widget::setPosition(Eigen::Vector2i(x,y));
	return true;
};

/**
 *
 */
bool NanoguiComponent::setSize(int w, int h)
{
	int oldW;	//
	int oldH;	//

	oldW = this->fixedWidth();
	oldH = this->fixedHeight();
	this->setFixedSize(Eigen::Vector2i(w, h));
	if
		(  nullptr != this->resizeEventEmitter
		&& (  w != oldW
		   || h != oldH))
	{
		this->resizeEventEmitter(this);
	}

	return w != oldW || h != oldH;
};

/**
 *
 */
bool NanoguiComponent::setVisible(bool value)
{
	bool oldV;	//

	oldV = this->visible();
	nanogui::Widget::setVisible(value);
	return value != oldV;
};

/**
 *
 */
bool NanoguiComponent::visible() const
{
	return nanogui::Widget::visible();
};

/**
 *
 */
bool NanoguiComponent::requestReflow()
{
	bool oldValue;	//

	oldValue = this->reflowNeeded;
	this->reflowNeeded = true;
	return !oldValue;
};

/**
 *
 */
bool NanoguiComponent::setComponentId(const std::string& value)
{
	std::string oldValue;	//

	oldValue = this->componentId;
	this->componentId = value;
	return oldValue != value;
};

/**
 *
 */
const std::string& NanoguiComponent::getComponentId() const
{
	return this->componentId;
};

/**
 *
 */
bool NanoguiComponent::setResizeEventEmitter(const std::function<void(IComponent*)>& emitter)
{
	if
		(nullptr != this->resizeEventEmitter)
	{
		/* Event emitter can only be set once*/
		return false;
	}

	this->resizeEventEmitter = emitter;
	return true;
};

/**
 *
 */
int NanoguiComponent::getWidth() const
{
	return this->fixedWidth();
};

/**
 *
 */
int NanoguiComponent::getHeight() const
{
	return this->fixedHeight();
};

/**
 *
 */
int NanoguiComponent::getWidthOffset() const
{
	return 0;
}

/**
 *
 */
int NanoguiComponent::getHeightOffset() const
{
	return 0;
}
