// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "ui/nanogui/nanogui_dict_field_editor.h"

#include "ui/nanogui/nanogui_add_property_form.h"

#include <iostream>

using namespace SEAScope;

/**
 *
 */
NanoguiDictFieldEditor::NanoguiDictFieldEditor(nanogui::Widget* parent)
: nanogui::Widget(parent),
  _backgroundColor(30, 30, 30, 255),
  _nameLabel(nullptr),
  //_addButton(nullptr),
  _deleteButton(nullptr),
  _addPropertyForm(nullptr),
  _deleteCallback(nullptr),
  _childUnicityCallback(nullptr),
  _addPropertyOkCallback(nullptr),
  _layoutChangeCallback(nullptr),
  _editing(false),
  _reflowNeeded(false)
{
    // Assuming 360px width
    this->setFixedWidth(342);
    this->setWidth(342);

    this->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                           nanogui::Alignment::Fill,
                                           0, 0));

    nanogui::Widget* spacer = new nanogui::Widget(this);
    spacer->setFixedHeight(10);
    spacer->setHeight(10);

    nanogui::Widget* innerBlock = new nanogui::Widget(this);
    innerBlock->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                 nanogui::Alignment::Middle,
                                                 0, 3));

    spacer = new nanogui::Widget(innerBlock);
    spacer->setFixedWidth(/*5*/10);
    spacer->setWidth(/*5*/10);

    /*
    nanogui::Widget* addButtonBlock = new nanogui::Widget(innerBlock);
    addButtonBlock->setFixedSize(Eigen::Vector2i(18, 18));
    addButtonBlock->setSize(Eigen::Vector2i(18, 18));

    this->_addButton = new nanogui::Button(addButtonBlock, "", ENTYPO_ICON_PLUS);
    this->_addButton->setFixedSize(Eigen::Vector2i(18, 18));
    this->_addButton->setSize(Eigen::Vector2i(18, 18));
    this->_addButton->setFontSize(16);
    this->_addButton->setCallback([this] () {
        this->_addPropertyForm->reset();
        this->_addPropertyForm->setVisible(true);
        this->_reflowNeeded = true;
        if
            (nullptr != this->_layoutChangeCallback)
        {
            this->_layoutChangeCallback();
        }
    });
    */

    this->_nameLabel = new nanogui::Label(innerBlock, "", "sans-bold");
    this->_nameLabel->setFixedWidth(297/*284*/);
    this->_nameLabel->setWidth(297/*284*/);

    nanogui::Widget* deleteButtonBlock = new nanogui::Widget(innerBlock);
    deleteButtonBlock->setFixedSize(Eigen::Vector2i(18, 18));
    deleteButtonBlock->setSize(Eigen::Vector2i(18, 18));

    this->_deleteButton = new nanogui::Button(deleteButtonBlock, "", ENTYPO_ICON_TRASH);
    this->_deleteButton->setFixedSize(Eigen::Vector2i(18, 18));
    this->_deleteButton->setFontSize(16);
    this->_deleteButton->setCallback([this] () {
        if
            (nullptr != this->_deleteCallback)
        {
            this->_deleteCallback("", this->getName());
        }
    });

    this->_addPropertyForm = new NanoguiAddPropertyForm(this, {"bool", "num", "text", "img"});
    this->_addPropertyForm->setValidateCallback([this] (const std::string& nodeType, const std::string& label) -> bool {
        if
            ("" == nodeType)
        {
            return false;
        }

        if
            ("" == label)
        {
            // Empty label not allowed
            return false;
        }

        bool ok(false);
        if
            (nullptr != this->_childUnicityCallback)
        {
            ok = this->_childUnicityCallback(this->_nameLabel->caption(), label);
        }

        return ok;
    });
    this->_addPropertyForm->setOkCallback([this] () {
        if
            (nullptr != this->_addPropertyOkCallback)
        {
            std::string nodeLabel;
            this->_addPropertyForm->getName(nodeLabel);

            std::string nodeType;
            this->_addPropertyForm->getType(nodeType);

            this->_addPropertyOkCallback(nodeType, this->_nameLabel->caption(), nodeLabel);

            this->_addPropertyForm->reset();
        }

        if
            (nullptr != this->_layoutChangeCallback)
        {
            this->_layoutChangeCallback();
        }
    });
    this->_addPropertyForm->setNokCallback([this] () {
        this->_addPropertyForm->reset();
    });

    this->_addPropertyForm->setVisible(false);
    //this->_addButton->setVisible(false);
    this->_deleteButton->setVisible(false);

    spacer = new nanogui::Widget(this);
    spacer->setFixedHeight(5);
    spacer->setHeight(5);

    this->_reflowNeeded = true;
}

/**
 *
 */
NanoguiDictFieldEditor::~NanoguiDictFieldEditor()
{

}

/**
 *
 */
bool NanoguiDictFieldEditor::activateEdition(bool activate)
{
    if
        (activate == this->_editing)
    {
        return false;
    }

    this->_editing = activate;

    //this->_addButton->setVisible(this->_editing);
    this->_addPropertyForm->reset();
    this->_addPropertyForm->setVisible(this->_editing);
    this->_deleteButton->setVisible(this->_editing);
    this->_reflowNeeded = true;

    return true;
}

/**
 *
 */
void NanoguiDictFieldEditor::draw(NVGcontext* ctx)
{
    float posX;
    float posY;
    float sizeX;
    float sizeY;

    if
        (true == this->_reflowNeeded)
    {
        this->performLayout(ctx);
        this->_reflowNeeded = false;
    }

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());

    nvgBeginPath(ctx);
    nvgRoundedRect(ctx, posX + 6.f, posY + 8.f, sizeX - 12.f, sizeY - 10.f, 3.f);
    nvgFillColor(ctx, this->_backgroundColor);
    nvgFill(ctx);
    nvgStrokeColor(ctx, nanogui::Color(20, 100, 170, 150));
    nvgStrokeWidth(ctx, 2.f);
    nvgStroke(ctx);

    nanogui::Widget::draw(ctx);
}

/**
 *
 */
void NanoguiDictFieldEditor::setName(const std::string& name)
{
    this->_nameLabel->setCaption(name);
}

/**
 *
 */
const std::string& NanoguiDictFieldEditor::getName() const
{
    return this->_nameLabel->caption();
}

/**
 *
 */
void NanoguiDictFieldEditor::setCategory(const std::string& category)
{
    this->_category = category;
}

/**
 *
 */
const std::string& NanoguiDictFieldEditor::getCategory() const
{
    return this->_category;
}

/**
 *
 */
void NanoguiDictFieldEditor::setBackgroundColor(const nanogui::Color& color)
{
    this->_backgroundColor = color;
}

/**
 *
 */
const nanogui::Color& NanoguiDictFieldEditor::getBackgroundColor() const
{
    return this->_backgroundColor;
}

/**
 *
 */
bool NanoguiDictFieldEditor::setDeleteCallback(const std::function<void(const std::string&, const std::string&)>& callback)
{
    if
        (nullptr != this->_deleteCallback)
    {
        return false;
    }

    this->_deleteCallback = callback;
    return true;
}

/**
 *
 */
bool NanoguiDictFieldEditor::setChildUnicityCallback(const std::function<bool(const std::string&, const std::string&)>& callback)
{
    if
        (nullptr != this->_childUnicityCallback)
    {
        return false;
    }

    this->_childUnicityCallback = callback;
    return true;
}

/**
 *
 */
bool NanoguiDictFieldEditor::setAddPropertyOkCallback(const std::function<void(const std::string&, const std::string&, const std::string&)>& callback)
{
    if
        (nullptr != this->_addPropertyOkCallback)
    {
        return false;
    }

    this->_addPropertyOkCallback = callback;
    return true;
}

/**
 *
 */
bool NanoguiDictFieldEditor::setLayoutChangeCallback(const std::function<void()>& callback)
{
    if
        (nullptr != this->_layoutChangeCallback)
    {
        return false;
    }

    this->_layoutChangeCallback = callback;
    return true;
}
