// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.

#include "ui/nanogui/nanogui_fading_line.h"

using namespace SEAScope;

/**
 *
 */
NanoguiFadingLine::NanoguiFadingLine(nanogui::Widget* parent, float lineWidth)
: nanogui::Widget(parent), _lineWidth(lineWidth)
{
}

/**
 *
 */
NanoguiFadingLine::~NanoguiFadingLine()
{
}

/**
 *
 */
void NanoguiFadingLine::draw(NVGcontext* ctx)
{
    float posX {0.f};   //
    float posY {0.f};   //
    float sizeX {0.f};  //
    float sizeY {0.f};  //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());

    float lineY1 {0.f}; //
    lineY1 = posY + std::floor(0.5f * (sizeY - this->_lineWidth));

    NVGpaint bg = nvgLinearGradient(ctx, posX, posY, sizeX, sizeY,
                                    nanogui::Color(255, 255, 200, 255), nanogui::Color(255, 255, 255, 0));

    nanogui::Widget::draw(ctx);

    nvgBeginPath(ctx);
    nvgRect(ctx, 0, lineY1, sizeX, this->_lineWidth);
    nvgFillPaint(ctx, bg);
    nvgFill(ctx);
}
