// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_fixed_size_label.h"

using namespace SEAScope;

/**
 *
 */
NanoguiFixedSizeLabel::NanoguiFixedSizeLabel(nanogui::Widget* parent, const std::string& font, int fontSize): nanogui::Label(parent, "", font, fontSize)
{
};

/**
 *
 */
NanoguiFixedSizeLabel::~NanoguiFixedSizeLabel()
{
};

/**
 *
 */
void NanoguiFixedSizeLabel::setCaption(const std::string&  caption)
{
	this->setTooltip(caption);
	nanogui::Label::setCaption(caption);
};

/**
 *
 */
void NanoguiFixedSizeLabel::draw(NVGcontext* ctx)
{
	NVGtextRow row;	//
	float captionWidth;	//

	if
		(0 < this->mFixedSize.x())
	{
	    Widget::draw(ctx);
		nvgFontFace(ctx, this->mFont.c_str());
		nvgFontSize(ctx, static_cast<float>(this->fontSize()));
		nvgFillColor(ctx, this->mColor);
        nvgTextAlign(ctx, NVG_ALIGN_LEFT | NVG_ALIGN_TOP);

		captionWidth = nvgTextBounds(ctx, 0.f, 0.f, this->mCaption.c_str(), nullptr, nullptr);
		if
			(captionWidth > static_cast<float>(this->mFixedSize.x()))
		{
			nvgTextBreakLines(ctx, this->mCaption.c_str(), nullptr, static_cast<float>(this->mFixedSize.x()), &row, 1);
			nvgText(ctx,
                    static_cast<float>(this->mPos.x()),
                    static_cast<float>(this->mPos.y()),
                    row.start,
                    row.end);
			nvgText(ctx,
                    static_cast<float>(this->mPos.x()) + row.width,
                    static_cast<float>(this->mPos.y()),
                    "...",
                    nullptr);
		}
		else
		{
			nvgText(ctx,
                    static_cast<float>(this->mPos.x()),
                    static_cast<float>(this->mPos.y()),
                    this->mCaption.c_str(),
                    nullptr);
		}
		return;
	}
	nanogui::Label::draw(ctx);
};

/**
 *
 */
Eigen::Vector2i NanoguiFixedSizeLabel::preferredSize(NVGcontext* ctx) const
{
	float width;	//

	if
		(  "" != this->mCaption
		&& 0 < this->mFixedSize.x())
	{
		nvgFontFace(ctx, this->mFont.c_str());
		nvgFontSize(ctx, static_cast<float>(this->fontSize()));
		nvgTextAlign(ctx, NVG_ALIGN_LEFT | NVG_ALIGN_TOP);
		width = nvgTextBounds(ctx, 0, 0, this->mCaption.c_str(), nullptr, nullptr);
		width = (width > static_cast<float>(this->mFixedSize.x())) ? static_cast<float>(this->mFixedSize.x()): width;
        return Eigen::Vector2i( width, this->mTheme->mStandardFontSize);
	}
	return nanogui::Label::preferredSize(ctx);
};
