// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "num_utils.h"
#include "core/log.h"
#include "ui/nanogui/nanogui_flat_button.h"

using namespace SEAScope;

/**
 *
 */
NanoguiFlatButton::NanoguiFlatButton(nanogui::Widget* parent): nanogui::Button(parent, ""), mFontFace("sans"), wraplineOffset(-1.f), wraplineWidth(-1.f)
{
};

/**
 *
 */
NanoguiFlatButton::~NanoguiFlatButton()
{
};

/**
 *
 */
void NanoguiFlatButton::draw(NVGcontext* ctx)
{
	float tw;	// Text width
	Eigen::Vector2f center;	// Center of the widget
	Eigen::Vector2f textPos;	// Relative position of text
	NVGcolor textColor;	//
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());

	nvgBeginPath(ctx);
	nvgRoundedRect( ctx
		   , posX, posY
		   , sizeX, sizeY
		   , 3);
	nvgStrokeColor(ctx, this->borderColor);
	nvgStroke(ctx);

	nvgRoundedRect( ctx
		   , posX + 1.f , posY + 1.f
		   , sizeX - 2.f, sizeY - 2.f
		   , 3);
	nvgFillColor(ctx, this->mBackgroundColor);
	nvgFill(ctx);

	if
		(  0 <= this->wraplineOffset
		&& 0 < this->wraplineWidth)
	{
		nvgBeginPath(ctx);
		nvgRect( ctx
			   , posX + this->wraplineOffset * sizeX
			   , posY + sizeY - 3.f
			   , this->wraplineWidth * sizeX
			   , 3);
		nvgFillColor(ctx, nanogui::Color(200, 30, 30, 200));
		nvgFill(ctx);

		nvgBeginPath(ctx);
		nvgRect( ctx
			   , posX + this->wraplineOffset * sizeX
			   , posY
			   , this->wraplineWidth * sizeX
			   , 2);
		nvgFillColor(ctx, nanogui::Color(200, 30, 30, 200));
		nvgFill(ctx);
	}

	nvgFontSize(ctx, static_cast<float>(this->mFontSize));
	nvgFontFace(ctx, this->mFontFace.c_str());
	tw = nvgTextBounds(ctx, 0,0, this->mCaption.c_str(), nullptr, nullptr);

    center = this->mPos.cast<float>() + this->mSize.cast<float>() * 0.5f;
    textPos = Eigen::Vector2f(center.x() - tw * 0.5f, center.y() - 1);
    textColor = floating_point_equality(this->mTextColor.w(), 0.f) ? this->mTheme->mTextColor : this->mTextColor;
    if
		(!this->mEnabled)
	{
        //textColor = this->mTheme->mDisabledTextColor;
		textColor = this->mBackgroundColor;
	}
	else if
		(true == this->mPushed)
	{
		nvgBeginPath(ctx);
		nvgStrokeWidth( ctx, 1.0f);
		nvgRect( ctx
			   , posX + 0.5f, posY + 0.5f
			   , sizeX - 1.f, sizeY - 1.f);
		nvgStrokeColor(ctx, this->mTheme->mBorderLight);
		nvgStroke(ctx);

		nvgBeginPath(ctx);
		nvgRect( ctx
			   , posX + 0.5f, posY + 0.5f
			   , sizeX - 1.f, sizeY - 2.f);
		nvgStrokeColor(ctx, this->mTheme->mBorderDark);
		nvgStroke(ctx);
	}

	nvgTextAlign(ctx, NVG_ALIGN_LEFT | NVG_ALIGN_MIDDLE);
    nvgFillColor(ctx, textColor);
	nvgText(ctx, textPos.x(), textPos.y(), this->mCaption.c_str(), nullptr);
};

/**
 *
 */
bool NanoguiFlatButton::setWrapline(float offset, float width)
{
	this->wraplineOffset = offset;
	this->wraplineWidth = width;
	return true;
};

/**
 *
 */
bool NanoguiFlatButton::setBorderColor(nanogui::Color color)
{
	if
		(color == this->borderColor)
	{
		return false;
	}
	this->borderColor = color;
	return true;
};
