// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_floating_label.h"

using namespace SEAScope;

/**
 *
 */
NanoguiFloatingLabel::NanoguiFloatingLabel(nanogui::Widget* parent):
nanogui::Widget(parent), NanoguiComponent(parent)
{
	this->setLayout( new nanogui::BoxLayout(nanogui::Orientation::Vertical,
											nanogui::Alignment::Fill,
											3, 3));
	this->label = new nanogui::Label(this, "");
	this->label->setFixedWidth(150);
    this->setVisible(false);
}

/**
 *
 */
NanoguiFloatingLabel::~NanoguiFloatingLabel()
{
}

bool NanoguiFloatingLabel::setCaption(const std::string& value)
{
	this->label->setCaption(value);
    if
        (0 >= value.size())
    {
        this->setVisible(false);
        return false;
    }
	Eigen::Vector2i size = this->label->preferredSize(this->screen()->nvgContext());
	this->setSize(size.x() + 6, size.y() + 6);
	nanogui::Widget::setSize(Eigen::Vector2i(size.x() + 6, size.y() + 6));
	this->label->setFixedSize(size);
	this->label->setSize(size);
    this->setVisible(true);
	this->requestReflow();
	return true;
}

/**
 *
 */
void NanoguiFloatingLabel::draw(NVGcontext* ctx)
{
	nvgBeginPath(ctx);
	nvgRoundedRect(ctx,
                   static_cast<float>(this->mPos.x() + 1),
                   static_cast<float>(this->mPos.y() + 1),
				   static_cast<float>(this->mSize.x() - 2),
                   static_cast<float>(this->mSize.y() - 2),
                   3);
	nvgFillColor(ctx, nanogui::Color(50, 50, 50, 170));
	nvgFill(ctx);

	NanoguiComponent::draw(ctx);
}
