// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

#include "core/log.h"
#include "num_utils.h"
#include "ui/nanogui/nanogui_gauge.h"


using namespace SEAScope;

/**
 *
 */
NanoguiGauge::NanoguiGauge(nanogui::Widget* parent)
: nanogui::Widget(parent), angle(0), scrollStep(0.01f), dragging(false),
  changeCallback(nullptr)
{
};

/**
 *
 */
NanoguiGauge::~NanoguiGauge()
{
};

/**
 *
 */
 void NanoguiGauge::draw(NVGcontext* ctx)
{
	float t;	//
	float x;	//
	float y;	//
	float cx0;	//
	float cy0;	//
	float cx1;	//
	float cy1;	//
	float cx2;	//
	float cy2;	//
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //
    double angleTmp;    //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());
    angleTmp = static_cast<double>(this->angle);

	nanogui::Widget::draw(ctx);

	// Background
	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 1.f
			 , posY + sizeY - 1.f);
	nvgLineTo( ctx
			 , posX + sizeX - 1.f
			 , posY + sizeY - 1.f);
	nvgQuadTo( ctx
			 , posX + sizeX - 1.f
			 , posY + 1.f
			 , posX + 0.5f * sizeX
			 , posY + 1.f);
	nvgQuadTo( ctx
			 , posX + 1.f
			 , posY + 1.f
			 , posX + 1.f
			 , posY + sizeY - 1.f);
	nvgFillColor(ctx, nanogui::Color(30,30,30,100));
	nvgFill(ctx);
	nvgStrokeColor(ctx, nanogui::Color(30,30,30,150));
	nvgStroke(ctx);

	// Tick: 0%
	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 0.5f * sizeX
			 , posY + sizeY - 1.f);
	nvgLineTo( ctx
			 , posX + 1.f
			 , posY + sizeY - 1.f);
	nvgStrokeColor(ctx, nanogui::Color( 230, 230, 230, 50));
	nvgStroke(ctx);

	// Tick: 25%	
	cx0 = posX + 1.f;
	cy0 = posY + sizeY - 1.f;
	cx1 = cx0;
	cy1 = posY + 1.f;
	cx2 = posX + 0.5f * sizeX;
	cy2 = cy1;
	x = 0.25f * cx0;
	x += 0.5f * cx1;
	x += 0.25f * cx2;
	y = 0.25f * cy0;
	y += 0.5f * cy1;
	y += 0.25f * cy2;
	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 0.5f * sizeX
			 , posY + sizeY - 1.f);
	nvgLineTo( ctx
			 , x
			 , y);
	nvgStrokeColor(ctx, nanogui::Color( 230, 230, 230, 50));
	nvgStroke(ctx);

	// Tick: 50%
	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 0.5f * sizeX
			 , posY + sizeY - 1.f);
	nvgLineTo( ctx
			 , posX + 0.5f * sizeX
			 , posY + 1.f);
	nvgStrokeColor(ctx, nanogui::Color( 230, 230, 230, 50));
	nvgStroke(ctx);

	// Tick: 75%
	cx0 = posX + 0.5f * sizeX;
	cy0 = posY + 1.f;
	cx1 = posX + sizeX - 1.f;
	cy1 = cy0;
	cx2 = cx1;
	cy2 = posY + sizeY - 1.f;
	x = 0.25f * cx0;
	x += 0.5f * cx1;
	x += 0.25f * cx2;
	y = 0.25f * cy0;
	y += 0.5f * cy1;
	y += 0.25f * cy2;
	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 0.5f * sizeX
			 , posY + sizeY - 1.f);
	nvgLineTo( ctx
			 , x
			 , y);
	nvgStrokeColor(ctx, nanogui::Color( 230, 230, 230, 50));
	nvgStroke(ctx);

	// Tick: 100%
	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 0.5f * sizeX
			 , posY + sizeY - 1.f);
	nvgLineTo( ctx
			 , posX + sizeX - 1.f
			 , posY + sizeY - 1.f);
	nvgStrokeColor(ctx, nanogui::Color( 230, 230, 230, 50));
	nvgStroke(ctx);

	// Value coordinates
	if
		(0.5 * M_PI < angleTmp)
	{
		t = static_cast<float>((angleTmp - 0.5 * M_PI) / (0.5 * M_PI));
		cx0 = posX + 0.5f * sizeX;
		cy0 = posY + 2.f;
		cx1 = posX + sizeX - 2.f;
		cy1 = cy0;
		cx2 = cx1;
		cy2 = posY + sizeY - 2.f;
	}
	else
	{
		t = static_cast<float>(angleTmp / (0.5 * M_PI));
		cx0 = posX + 2.f;
		cy0 = posY + sizeY - 2.f;
		cx1 = cx0;
		cy1 = posY + 2.f;
		cx2 = posX + 0.5f * sizeX;
		cy2 = cy1;
	}

	// Compute x,y coordinates using Bezier quad equation
	x = (1-t) * (1-t) * cx0;
	x += 2 * (1-t) * t * cx1;
	x += t * t * cx2;
	y = (1-t) * (1-t) * cy0;
	y += 2 * (1-t) * t * cy1;
	y += t * t * cy2;

	// Value area
	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 0.5f * sizeX
			 , posY + sizeY - 2.f);
	nvgLineTo( ctx
			 , posX + 2.f
			 , posY + sizeY - 2.f);
	if
		(0.01 < angleTmp)
	{
		if
			(0.5 * M_PI < angleTmp)
		{
			nvgQuadTo( ctx
					 , posX + 2.f
					 , posY + 2.f
					 , posX + 0.5f * sizeX
					 , posY + 2.f);
			nvgQuadTo( ctx
					 , (posX + 0.5f * sizeX) * (1 - t) + t * (posX + sizeX - 2.f)
					 , posY + 2.f
					 , x
					 , y);
			nvgLineTo( ctx
					 , posX + 0.5f * sizeX
					 , posY + sizeY - 2.f);
		}
		else
		{
			nvgQuadTo( ctx
					 , posX + 2.f
					 , (posY + sizeY - 2.f) * (1 - t) + t * (posY + 2.f)
					 , x
					 , y);
			nvgLineTo( ctx
					 , posX + 0.5f * sizeX
					 , posY + sizeY - 2.f);
		}
		nvgFillColor(ctx, nanogui::Color(200, 250, 30, 110));
		nvgFill(ctx);
	}

	// Value line
	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 0.5f * sizeX
			 , posY + sizeY - 1.f);
	nvgLineTo( ctx
			 , x
			 , y);
	if
		(this->dragging)
	{
		nvgStrokeColor(ctx, nanogui::Color(255,50,30,255));
	}
	else
	{
		nvgStrokeColor(ctx, nanogui::Color(30,30,30,255));
	}
	nvgStroke(ctx);
};

/**
 *
 */
bool NanoguiGauge::setPercent(float percent)
{
	float radians;	//

	percent = static_cast<float>(std::min(1.f, percent));
	percent = static_cast<float>(std::max(0.f, percent));
	radians = static_cast<float>(static_cast<double>(percent) * M_PI);
	if
		(false == SEAScope::floating_point_equality(radians, this->angle))
	{
		this->angle = radians;
		return true;
	}
	return false;
};

/**
 *
 */
bool NanoguiGauge::updateAngle(float dx, float dy)
{
    double dxTmp;   //
    double dyTmp;   //
	double angleTmp;	//
	float rel;	//

    dxTmp = static_cast<double>(dx);
    dyTmp = static_cast<double>(dy);
	angleTmp = std::atan2(dyTmp, dxTmp) + M_PI;
	if
		(1.5 * M_PI < angleTmp)
	{
		angleTmp = 0.0;
	}
	else if
		(M_PI < angleTmp)
	{
		angleTmp = M_PI;
	}

	if
		(false == floating_point_equality(static_cast<float>(angleTmp), this->angle))
	{
		this->angle = static_cast<float>(angleTmp);
		if
			(nullptr != this->changeCallback)
		{
			rel = static_cast<float>(static_cast<double>(this->angle) / M_PI);
			this->changeCallback(rel);
		}
		return true;
	}

	return false;
};

/**
 *
 */
bool NanoguiGauge::mouseButtonEvent(const Eigen::Vector2i& pos, int buttons, bool pressed, int mods)
{
	float dx;		//
	float dy;		//
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //
    float mousePosX;    //
    float mousePosY;    //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());
    mousePosX = static_cast<float>(pos.x());
    mousePosY = static_cast<float>(pos.y());

	nanogui::Widget::mouseButtonEvent(pos, buttons, pressed, mods);
	if
		(false == pressed)
	{
		dx = mousePosX - (posX + 0.5f * sizeX);
		dy = mousePosY - (posY + sizeY);
		this->updateAngle(dx, dy);
		this->dragging = false;
	}
	return true;
};

/**
 *
 */
bool NanoguiGauge::mouseDragEvent(const Eigen::Vector2i& pos, const Eigen::Vector2i& /*rel*/, int /*buttons*/, int /*mods*/)
{
	float dx;		//
	float dy;		//
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //
    float mousePosX;    //
    float mousePosY;    //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());
    mousePosX = static_cast<float>(pos.x());
    mousePosY = static_cast<float>(pos.y());

	dx = mousePosX - (posX + 0.5f * sizeX);
	dy = mousePosY - (posY + sizeY);
	this->updateAngle(dx, dy);
	this->dragging = true;
	return true;
};

/**
 *
 */
bool NanoguiGauge::setChangeCallback(const std::function<bool(float)>& callback)
{
	this->changeCallback = callback;
	return true;
};

/**
 *
 */
bool NanoguiGauge::scrollEvent(const Eigen::Vector2i& /*p*/, const Eigen::Vector2f& rel)
{
	bool changed;	//
	double percent;	//
	double radians;	//
    double angleTmp; //
    double scrollStepTmp;   //

	changed = false;
    angleTmp = static_cast<double>(this->angle);
    scrollStepTmp = static_cast<double>(this->scrollStep);

	percent = angleTmp / M_PI + scrollStepTmp * static_cast<double>(rel.y());

	percent = std::min(1.0, percent);
	percent = std::max(0.0, percent);
	radians = percent * M_PI;

	changed = (false == floating_point_equality(radians, angleTmp));

	if
		(  changed
		&& nullptr != this->changeCallback)
	{
		this->changeCallback(static_cast<float>(percent));
	}

	return true;
};
