// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <ctime>
#include <sstream>

#include "vendors/_nanogui.h"

#include "core/log.h"
#include "core/granule_metadata.h"
#include "core/granule_info.h"
#include "ui/nanogui/nanogui_tag.h"
#include "ui/nanogui/nanogui_granule_showcase.h"
#include "ui/nanogui/nanogui_calendar.h"
#include "ui/nanogui/nanogui_flat_button.h"

using namespace SEAScope;

/**
 *
 */
NanoguiGranuleShowcase::NanoguiGranuleShowcase(nanogui::Widget* parent)
: nanogui::Widget(parent), NanoguiScrollableWithTitle(parent), uri(nullptr)
, collectionLbl(nullptr), collectionTags(nullptr), variableLbl(nullptr)
, variableUnitsLbl(nullptr), variableTags(nullptr), dataId(nullptr)
, dataModel(nullptr), start(nullptr), stop(nullptr), granuleName(nullptr)
, institution(nullptr), comment(nullptr), fileId(nullptr)
, productVersion(nullptr), latMin(nullptr), latMax(nullptr), lonMin(nullptr)
, lonMax(nullptr), creatorEmail(nullptr), station(nullptr), platform(nullptr)
, sensor(nullptr), resContainer(nullptr), datesContainer(nullptr)
, monthCallback(nullptr), yearCallback(nullptr)
{
	nanogui::Widget* wrap;	//
	nanogui::Widget* main;	//
	nanogui::Widget* side;	//
	nanogui::Widget* block;	//
	nanogui::Widget* block2;	//
	nanogui::Label* lbl;	//

    this->header->setFixedHeight(10);  // add extra margin on top
    this->footer->setFixedHeight(5);  // add extra margin at the bottom

	this->scrollable->content->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
										   , nanogui::Alignment::Minimum
										   , 8, 0));

	wrap = new nanogui::Widget(this->scrollable->content);
	wrap->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
										    , nanogui::Alignment::Minimum
											, 0, 0));
	wrap->setFixedWidth(304);
	wrap->setWidth(304);

	main = new nanogui::Widget(wrap);
	main->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										   , nanogui::Alignment::Minimum
										   , 0, 0));
	main->setFixedWidth(216);
	main->setWidth(216);

	side = new nanogui::Widget(wrap);
	side->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										   , nanogui::Alignment::Minimum
										   , 0, 0));
	side->setFixedWidth(86);
	side->setWidth(86);

	/* Collection */
	lbl = new nanogui::Label(main, "Collection", "sans-bold");
	lbl->setFontSize(18);
	lbl->setFixedSize(Eigen::Vector2i(216, 18));
	block = new nanogui::Widget(main);
	block->setFixedSize(Eigen::Vector2i(216, 2));
	this->collectionLbl = new nanogui::Label(main, "");
	this->collectionLbl->setFixedSize(Eigen::Vector2i(216, 16));

	block2 = new nanogui::Widget(main);
	block2->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
											 , nanogui::Alignment::Maximum
											 , 0, 0));
	block2->setFixedWidth(212);
	block2->setWidth(212);
	this->collectionTags = new nanogui::Widget(block2);
	this->collectionTags->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
														   , nanogui::Alignment::Middle
														   , 0, 2));
	block = new nanogui::Widget(main);
	block->setSize(Eigen::Vector2i(216, 5));

	/* Variable */
	lbl = new nanogui::Label(main, "Variable", "sans-bold");
	lbl->setFontSize(18);
	lbl->setFixedSize(Eigen::Vector2i(216,18));
	block = new nanogui::Widget(main);
	block->setFixedSize(Eigen::Vector2i(216, 2));
	this->variableLbl = new nanogui::Label(main, "");
	this->variableLbl->setFixedSize(Eigen::Vector2i(216, 16));

	block2 = new nanogui::Widget(main);
	block2->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
											 , nanogui::Alignment::Maximum
											 , 0, 0));
	block2->setFixedWidth(212);
	block2->setWidth(212);

	this->variableUnitsLbl = new nanogui::Label(block2, "");

	this->variableTags = new nanogui::Widget(block2);
	this->variableTags->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
														 , nanogui::Alignment::Middle
														 , 0, 2));
	block = new nanogui::Widget(main);
	block->setSize(Eigen::Vector2i(216, 5));

	/* Granule */
	lbl = new nanogui::Label(main, "Granule", "sans-bold");
	lbl->setFontSize(18);
	lbl->setFixedSize(Eigen::Vector2i(216,18));
	block = new nanogui::Widget(main);
	block->setFixedSize(Eigen::Vector2i(216, 2));

	this->dataId = new NanoguiAttribute(main);
	this->dataId->setName("¤ Data ID");
	this->dataModel = new NanoguiAttribute(main);
	this->dataModel->setName("¤ Data model");
	this->start = new NanoguiAttribute(main);
	this->start->setName("¤ Start");
	this->stop = new NanoguiAttribute(main);
	this->stop->setName("¤ Stop");
	this->granuleName = new NanoguiAttribute(main);
	this->granuleName->setName("¤ Title");
	this->institution = new NanoguiAttribute(main);
	this->institution->setName("¤ Institution");
	this->comment = new NanoguiAttribute(main);
	this->comment->setName("¤ Comment");
	this->fileId = new NanoguiAttribute(main);
	this->fileId->setName("¤ File ID");
	this->productVersion = new NanoguiAttribute(main);
	this->productVersion->setName("¤ Product version");
	this->latMin = new NanoguiAttribute(main);
	this->latMin->setName("¤ Lat min");
	this->latMax = new NanoguiAttribute(main);
	this->latMax->setName("¤ Lat max");
	this->lonMin = new NanoguiAttribute(main);
	this->lonMin->setName("¤ Lon min");
	this->lonMax = new NanoguiAttribute(main);
	this->lonMax->setName("¤ Lon max");
	this->creatorEmail = new NanoguiAttribute(main);
	this->creatorEmail->setName("¤ Creator email");
	this->station = new NanoguiAttribute(main);
	this->station->setName("¤ Station");
	this->platform = new NanoguiAttribute(main);
	this->platform->setName("¤ Platform");
	this->sensor = new NanoguiAttribute(main);
	this->sensor->setName("¤ Sensor");

	/* Side */
	lbl = new nanogui::Label(side, "<> Resolutions", "sans-bold");
	lbl->setColor(nanogui::Color(180,180,180,150));
	this->resContainer = new nanogui::Widget(side);
	this->resContainer->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
														 , nanogui::Alignment::Middle
														 ,0, 0));
	block = new nanogui::Widget(side);
	block->setSize(Eigen::Vector2i(86, 3));

	lbl = new nanogui::Label(side, "<> Dates", "sans-bold");
	lbl->setColor(nanogui::Color(180,180,180,150));
	this->datesContainer = new nanogui::Widget(side);
	this->datesContainer->setFixedWidth(86);
	this->datesContainer->setWidth(86);
	this->datesContainer->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
											, nanogui::Alignment::Middle
											, 0, 3));

	wrap = nullptr;
	main = nullptr;
	side = nullptr;
	block = nullptr;
	block2 = nullptr;
	lbl = nullptr;
};

/**
 *
 */
NanoguiGranuleShowcase::~NanoguiGranuleShowcase()
{
    this->uri = nullptr;
    this->collectionLbl = nullptr;
    this->collectionTags = nullptr;
    this->variableLbl = nullptr;
    this->variableUnitsLbl = nullptr;
    this->variableTags = nullptr;
    this->dataId = nullptr;
    this->dataModel = nullptr;
    this->start = nullptr;
    this->stop = nullptr;
    this->granuleName = nullptr;
    this->institution = nullptr;
    this->comment = nullptr;
    this->fileId = nullptr;
    this->productVersion = nullptr;
    this->latMin = nullptr;
    this->latMax = nullptr;
    this->lonMin = nullptr;
    this->lonMax = nullptr;
    this->creatorEmail = nullptr;
    this->station = nullptr;
    this->platform = nullptr;
    this->sensor = nullptr;
    this->resContainer = nullptr;
    this->datesContainer = nullptr;
    this->monthCallback = nullptr;
    this->yearCallback = nullptr;
};

/**
 *
 */
bool NanoguiGranuleShowcase::displayInfo(const GranuleInfo& info)
{
	NanoguiTag* t;	//
	NanoguiCalendar* calendar;	//
	NanoguiFlatButton* btn;	//
	std::vector<std::uint16_t> years;	//
	std::vector<std::uint8_t> months;	//
	std::uint8_t month;	//
	std::ostringstream buffer;	//
	std::string dataModelLabel;	//
	char startStr[20];	//
	char stopStr[20];	//
	time_t timestampStart;	//
	time_t timestampStop;	//
    double startTmp;    //
    double stopTmp; //
	std::tm* ltimeStart;	//
	std::tm* ltimeStop;	//
	bool hasAttr;	//
	bool ok;	//
	int l;	//

	ok = info.coverage.getYears(years);
	if
		(false == ok)
	{
		return false;
	}

	this->collectionLbl->setCaption(info.collection.label);
	l = static_cast<int>(this->collectionTags->children().size());
	while
		(--l > -1)
	{
		this->collectionTags->removeChild(l);
	}
	for
		(const auto& it: info.collection.tags)
	{
		t = new NanoguiTag(this->collectionTags);
		t->setNameColor(nanogui::Color(25, 25, 25, 200));
		t->setValueColor(nanogui::Color(30, 30, 100, 200));
		t->setValue(it.first, it.second);
	}

	this->variableLbl->setCaption(info.variable.label);
	this->variableUnitsLbl->setCaption("units: " + info.variable.units);
	l = static_cast<int>(this->variableTags->children().size());
	while
		(--l > -1)
	{
		this->variableTags->removeChild(l);
	}
	for
		(const auto& it: info.variable.tags)
	{
		t = new NanoguiTag(this->variableTags);
		t->setNameColor(nanogui::Color(25, 25, 25, 200));
		t->setValueColor(nanogui::Color(30, 30, 100, 200));
		t->setValue(it.first, it.second);
	}

	this->dataId->setValue(info.metadata.dataId());

	/* dataModel */
    DataModel _dataModel {info.metadata.dataModel()};
	if
		(DataModel::LAT_LON == _dataModel)
	{
		dataModelLabel = "Lat / Lon";
	}
	else if
		(DataModel::Y_X == _dataModel)
	{
		dataModelLabel = "Y / X";
	}
	else if
		(DataModel::NJ_NI == _dataModel)
	{
		dataModelLabel = "Nj / Ni";
	}
	else if
		(DataModel::ROW_CELL == _dataModel)
	{
		dataModelLabel = "Row / Cell";
	}
	else if
		(DataModel::TIME_STATION == _dataModel)
	{
		dataModelLabel = "Time / Station";
	}
	else if
		(DataModel::TIME == _dataModel)
	{
		dataModelLabel = "Time";
	}
	else
	{
		LOG( LogLevel::Error, "NanoguiGranuleShowcase"
				, "Unknown data model");
		return false;
	}
	this->dataModel->setValue(dataModelLabel);

	/* start */
    startTmp = static_cast<double>(info.metadata.timeCoverageStart());
	timestampStart = static_cast<long>(.001 * startTmp);
	ltimeStart = std::localtime(&timestampStart);
	strftime(startStr, sizeof(startStr), "%Y-%m-%dT%H:%M:%S", ltimeStart);
	this->start->setValue(startStr);

	/* stop */
    stopTmp = static_cast<double>(info.metadata.timeCoverageEnd());
	timestampStop = static_cast<long>(.001 * stopTmp);
	ltimeStop = std::localtime(&timestampStop);
	strftime(stopStr, sizeof(stopStr), "%Y-%m-%dT%H:%M:%S", ltimeStop);
	this->stop->setValue(stopStr);

	/* title */
	this->granuleName->setValue(info.metadata.title());
    hasAttr = info.metadata.has(IDFAttributes::TITLE);
	this->granuleName->setVisible(hasAttr);

	/* institution */
	this->institution->setValue(info.metadata.institution());
    hasAttr = info.metadata.has(IDFAttributes::INSTITUTION);
	this->institution->setVisible(hasAttr);

	/* comment */
	this->comment->setValue(info.metadata.comment());
    hasAttr = info.metadata.has(IDFAttributes::COMMENT);
	this->comment->setVisible(hasAttr);

	/* fileId */
	this->fileId->setValue(info.metadata.fileId());
    hasAttr = info.metadata.has(IDFAttributes::FILE_ID);
	this->fileId->setVisible(hasAttr);

	/* productVersion */
	this->productVersion->setValue(info.metadata.productVersion());
    hasAttr = info.metadata.has(IDFAttributes::PRODUCT_VERSION);
	this->productVersion->setVisible(hasAttr);

	/* latMin */
	buffer.str("");
	buffer << info.metadata.latMin() << " degrees north";
	this->latMin->setValue(buffer.str());
    hasAttr = info.metadata.has(IDFAttributes::LAT_MIN);
	this->latMin->setVisible(hasAttr);

	/* latMax */
	buffer.str("");
	buffer << info.metadata.latMax() << " degrees north";
	this->latMax->setValue(buffer.str());
    hasAttr = info.metadata.has(IDFAttributes::LAT_MAX);
	this->latMax->setVisible(hasAttr);

	/* lonMin */
	buffer.str("");
	buffer << info.metadata.lonMin() << " degrees east";
	this->lonMin->setValue(buffer.str());
    hasAttr = info.metadata.has(IDFAttributes::LON_MIN);
	this->lonMin->setVisible(hasAttr);

	/* lonMax */
	buffer.str("");
	buffer << info.metadata.lonMax() << " degrees east";
	this->lonMax->setValue(buffer.str());
    hasAttr = info.metadata.has(IDFAttributes::LON_MAX);
	this->lonMax->setVisible(hasAttr);

	/* creatorEmail */
	this->creatorEmail->setValue(info.metadata.creatorEmail());
    hasAttr = info.metadata.has(IDFAttributes::CREATOR_EMAIL);
	this->creatorEmail->setVisible(hasAttr);

	/* stationId */
	this->station->setValue(info.metadata.stationId());
    hasAttr = info.metadata.has(IDFAttributes::STATION_ID);
	this->station->setVisible(hasAttr);

	/* platform */
	this->platform->setValue(info.metadata.platform());
    hasAttr = info.metadata.has(IDFAttributes::PLATFORM);
	this->platform->setVisible(hasAttr);

	/* sensor */
	this->sensor->setValue(info.metadata.sensor());
    hasAttr = info.metadata.has(IDFAttributes::SENSOR);
	this->sensor->setVisible(hasAttr);

	/* resolutions */
	l = static_cast<int>(this->resContainer->children().size());
	while
		(--l > -1)
	{
		this->resContainer->removeChild(l);
	}
	for
		(std::uint32_t i = 0; i < info.metadata.viewsCount(); ++i)
	{
        IDFDescriptor idf;
        info.metadata.getView(i, idf);
		buffer.str("");
		buffer << idf.resolution << "m";
		btn = new NanoguiFlatButton(this->resContainer);
		btn->setCaption(buffer.str());
		btn->setFixedSize(Eigen::Vector2i(86, 14));
		btn->setFontSize(14);
		btn->setBackgroundColor(nanogui::Color(50, 50, 50, 0));
		if
			(idf.uri == info.uri)
		{
			btn->setTextColor(nanogui::Color(250, 250, 30, 255));
		}
		else
		{
			btn->setTextColor(nanogui::Color(250, 250, 250, 255));
		}
		buffer.str("");
	}

	/* coverage */
	l = static_cast<int>(this->datesContainer->children().size());
	while
		(--l > -1)
	{
		this->datesContainer->removeChild(l);
	}
	for
		(const auto& year : years)
	{
		months.clear();
		ok = info.coverage.getMonths(year, months);
		calendar = new NanoguiCalendar(this->datesContainer);
		calendar->setYear(year);
		calendar->setYearBgColor(nanogui::Color(0,0,0,50));
		calendar->setYearFgColor(nanogui::Color(250, 250, 250, 255));
		calendar->yearBtn->setFontSize(12);
		calendar->yearBtn->setFixedHeight(12);
		month = 1;
		for
			(; month < 13; ++month)
		{
			auto monthFound = std::find( months.begin()
									   , months.end()
									   , month);
			if
				(months.end() != monthFound)
			{
				calendar->setMonthBgColor( month
										 , nanogui::Color(250, 250, 250, 255));
			}
			else
			{
				calendar->setMonthBgColor( month
										 , nanogui::Color(90, 90, 90, 255));
			}
		}

		calendar->monthCallback = [this] (std::uint16_t yearIn, std::uint8_t monthIn) {
			if
				(nullptr != this->monthCallback)
			{
				this->monthCallback(yearIn, monthIn);
			}
		};

		calendar->yearCallback = [this] (std::uint16_t yearIn) {
			if
				(nullptr != this->yearCallback)
			{
				this->yearCallback(yearIn);
			}
		};
	}

	this->reflowNeeded = true;

	t = nullptr;
	calendar = nullptr;
	btn = nullptr;
    ltimeStart = nullptr;
    ltimeStop = nullptr;

	return false;
};

/**
 *
 */
nanogui::Label* NanoguiGranuleShowcase::addAttribute( nanogui::Widget* parent
													, const std::string& name)
{
	nanogui::Label* lbl;	//
	nanogui::Widget* valueWrapper;	//
	nanogui::Widget* spacer;	//

	lbl = new nanogui::Label(parent, name, "sans-bold");
	lbl->setFixedWidth(216);
	lbl->setColor(nanogui::Color(180,180,180,150));

	valueWrapper = new nanogui::Widget(parent);
	valueWrapper->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
												   , nanogui::Alignment::Middle
												   , 0, 0));
	spacer = new Widget(valueWrapper);
	spacer->setFixedWidth(10);

	lbl = new nanogui::Label(valueWrapper, "", "sans-bold");
	lbl->setFixedWidth(201);

    valueWrapper = nullptr;
    spacer = nullptr;

	return lbl;
};

/**
 *
 */
void NanoguiGranuleShowcase::draw(NVGcontext* ctx)
{
	float uriHeight;	//
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //
    float titleHeight;  //

	NanoguiScrollableWithTitle::draw(ctx);

	uriHeight = 0.0f;
    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());
    titleHeight = static_cast<float>(this->titlebar->fixedHeight());

	if
		(nullptr != this->uri)
	{
		uriHeight = static_cast<float>(this->uri->height());

		nvgBeginPath(ctx);
		nvgMoveTo( ctx
				 , posX + 1.0f
				 , posY + titleHeight + uriHeight + 2.0f);
		nvgLineTo( ctx
				 , sizeX - 1.0f
				 , posY + titleHeight + uriHeight + 2.0f);
		nvgStrokeColor(ctx, nanogui::Color(255, 255, 255, 130));
		nvgStroke(ctx);
	}

	nvgBeginPath(ctx);
	nvgMoveTo( ctx
			 , posX + 8.0f + 216.0f
			 , posY + titleHeight + uriHeight + 12.0f);
	nvgLineTo( ctx
			 , posX + 8.0f + 216.0f
			 , posY + sizeY);
	nvgStrokeColor(ctx, nanogui::Color(255, 255, 255, 40));
	nvgStroke(ctx);
};
