// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "core/log.h"
#include "ui/nanogui/nanogui_icon.h"

using namespace SEAScope;

/**
 *
 */
NanoguiIcon::NanoguiIcon(nanogui::Widget* parent, int icon)
: nanogui::Widget(parent),
  _icon(icon),
  _color(nanogui::Color(200,200,200,200)),
  _backgroundColor(nanogui::Color(0, 0, 0, 0))
{
}

/**
 *
 */
NanoguiIcon::~NanoguiIcon()
{
}

/**
 *
 */
bool NanoguiIcon::setColor(nanogui::Color color)
{
	if
		(this->_color != color)
	{
		this->_color = color;
		return true;
	}
	return false;
}

/**
 *
 */
bool NanoguiIcon::setBackgroundColor(nanogui::Color color)
{
	if
		(this->_backgroundColor != color)
	{
		this->_backgroundColor = color;
		return true;
	}
	return false;
}

/**
 *
 */
Eigen::Vector2i NanoguiIcon::preferredSize(NVGcontext* ctx) const
{
	float iw;	//
	float ih;	//
	int w;	//
	int h;	//
    float sizeY;    //
    float wTmp; //
    float hTmp; //

	iw = 0.0f;
	ih = static_cast<float>(this->mFontSize);
    sizeY = static_cast<float>(this->mSize.y());

	if
		(nanogui::nvgIsFontIcon(this->_icon))
	{
		ih *= this->icon_scale();
		nvgFontFace(ctx, "icons");
		nvgFontSize(ctx, ih);
		auto str = nanogui::utf8(this->_icon);
		iw = nvgTextBounds(ctx, 0, 0, str.data(), nullptr, nullptr)
			+ sizeY * 0.15f;
	}
	else
	{
		ih *= 0.9f;
		nvgImageSize(ctx, this->_icon, &w, &h);
        wTmp = static_cast<float>(w);
        hTmp = static_cast<float>(h);
		iw = wTmp * ih / hTmp;
	}
    return Eigen::Vector2i(static_cast<int>(iw) + 20, this->mFontSize + 10);
}

/**
 *
 */
void NanoguiIcon::draw(NVGcontext* ctx)
{
	float iw;	//
	float ih;	//
    int w;	//
	int h;	//
    float wTmp; //
    float hTmp; //
    float posX;    //
    float posY;    //
    float sizeX;    //
    float sizeY;    //
    Eigen::Vector2f iconPos;	//

	nanogui::Widget::draw(ctx);

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());

    nvgBeginPath(ctx);
    nvgRect(ctx, posX, posY, posX + sizeX, posY + sizeY);
    nvgFillColor(ctx, this->_backgroundColor);
    nvgFill(ctx);
    nvgClosePath(ctx);

	auto iconTmp = nanogui::utf8(this->_icon);

    ih = static_cast<float>(this->mFontSize);
    if
		(nanogui::nvgIsFontIcon(this->_icon))
	{
		ih *= this->icon_scale();
		nvgFontSize(ctx, ih);
		nvgFontFace(ctx, "icons");
		iw = nvgTextBounds(ctx, 0, 0, iconTmp.data(), nullptr, nullptr);
    }
	else
	{
        ih *= 0.9f;
        nvgImageSize(ctx, this->_icon, &w, &h);
        wTmp = static_cast<float>(w);
        hTmp = static_cast<float>(h);
        iw = wTmp * ih / hTmp;
    }

    nvgFillColor(ctx, this->_color);
    nvgTextAlign(ctx, NVG_ALIGN_LEFT | NVG_ALIGN_MIDDLE);
	iconPos = this->mPos.cast<float>() + this->mSize.cast<float>() * 0.5f;
    iconPos.y() -= 1;
	iconPos.x() -= iw * 0.5f;

	if
		(nanogui::nvgIsFontIcon(this->_icon))
	{
		nvgText(ctx, iconPos.x(), iconPos.y()+1, iconTmp.data(), nullptr);
	}
	else
	{
		NVGpaint imgPaint = nvgImagePattern(ctx,
				iconPos.x(), iconPos.y() - ih/2, iw, ih, 0, this->_icon, this->mEnabled ? 0.5f : 0.25f);

		nvgFillPaint(ctx, imgPaint);
		nvgFill(ctx);
	}
}

/**
 *
 */
bool NanoguiIcon::setIcon(int value)
{
    if
        (value == this->_icon)
    {
        return false;
    }
    this->_icon = value;
    return true;
}
