// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "ui/nanogui/nanogui_icons.h"
#include "3d/icons.h"
#include "3d/texture.h"

#include "vendors/_glad.h"
#include "vendors/_nanovg.h"

NanoguiIcons::NanoguiIcons()
{
}

NanoguiIcons::~NanoguiIcons()
{
}

void NanoguiIcons::setTextures(NVGcontext* ctx, const Icons& icons)
{
    auto textures = icons.getTextures();

    auto it = textures.begin();
    auto itEnd = textures.end();

    for
        (; it != itEnd; ++it)
    {
        std::string name = icons.getName(it->first);
        
        _idToName[it->first] = name;
        _nameToId[name] = it->first;

        OGL::Texture* tex = it->second;

        // Create a nanovg image from the OpenGL texture without copy
        int nvgTexture = nvglCreateImageFromHandleGL3(ctx, it->second->getHandle(), static_cast<int>(tex->width()), static_cast<int>(tex->height()), NVG_IMAGE_NODELETE);

        _nvgtextures[it->first] = nvgTexture;
    }
}

int NanoguiIcons::getNVGTexture(IconType iconType, bool& ok) const
{
    return getNVGTexture(static_cast<std::uint16_t>(iconType), ok);
}

int NanoguiIcons::getNVGTexture(const std::string& name, bool& ok) const
{
    auto found = this->_nameToId.find(name);
    if
        (this->_nameToId.end() == found)
    {
        ok = false;
        return  -1;
    }

    return getNVGTexture(found->second, ok);
}

int NanoguiIcons::getNVGTexture(std::uint16_t iconType, bool& ok) const
{
    auto found = this->_nvgtextures.find(iconType);
    if
        (this->_nvgtextures.end() == found)
    {
        ok = false;
        return  -1;
    }

    ok = true;
    return found->second;
}

std::string NanoguiIcons::getName(const std::uint16_t& id) const
{
    auto it = _idToName.find(id);

    if
        (it != _idToName.end())
    {
        return it->second;
    }

    return std::string("");
}

std::uint16_t NanoguiIcons::getId(const std::string& name) const
{
    auto it = _nameToId.find(name);

    if
        (it != _nameToId.end())
    {
        return it->second;
    }

    return 0;
}
