// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "core/log.h"
#include "ui/nanogui/nanogui_layout_settings.h"

using namespace SEAScope;

/**
 *
 */
NanoguiLayoutSettings::NanoguiLayoutSettings(nanogui::Widget* parent): nanogui::Widget(parent)
{
	nanogui::Label* lbl;	//
	this->setLayout(new nanogui::BoxLayout( nanogui::Orientation::Vertical
										  , nanogui::Alignment::Middle
										  , 0, 10));
	lbl = new nanogui::Label(this, "Modify the size and position of user interface elements");
	lbl->setFixedWidth(300);
	this->componentsList = new nanogui::Widget(this);
	this->componentsList->setLayout( new nanogui::GridLayout( nanogui::Orientation::Horizontal, 2
															, nanogui::Alignment::Middle
															, 2, 2));
	this->saveButton = new nanogui::Button(this, "Save", ENTYPO_ICON_SAVE);
	this->saveButton->setFixedSize(Eigen::Vector2i(75, 30));
	this->saveButton->setBackgroundColor(nanogui::Color(30, 135, 50, 250));
	this->saveButton->setEnabled(false);
	this->saveButton->setCallback([this]() {
		if
			(nullptr != this->saveCallback)
		{
			this->saveCallback();
		}
		this->saveButton->setEnabled(false);
	});
};

/**
 *
 */
NanoguiLayoutSettings::~NanoguiLayoutSettings()
{

};

/**
 *
 */
bool NanoguiLayoutSettings::addBlueprint(const std::string& label, NanoguiBlueprint* blueprint)
{
	nanogui::Label* lbl;	//
	NanoguiBlueprintEditor* editor;	//

	lbl = new nanogui::Label(this->componentsList, label, "sans-bold");
	lbl->setFontSize(16);

	editor = new NanoguiBlueprintEditor(this->componentsList, blueprint);
	editor->setCallback([this]() {
		this->saveButton->setEnabled(true);
	});
	return true;
};

/**
 *
 */
bool NanoguiLayoutSettings::setSaveCallback(const std::function<void()>& callback)
{
	this->saveCallback = callback;
	return true;
};
