// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <cmath>
#include "core/log.h"
#include "ui/nanogui/nanogui_list.h"

using namespace SEAScope;

/**
 *
 */
NanoguiList::NanoguiList(nanogui::Widget* parent)
: nanogui::VScrollPanel(parent), content(nullptr)
{
	this->content = new NanoguiSortedList(this);
};

/**
 *
 */
NanoguiList::~NanoguiList()
{
};

/**
 *
 */
void NanoguiList::update(bool resetScroll)
{
    float prevHeight = static_cast<float>(this->content->height());
    this->content->update();

    if
        (true == resetScroll)
    {
        this->mScroll = 0.f;
    }
    else
    {
        // Try to adapt scrolling to the new height
        this->mScroll = this->mScroll + static_cast<float>(this->content->height()) - prevHeight; //0.f;
    }
}

/**
 *
 */
bool NanoguiList::scrollEvent(const Eigen::Vector2i &p, const Eigen::Vector2f &rel) {
	bool ok;	//
    float prevScroll;	//
	float scrollRatio;	//
	float scrollAmount;	//
	float scrollh;	//
    float sizeY;    //
    float heightTmp;    //
    float childPreferredHeightTmp;  //

    heightTmp = static_cast<float>(this->height());
    childPreferredHeightTmp = static_cast<float>(this->mChildPreferredHeight);
    sizeY = static_cast<float>(this->mSize.y());
	const float scrollStep = sizeY / 20.0f;
	const float SCROLL_EPSILON = 0.001f;

	ok = false;
	prevScroll = 0.0f;
	scrollRatio = 0.0f;
	scrollAmount = 0.0f;
	scrollh = 0.0f;

	ok = Widget::scrollEvent(p, rel);
    if (  false == ok
	   && false == this->mChildren.empty()
	   && this->mChildPreferredHeight > this->mSize.y())
	{
        scrollAmount = rel.y() * scrollStep;
		scrollRatio = heightTmp / childPreferredHeightTmp;
        scrollh = heightTmp * std::min(1.0f, scrollRatio);

		prevScroll = this->mScroll;
        this->mScroll = std::max( 0.0f
								, std::min( 1.0f
										  , this->mScroll - scrollAmount / std::fabs(sizeY - 8.0f - scrollh)));
        ok = std::fabs(this->mScroll - prevScroll) > SCROLL_EPSILON;
        this->mUpdateLayout = ok;
    }

    for
        (auto child : this->content->children())
    {
        if
            (false == child->visible())
        {
            continue;
        }

        if
            ((true == child->focused()) && (false == child->contains(p - this->mPos - this->content->position())))
        {
            child->mouseEnterEvent(p - this->mPos - this->content->position(), false);
        }
        else if
            ((false == child->focused()) && (true == child->contains(p - this->mPos - this->content->position())))
        {
            child->mouseEnterEvent(p - this->mPos - this->content->position(), true);
        }
    }

	return ok;
}
