// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_map_caption.h"
#include <nanogui/opengl.h>

using namespace SEAScope;

/**
 *
 */
NanoguiMapCaption::NanoguiMapCaption(nanogui::Widget* parent, std::vector<std::string>& textlines)
: nanogui::Widget(parent), NanoguiComponent(parent)
{
	nanogui::Label* lbl;
	auto it = textlines.begin();

	this->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										   , nanogui::Alignment::Middle
										   , 2, 3));
	for
		(;it != textlines.end(); it++)
	{
		lbl = new nanogui::Label(this, *it, "sans-bold");
		lbl->setFontSize(14);
		lbl->setColor(nanogui::Color(250,250,250,255));
	}
};

/**
 *
 */
NanoguiMapCaption::~NanoguiMapCaption()
{
};

/**
 *
 */
bool NanoguiMapCaption::isResizable() const
{
	return false;
};

/**
 *
 */
bool NanoguiMapCaption::isMovable() const
{
	return false;
};

/**
 *
 */
void NanoguiMapCaption::draw(NVGcontext* ctx)
{
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());

	if
		(true == this->reflowNeeded)
	{
		this->performLayout(ctx);
		this->reflowNeeded = false;
	}
	nvgBeginPath(ctx);
	nvgRoundedRect( ctx, posX + 1.f, posY + 1.f
				  , sizeX - 2.f, sizeY - 1.f
				  , 2);
	nvgFillColor(ctx, nanogui::Color(0,0,0,90));
	nvgFill(ctx);

	nanogui::Widget::draw(ctx);
};
