// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_icon.h"
#include "ui/nanogui/nanogui_rendering_menu.h"

using namespace SEAScope;

/**
 *
 */
NanoguiRenderingMenu::NanoguiRenderingMenu()
: window(nullptr), control(nullptr), advanced(nullptr), headerbar(nullptr),
  moreBtn(nullptr), reduceBtn(nullptr), expandBtn(nullptr), title(nullptr),
  spacer(nullptr), moreDataCallback(nullptr)
{
};

/**
 *
 */
NanoguiRenderingMenu::~NanoguiRenderingMenu()
{
};

/**
 *
 */
bool NanoguiRenderingMenu::init(nanogui::Widget* screen)
{
	std::uint32_t headerHeight;	//
    std::uint32_t windowHeight; //
    std::uint32_t containerHeight;  //

    std::uint32_t windowWidth = 480;    // 365

	if
		(nullptr != this->window)
	{
		/* Already initialize */
		return false;
	}

	headerHeight = 24;
    windowHeight = 324;
    containerHeight = windowHeight - (headerHeight + 4);

	this->window = new NanoguiDraggablePane(screen);
	this->window->headerHeight = headerHeight;

	this->headerbar = new nanogui::Widget(this->window);
	this->headerbar->setFixedSize(Eigen::Vector2i(windowWidth, headerHeight));
	this->headerbar->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
												nanogui::Alignment::Middle,
												2, 3));

    this->spacer = new nanogui::Widget(this->window);
    this->spacer->setFixedHeight(4);

    nanogui::Widget* container = new nanogui::Widget(this->window);
    container->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                nanogui::Alignment::Middle,
                                                0, 0));

	this->control = new NanoguiRenderingControl(container);
    this->control->setFixedSize(Eigen::Vector2i(365, containerHeight));
    this->control->scroller->setFixedSize(Eigen::Vector2i(365, containerHeight));

    this->advanced = new NanoguiAdvancedRenderingControl(container);
    this->advanced->resize(Eigen::Vector2i(230, containerHeight));
    this->advanced->setFixedSize(Eigen::Vector2i(230, containerHeight));
    this->advanced->setActivationCallback([this](bool activated) {
        if
            (activated)
        {
            if
                (this->control->visible())
            {
                this->headerbar->setFixedSize(Eigen::Vector2i(480, 24));
            }
            else
            {
                this->headerbar->setFixedSize(Eigen::Vector2i(115, 24));
            }
        }
        else
        {
            if
                (this->control->visible())
            {
                this->headerbar->setFixedSize(Eigen::Vector2i(365, 24));
            }
            else
            {
                this->headerbar->setFixedSize(Eigen::Vector2i(50, 24));
            }
        }
		this->window->updateLayout();
    });
    //this->advanced->hide();

	NanoguiIcon* icon = new NanoguiIcon(this->headerbar, ENTYPO_ICON_SOUND_MIX);
	icon->setFixedSize(Eigen::Vector2i(20, 20));
	icon->setFontSize(18);
    icon->setBackgroundColor(nanogui::Color(48, 139, 87, 255));
	icon->setColor(nanogui::Color(255,255,255,255));
	icon->setTooltip("Rendering menu.");

	this->title = new nanogui::Label(this->headerbar, "Display data", "sans-bold");
    this->title->setFontSize(16);
	this->title->setFixedSize(Eigen::Vector2i(251, 18));

	this->moreBtn = new nanogui::Button(this->headerbar, "More", ENTYPO_ICON_PLUS);
	this->moreBtn->setFontSize(16);
	this->moreBtn->setTextColor(nanogui::Color(255,255,255,255));
	this->moreBtn->setFixedSize(Eigen::Vector2i(60, 18));
	this->moreBtn->setBackgroundColor(nanogui::Color(48, 87, 139, 255));

	this->expandBtn = new nanogui::Button(this->headerbar, "", ENTYPO_ICON_RESIZE_FULL_SCREEN);
	this->expandBtn->setTooltip("Show rendering menu.");
	this->expandBtn->setFixedSize(Eigen::Vector2i(18,18));
	this->expandBtn->setFontSize(16);
	this->expandBtn->setVisible(false);

	this->reduceBtn = new nanogui::Button(this->headerbar, "", ENTYPO_ICON_RESIZE_100_PERCENT);
	this->reduceBtn->setTooltip("Hide rendering menu.");
	this->reduceBtn->setFixedSize(Eigen::Vector2i(18,18));
	this->reduceBtn->setFontSize(16);

	this->moreBtn->setCallback([this] () {
		if
			(nullptr != this->moreDataCallback)
		{
			this->moreDataCallback();
		}
	});

	this->reduceBtn->setCallback([this]() {
		this->title->setVisible(false);
		this->control->setVisible(false);
        this->moreBtn->setVisible(false);
		this->reduceBtn->setVisible(false);
		this->spacer->setVisible(false);

		this->expandBtn->setVisible(true);

        if
            (this->advanced->visible())
        {
		    this->headerbar->setFixedSize(Eigen::Vector2i(115, 24));
        }
        else
        {
		    this->headerbar->setFixedSize(Eigen::Vector2i(50, 24));
        }
		this->window->updateLayout();
	});

	this->expandBtn->setCallback([this]() {
		this->title->setVisible(true);
		this->control->setVisible(true);
        this->moreBtn->setVisible(true);
		this->reduceBtn->setVisible(true);
		this->spacer->setVisible(true);

		this->expandBtn->setVisible(false);

        if
            (this->advanced->visible())
        {
		    this->headerbar->setFixedSize(Eigen::Vector2i(480, 24));
        }
        else
        {
		    this->headerbar->setFixedSize(Eigen::Vector2i(365, 24));
        }

		this->window->updateLayout();
	});

	this->window->updateLayout();
	return true;
};
