// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <sstream>
#include <iomanip>

#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_scale.h"

#include "core/log.h"

using namespace SEAScope;

/**
 *
 */
NanoguiScale::NanoguiScale(nanogui::Widget* parent)
: nanogui::Widget(parent), resolution(nullptr), reflowNeeded(false)
{
    this->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                           nanogui::Alignment::Middle, 0, 0));

    this->resolution = new nanogui::Label(this, "", "sans-bold");
    this->resolution->setFontSize(16);
};

/**
 *
 */
NanoguiScale::~NanoguiScale()
{
};

/**
 * resolution in meters per pixel
 */
void NanoguiScale::setResolution(float resolutionIn)
{
    std::ostringstream buffer;  //
    float scaledRes;    //
    float sizeX;    //

    sizeX = static_cast<float>(this->mSize.x());

	buffer << std::fixed << std::setprecision(0);
	buffer << "1 pixel represents " << resolutionIn << "m";
	buffer << " \n(at the center of the screen)";
	this->setTooltip(buffer.str());

    scaledRes = resolutionIn * sizeX;

	LOG(LogLevel::Debug, "NanoguiScale", "resolution: {}, scaledResolution: {}", resolutionIn ,scaledRes);

	buffer.str("");
    if
        (scaledRes < 5000.0f)
    {
        buffer << std::fixed << std::setprecision(0) << scaledRes << "m";
    }
    else
    {
        scaledRes = 0.001f * scaledRes;
        buffer << std::fixed << std::setprecision(1) << scaledRes << "km";
    }
    this->resolution->setCaption(buffer.str());
	this->reflowNeeded = true;
};

/**
 *
 */
void NanoguiScale::draw(NVGcontext* ctx)
{
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());

	if
		(this->reflowNeeded)
	{
		this->performLayout(ctx);
		this->reflowNeeded = false;
	}

    nvgBeginPath(ctx);
    nvgMoveTo(ctx, posX + 2.0f, posY + sizeY - 2.0f);
    nvgLineTo(ctx, posX + 2.0f, posY + sizeY - 7.0f);
    nvgLineTo(ctx, posX + sizeX - 2.0f, posY + sizeY - 7.0f);
    nvgLineTo(ctx, posX + sizeX - 2.0f, posY + sizeY - 2.0f);
    nvgStrokeColor(ctx, nanogui::Color(255,255,255,255));
    nvgStroke(ctx);

    nanogui::Widget::draw(ctx);
};
