// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_glfw3.h"

#include "ui/nanogui/nanogui_selectable_label.h"
#include "core/log.h"

using namespace SEAScope;

NanoguiSelectableLabel::NanoguiSelectableLabel(nanogui::Widget* parent)
: nanogui::TextBox(parent), screen(nullptr)
{
    nanogui::Screen* sc;
    nanogui::Widget* widget;

    this->setEditable(true);

    widget = parent;
    while
        (nullptr != widget)
    {
        sc = dynamic_cast<nanogui::Screen*>(widget);
        if
            (nullptr != sc)
        {
            this->screen = sc;
            break;
        }
        widget = widget->parent();
    }
};

NanoguiSelectableLabel::~NanoguiSelectableLabel()
{
};

bool NanoguiSelectableLabel::keyboardEvent(int, int, int, int)
{
    return false;
};

bool NanoguiSelectableLabel::keyboardCharacterEvent(unsigned int)
{
    return false;
};

bool NanoguiSelectableLabel::mouseButtonEvent(const Eigen::Vector2i& p, int button, bool down, int modifiers)
{
    bool ok;    //

    ok = nanogui::TextBox::mouseButtonEvent(p, button, down, modifiers);
    if
        (false == down)
    {
        this->copySelection();
    }
    return ok;
};

bool NanoguiSelectableLabel::copySelection()
{
    int begin;  //
    int end;    //
    std::size_t beginIndex; //
    std::size_t endIndex;   //

    if
        (  -1 >= this->mSelectionPos
        || nullptr == this->screen)
    {
        return false;
    }

    begin = this->mCursorPos;
    end = this->mSelectionPos;

    if
        (begin > end)
    {
        std::swap(begin, end);
    }

    beginIndex = static_cast<std::size_t>(begin);
    endIndex = static_cast<std::size_t>(end);

    glfwSetClipboardString(this->screen->glfwWindow(),
                           this->mValueTemp.substr(beginIndex, endIndex).c_str());

    return true;
}
