// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "core/log.h"
#include "ui/nanogui/nanogui_settings.h"
#include "ui/nanogui/nanogui_renderer.h"

using namespace SEAScope;

/**
 *
 */
NanoguiSettings::NanoguiSettings()
: root{nullptr},
  currentTab{nullptr},
  currentBtn{nullptr},
  window{nullptr},
  generalSettings{nullptr},
  layoutSettings{nullptr}
{

};

/**
 *
 */
NanoguiSettings::~NanoguiSettings()
{

};

/**
 *
 */
bool NanoguiSettings::setVisible(bool visible)
{
	this->window->setVisible(visible);
	return true;
};

/**
 *
 */
bool NanoguiSettings::visible() const
{
	return this->window->visible();
};

/**
 *
 */
bool NanoguiSettings::init(IRenderer& renderer)
{
	NanoguiRenderer* r;	//
	nanogui::Window* w;	//
	nanogui::Widget* headers;	//
	nanogui::Widget* contents;	//
	std::vector<nanogui::Button*> headBtns(2);	//


	r = dynamic_cast<NanoguiRenderer*>(&renderer);
	if
		(nullptr == r)
	{
		return false;
	}

	this->root = r;

	w = new nanogui::Window(this->root, "Settings");
	w->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
										, nanogui::Alignment::Minimum
										, 3, 1));

	headers = new nanogui::Widget(w);
	headers->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
										   , nanogui::Alignment::Minimum, 3, 1));

	headBtns[0] = new nanogui::Button(headers, "General");
	headBtns[1] = new nanogui::Button(headers, "Layout");

	headBtns[0]->setButtonGroup(headBtns);
	headBtns[0]->setFlags(nanogui::Button::RadioButton);
	headBtns[0]->setFixedSize(Eigen::Vector2i(100,24));
	headBtns[0]->setFontSize(18);

	headBtns[1]->setButtonGroup(headBtns);
	headBtns[1]->setFlags(nanogui::Button::RadioButton);
	headBtns[1]->setFixedSize(Eigen::Vector2i(100,24));
	headBtns[1]->setFontSize(18);

	contents = new nanogui::Widget(w);
	contents->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical));

	this->generalSettings = new NanoguiGeneralSettings(contents);
	this->generalSettings->setVisible(false);
	this->layoutSettings = new NanoguiLayoutSettings(contents);
	this->layoutSettings->setVisible(false);

	// Init
	headBtns[0]->setPushed(true);
	this->currentTab = generalSettings;
	this->currentTab->setVisible(true);

	headBtns[0]->setCallback([this] { 
		this->currentTab->setVisible(false);
		this->currentTab = this->generalSettings;
		this->currentTab->setVisible(true);
		this->root->performLayout();});
	headBtns[1]->setCallback([this] {
		this->currentTab->setVisible(false);
		this->currentTab = this->layoutSettings;
		this->currentTab->setVisible(true);
		this->root->performLayout();});
	w->center();

	this->window = w;
	return true;
};

