// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <nanogui/entypo.h>
#include "core/log.h"
#include "ui/nanogui/nanogui_icon.h"
#include "ui/nanogui/nanogui_side_menu.h"

using namespace SEAScope;

/**
 *
 */
NanoguiSideMenu::NanoguiSideMenu(nanogui::Widget* parent)
: nanogui::Widget(parent), anchor(nullptr), header(nullptr), content(nullptr)
, reflowNeeded(true)
{
	NanoguiIcon* icon;	//

	this->anchor = new NanoguiCornerAnchor(parent);

	icon = new NanoguiIcon(this->anchor, ENTYPO_ICON_LIST);
	icon->setFontSize(30);
	icon->setFixedSize(Eigen::Vector2i(32,32));
	icon->setColor(nanogui::Color(250,250,250,250));
	icon->setTooltip("");

	this->header = new nanogui::Widget(this);
	this->anchor->updateCallback = [this] (int /*x*/, int /*y*/, int /*w*/, int /*h*/) {
		if
			(NanoguiCorner::TOP_LEFT == this->anchor->corner)
		{
			this->header->setPosition(Eigen::Vector2i(this->anchor->width(), 0));
			this->content->setPosition(Eigen::Vector2i(0, 10 + this->anchor->height()));
			this->setPosition(this->anchor->position());
		}
		else if
			(NanoguiCorner::TOP_RIGHT == this->anchor->corner)
		{
			this->header->setPosition(Eigen::Vector2i(0, 0));
			this->content->setPosition(Eigen::Vector2i(0, 10 + this->anchor->height()));
			this->setPosition(this->anchor->position() - Eigen::Vector2i(this->width() - this->anchor->width(), 0));
		}
		else if
			(NanoguiCorner::BOTTOM_LEFT == this->anchor->corner)
		{
			this->header->setPosition(Eigen::Vector2i(this->anchor->width(), this->fixedHeight() - this->anchor->height()));
			this->content->setPosition(Eigen::Vector2i(0, 0));
			this->setPosition(Eigen::Vector2i(0, this->anchor->position().y() + this->anchor->height() - this->height()));
		}
		else
		{
			this->header->setPosition(Eigen::Vector2i(0, this->fixedHeight() - this->anchor->height()));
			this->content->setPosition(Eigen::Vector2i(0, 0));
			this->setPosition(this->anchor->position() + this->anchor->size() - this->size());
		}
	};

	this->anchor->clickCallback = [this] () {
		this->setVisible( !this->visible());
	};
};

/**
 *
 */
NanoguiSideMenu::~NanoguiSideMenu()
{
};

/**
 *
 */
void NanoguiSideMenu::draw(NVGcontext* ctx)
{
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());

	if
		(true == this->reflowNeeded)
	{
		this->performLayout(ctx);
		this->reflowNeeded = false;
		LOG(LogLevel::Debug, "NanoguiSideMenu", "Draw after reflow: {}x{} {}x{}", this->mSize.x(), this->mSize.y(), this->width(), this->height());
	}
	nvgBeginPath(ctx);
	nvgRect( ctx
		   , posX, posY
		   , sizeX, sizeY);
	nvgFillColor(ctx, nanogui::Color(0, 0, 0, 150));
	nvgFill(ctx);

	nanogui::Widget::draw(ctx);
};

/**
 *
 */
void NanoguiSideMenu::setSize(const Eigen::Vector2i& size)
{
    nanogui::Widget::setSize(size);
	this->header->setSize(Eigen::Vector2i( size.x() - this->anchor->width()
										 , this->anchor->height()));
    if
        (nullptr != this->content)
    {
		if
			(  NanoguiCorner::TOP_LEFT == this->anchor->corner
			|| NanoguiCorner::TOP_RIGHT == this->anchor->corner)
		{
			this->content->setPosition(Eigen::Vector2i(0, 10 + this->anchor->height()));
		}
		else
		{
			this->content->setPosition(Eigen::Vector2i(0, 0));
		}

        this->content->setFixedSize(size - Eigen::Vector2i(0, 10 + this->anchor->height()));
        this->content->setSize(size - Eigen::Vector2i(0, 10 + this->anchor->height()));
    }
};

/**
 *
 */
bool NanoguiSideMenu::mouseButtonEvent(const Eigen::Vector2i& pos, int button, bool pressed, int mods)
{
	nanogui::Widget::mouseButtonEvent(pos, button, pressed, mods);
	return true;
};
