// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "core/log.h"
#include "ui/nanogui/nanogui_source_widget.h"

using namespace SEAScope;

/**
 *
 */
NanoguiSourceWidget::NanoguiSourceWidget(nanogui::Widget* parent): nanogui::Widget(parent), titleLabel(nullptr), uriLabel(nullptr), selectionCb(nullptr), selectionCallback(nullptr), infoCallback(nullptr)
{
	nanogui::Widget* labels;	//
	nanogui::Button* infoButton;	//

	this->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
										   , nanogui::Alignment::Middle
										   , 2, 5));

	this->selectionCb = new NanoguiCheckbox(this, "");
	this->selectionCb->setFixedSize(Eigen::Vector2i(20,20));
	this->selectionCb->setCallback([this](bool selected) {
		if
			(nullptr != this->selectionCallback)
		{
			this->selectionCallback(this, selected);
		}
	});

	labels = new nanogui::Widget(this);
	labels->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Vertical
											 , nanogui::Alignment::Fill
											 , 0, 0));
	labels->setFixedSize(Eigen::Vector2i(250, 28));
	this->titleLabel = new nanogui::Label(labels, "", "sans-bold");
	this->titleLabel->setFixedSize(Eigen::Vector2i(250, 14));
	this->titleLabel->setFontSize(14);
	this->uriLabel = new nanogui::Label(labels, "");
	this->uriLabel->setFixedSize(Eigen::Vector2i(250, 14));
	this->uriLabel->setFontSize(14);

	infoButton = new nanogui::Button(this, "", ENTYPO_ICON_INFO);
	infoButton->setFixedSize(Eigen::Vector2i(20, 20));
	infoButton->setCallback([this]() {
		if
			(nullptr != this->infoCallback)
		{
			this->infoCallback(this);
		}
	});
};

/**
 *
 */
NanoguiSourceWidget::~NanoguiSourceWidget()
{
};

/**
 *
 */
bool NanoguiSourceWidget::setTitle(const std::string& title)
{
	this->titleLabel->setCaption(title);
	return true;
};

/**
 *
 */
bool NanoguiSourceWidget::setURI(const std::string& uri)
{
	this->uriLabel->setCaption(uri);
	return true;
};

/**
 *
 */
void NanoguiSourceWidget::draw(NVGcontext* ctx)
{
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());

	/* Draw light semi-transparent background */
	nvgBeginPath(ctx);
	nvgRoundedRect( ctx
				  , posX, posY
				  , sizeX, sizeY
				  , 3);
	nvgFillColor(ctx, nanogui::Color(255,255,255,10));
	nvgFill(ctx);

	nanogui::Widget::draw(ctx);
};

/**
 *
 */
bool NanoguiSourceWidget::setInfoCallback(const std::function<void(ISourceWidget*)>& callback)
{
	this->infoCallback = callback;
	return true;
};

/**
 *
 */
bool NanoguiSourceWidget::setSelectionCallback(const std::function<void(ISourceWidget*, bool)>& callback)
{
	this->selectionCallback = callback;
	return true;
};
