// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_tag.h"

using namespace SEAScope;

/**
 *
 */
NanoguiTag::NanoguiTag(nanogui::Widget* parent)
: nanogui::Widget(parent), nameLabel(nullptr), valueLabel(nullptr)
{
	this->setLayout( new nanogui::BoxLayout( nanogui::Orientation::Horizontal
										   , nanogui::Alignment::Middle
										   , 2, 2));
	this->nameLabel = new nanogui::Label(this, "");
	this->valueLabel = new nanogui::Label(this, "");
};

/**
 *
 */
NanoguiTag::~NanoguiTag()
{
};

/**
 *
 */
bool NanoguiTag::setValue(const std::string& name, const std::string& value)
{
	this->nameLabel->setCaption(name);
	this->valueLabel->setCaption(value);
	return true;
};

/**
 *
 */
bool NanoguiTag::setNameColor(const nanogui::Color& color)
{
	if
		(this->nameColor != color)
	{
		this->nameColor = color;
		return true;
	}

	return false;
};

/**
 *
 */
bool NanoguiTag::setValueColor(const nanogui::Color& color)
{
	if
		(this->valueColor != color)
	{
		this->valueColor = color;
		return true;
	}
	return false;
};

/**
 *
 */
void NanoguiTag::draw(NVGcontext* ctx)
{
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //
    float labelSizeX;   //

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());
    labelSizeX = static_cast<float>(this->nameLabel->width());

	nvgBeginPath(ctx);
	nvgRoundedRectVarying( ctx
				  , posX - 1.0f
				  , posY
				  , labelSizeX + 4.0f
				  , sizeY
				  , 0.3f * sizeY, 0, 0, 0.3f * sizeY);
	nvgFillColor(ctx, this->nameColor);
	nvgFill(ctx);

	nvgBeginPath(ctx);
	nvgRoundedRectVarying( ctx
				  , posX + labelSizeX + 2.0f
				  , posY
				  , sizeX - labelSizeX
				  , sizeY
				  , 0, 0.3f * sizeY, 0.3f * sizeY, 0);
	nvgFillColor(ctx, this->valueColor);
	nvgFill(ctx);

	nanogui::Widget::draw(ctx);
};
