// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "ui/nanogui/nanogui_texture_panel.h"

#include "ui/nanogui/nanogui_icons.h"
#include "ui/nanogui/nanogui_renderer.h"

namespace SEAScope
{

NanoguiTexturePanel::NanoguiTexturePanel(nanogui::Widget* parent, const std::string& str)
 : NanoguiWindow(parent, str), _vscroll(nullptr), _imgPanel(nullptr)
{
}

NanoguiTexturePanel::~NanoguiTexturePanel()
{
}

void NanoguiTexturePanel::init()
{
    setSize(nanogui::Vector2i(400, 300));
    setFixedSize(nanogui::Vector2i(400, 300));


    _vscroll = new nanogui::VScrollPanel(this);

    _imgPanel = new nanogui::ImagePanel(_vscroll);
    _imgPanel->setCallback([this](int i) {
            _selectionChangedCallback(_proxyNVGTex[static_cast<std::size_t>(i)],
                                      iconList[static_cast<std::size_t>(i)].second);
        });

    _vscroll->setFixedSize(nanogui::Vector2i(400, 300 - 32));
    this->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical, nanogui::Alignment::Fill));

    SEAScope::NanoguiRenderer* nr = getRenderer();

    if
        (nullptr != nr)
    {
        const NanoguiIcons& icons = nr->getIcons();
        auto& textures = icons.getNVGTextures();

        auto it = textures.cbegin();
        auto itEnd = textures.cend();

        iconList.emplace_back(-1, "");
        _proxy.emplace_back(static_cast<uint16_t>(65535));
        _proxyNVGTex.emplace_back(-1);

        for
            (; it != itEnd; ++it)
        {
            iconList.emplace_back(it->second, icons.getName(it->first));
            _proxy.emplace_back(it->first);
            _proxyNVGTex.emplace_back(it->second);
        }

        _imgPanel->setImages(iconList);
    }
}

void NanoguiTexturePanel::setSelectionChangedCallback(const std::function<void(int, const std::string& name)>& func)
{
    _selectionChangedCallback = func;
}

} // namespace SEAScope
