// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "ui/nanogui/nanogui_time_coverage_editor.h"

#include "ui/nanogui/nanogui_datetimebox.h"

#include <iostream>

using namespace SEAScope;

/**
 *
 */
NanoguiTimeCoverageEditor::NanoguiTimeCoverageEditor(nanogui::Widget* parent)
: nanogui::Widget(parent),
  _startField(nullptr),
  _stopField(nullptr),
  _updateButton(nullptr),
  _startOriginal(""),
  _stopOriginal(""),
  _callback(nullptr)
{
    nanogui::Label* lbl(nullptr);
    nanogui::Widget* wrapper(nullptr);
    nanogui::Widget* spacer(nullptr);

    this->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                           nanogui::Alignment::Minimum,
                                           0, 0));

    lbl = new nanogui::Label(this, "Time coverage", "sans-bold");
    lbl->setColor(nanogui::Color(180, 180, 180, 150));

    wrapper = new nanogui::Widget(this);
    wrapper->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                              nanogui::Alignment::Middle,
                                              0, 0));

    spacer = new nanogui::Widget(wrapper);
    spacer->setFixedWidth(10);

    this->_startField = new NanoguiDatetimeBox(wrapper, 2000, 1, 1, 0, 0, 0);
    this->_startField->setFontSize(16);
    this->_startField->setFixedSize(Eigen::Vector2i(130, 20));
    this->_startField->setCallback([this] (const std::string& value) -> bool {
        if
            (value != this->_startOriginal)
        {
            this->_updateButton->setEnabled(true);
        }
        else if
            (this->_stopField->value() == this->_stopOriginal)
        {
            this->_updateButton->setEnabled(false);
        }
        return true;
    });

    new nanogui::Label(wrapper, " ... ");

    this->_stopField = new NanoguiDatetimeBox(wrapper, 2000, 1, 1, 0, 0, 0);
    this->_stopField->setFontSize(16);
    this->_stopField->setFixedSize(Eigen::Vector2i(130, 20));
    this->_stopField->setCallback([this] (const std::string& value) -> bool {
        if
            (value != this->_stopOriginal)
        {
            this->_updateButton->setEnabled(true);
        }
        else if
            (this->_startField->value() == this->_startOriginal)
        {
            this->_updateButton->setEnabled(false);
        }
        return true;
    });

    this->_updateButton = new nanogui::Button(wrapper, "", ENTYPO_ICON_CYCLE);
    this->_updateButton->setFontSize(16);
    this->_updateButton->setFixedSize(Eigen::Vector2i(20, 20));
    this->_updateButton->setCallback([this] () {
        if
            (nullptr != this->_callback)
        {
            this->_callback(this->_startField->value(),
                            this->_stopField->value());
        }
    });
}

/**
 *
 */
NanoguiTimeCoverageEditor::~NanoguiTimeCoverageEditor()
{

}

/**
 *
 */
void NanoguiTimeCoverageEditor::setValue(const std::string& startStr, const std::string& stopStr)
{
    this->_startOriginal = startStr;
    this->_stopOriginal = stopStr;

    this->_startField->setValue(startStr);
    this->_stopField->setValue(stopStr);

    this->_updateButton->setEnabled(false);
}

/**
 *
 */
void NanoguiTimeCoverageEditor::setCallback(const std::function<void(const std::string&, const std::string&)>& callback)
{
    this->_callback = callback;
}
