// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "ui/nanogui/nanovg_wrapper.h"

void nvg::rect(NVGcontext* ctx, const nanogui::Vector2i& origin, const nanogui::Vector2i& size)
{
    nvgRect(
        ctx,
        static_cast<float>(origin.x()),
        static_cast<float>(origin.y()),
        static_cast<float>(size.x()),
        static_cast<float>(size.y()));
}

void nvg::roundRect(NVGcontext* ctx, const nanogui::Vector2i& origin, const nanogui::Vector2i& size, int radius)
{
    nvgRoundedRect(
        ctx,
        static_cast<float>(origin.x()),
        static_cast<float>(origin.y()),
        static_cast<float>(size.x()),
        static_cast<float>(size.y()),
        static_cast<float>(radius));
}

NVGpaint nvg::linearGradient(NVGcontext* ctx, const nanogui::Vector2i& start, const nanogui::Vector2i& end, const nanogui::Color& top, const nanogui::Color& bottom)
{
    return nvgLinearGradient(
        ctx,
        static_cast<float>(start.x()),
        static_cast<float>(start.y()),
        static_cast<float>(end.x()),
        static_cast<float>(end.y()),
        static_cast<NVGcolor>(top),
        static_cast<NVGcolor>(bottom));
}

NVGpaint nvg::imagePattern(NVGcontext* ctx, const nanogui::Vector2i& pos, const nanogui::Vector2i& size, float angle, int texture, float alpha)
{
    return nvgImagePattern(
        ctx,                                            
        static_cast<float>(pos.x()),
        static_cast<float>(pos.y()),
        static_cast<float>(size.x()),
        static_cast<float>(size.y()),
        angle,
        texture,
        alpha);
}

void nvg::text(NVGcontext* ctx, const nanogui::Vector2i& pos, const std::string& str)
{
    nvgText(
        ctx, 
        static_cast<float>(pos.x()),
        static_cast<float>(pos.y()),
        str.c_str(),
        nullptr);
}
