// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <iostream>
#include "platform/dialog_impl.h"

#include "vendors/_pfd.h" 

using namespace SEAScope;

/**
 *
 */
DialogImpl::DialogImpl()
{

}

/**
 *
 */
DialogImpl::~DialogImpl()
{
}

bool DialogImpl::available()
{
    return pfd::settings::available();
}

/**
 *
 */
bool DialogImpl::askYesNoQuestion(const std::string& prompt, const std::string& /*yesLabel*/, const std::string& /*noLabel*/)
{
	bool result = false;
	
	if 
		(false == pfd::settings::available())
	{
		char answer;	//

		std::cout << "\n\n\033[1;33m[User input required]" << "\033[0m\n";

		do
		{
			std::cout << "\033[1;34m" << prompt << "\033[0m" << std::endl;
			std::cout << "Your answer [y/n]: ";
			std::cin >> answer;
		}
		while (!std::cin.fail() && answer != 'y' && answer != 'n');


		result = 'y' == answer;
	}
	else
	{
		pfd::message dlg("Question", prompt, pfd::choice::yes_no, pfd::icon::question);
		pfd::button btn = dlg.result();
		result = btn == pfd::button::yes;
	}
	
	return result;
}

std::string DialogImpl::fileDialog(const std::string& currentPath, bool save)
{
    if
        (false == save)
    {
        pfd::open_file dlg("Open file", currentPath, { "All Files", "*" }, pfd::opt::force_path);
        std::vector<std::string> str = dlg.result();
        return str.size() > 0 ? str[0] : "";
    }
    else
    {
        pfd::save_file dlg("Save file", currentPath, { "All Files", "*" }, pfd::opt::force_path);
        std::string str = dlg.result();
        return str;
    }
}

std::string DialogImpl::dirDialog(const std::string& currentPath)
{
    pfd::select_folder dlg("Select folder", currentPath);
    return dlg.result();
}
