// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef WIN32_PATH_H
#define WIN32_PATH_H

#include <ctime>
#include <sys/stat.h>
#include <string>
#include <forward_list>

#include "core/interfaces/ipath_handler.h"

namespace SEAScope
{
	class PathImpl : public virtual IPathHandler
	{
		public:
			PathImpl();
			~PathImpl();

			bool dirExists(const std::string&) override;
			bool fileExists(const std::string&) override;
			bool fileSize(const std::string&, std::int64_t&) override;
			bool writeFile(const std::string&) override;
			bool createDir(const std::string&, mode_t) override;
			bool createDirs(const std::string&, mode_t) override;
			void splitPaths(const std::string&, const std::string&, std::forward_list<std::string>&) override;
			void joinPaths(const std::vector<std::string>&, std::string&) override;
			bool listFiles(const std::string&, std::forward_list<std::string>&) override;
			bool listFilesRecursive(const std::string&, std::forward_list<std::string>&) override;
			bool deleteFile(const std::string&) override;
			char separator() const override;
			void fixPlatformPath(std::string& str) override;
			bool lastModified(const std::string&, std::time_t&) override;
			bool currentWorkingDir(std::string& path) override;
			bool canonicalPath(const std::string&, std::string&) override;
            bool homeDir(std::string&) override;
            bool absolutePath(const std::string&, std::string&) override;
            bool getParentDir(const std::string&, std::string&) override;

	};
};

#endif
