// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SERVER_H
#define SERVER_H

#include <array>
#include "core\perfs.h"
#include "core\sequential_buffer.h"

#include <concrt.h>
#include <WinSock2.h>

namespace SEAScope
{
	class ControlProtocolHandler;
	class IProtocolHandler;
	class SocketsManager;
}

class SocketData
{
public:
	HANDLE event;
	SOCKET socket;
	SEAScope::IProtocolHandler* protocol;
	SEAScope::SequentialBuffer<4096> buffer;

	SocketData()
	{
		event = 0;
		socket = 0;
		protocol = nullptr;
	}

	~SocketData()
	{
	}
};

class Server
{
public:
	Server();
	~Server();

	void initialize(const std::string& address, int port, SEAScope::SocketsManager*);
	bool run();
	void stop();

protected:
	bool acceptConnection(SocketData& s);
	bool readSocket(SocketData& s);
	bool closeSocket(SocketData& s);
	void updateEvents();

private:
	std::string _address;
	int _port;
	bool _stopThread;
	std::mutex _mutex;
	std::vector<HANDLE> _events;
	SOCKET* sockets;
	SocketData _listener;
	std::vector<SEAScope::IProtocolHandler*> _protocolHandler;
	std::vector<SocketData*> _clients;
	SEAScope::SequentialBuffer<SOCKET_CHUNKSIZE>* _buffers;
	int count;
	SEAScope::ControlProtocolHandler* _protocol;
	SEAScope::SocketsManager* _socketManager;
};

#endif // SERVER_H
