// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef WIN32_SOCKETS_MONITOR_H
#define WIN32_SOCKETS_MONITOR_H

#include <map>

#include "core/types.h"
#include "core/interfaces/iprotocol_handler.h"
#include "core/interfaces/isockets_monitor.h"
#include "core/perfs.h"
#include "core/sequential_buffer.h"
#include "core/sequential_buffer_pool.h"

namespace SEAScope
{
	struct MonitoredSocket
	{
		IProtocolHandler* ioHandler;
		ProtocolHandlerFactory clientHandler;
		SequentialBuffer<SOCKET_CHUNKSIZE>* buffer;
	};

	class SocketsMonitorImpl: public virtual SEAScope::ISocketsMonitor
	{
		public:
			SocketsMonitorImpl();
			virtual ~SocketsMonitorImpl();

			virtual bool init(const std::function<void()>&, int&);
			virtual bool handleEvent(const SocketJob&);
			virtual bool monitorSockets();
			virtual bool triggerEventsProcessing();
			virtual bool sendData(socket_t, std::uint8_t*, std::uint64_t);

			bool subscribe(socket_t, IProtocolHandler*, const ProtocolHandlerFactory&);
			bool unsubscribe(socket_t);

			SequentialBufferPool<20, SOCKET_CHUNKSIZE> bufferPool;
			std::map<socket_t, SEAScope::MonitoredSocket> monitored;
			std::function<void()> signalHandler;
			bool terminationRequested;
			int epfd;
			int read_pipe;
			int write_pipe;
	};
};

#endif
