// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <iostream>
#include <algorithm>
#include <string>

#include "platform/app_launcher_impl.h"
#include "core/path.h"
#include "vendors/_mini.h"

using namespace SEAScope;

/**
 *
 */
AppLauncherImpl::AppLauncherImpl()
{
}

/**
 *
 */
AppLauncherImpl::~AppLauncherImpl()
{
}

/**
 *
 */
bool AppLauncherImpl::initWorkspace(const std::string& programPath,
                                    bool hasWorkspace, std::string& workspacePath,
                                    bool hasConfig, std::string& configPath,
                                    bool hasListen, std::string& listenURL,
                                    bool hasLogFilters, std::string& logFilters) const
{
    if
        (false == hasWorkspace)
    {
        size_t pos = programPath.rfind('\\');
        pos = programPath.rfind('\\', pos-1);
        workspacePath = programPath.substr(0, pos);
    }
    else
    {
        std::string workspacePathTmp = workspacePath;
        Path::absolutePath(workspacePathTmp, workspacePath);
    }
    Path::setPathValid(workspacePath);

    if
        (false == hasConfig)
    {
        configPath = workspacePath + Path::separator() + "config.ini";
    }
    else
    {
        std::string configPathTmp = configPath;
        Path::absolutePath(configPathTmp, configPath);
    }
    Path::setPathValid(configPath);

    if
        (false == Path::fileExists(configPath))
    {
        bool canBeFixed {false};
        canBeFixed = Path::writeFile(configPath);
        if
            (false == canBeFixed)
        {
            std::cerr << "Cannot write to file '" << configPath << "', please check permissions" << std::endl;
            return false;
        }

        // Default config
        mINI::INIStructure root;
        root["paths"];
        root["paths"]["colormaps"] = workspacePath + Path::separator() + "colormaps";
        root["paths"]["data"] = workspacePath + Path::separator() + "data";
        root["paths"]["state"] = workspacePath + Path::separator() + "state.db";
        root["paths"]["index"] = workspacePath + Path::separator() + "index.fb";
        root["paths"]["annotations"] = workspacePath + Path::separator() + "annotations.db";
        root["paths"]["custom"] = workspacePath + Path::separator() + "custom";

        mINI::INIFile parser(configPath);
        bool ok = parser.write(root, true /*pretty print*/);
        if
            (false == ok)
        {
            std::cerr << "Could not write default configuration to '" << configPath << "'" << std::endl;
            return false;
        }
    }

    if
        (false == hasListen)
    {
        listenURL = "127.0.0.1:11155";
    }

    if
        (false == hasLogFilters)
    {
        logFilters = "info";
    }

    return true;
}
