// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <windows.h>

#include "platform/dialog_impl.h"

#include "vendors/_pfd.h" 

namespace SEAScope
{

/**
 *
 */
DialogImpl::DialogImpl()
{
}

/**
 *
 */
DialogImpl::~DialogImpl()
{
}

bool DialogImpl::available()
{
    return pfd::settings::available();
}

/**
 *
 */
bool DialogImpl::askYesNoQuestion(const std::string& prompt, const std::string& /*yesLabel*/, const std::string& /*noLabel*/)
{
    pfd::message dlg("Question", prompt, pfd::choice::yes_no, pfd::icon::question);
    pfd::button btn = dlg.result();
    return btn == pfd::button::yes;
}

std::string DialogImpl::fileDialog(const std::string& currentPath, bool save)
{
    std::string path;

    if
        (currentPath.substr(0, 4) == R"(\\?\)")
    {
        path = currentPath.substr(4);
    }
    else
    {
        path = currentPath;
    }

    if
        (false == save)
    {
        pfd::open_file dlg("Open file", path, { "All Files", "*" }, pfd::opt::force_path);
        std::vector<std::string> str = dlg.result();
        return str.size() > 0 ? str[0] : "";
    }
    else
    {
        pfd::save_file dlg("Save file", path, { "All Files", "*" }, pfd::opt::force_path);
        std::string str = dlg.result();
        return str;
    }
}

std::string DialogImpl::dirDialog(const std::string& currentPath)
{
    std::string path;

    if
        (currentPath.substr(0, 4) == R"(\\?\)")
    {
        path = currentPath.substr(4);
    }
    else
    {
        path = currentPath;
    }

    pfd::select_folder dlg("Select folder", path, pfd::opt::force_path);
    return dlg.result();
}

} // namespace SEAScope
