// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <iostream>
#include <thread>
#include <WinSock2.h>

#include "platform/sockets_monitor_impl.h"

#include "core/log.h"

using namespace SEAScope;

/**
 *
 */
SocketsMonitorImpl::SocketsMonitorImpl()
:	signalHandler(nullptr),
	terminationRequested(false), epfd(-1),
	read_pipe(-1),
	write_pipe(-1)
{
};

/**
 *
 */
SocketsMonitorImpl::~SocketsMonitorImpl()
{
};

/**
 *
 */
bool SocketsMonitorImpl::handleEvent(const SocketJob&)
{
	return false;
};

/**
 *
 */
bool SocketsMonitorImpl::subscribe(socket_t fd, IProtocolHandler* handler, const ProtocolHandlerFactory& handlerFactory)
{
	SequentialBuffer<SOCKET_CHUNKSIZE>* buffer;	//
	SEAScope::MonitoredSocket monitorEntry;	//
	bool ok;	//
	int rc;	// Return code

	LOG(LogLevel::Debug, "sockets monitor", "Subscribe {}", fd);
	LOG(LogLevel::Debug, "sockets monitor", "with handler? {}", nullptr != handler);
	LOG(LogLevel::Debug, "sockets monitor", "with handler factory? {}", nullptr != handlerFactory);
	auto found = this->monitored.find(fd);
	if
		(this->monitored.end() != found)
	{
		LOG(LogLevel::Error, "sockets monitor"
			, "File descriptor {} is already monitored"
			, fd);
		return false;
	}

	ok = false;
	buffer = this->bufferPool.acquire(ok);
	if
		(false == ok)
	{
		LOG(LogLevel::Error, "sockets monitor"
			, "No buffer available in the pool!");
		return false;
	}

	monitorEntry.ioHandler = handler;
	monitorEntry.clientHandler = handlerFactory;
	monitorEntry.buffer = buffer;
	//monitorEntry.epollEvent.data.fd = fd;
	//monitorEntry.epollEvent.events = EPOLLIN | EPOLLET;

	rc = 0;

	//rc = epoll_ctl(this->epfd
	//	, EPOLL_CTL_ADD
	//	, fd
	//	, &(monitorEntry.epollEvent));
	if
		(-1 == rc)
	{
		// release buffer
		this->bufferPool.release(buffer);

		return false;
	}

	this->monitored[fd] = monitorEntry;
	return true;
};

/**
 *
 */
bool SocketsMonitorImpl::unsubscribe(socket_t)
{
	return true;
};

/**
 *
 */
bool SocketsMonitorImpl::init(const std::function<void()>& sh, int&)
{
	this->signalHandler = sh;
	return true;
};

/**
 *
 */
bool SocketsMonitorImpl::monitorSockets()
{
	//std::cout << "thread running " << std::this_thread::get_id() << std::endl;



	//auto pair = monitored.begin();

	//SOCKET s = (SOCKET)pair->first;
	//HANDLE event = WSACreateEvent();

	//int res = WSAEventSelect(s, event, FD_ACCEPT | FD_CLOSE | FD_READ);

	//bool done = false;
	//DWORD dw;

	//while (false == done)
	//{
	//	dw = WSAWaitForMultipleEvents(1, &event, FALSE, WSA_INFINITE, FALSE);
	//	dw -= WSA_WAIT_EVENT_0;

	//	if (WSA_WAIT_EVENT_0 == dw)
	//	{
	//		int i = 0;
	//	}
	//}

	return true;
};

/**
 *
 */
bool SocketsMonitorImpl::triggerEventsProcessing()
{
	return true;
};

/**
 *
 */
bool SocketsMonitorImpl::sendData(socket_t sock, std::uint8_t* buffer, std::uint64_t dataSize)
{
	std::int64_t remaining;	//
	std::int64_t sent;	//

						// Send data
	remaining = static_cast<std::int64_t>(dataSize);
	while
		(0 < remaining)
	{
		sent = send(static_cast<SOCKET>(sock), (const char*)buffer, static_cast<int>(remaining), 0);
		if
			(-1 == sent)
		{
			if
				((errno == EWOULDBLOCK) || (errno == EAGAIN))
			{
				continue;
			}

			/* Something wrong happened */
			break;
		}

		/* Prepare next write */
		remaining -= sent;
		buffer += sent;
	}

	return 0 == remaining;
};
