#!/bin/bash

# =============================================================================
# This file is part of SEAScope, a 3D visualisation and analysis application
# for satellite, in-situ and numerical model data.
#
# Copyright (C) 2014-2025 OceanDataLab
#
# SEAScope is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
# =============================================================================

set -o nounset
set -o errexit
set -o pipefail

readonly ARGS=( "${@}" );

get_arch()
{
	uname -m
}

get_hostname()
{
	hostname
}

get_packager_name()
{
	git config user.name
}

get_tag_or_branch()
{
	local commit;
	local tag;

  commit="$(git rev-parse HEAD)";
  tag="$(git tag --contains "${commit}" | head -1)";

	if [ -n "${tag}" ];
	then
		echo "${tag}"
	else
		 git rev-parse --abbrev-ref HEAD
	fi	
}

get_commit()
{
	git rev-parse --short HEAD
}

build_info()
{
	local dst_path;

    dst_path="${1}"; shift;

	cat > "${dst_path}" <<EOF
#ifndef SEASCOPE_VERSION_H
#define SEASCOPE_VERSION_H

#include <map>
#include <sstream>

namespace SEAScope
{
	static std::map<std::string, std::string> getVersion() 
	{
		std::map<std::string, std::string> v ({
		  {"name", "$(get_tag_or_branch)"},
		  {"ref", "$(get_commit)"},
		  {"date", __DATE__},
		  {"time", __TIME__},
		  {"packager", "$(get_packager_name)"},
		  {"arch",_M_X64 == 100 ? "x86_64" : "x86"},
		  {"cl version",std::to_string(_MSC_VER)},
		  {"c++ version",std::to_string(_MSVC_LANG)}
		});
		return v;
	};
};
#endif
EOF
}

main()
{
	local cmd;

	cmd="${1}"; shift;
	
	build_info "${cmd}/seascope_version.h"
}

main "${ARGS[@]}"
exit 0
