:: =============================================================================
:: @author: <guillaume.leseach@oceandatalab.com>
:: @date: 2019-06-28
:: 
:: This file is part of SEAScope, a 3D visualisation and analysis application
:: for satellite, in-situ and numerical model data.
::
:: Copyright (C) 2014-2025 OceanDataLab
::
:: SEAScope is free software: you can redistribute it and/or modify it under
:: the terms of the GNU Lesser General Public License as published by the Free
:: Software Foundation, either version 3 of the License, or (at your option)
:: any later version.
::
:: SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
:: WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
:: FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
:: details.
::
:: You should have received a copy of the GNU Lesser General Public License
:: along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
:: =============================================================================
@echo off 
setlocal
SET _devdir=%~dp0
SET _devdir=%_devdir:~0,-1%
for %%i in ("%_devdir%") do set _projectdir=%%~dpi
SET _clientdir=%_projectdir:~0,-1%
SET _pluginsdir=%_clientdir%\..\embedded-server

SET _compilationTarget=Release

if "%1" == "debug" (
	SET _compilationTarget=Debug
)

SET PLUGINS_WORKSPACE=%_pluginsdir%\thirdparty
SET WORKSPACE=%_clientdir%\thirdparty
SET _bindir=%WORKSPACE%\bin\%_compilationTarget%
SET _exedir=%WORKSPACE%\bin\exe
SET _libdir=%WORKSPACE%\lib\%_compilationTarget%
SET _seascopebindir=%_clientdir%\bin\%_compilationTarget%

mkdir %WORKSPACE%
cd %WORKSPACE%
mkdir bin
mkdir %_bindir%
mkdir bin\exe
mkdir lib
mkdir lib\%_compilationTarget%
mkdir tmp

cd %_clientdir%
mkdir bin
mkdir bin\%_compilationTarget%

SET _fmtdir=%WORKSPACE%\fmt
SET _glfwdir=%WORKSPACE%\glfw
SET _glmdir=%WORKSPACE%\glm
SET _cxxoptsdir=%WORKSPACE%\cxxopts
REM SET _zlibdir=%WORKSPACE%\zlib
SET _netcdfdir=%WORKSPACE%\netCDF
SET _netcdfcxxdir=%WORKSPACE%\netcdf-cxx4
SET _sqlitedir=%WORKSPACE%\sqlite
SET _nanoguidir=%WORKSPACE%\nanogui
SET _flatbuffersdir=%WORKSPACE%\flatbuffers
SET _tmpdir=%WORKSPACE%\tmp
SET _picojsondir=%WORKSPACE%\picojson
SET _minidir=%WORKSPACE%\mINI
SET _pfddir=%WORKSPACE%\pfd

if %_compilationTarget% == Release (
	SET _fmtlib=fmt.lib
REM SET _zlib=zlibstatic.lib
) else (
	if %_compilationTarget% == Debug (
		SET _fmtlib=fmtd.lib
REM 	SET _zlib=zlibstaticd.lib
	)
)

SET _glfwlib=glfw3.lib
SET _sqlitelib=sqlite3.lib
SET _nanoguilib=nanogui-obj.lib
SET _flatcexe=flatc.exe

SET _fmturl=https://github.com/fmtlib/fmt.git
SET _glfwurl=https://github.com/glfw/glfw/releases/download/3.3.8/glfw-3.3.8.bin.WIN64.zip
SET _glmurl=https://github.com/g-truc/glm.git
SET _cxxoptsurl=https://github.com/jarro2783/cxxopts.git
REM SET _zliburl=https://www.zlib.net/zlib1213.zip
SET _sqliteurl=https://www.sqlite.org/2019/sqlite-amalgamation-3280000.zip
SET _nanoguiurl=https://github.com/wjakob/nanogui.git
SET _flatbuffersurl=https://github.com/google/flatbuffers.git
SET _picojsonurl=https://github.com/kazuho/picojson.git
SET _miniurl=https://github.com/pulzed/mINI.git
SET _pfdurl=https://github.com/samhocevar/portable-file-dialogs.git

::
::
::	Downloads
::
::

REM FMT
cd %WORKSPACE%
if not exist %_fmtdir% (
	git clone %_fmturl% %_fmtdir%
	cd %_fmtdir%
	git checkout 492a99c964133a961c212f807b97ff191ed9f56a
	cmake -G "Visual Studio 17 2022" -A x64 .
)

REM GLFW
cd %WORKSPACE%
if not exist %_glfwdir% (
	if not exist %_tmpdir%\glfw.zip (
		powershell -Command "(New-Object Net.WebClient).DownloadFile('%_glfwurl%', '%_tmpdir%\glfw.zip')"
	)
	powershell.exe -NoP -NonI -Command "Expand-Archive -Path '%_tmpdir%\glfw.zip' -DestinationPath '%WORKSPACE%'"
	rename glfw-3.3.8.bin.WIN64 glfw
	copy glfw\lib-vc2022\glfw3.lib %_libdir%
)

REM GLM
cd %WORKSPACE%
if not exist %_glmdir% (
	git clone "%_glmurl%" glm
	cd glm
	git checkout 0.9.9.0
)

REM CXXOPTS
cd %WORKSPACE%
if not exist %_cxxoptsdir% (
	git clone "%_cxxoptsurl%" cxxopts
	cd cxxopts
	git checkout 3876c0907237e5fa89c5850ed1ee688a3bcb62b3
)

REM ZLIB
REM cd %WORKSPACE%
REM if not exist %_zlibdir% (
REM 	if not exist %_tmpdir%\zlib.zip (
REM 		powershell -Command "(New-Object Net.WebClient).DownloadFile('%_zliburl%', '%_tmpdir%\zlib.zip')"
REM 	)
REM 	powershell.exe -NoP -NonI -Command "Expand-Archive -Path '%_tmpdir%\zlib.zip' -DestinationPath '%WORKSPACE%'"
REM 	rename zlib-1.2.13 zlib
REM 	cd %_zlibdir%
REM 	cmake -G "Visual Studio 17 2022" -A x64 .
REM )

REM SQLITE
cd %WORKSPACE%
if not exist %_sqlitedir% (
	if not exist %_tmpdir%\sqlite.zip (
		powershell -Command "(New-Object Net.WebClient).DownloadFile('%_sqliteurl%', '%_tmpdir%\sqlite.zip')"
	)
	powershell.exe -NoP -NonI -Command "Expand-Archive '%_tmpdir%\sqlite.zip'  -DestinationPath '%WORKSPACE%'"
	rename sqlite-amalgamation-3280000 sqlite
)

REM NANOGUI
cd %WORKSPACE%
if not exist %_nanoguidir% (
	git clone %_nanoguiurl% --recursive %_nanoguidir%
	cd %_nanoguidir%
	cmake -G "Visual Studio 17 2022" -A x64 .
)

REM FLATBUFFERS
cd %WORKSPACE%
if not exist %_flatbuffersdir% (
	git clone %_flatbuffersurl% %_flatbuffersdir%
	cd %_flatbuffersdir%
	git checkout 63cc0eec4e59015c91ab09894cb6822191a28151
	cmake -G "Visual Studio 17 2022" -A x64 .
	powershell -Command "(Get-Content flatc.vcxproj) | Foreach-Object {$_ -replace '<TreatWarningAsError>true</TreatWarningAsError>','<TreatWarningAsError>false</TreatWarningAsError>'} | Out-File flatc.vcxproj"
)

REM PICOJSON
cd %WORKSPACE%
if not exist %_picojsondir% (
	git clone %_picojsonurl% %_picojsondir%
	cd %_picojsondir%
	git checkout 8ba7113451b745b6e4bd83db12f73eefbb315d2e
)

REM mINI
cd %WORKSPACE%
if not exist %_minidir% (
	git clone %_miniurl% %_minidir%
)

REM PFD
cd %WORKSPACE%
if not exist %_pfddir% (
	git clone %_pfdurl% %_pfddir%
    cd %_pfddir%
    git checkout 7f852d88a480020d7f91957cbcefe514fc95000c
)

REM
REM
REM	Compilation
REM
REM


REM FMT
if not exist %_libdir%\%_fmtlib% (
	cd %_fmtdir%
	msbuild fmt.vcxproj /property:Configuration=%_compilationTarget%
	copy %_compilationTarget%\%_fmtlib% %_libdir%
)

REM GLFW
if not exist %_libdir%\%_glfwlib% (
	cd %WORKSPACE%
	copy %_glfwdir%\lib-vc2022\%_glfwlib% %_libdir%
)

REM GLM

REM CXXOPTS

REM ZLIB
REM if not exist %_libdir%\%_zlib% (
REM 	cd %_zlibdir%
REM 	msbuild zlibstatic.vcxproj /property:Configuration=%_compilationTarget%
REM 	copy %_compilationTarget%\%_zlib% %_libdir%
REM )

REM SQLITE
if not exist %_libdir%\%_sqlitelib% (
	cd %_sqlitedir%
	cl /c /EHsc sqlite3.c
	cl /c /EHsc shell.c
	lib sqlite3.obj shell.obj /OUT:%_sqlitelib%
	copy %_sqlitelib% %_libdir%
)

REM NANOGUI
if not exist %_libdir%\%_nanoguilib% (
	cd %_nanoguidir%
	msbuild nanogui-obj.vcxproj /property:Configuration=%_compilationTarget%
	copy nanogui-obj.dir\%_compilationTarget%\%_nanoguilib% %_libdir%
)

REM FLATBUFFERS
if not exist %_exedir%\%_flatcexe% (
	cd %_flatbuffersdir%
	msbuild flatc.vcxproj /property:Configuration=%_compilationTarget%
	copy %_compilationTarget%\%_flatcexe% %_exedir%
)

cd %WORKSPACE%
mkdir ..\cpp\include\api
%_exedir%\%_flatcexe% -o ..\cpp\include\api --cpp ..\share\flatbuffers\api.fbs

cd %WORKSPACE%/..

REM
REM
REM	Seascope compilation
REM
REM

REM SEASCOPE
if exist %_libdir%\%_fmtlib% (
	if exist %_libdir%\%_glfwlib% (
REM		if exist %_libdir%\%_zlib% (
			if exist %_libdir%\%_sqlitelib% (
				if exist %_libdir%\%_nanoguilib% (
					msbuild developer/win32/seascope.sln /p:Configuration=%_compilationTarget%;Platform=x64
					REM Copy executable and DLLs
					copy developer\win32\x64\%_compilationTarget%\seascope.exe %_seascopebindir%
					copy %PLUGINS_WORKSPACE%\netCDF\bin\netcdf.dll %_seascopebindir%
					copy %PLUGINS_WORKSPACE%\netCDF\bin\hdf5.dll %_seascopebindir%
					copy %PLUGINS_WORKSPACE%\netCDF\bin\hdf5_hl.dll %_seascopebindir%
					copy %PLUGINS_WORKSPACE%\netCDF\bin\msvcp140.dll %_seascopebindir%
					copy %PLUGINS_WORKSPACE%\netCDF\bin\vcruntime140.dll %_seascopebindir%
					copy %PLUGINS_WORKSPACE%\netCDF\bin\zlib1.dll %_seascopebindir%
				)
			)
REM		)
	)
)
