// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef COLLECTIONS_EXPLORER_H
#define COLLECTIONS_EXPLORER_H

#include <map>
#include <set>
#include <string>
#include <cstdint>

#include "core/types.h"
#include "core/structs.h"
#include "core/rendering_cfg.h"
#include "local/collection_ini_parser.h"
#include "platform.h"


namespace SEAScope
{
    // Forward declarations
    class IReader;
    class IGranulesIndex;
    class ReaderCfg;
    class CollectionsCatalogue;

	class CollectionsExplorer
	{
		public:
			CollectionsExplorer();
			virtual ~CollectionsExplorer();

            bool initCatalogue(const IGranulesIndex&, CollectionsCatalogue&);
            bool initIndex(const std::string&, const std::string&, IGranulesIndex&, bool);

            static bool isIndexUpdateNeeded(const std::string&, const std::vector<std::string>&, std::vector<bool>&);
            static bool listCollections(const std::string&, std::vector<std::string>&);
            static bool migrateLegacyCollections(const std::string&);

        protected:
			bool findGranules(const std::string&, std::uint32_t, const ReaderCfg&, IGranulesIndex&);
			bool indexGranule(const std::string&, std::uint32_t, const ReaderCfg&, IGranulesIndex&);

            std::unordered_map<std::string, const IReader*> _readers;
	};
};

#endif
