// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef EMBEDDED_DATASTREAM_H
#define EMBEDDED_DATASTREAM_H

#include <unordered_map>
#include <string>
#include "core/interfaces/idatastream_service_connector.h"
#include "platform.h"


namespace SEAScope
{
    // Forward declarations
    class IReader;
    class ILogHandler;
    class IPathHandler;

	class EmbeddedDatastream final: public virtual IDatastreamServiceConnector
	{
		public:
			EmbeddedDatastream();
			~EmbeddedDatastream() override;

			bool init(ILogHandler*, IPathHandler*) override;
            bool reset() override;
			bool loadData(const GranuleId&, const std::string&,
						  DataModel, const std::unordered_map<std::string, DataBucket*>&) const override;
			bool canHandle(const std::string&) const override;

			static EmbeddedDatastream* create();

        protected:
            std::unordered_map<std::string, IReader*> _readers;
	};
};
#endif
