// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef EMBEDDED_INDEX_H
#define EMBEDDED_INDEX_H

#include "core/interfaces/iindex_service_connector.h"
#include "local/collections_explorer.h"
#include "local/collections_catalogue.h"
#include "sqlite_granules_index.h"
#include "platform.h"

namespace SEAScope
{
    // Forward declarations
    class ILogHandler;
    class IPathHandler;
    class Settings;

	class EmbeddedIndex final: public virtual IIndexServiceConnector
	{
		public:
			EmbeddedIndex();
			~EmbeddedIndex() override;

			bool init(const Settings&, ILogHandler*, IPathHandler*, IDialogHandler*) override;
            bool reset() override;
			bool searchGranules( const SearchCriteria&
									   , std::vector<GranuleMetadata>&) const override;
			std::array<bool,2> searchPrevNext( timestamp_t
									   , const std::set<CollectionId>&
									   , timestamp_t&, timestamp_t&, timestamp_t) const override;
			bool searchClosestBound( timestamp_t
								   , const std::set<CollectionId>&
								   , timestamp_t&
                                   , const timestamp_t
                                   , const timestamp_t) const override;
			bool getDataCoverage(const std::set<CollectionId>&, EventsCoverage&) const override;
			bool getDataCoverage(const CollectionId&, EventsCoverage&) const override;
			bool listCollections(std::set<std::uint32_t>&) const  override;
			bool getCollectionCfg(std::uint32_t, CollectionCfg&, std::vector<RenderingCfg*>&) const  override;
            bool saveVariableRenderingCfg(const SEAScope::RenderingCfg*) const override;
			bool canHandle(const std::string&) const override;

			static EmbeddedIndex* create();

        protected:
            SQLiteGranulesIndex _index;
            CollectionsCatalogue _catalogue;
			CollectionsExplorer _explorer;

	};
};
#endif
