// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef GCPS_COMPUTER_H
#define GCPS_COMPUTER_H

#include <cstdint>
#include <vector>

namespace SEAScope
{
    // Forward declarations
    struct Shape;

    enum class CoordinatesPlacement : std::uint8_t {
        NONE = 0,
        START = 1,
        MIDDLE = 2,
        END = 3,
        COUNT = 4
    };

    namespace GCPsComputer
    {
        bool getGCPs(std::uint64_t timeValue,
                     std::vector<double>& inputMajorAxisValues,
                     std::vector<double>& inputMinorAxisValues,
                     CoordinatesPlacement inputMajorAxisCoordinatesPlacement,
                     CoordinatesPlacement inputMinorAxisCoordinatesPlacement,
                     std::uint16_t outputMajorAxisSpacing,
                     std::uint16_t outputMinorAxisSpacing,
                     Shape& output);
    };
}

#endif
