// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NETCDF_UTILS_H
#define NETCDF_UTILS_H

#include <ctime>
#include <string>
#include "core/types.h"

namespace netCDF
{
  class NcVar;
  class NcVarAtt;
  class NcFile;
}

namespace SEAScope
{
    struct IDFDescriptor;
    class GranuleMetadata;

    struct IDFOptionalAttributes
    {
        bool hasLatMin;
        float latMin;
        bool hasLatMax;
        float latMax;
        bool hasLonMin;
        float lonMin;
        bool hasLonMax;
        float lonMax;
        std::string title;
        std::string institution;
        std::string comment;
        std::string fileId;
        std::string productVersion;
        std::string creatorEmail;
        std::string stationId;
        std::string platform;
        std::string sensor;
    };

    netCDF::NcVarAtt readAttribute(netCDF::NcVar&, const std::string&, bool&);

    /*
    * Read the value of the attribute. char array or string types are valid.
    * If the type is nc_Type::string, take only the first string.
    */
    bool readAttributeValueString(netCDF::NcFile& fileHandler, const char* name, std::string& str);
    bool readAttributeValueFloat(netCDF::NcFile& fileHandler, const char* name, float val);

    bool extractOptionalAttributes(netCDF::NcFile&, IDFOptionalAttributes&);
    bool str2timestamp(const std::string& timeStr, std::time_t& result);
    bool parseTimeUnits(const std::string&, std::time_t& epoch, double& multiplier);
    bool getDepthIndex(const netCDF::NcFile& fileHandler, const std::string& depthName, float depthValue, std::size_t& result);

    /*
    * Try to get a NcVar from a NcFile, if the NcVar is null throw an exception.
    */
    netCDF::NcVar getVarFromNcFile(netCDF::NcFile& file, const char* name);
}
#endif
