// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef READER_CFG_H
#define READER_CFG_H

namespace SEAScope
{
    class ReaderCfg
    {
        public:
            void setTimeVariable(const std::string& value) {this->_timeVariable = value;}
            const std::string& getTimeVariable() const {return this->_timeVariable;}

            void setLatVariable(const std::string& value) {this->_latVariable = value;}
            const std::string& getLatVariable() const {return this->_latVariable;}

            void setLonVariable(const std::string& value) {this->_lonVariable = value;}
            const std::string& getLonVariable() const {return this->_lonVariable;}

            void setGCPsMajorAxisSpacing(std::uint16_t value) {this->_gcpsMajorAxisSpacing = value;}
            std::uint16_t getGCPsMajorAxisSpacing() const {return this->_gcpsMajorAxisSpacing;}

            void setGCPsMinorAxisSpacing(std::uint16_t value) {this->_gcpsMinorAxisSpacing = value;}
            std::uint16_t getGCPsMinorAxisSpacing() const {return this->_gcpsMinorAxisSpacing;}

            void setRelativeTimeCoverage(std::int64_t startOffset, std::int64_t stopOffset) {this->_relativeTimeCoverageStart = startOffset; this->_relativeTimeCoverageEnd = stopOffset; this->_hasRelativeTimeCoverage = true;}
            std::int64_t getRelativeTimeCoverageStart() const {return this->_relativeTimeCoverageStart;}
            std::int64_t getRelativeTimeCoverageEnd() const {return this->_relativeTimeCoverageEnd;}
            bool hasRelativeTimeCoverage() const {return this->_hasRelativeTimeCoverage;}

            void setDepth(const std::string& name, float value) {this->_depthVariable = name; this->_depthValue = value;}
            const std::string& getDepthVariable() const {return this->_depthVariable;}
            float getDepthValue() const {return this->_depthValue;}

            void setType(const std::string& value) {this->_type = value;}
            const std::string& getType() const {return this->_type;}

        protected:
            std::string _type = "idf";
            bool _hasRelativeTimeCoverage = false;
            std::string _timeVariable = "time";
            std::string _latVariable = "latitude";
            std::string _lonVariable = "longitude";
            std::string _depthVariable = "";
            std::uint16_t _gcpsMajorAxisSpacing = 32;
            std::uint16_t _gcpsMinorAxisSpacing = 32;
            float _depthValue = 0.f;
            std::int64_t _relativeTimeCoverageStart = 0;
            std::int64_t _relativeTimeCoverageEnd = 0;
    };
}

#endif
