// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2025 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SIMPLE_NETCDF_READER_H
#define SIMPLE_NETCDF_READER_H

#include "local/ireader.h"

namespace netCDF
{
    class NcFile;
}

namespace SEAScope
{
    class SimpleNetCDFReader: public virtual IReader
    {
        public:
            ~SimpleNetCDFReader() override;

			bool extractGranuleInfo(const std::string&, std::vector<GranuleMetadata>&, const ReaderCfg&) const override;
			bool extractGranuleData(const std::string&, DataModel, const std::unordered_map<std::string, DataBucket*>&) const override;

        protected:
            bool createGranule(netCDF::NcFile& fileHandler,
                               const std::string& path,
                               const ReaderCfg& readerCfg,
                               GranuleMetadata& granule,
                               const Shape& shape,
                               float resolution,
                               int timeIndex,
                               int depthIndex,
                               std::time_t,
                               double,
                               std::uint32_t dataMajorAxisSize,
                               std::uint32_t dataMinorAxisSize) const;
    };
}

#endif
