:: =============================================================================
:: @author: <guillaume.leseach@oceandatalab.com>
:: @date: 2019-06-28
:: 
:: This file is part of SEAScope, a 3D visualisation and analysis application
:: for satellite, in-situ and numerical model data.
::
:: Copyright (C) 2014-2025 OceanDataLab
::
:: SEAScope is free software: you can redistribute it and/or modify it under
:: the terms of the GNU Lesser General Public License as published by the Free
:: Software Foundation, either version 3 of the License, or (at your option)
:: any later version.
::
:: SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
:: WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
:: FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
:: details.
::
:: You should have received a copy of the GNU Lesser General Public License
:: along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
:: =============================================================================
@echo off
setlocal
SET _devdir=%~dp0
SET _devdir=%_devdir:~0,-1%
for %%i in ("%_devdir%") do set _projectdir=%%~dpi
SET _pluginsdir=%_projectdir:~0,-1%

:: Check if arg1 and arg2 exist
if "%1" == "" (
	call :missing_argument 
	goto :eof
) else (
	if '%2%' == '' (
		call :missing_argument 
		goto :eof
	) else (
		if '%3%' == '' (
			call :missing_argument 
			goto :eof
		)
	)
)

SET _compilationTarget=release

:: Check if arg1 contains debug or release string
if "%1" == "debug" (
	SET _compilationTarget=Debug
) else (
	if "%1" == "release" (
		SET _compilationTarget=Release
	) else (
		echo Error arg 1. Invalid compilation mode. Use debug or release.
		goto :eof
	)
)

:: Check if path in arg2 is valid
if not exist %2 (
	echo Error arg 2. Invalid path.
	goto :eof
)

:: Set all paths
SET netcdfdir=%2
SET WORKSPACE=%_pluginsdir%\thirdparty
SET SEASCOPE_WORKSPACE=%3
SET SEASCOPE_SRC_DIR=%SEASCOPE_WORKSPACE%\cpp\src
SET SEASCOPE_INC_DIR=%SEASCOPE_WORKSPACE%\cpp\include
SET _bindir=%WORKSPACE%\bin\%_compilationTarget%
SET _exedir=%WORKSPACE%\bin\exe
SET _libdir=%WORKSPACE%\lib\%_compilationTarget%
SET _pluginsbindir=%_pluginsdir%\bin\%_compilationTarget%

:: Create directories
cd %_pluginsdir%
mkdir bin
mkdir %_pluginsbindir%
mkdir thirdparty
mkdir tmp
mkdir tmp\src
mkdir build

cd %WORKSPACE%
mkdir bin
mkdir %_bindir%
mkdir %_exedir%
mkdir lib
mkdir %_libdir%
mkdir tmp
mkdir tmp\src
:: Make a junction with the NetCDF directory in thirdparty directory
mklink /j netCDF %netcdfdir%

:: Set path for client sources
SET CLIENT_SRC_DIR=%_pluginsdir%\tmp\src

:: Set paths for libraries
SET _netcdfcxxdir=%WORKSPACE%\netcdf-cxx4
SET _tmpdir=%WORKSPACE%\tmp

::
:: Libraries name
::

SET _netcdfcxxlib=netcdf-cxx4.lib

:: git url for libraries
SET _netcdfcxxurl=https://github.com/Unidata/netcdf-cxx4/archive/v4.3.0.zip

::
::
::	Downloads
::
::

:: NETCDF-CXX
cd %WORKSPACE%
if not exist %_netcdfcxxdir% (
	powershell -Command "(New-Object Net.WebClient).DownloadFile('%_netcdfcxxurl%', '%_tmpdir%\netcdfcxx.zip')"
	powershell.exe -NoP -NonI -Command "Expand-Archive -Path '%_tmpdir%\netcdfcxx.zip' -DestinationPath '%WORKSPACE%'"
	rename netcdf-cxx4-4.3.0 netcdf-cxx4
	cd %_netcdfcxxdir%
	cmake -G "Visual Studio 17 2022" -A x64 -DBUILD_SHARED_LIBS=OFF .
)

::
::
::	Compilation
::
::

:: NETCDF-CXX
if not exist %_libdir%\%_compilationTarget%\%_netcdfcxxlib% (
	cd %_netcdfcxxdir%
	msbuild cxx4\netcdf-cxx4.vcxproj /property:Configuration=%_compilationTarget%
	copy cxx4\%_compilationTarget%\%_netcdfcxxlib% %_libdir%
)

::
::
::	Src copy
::
::

copy "%SEASCOPE_SRC_DIR%\core\rendering_cfg.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\log.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\simplelogger.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\filter_logger.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\..\win32\src\path_impl.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\path.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\data\image.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\settings.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\timeline_cfg.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\flatbuffers_granules_index.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\sqlite_granules_index.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\sqlite_granules_index_schema.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\sqlite_utils.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\granule_metadata.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\local\collections_catalogue.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\utils.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\api_in.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\api_out.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\igranules_index.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\ilog_handler.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\ipath_handler.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\idatastream_service_connector.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\iindex_service_connector.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\data\events_coverage.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\time_utils.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\dialog.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\idialog_handler.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\data_bucket.cpp" "%CLIENT_SRC_DIR%"

:: Script exit
EXIT /B 0

::
:: Function
::

:missing_argument
echo An argument is missing.
echo First argument is compilation mode (debug / release)
echo Second argument is NetCDF path
echo Third argument is SEAScope dir path
EXIT /B 0
